/*
  Broadcast 2.0 multitrack audio editing
  (c) 1997 Adam Williams

  This program is distributed with the intent that it will be useful, without
  any warranty.  The code is being updated and has many bugs which are
  constantly being elucidated so changes should not be made without notifying
  the author.
*/

/* audio.c      audio hardware interfaces for all the supported platforms */

#include "defs.h"
#include "record.h"
#include "audio.h"

#ifdef _HPUX_
int dsp;

int openaudio(){
  if((dsp = open("/dev/audio", O_RDWR | O_NDELAY)) < 0){
    printf("Can't open audio device.\n");
    close(dsp);
    return 1;
  }
  return 0;
}

int closeaudio(){
  resetaudio();
  close(dsp);
  return 0;
}

int writeaudio(char *buffer, long bufferlength){
  if(write(dsp, buffer, bufferlength) < 0){
    printf("Audio write failed\n");
    return 1;
  }
  return 0;
}

int readaudio(char *buffer, long bufferlength){
  if(read(dsp, buffer, bufferlength) < 0){
    printf("Audio read failed\n");
    return 1;
  }
  return 0;
}

void initplaybackaudio(int channels){
  ioctl(dsp, AUDIO_SET_DATA_FORMAT, AUDIO_FORMAT_LINEAR16BIT);
  ioctl(dsp, AUDIO_SET_SAMPLE_RATE, samplerate);
  ioctl(dsp, AUDIO_SET_CHANNELS, channels);
  ioctl(dsp, AUDIO_SET_TXBUFSIZE, PLAYBUFFER);
  resumeaudiooutput();
}

void pauseaudiooutput(){
   ioctl(dsp, AUDIO_PAUSE, AUDIO_TRANSMIT);
}

void resetaudio(){
   ioctl(dsp, AUDIO_RESET, AUDIO_TRANSMIT);
}

void flushaudio(){
   ioctl(dsp, AUDIO_DRAIN, AUDIO_TRANSMIT);
}

void resumeaudiooutput(){
  ioctl(dsp, AUDIO_RESUME, AUDIO_TRANSMIT);
}

long getaudiotxlength(){
  static struct audio_status status;

  if(ioctl(dsp, AUDIO_GET_STATUS, &status) < 0){
    printf("audiotxlength failed\n");
    return 0;
  }
  
  return status.transmit_buffer_count;
}

void initrecordaudio(char input, char *buffer, int channels){
  ioctl(dsp, AUDIO_SET_DATA_FORMAT, AUDIO_FORMAT_LINEAR16BIT);
  ioctl(dsp, AUDIO_SET_SAMPLE_RATE, samplerate);
  ioctl(dsp, AUDIO_SET_CHANNELS, channels);
  ioctl(dsp, AUDIO_SET_RXBUFSIZE, RECBUFFER);
  ioctl(dsp, AUDIO_RESUME, AUDIO_RECEIVE);
  if(input == LINE) ioctl(dsp, AUDIO_SET_INPUT, AUDIO_IN_LINE);
  if(input == MIC) ioctl(dsp, AUDIO_SET_INPUT, AUDIO_IN_MIKE);
}

long getaudiorxlength(int frame){
  static struct audio_status status;

  if(ioctl(dsp, AUDIO_GET_STATUS, &status) < 0){
    printf("getaudiorxlength failed\n");
    return 0;
  }
  return status.receive_buffer_count-frame;
}

void setrecordgain(int channel, int level){
  static struct audio_gain gain;

  ioctl(dsp, AUDIO_GET_GAINS, &gain);
  gain.cgain[channel].receive_gain = level / 5;
  ioctl(dsp, AUDIO_SET_GAINS, &gain);
}

void getrecordgain(int channels, int *level){
  static struct audio_gain gain;
  int i;

  ioctl(dsp, AUDIO_GET_GAINS, &gain);
  for(i = 0; i < channels; i++){
    *level++ = gain.cgain[i].receive_gain  * 5;
  }
}
#endif









#ifdef _LINUX_
#include <sys/soundcard.h>

int dsp;
audio_buf_info playinfo, recinfo;
int firstbuffer;
int paused, bufferloaded;
char *tmpbuffer;
long tmplength;

/* open the audio device for reading or writing */
/* used for recording and/or playback */
int openaudio(){
	int buffer = 0xffff000e;

  if((dsp = open("/dev/dsp", O_RDWR | O_NDELAY)) < 0){
    printf("Can't open audio device.\n");
    close(dsp);
    return 1;
  }
     
	if(duplexenable && ioctl(dsp, SNDCTL_DSP_SETDUPLEX, 0) == -1){
		printf("Couldn't enable full duplex.\n");
		duplexenable = 0;
	}

/* 	if(ioctl(dsp, SNDCTL_DSP_SETFRAGMENT, &buffer)==-1)
	{
		printf("Can't set buffer size.\n");
	}
 */
	paused = 0;
	bufferloaded = 0;
  return 0;
}

/* close the audio device */
int closeaudio(){
  resetaudio();
  close(dsp);
  return 0;
}

/* initialize the audio device for playback */
/* with channels number of channels */
/* must use little endian 16 bit */
/* use PLAYBUFFER as size of buffer in bytes */
void initplaybackaudio(int channels){
	int format = AFMT_S16_LE;

  if(ioctl(dsp, SNDCTL_DSP_SETFMT, &format) < 0)
     printf("playback file format failed\n");
  if(ioctl(dsp, SNDCTL_DSP_CHANNELS, &channels) < 0)
     printf("playback channel allocation failed\n");
  if(ioctl(dsp, SNDCTL_DSP_SPEED, &samplerate) < 0)
     printf("playback sample rate set failed\n");

  ioctl(dsp, SNDCTL_DSP_GETOSPACE, &playinfo);

/*   printf("%d %d %d %d\n", playinfo.fragments, 
  	playinfo.fragstotal, playinfo.fragsize, playinfo.bytes);
 */
	firstbuffer = 1;
}

/* write buffer of length bufferlength in bytes to the audio device */
int writeaudio(char *buffer, long bufferlength){
	if(!paused){
		if(write(dsp, buffer, bufferlength) < 0){
			printf("Audio write failed\n");
		}
	}else{
		tmpbuffer = buffer;
		tmplength = bufferlength;
		bufferloaded = 1;
	}
  return 0;
}

/* write the next fragment if not paused */
/* return the number of bytes remaining in the audio device to be played */
long getaudiotxlength(){
	static audio_buf_info info;
	
	ioctl(dsp, SNDCTL_DSP_GETOSPACE, &info);

/*
	printf("%d\n", info.bytes);
*/

  if(!firstbuffer) return playinfo.bytes - info.bytes;
  else{
  	return 0;
  }
}

/* initialize the audio input for recording and start recording */
/* input is ignored in linux */
void initrecordaudio(char input, char *buffer, int channels){
	int format = AFMT_S16_LE;
	char *startbuffer;

  if(ioctl(dsp, SNDCTL_DSP_SETFMT, &format) < 0)
     printf("record file format failed\n");
  if(ioctl(dsp, SNDCTL_DSP_CHANNELS, &channels) < 0)
     printf("record channel allocation failed\n");
  if(ioctl(dsp, SNDCTL_DSP_SPEED, &samplerate) < 0)
     printf("record sample rate set failed\n");
     
  ioctl(dsp, SNDCTL_DSP_GETISPACE, &recinfo);
  
  recinfo.bytes = recinfo.fragsize * recinfo.fragstotal;

/*   printf("record %d %d %d %d\n", recinfo.fragments, 
  	recinfo.fragstotal, recinfo.fragsize, recinfo.bytes);
 */  	
	startbuffer = calloc(64000, 1);
  if((read(dsp, startbuffer, 64000)) < 0){       /* start the recording */
    printf("Audio startup failed\n");
  }
	free(startbuffer);
}

/* reset the variables since the buffer is already loaded */	
/* read buffer of length bufferlength from audio device */
int readaudio(char *buffer, long bufferlength){
  if((read(dsp, buffer, bufferlength)) < 0){
    printf("Audio read failed\n");
    return 1;
  }

  return 0;
}

/* read the next fragment and */
/* return number of bytes read into the input buffer since the last readaudio */
long getaudiorxlength(int frame){
	static audio_buf_info info;
	
	ioctl(dsp, SNDCTL_DSP_GETISPACE, &info);

 	/* printf("%d ", info.bytes);     fflush(stdout); */
 
	if(info.bytes <= 0) info.bytes = 131072;
  return info.bytes/* recupdate */;
}

void resetaudio(){
   ioctl(dsp, SNDCTL_DSP_RESET);
}

void flushaudio(){
   ioctl(dsp, SNDCTL_DSP_SYNC, 0);
}

/*
It is currently NOT possible
to stop or restart a device that has already been active
without first reopening the device file.  
*/
void pauseaudiooutput(){
	paused = 1;
	bufferloaded = 0;
}

void resumeaudiooutput(){
	if(paused){
		paused = 0;
		if(bufferloaded){
			bufferloaded = 0;
			writeaudio(tmpbuffer, tmplength);
		}
	}
}

/* use a third party mixer */
void setrecordgain(int channel, int level){
}

void getrecordgain(int channels, int *level){
}
#endif








#ifdef _UNDEFINED_
int openaudio(){
}

int closeaudio(){
}

int writeaudio(char *buffer, long bufferlength){
}

int readaudio(char *buffer, long bufferlength){
}

void initplaybackaudio(int channels){
}

void initrecordaudio(char input, char *buffer, int channels){
}

void resetaudio(){
}

void flushaudio(){
}

void pauseaudiooutput(){
}

void resumeaudiooutput(){
}

long getaudiotxlength(){
}

long getaudiorxlength(int frame){
}

void setrecordgain(int channel, int level){
}

void getrecordgain(int channels, int *level){
}
#endif
