/*
  Broadcast 2.0 multitrack audio editing
  (c) 1997 Adam Williams

  This program is distributed with the intent that it will be useful, without
  any warranty.  The code is being updated and has many bugs which are
  constantly being elucidated so changes should not be made without notifying
  the author.
*/

/* automate.c      automation handlers, editors, persistent storage  */


#include "automate.h"
#include "undo.h"
#include "database.h"
#include "record.h"
#include "play.h"
#include "units.h"

struct autoevent_ autoevent = {0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0};

void initautoevents(){
  static i;

  autoevent.play = 0;
  autoevent.showmute = 0;
  autoevent.showfade = 0;
  autoevent.showinvert = 0;
  autoevent.automation = 0;
  autoevent.showpan = malloc(outchannels);
  autoevent.selectpan = malloc(outchannels);
  autoevent.selectmute = 0;
  autoevent.selectfade = 0;
  autoevent.selectinvert = 0;
  for(i = 0; i < outchannels; i++){
    autoevent.showpan[i] = 0;
    autoevent.selectpan[i] = 0;
  }
}

/* range of values for floating point autos */
  const float maxfloat = 3;
  const float minfloat = INFINITYGAIN;

void loadshowpans(FILE *in, float value){
  int i;

  autoevent.showpan[0] = value;
  for(i = 1; i < outchannels; i++){
    fscanf(in, "%d", &value);
    autoevent.showpan[i] = value;
  }
}

void saveshowpans(FILE *out){
  int i;

  fprintf(out, "SHOWPANS");
  for(i = 0; i < outchannels; i++){
    fprintf(out, " %d", autoevent.showpan[i]);
  }
  fprintf(out, "\n");
}

int totalautos(struct Auto *firstauto){
  static struct Auto *currentauto;
  static int i;

  for(i = 0, currentauto = firstauto; currentauto; currentauto = currentauto->nextauto, i++)
    ;
  return i;
}

/* give the auto containing x */
struct Auto *autoof(long x, struct Auto *firstauto){
  static struct Auto *currentauto;

  for(currentauto = firstauto;
    currentauto && currentauto->position < x;
    currentauto = currentauto->nextauto)
    ;

  if(currentauto && currentauto->position > x)
    currentauto = currentauto->prevauto;

  return currentauto;
}

void initautomation(int starttrack, long length){
  struct Track *track;
  int i;

  for(track = trackof(starttrack); track; track = track->nexttrack){
/*=============================allocate memory==============================*/
    track->firstautomute = malloc(sizeof(struct Auto));
    track->lastautomute = malloc(sizeof(struct Auto));

    track->firstautofade = malloc(sizeof(struct Auto));
    track->lastautofade = malloc(sizeof(struct Auto));

    track->firstautopan = malloc(sizeof(struct Auto) * outchannels);
    track->lastautopan = malloc(sizeof(struct Auto) * outchannels);
    for(i = 0; i < outchannels; i++){
      track->firstautopan[i] = malloc(sizeof(struct Auto));
      track->lastautopan[i] = malloc(sizeof(struct Auto));
    }

    track->firstautoinvert = malloc(sizeof(struct Auto));
    track->lastautoinvert = malloc(sizeof(struct Auto));

/*================================position pointers=======================*/

    track->firstautomute->prevauto = 0;
    track->firstautomute->nextauto = track->lastautomute;
    track->lastautomute->prevauto = track->firstautomute;
    track->lastautomute->nextauto = 0;

    track->firstautofade->prevauto = 0;
    track->firstautofade->nextauto = track->lastautofade;
    track->lastautofade->prevauto = track->firstautofade;
    track->lastautofade->nextauto = 0;

    for(i = 0; i < outchannels; i++){
      track->firstautopan[i]->prevauto = 0;
      track->firstautopan[i]->nextauto = track->lastautopan[i];
      track->lastautopan[i]->prevauto = track->firstautopan[i];
      track->lastautopan[i]->nextauto = 0;
    }

    track->firstautoinvert->prevauto = 0;
    track->firstautoinvert->nextauto = track->lastautoinvert;
    track->lastautoinvert->prevauto = track->firstautoinvert;
    track->lastautoinvert->nextauto = 0;

/*=================================store default values======================*/
    track->firstautomute->value = 0;
    track->lastautomute->value = 0;
    track->firstautomute->position = 0;
    track->lastautomute->position = length;

    track->firstautofade->value = 0;
    track->lastautofade->value = 0;
    track->firstautofade->position = 0;
    track->lastautofade->position = length;

    for(i = 0; i < outchannels; i++){
      track->firstautopan[i]->value = 0;
      track->lastautopan[i]->value = 0;
      track->firstautopan[i]->position = 0;
      track->lastautopan[i]->position = length;
    }

    track->firstautoinvert->value = 0;
    track->lastautoinvert->value = 0;
    track->firstautoinvert->position = 0;
    track->lastautoinvert->position = length;
  }
}

void destroyautos(struct Auto *firstauto){
  static struct Auto *temp, *currentauto;

  for(currentauto = firstauto; currentauto; currentauto = temp){
    temp = currentauto->nextauto;
    free(currentauto);
  }
}

void destroyautomation(int starttrack){
  struct Track *track;
  int i;

  for(track = trackof(starttrack); track; track = track->nexttrack){
    destroyautos(track->firstautomute);
    destroyautos(track->firstautofade);

    for(i = 0; i < outchannels; i++){
      destroyautos(track->firstautopan[i]);
    }
    free(track->firstautopan);
    free(track->lastautopan);

    destroyautos(track->firstautoinvert);
  }
}

struct Auto *createnewauto(struct Auto *auto1, struct Auto *auto2){
  struct Auto *currentauto;

  currentauto = malloc(sizeof(struct Auto));
  currentauto->prevauto = auto1;
  currentauto->nextauto = auto2;
  auto1->nextauto = currentauto;
  auto2->prevauto = currentauto;
  return currentauto;
}

void deleteauto(struct Auto *currentauto){
  struct Auto *frontauto, *backauto;

  frontauto = currentauto->prevauto;
  backauto = currentauto->nextauto;
  frontauto->nextauto = backauto;
  backauto->prevauto = frontauto;
  free(currentauto);
}

void optimizeautos(struct Auto *firstauto, struct Auto *lastauto,
  struct Track *currenttrack, char operation)
{
  static struct Auto *prevauto, *currentauto, *nextauto;
  static int k;
  static long endoftrack;

  if(operation != EFFECT){
    do{        /* test for position beyond track length */
      currentauto = firstauto;

			if(currenttrack->lastedit){
      	endoftrack = currenttrack->lastedit->start1 +
        	currenttrack->lastedit->length;
      }else{
       	endoftrack = 0;
      }
      k = 0;

      while(currentauto && !k){
        /* use > to prevent infinite loop */

        if(currentauto->position > endoftrack){
          if(currentauto != lastauto){
            deleteauto(currentauto);
          }else{
            currentauto->position = endoftrack;
          }
          k = 1;
        }
        currentauto = currentauto->nextauto;
      }
    }while(k);
  }

  do{       /* test for same value */
    prevauto = firstauto;
    if(!prevauto) goto end;
    currentauto = prevauto->nextauto;
    if(!currentauto) goto end;
    nextauto = currentauto->nextauto;
    if(!nextauto) goto end;
    k = 0;

    while(nextauto && !k){
      if(prevauto->value == currentauto->value &&
        currentauto->value == nextauto->value){
        deleteauto(currentauto);
        k = 1;
      }
      prevauto = currentauto;
      currentauto = nextauto;
      nextauto = nextauto->nextauto;
    }
  }while(k);

  do{        /* test for same position */
    prevauto = firstauto;
    if(!prevauto) goto end;
    currentauto = prevauto->nextauto;
    if(!currentauto) goto end;
    nextauto = currentauto->nextauto;
    if(!nextauto) goto end;
    k = 0;

    while(nextauto && !k){
      if(prevauto->position == currentauto->position){
        deleteauto(currentauto);
        k = 1;
      }
      prevauto = currentauto;
      currentauto = nextauto;
      nextauto = nextauto->nextauto;
    }
    if(lastauto->position == lastauto->prevauto->position &&
      lastauto->position != firstauto->position)
      deleteauto(lastauto->prevauto);
  }while(k);
end:
  ;
}

void optimizeautomation(char operation){
  struct Track *track;
  char i;

  for(track = firsttrack; track; track = track->nexttrack){
    optimizeautos(track->firstautomute, track->lastautomute, track, operation);
    optimizeautos(track->firstautofade, track->lastautofade, track, operation);
    optimizeautos(track->firstautoinvert, track->lastautoinvert, track,
      operation);
    for(i = 0; i < outchannels; i++){
      optimizeautos(track->firstautopan[i], track->lastautopan[i], track,
        operation);
    }
  }
}

void readautos(FILE *in, struct Auto **firstauto, struct Auto **lastauto){
  static struct Auto *currentauto;
  static TWO newautos, autonumber;

  newautos = gettwo(in);

  for(autonumber = 0; autonumber < newautos; autonumber++){
    if(autonumber == 0){
      *firstauto = *lastauto = currentauto = malloc(sizeof(struct Auto));
      currentauto->prevauto = 0;
      currentauto->nextauto = 0;
    }else{
      currentauto = malloc(sizeof(struct Auto));
      currentauto->prevauto = *lastauto;
      currentauto->nextauto = (*lastauto)->nextauto;
      (*lastauto)->nextauto = currentauto;
      (*lastauto) = currentauto;
    }
    currentauto->position = getfour(in);
    currentauto->value = getfloat(in);
  }
}

void readautomation(FILE *in, int firsttrack, int newtracks, int newoutchannels){
  static TWO newautos;
  int tracknumber, i;
  struct Track *track;
  struct Auto *frontauto, *backauto;

  for(track = trackof(firsttrack), tracknumber = 0;
    track && tracknumber < newtracks;
    track = track->nexttrack, tracknumber++)
  {
    readautos(in, &track->firstautomute, &track->lastautomute);
    readautos(in, &track->firstautofade, &track->lastautofade);
    track->firstautopan = malloc(sizeof(struct Auto) * outchannels);
    track->lastautopan = malloc(sizeof(struct Auto) * outchannels);
                                       /* read pans */
    for(i = 0; i < newoutchannels && i < outchannels; i++){
      readautos(in, &track->firstautopan[i], &track->lastautopan[i]);
    }
    for(; i < newoutchannels; i++){        /* skip extra pans */
      newautos = gettwo(in);
      fseek(in, newautos * 8, SEEK_CUR);
    }
    for(; i < outchannels; i++){           /* fill extra pans */
      frontauto = track->firstautopan[i] = malloc(8 * 2);
      frontauto->prevauto = 0;
      frontauto->position = 0;
      backauto = track->lastautopan[i] = malloc(8 * 2);
      backauto->prevauto = frontauto;
      frontauto->nextauto = backauto;
      backauto->nextauto = 0;

      frontauto->value = 0;
      backauto->value = 0;
      frontauto->position = 0;
      backauto->position = track->lastedit->start1 + track->lastedit->length;
    }

    readautos(in, &track->firstautoinvert, &track->lastautoinvert);
  }
}

void writeautos(FILE *out, struct Auto *firstauto){
  static struct Auto *currentauto;

  puttwo(totalautos(firstauto), out);
  for(currentauto = firstauto; currentauto; currentauto = currentauto->nextauto){
    putfour(currentauto->position, out);
    putfloat(currentauto->value, out);
  }
}

void writeautomation(FILE *out){
  struct Track *track;
  int i;

  for(track = firsttrack; track; track = track->nexttrack){
    writeautos(out, track->firstautomute);
    writeautos(out, track->firstautofade);

    for(i = 0; i < outchannels; i++){
      writeautos(out, track->firstautopan[i]);
    }

    writeautos(out, track->firstautoinvert);
  }
}

void writeeffectautos(FILE *out, FOUR length, float value){
  puttwo(2, out);
  putfour(0, out);
  putfloat(value, out);
  putfour(length, out);
  putfloat(value, out);
}

void writeeffectautomation(FILE *out, struct CastHeader *castheader){
  struct Track *currenttrack;
  int i;

  for(currenttrack = firsttrack;                 /* write automation */
    currenttrack;
    currenttrack = currenttrack->nexttrack)
  {
    if(currenttrack->record){
      writeeffectautos(out, castheader->length / 2, 1);     /* mute */
      writeeffectautos(out, castheader->length / 2, 1);     /* fade */
      for(i = 0; i < outchannels; i++){
        writeeffectautos(out, castheader->length / 2, 1);   /* pan */
      }
      writeeffectautos(out, castheader->length / 2, 0);     /* invert */
      castheader++;
    }
  }
}

/* move autos for edit alterations */
void shiftautos(struct Auto *firstauto, struct Auto *lastauto,
  long start, long length)
{
	static struct Auto *currentauto, *tempauto;
	
	currentauto = autoof(start, firstauto);

	if(currentauto && currentauto != lastauto) currentauto = currentauto->nextauto;
	
	while(currentauto)
	{
		currentauto->position += length;
		       /* in case of a negative length delete overlapping autos */
		if(currentauto->position < start && currentauto != lastauto) {
			tempauto = currentauto->nextauto;
			deleteauto(currentauto);
			currentauto = tempauto;
		}else{
			currentauto = currentauto->nextauto;
		}
	}
}

/* move automation in track by length */
void shiftautomation(struct Track *currenttrack, long start, long length)
{
	int i;
	/* in case of an edit shift before start of track */
	if(start < 0) start = 0;
	
	shiftautos(currenttrack->firstautomute, 
		currenttrack->lastautomute, start, length);
	shiftautos(currenttrack->firstautofade, 
		currenttrack->lastautofade, start, length);
	shiftautos(currenttrack->firstautoinvert,
		currenttrack->lastautoinvert, start, length);
	for(i = 0; i < outchannels; i++){
		shiftautos(currenttrack->firstautopan[i], 
			currenttrack->lastautopan[i], start, length);
	}
	optimizeautomation(0);
}

void pasteautos(struct Track *currenttrack, struct Auto *firstauto, struct Auto *lastauto, FILE *clipboard,
  long start, long length, char operation)
{
  static struct Auto *frontauto, *currentauto, *backauto;

  frontauto = autoof(start, firstauto);
  if(!frontauto){      /* insert before lastauto if beyond end of track*/
    frontauto = lastauto->prevauto;
    length += start - lastauto->position;
  }

  backauto = frontauto->nextauto;
  if(!backauto){
    backauto = frontauto;
    frontauto = frontauto->prevauto;
  }

  if(operation == PASTE){                                    /* paste operation */
    static TWO newautos;
    static int i;
    static float value;
    static FOUR position;

    newautos = gettwo(clipboard);

    for(i = 0; i < newautos; i++){
      position = getfour(clipboard) + start;
      value = getfloat(clipboard);

      if(firstauto->position == position){
        firstauto->value = value;
      }else
      if(lastauto->position + length == position){

        lastauto->value = value;
      }else{
        currentauto = createnewauto(frontauto, backauto);
        currentauto->position = position;
        currentauto->value = value;
        frontauto = currentauto;
      }
    }
    for(currentauto = backauto; currentauto; currentauto = currentauto->nextauto){
      currentauto->position += length;
    }
  }else
  if(operation == RECORD){                /* record operation */
                                          /* shift autos */
    for(currentauto = backauto; currentauto; currentauto =
      currentauto->nextauto)
    {
      currentauto->position += length;
    }
  }else
                               /* effect operation changes length only */
                               /* if the effect extended the last edit */
  if(operation == EFFECT){
                       /* change length of automation track */
    static long tracklength;

		if(currenttrack->lastedit){
    	tracklength = currenttrack->lastedit->start1 +
      	currenttrack->lastedit->length;

    	if(lastauto->position < tracklength) lastauto->position = tracklength;
		}
  }
}

/* paste automation data from clipboard file */
/* needs length because selection for paste can have arbitrary length */
void pasteautomation(FILE *clipboard, long start, long length, char operation){
  struct Track *track;
  TWO newautopans;
  int i;
  TWO newtracks;

  if(operation == PASTE){
    newautopans = gettwo(clipboard);                         /* new pans */
    newtracks = gettwo(clipboard);                           /* new tracks */
  }else{
    newautopans = outchannels;
    newtracks = recevent.channels;
  }

  for(track = firsttrack; track; track = track->nexttrack){
    if(track->record){
      pasteautos(track, track->firstautomute, track->lastautomute,
        clipboard, start, length, operation);
      pasteautos(track, track->firstautofade, track->lastautofade,
        clipboard, start, length, operation);
      pasteautos(track, track->firstautoinvert, track->lastautoinvert,
        clipboard, start, length, operation);                /* paste pans */
      for(i = 0; i < outchannels && i < newautopans; i++)
        pasteautos(track, track->firstautopan[i], track->lastautopan[i],
          clipboard, start, length, operation);
      for(; i < outchannels; i++){                     /* make up filler pans */
                       /* since length must be changed */
        pasteautos(track, track->firstautopan[i], track->lastautopan[i],
          clipboard, start, length, RECORD);
      }
    }
  }

  optimizeautomation(operation);
}

void copyautos(struct Auto *firstauto, FILE *clipboard, long start, long end)
{
  static struct Auto *currentauto;
  static TWO totalautos;

  totalautos = 0;
  currentauto = autoof(start, firstauto);
  if(!currentauto){                           /* start is after track end */
    puttwo(0, clipboard);
  }else{                                             /* start is in track */
    for(; currentauto && currentauto->position <= end; currentauto = currentauto->nextauto){
      if(currentauto->position >= start) totalautos++;
    }

    puttwo(totalautos, clipboard);                /* put total autos */
    currentauto = autoof(start, firstauto);
    for(; currentauto && currentauto->position <= end;
      currentauto = currentauto->nextauto)
    {
      if(currentauto->position >= start){
        putfour(currentauto->position - start, clipboard);
        putfloat(currentauto->value, clipboard);
      }
    }
  }
}

                /* write the automation selected to the clipboard file */
void copyautomation(FILE *clipboard, long start, long end){
  struct Track *track;
  FOUR fileposition;
  TWO totaltracks;
  int i;

  puttwo(outchannels, clipboard);                      /* # of autopans */
  totaltracks = recordabletracks();
  puttwo(totaltracks, clipboard);

  for(track = firsttrack; track; track = track->nexttrack){
    if(track->record){
      copyautos(track->firstautomute, clipboard, start, end);
      copyautos(track->firstautofade, clipboard, start, end);
      copyautos(track->firstautoinvert, clipboard, start, end);
      for(i = 0; i < outchannels; i++){
        copyautos(track->firstautopan[i], clipboard, start, end);
      }
    }
  }
}

void clearautos(struct Auto *firstauto, struct Auto *lastauto,
  long start, long end, char operation)
{
  static struct Auto *currentauto, *temp;
  static long length;

          /* if selection is beyond track length, modify selection */
  if(lastauto->position < end) length = lastauto->position - start;
  else length = end - start;

          /* delete autos but don't delete the last auto or the first auto */

  for(currentauto = autoof(start, firstauto);
    currentauto && currentauto->position < end && currentauto != lastauto;
    currentauto = temp)
  {
    temp = currentauto->nextauto;
    if(currentauto != firstauto && currentauto->position >= start)
      deleteauto(currentauto);
  }

           /* shift autos back since audio data has changed */
  for(; currentauto; currentauto = currentauto->nextauto){
    currentauto->position -= length;
  }
}

void clearautomation(long start, long end, char operation){
  struct Track *track;
  int i;
                          /* want to keep automation if effect */
  if(operation != EFFECT){
    for(track = firsttrack; track; track = track->nexttrack){
      if(track->record){
        clearautos(track->firstautomute, track->lastautomute, start, end,
          operation);
        clearautos(track->firstautofade, track->lastautofade, start, end,
          operation);
        clearautos(track->firstautoinvert, track->lastautoinvert, start, end,
          operation);
        for(i = 0; i < outchannels; i++){
          clearautos(track->firstautopan[i], track->lastautopan[i], start, end,
            operation);
        }
      }
    }
  }

  optimizeautomation(operation);
}

void changeautomationchannels(int newchannels){
  int i;
  char *shownewpan;
  char *selectnewpan;
  struct Track *track;
  struct Auto **newfirstautopan;
  struct Auto **newlastautopan;

                                  /* update events */
  shownewpan = malloc(newchannels);
  selectnewpan = malloc(newchannels);
  for(i = 0; i < outchannels && i < newchannels; i++){
    shownewpan[i] = autoevent.showpan[i];
    selectnewpan[i] = autoevent.selectpan[i];
  }

  for(; i < newchannels; i++){
    shownewpan[i] = 0;
    selectnewpan[i] = 0;
  }

  free(autoevent.selectpan);
  free(autoevent.showpan);

  autoevent.selectpan = selectnewpan;
  autoevent.showpan = shownewpan;

                                 /* update every track */
  for(track = firsttrack; track; track = track->nexttrack){
    newfirstautopan = malloc(sizeof(struct Auto) * newchannels);
    newlastautopan = malloc(sizeof(struct Auto) * newchannels);
    for(i = 0; i < outchannels && i < newchannels; i++){
      newfirstautopan[i] = track->firstautopan[i];
      newlastautopan[i] = track->lastautopan[i];
    }
    free(track->firstautopan);
    free(track->lastautopan);
    track->firstautopan = newfirstautopan;
    track->lastautopan = newlastautopan;

    for(; i < newchannels; i++){
      track->firstautopan[i] = malloc(sizeof(struct Auto));
      track->lastautopan[i] = malloc(sizeof(struct Auto));

      track->firstautopan[i]->prevauto = 0;
      track->firstautopan[i]->nextauto = track->lastautopan[i];
      track->lastautopan[i]->prevauto = track->firstautopan[i];
      track->lastautopan[i]->nextauto = 0;

      track->firstautopan[i]->value = 1;
      track->lastautopan[i]->value = 1;
      track->firstautopan[i]->position = 0;
      
      if(track->lastedit){
      	track->lastautopan[i]->position = track->lastedit->start1 +
        	track->lastedit->length;
      }
      else track->lastautopan[i]->position = 0;
    }
  }

	if(newchannels < outchannels){
		outchannels = newchannels;
  	if(automationisvisible()){
    	drawwave(0, 0, wavewindow.w, wavewindow.h, startx, starty, NO_BLANK, FLASH,
      	DRAWCURSOR);
  	}
  }

	outchannels = newchannels;
}

/* test the handle specified by handlex and handley */
/* in the track starting at row1 */
/* return 1 if true */
int testhandle(int x, int y, int row1, int handlex, int handley){
	static int maxrow, minrow;
	
	minrow = handley - BOXSIZE / 2;
	maxrow = handley + BOXSIZE / 2;
	
	if(minrow < row1 - BOXSIZE / 2) minrow = row1 - BOXSIZE / 2;
	if(maxrow > row1 + yscaletrack - BOXSIZE / 2) 
		maxrow = row1 + yscaletrack - BOXSIZE / 2;
	
	if(x > handlex - BOXSIZE / 2 && x < handlex + BOXSIZE / 2 &&
		y > minrow && y < maxrow) return 1;
	else return 0;
}

int testtoggleautos(int x, long position, int y, int row1,
  struct Auto **currentauto, struct Auto *firstauto, struct Auto *lastauto)
{
  static struct Auto *auto1, *auto2;
  static long endx;
  static int minrow, maxrow, handlex, handley;
  	
  auto1 = autoof(position, firstauto);
  if(!auto1) auto1 = lastauto;
  auto2 = auto1->nextauto;
  endx = startx + wavewindow.w * xscale;

                 /* test handles */
  if(auto1->position >= startx){
    handlex = (auto1->position - startx) / xscale;

    if(auto1->value == 1)
      handley = row1 + TOGGLEY;
    else
      handley = row1 + yscaletrack - TOGGLEY;

    if(testhandle(x, y, row1, handlex, handley)){
      *currentauto = auto1;
      return 1;
    }
  }

  if(auto2){
    if(auto2->position <= endx){
      handlex = (auto2->position - startx) / xscale;

      if(auto2->value == 1)
        handley = row1 + TOGGLEY;
      else
        handley = row1 + yscaletrack - TOGGLEY;

      if(testhandle(x, y, row1, handlex, handley)){
        *currentauto = auto2;
        return 1;
      }
    }
    
    minrow = row1;
    
            /* test line */
    if(auto1->value == 1)
      minrow += TOGGLEY;
    else
      minrow += yscaletrack - TOGGLEY;

           /* set min and max boundaries */
		maxrow = minrow + BOXSIZE / 2;
		minrow -= BOXSIZE / 2;
		
		if(minrow < row1) minrow = row1;
		if(maxrow > row1 + yscaletrack) maxrow = row1 + yscaletrack;
    if(y >= minrow && y <= maxrow){
      *currentauto = createnewauto(auto1, auto2);
      (*currentauto)->position = position;
      (*currentauto)->value = auto1->value;
      return 1;
    }
  }
  return 0;
}

int testfloatautos(int x, long position, int y, int row1,
  struct Auto **currentauto, struct Auto *firstauto, struct Auto *lastauto)
{
  static struct Auto *auto1, *auto2;
  static int column;
  static long endx;
  static float range;
  static float slope;
  static int minrow, maxrow, handlex, handley;

	row1 += BOXSIZE / 2;
  auto1 = autoof(position, firstauto);
  if(!auto1) auto1 = lastauto;
  auto2 = auto1->nextauto;
  endx = startx + wavewindow.w * xscale;

/* value used to scale the auto value to the range of yscaletrack */
  range = (float)(yscaletrack - BOXSIZE) / (maxfloat - minfloat);

          /* test adjacent handles */
  if(auto1->position >= startx){
    handlex = (auto1->position - startx) / xscale;
    handley = row1 + yscaletrack - BOXSIZE;
    handley -= (auto1->value - minfloat) * range;

    if(testhandle(x, y, row1, handlex, handley)){
      *currentauto = auto1;
      return 1;
    }
  }

  if(auto2){
    if(auto2->position <= endx){
      handlex = (auto2->position - startx) / xscale;
      handley = row1;
      handley += yscaletrack - BOXSIZE -
      	(auto2->value - minfloat) * range;

      if(testhandle(x, y, row1, handlex, handley)){
        *currentauto = auto2;
        return 1;
      }
    }

                         /* test intervening line */
    slope = (auto2->value - auto1->value) /
      ((auto2->position - auto1->position) / xscale);

    minrow = row1;
    minrow += yscaletrack - BOXSIZE - (auto1->value - minfloat) * range;
    minrow -= (position - auto1->position) / xscale * slope * range;
    
                           /* set upper and lower boundaries */
    maxrow = minrow + BOXSIZE / 2;
    minrow -= BOXSIZE / 2;
		if(minrow < row1 - BOXSIZE / 2) minrow = row1 - BOXSIZE / 2;
    if(maxrow > row1 + yscaletrack - BOXSIZE / 2) maxrow = row1 + yscaletrack -
    	BOXSIZE / 2;

    if(y >= minrow && y <= maxrow){
      *currentauto = createnewauto(auto1, auto2);
      (*currentauto)->position = position;
      (*currentauto)->value = auto1->value +
        (position - auto1->position) * (auto2->value - auto1->value) /
        (auto2->position - auto1->position);
      return 1;
    }
  }
  return 0;
}

/* draw a handle with row1 as minimum row */
void drawhandle(int x, int y, int row1){
	static int minx, miny, maxy, w, h;
	
	minx = x - BOXSIZE / 2;
	miny = y - BOXSIZE / 2;
	maxy = miny + BOXSIZE;
	w = BOXSIZE;
	row1 += BOXSIZE / 2;
	
/*
	if(miny < row1) miny = row1;
	if(maxy > row1 + yscaletrack - BOXSIZE) maxy = row1 + yscaletrack;
*/
	
	h = maxy - miny;
	
	XFillRectangle(display, wavpixmap, gc,
  	minx, miny, w, w);
}

void drawtoggleautos(int column1, int row1, long startx,
  struct Auto *firstauto, int column2)
{
  static struct Auto *auto1, *auto2;
  static int column, column_, row;
  static long endx;

  auto1 = autoof(startx, firstauto);
  if(!auto1) goto end;
  endx = startx + (column2 - column1) * xscale;
  auto2 = auto1->nextauto;
  column = column1;

  while(auto1 && auto1->position < endx){
    if(auto1->value == 1) row = row1 + TOGGLEY;
    else row = row1 + yscaletrack - TOGGLEY;

    if(auto1->position >= startx) drawhandle(column, row, row1);

    if(auto2){
      if(auto2->position <= endx)
        column_ = (auto2->position - startx) / xscale + column1;
      else
        column_ = column2;

      XDrawLine(display, wavpixmap, gc, column, row, column_, row);
    }

    if(auto1) auto1 = auto2;
    if(auto2) auto2 = auto2->nextauto;
    column = column_;
  }
end:
  ;
}

void drawfloatautos(int column1, int row1, long startx,
  struct Auto *firstauto, int column2)
{
  static struct Auto *auto1, *auto2;
  static int column, column_, row, row_;
  static long endx;
  static float range;
  static float slope;

	row1 += BOXSIZE / 2;
  auto1 = autoof(startx, firstauto);
  if(!auto1) goto end;
  auto2 = auto1->nextauto;
  endx = startx + (column2 - column1) * xscale;
  range = (float)(yscaletrack - BOXSIZE) / (maxfloat - minfloat);
  column = column1;
  row = row1 + yscaletrack - BOXSIZE -
  	(auto1->value - minfloat) * range;

  while(auto1 && auto1->position < endx){
    if(auto1->position >= startx){
      drawhandle(column, row, row1);
    }else{
      slope = (auto2->value - auto1->value) /
        ((auto2->position - auto1->position) / xscale);

      row -= (startx - auto1->position) / xscale * slope * range;
    }

    if(auto2){
      if(auto2->value == auto1->value)
        row_ = row;
      else
        row_ = row1 + yscaletrack - BOXSIZE -
        	(auto2->value - minfloat) * range;

      if(auto2->position <= endx){
        column_ = (auto2->position - startx) / xscale + column1;
      }else{
        column_ = column2;

        slope = (auto2->value - auto1->value) /
          ((auto2->position - auto1->position) / xscale);
        if(auto2->value != auto1->value)
          row_ += (auto2->position - endx) / 
          	xscale * slope * range;
      }

      XDrawLine(display, wavpixmap, gc, column, row, column_, row_);
      column = column_;
      row = row_;
    }
    if(auto1) auto1 = auto2;
    if(auto2) auto2 = auto2->nextauto;
  }
end:
  ;
}

                              /* select an automation point */
                              /* select only one point at a time */
                              /* can only move selection with vertical boundries
                              of track and horizontal positions of adjacent
                              autos */
                              /* delete an auto by moving out of its horizontal
                              boundries */
int selectautomation(XEvent *report){
  static struct Track *track;
  static char result;
  static int x, y, row, i, tracknumber;
  static long position;
  static float range;
  static char string[32];

  result = 0;
  x = report->xbutton.x;
  y = report->xbutton.y;
  position = x * xscale + startx;
  row = 0;
  tracknumber = starty;
  range = (float)(yscaletrack - BOXSIZE) / (maxfloat - minfloat);

  for(track = trackof(starty); track && !result && row < y; track = track->nexttrack){
    if(autoevent.showmute){
      if(testtoggleautos(x, position, y, row,
        &autoevent.selected, track->firstautomute, track->lastautomute))
      {
        autoevent.firstautoselected = track->firstautomute;
        autoevent.selectmute = 1;
        result = 1;
        goto adjust;
      }
    }

    if(autoevent.showinvert){
      if(testtoggleautos(x, position, y, row,
        &autoevent.selected, track->firstautoinvert, track->lastautoinvert))
      {
        autoevent.firstautoselected = track->firstautoinvert;
        autoevent.selectinvert = 1;
        result = 1;
        goto adjust;
      }
    }

    if(autoevent.showfade){
      if(testfloatautos(x, position, y, row,
        &autoevent.selected, track->firstautofade, track->lastautofade))
      {
        autoevent.firstautoselected = track->firstautofade;
        autoevent.selectfade = 1;
        result = 1;
        goto adjust;
      }
    }

    for(i = 0; i < outchannels && !result; i++){
      if(autoevent.showpan[i])
        if(testfloatautos(x, position, y, row,
          &autoevent.selected, track->firstautopan[i], track->lastautopan[i]))
        {
          autoevent.firstautoselected = track->firstautopan[i];
          autoevent.selectpan[i] = 1;
          result = 1;
          goto adjust;
        }
    }

    row += yscaletrack;
    tracknumber++;
  }

/* =====================================user has selected a handle ===============*/
adjust:
  if(result){                   /* adjust selection */
		updateundo(TWEEKAUTOS);
		changesmade = 1;
  
    XSetForeground(display, gc, red.pixel);
    XSetFunction(display, gc, GXxor);

    if(autoevent.selectmute | autoevent.selectinvert)
      drawtoggleautos(0, row, startx, autoevent.firstautoselected,
        wavewindow.w);
    else
      drawfloatautos(0, row, startx, autoevent.firstautoselected,
        wavewindow.w);
    XSetFunction(display, gc, GXcopy);
    XCopyArea(display, wavpixmap, wavwin, gc, 0, row,
      wavewindow.w, yscaletrack, 0, row);

/* draw value in win so it doesn't stick */
/* toggled auto */
      if(autoevent.selectmute | autoevent.selectinvert){
      	if(autoevent.selected->value) sprintf(string, "ON");
      	else sprintf(string, "OFF");
      }else{
/* floating point auto so print DB */
      	if(autoevent.selected->value >= 0) sprintf(string, "+%.1f", autoevent.selected->value);
      	else
      	if(autoevent.selected->value > INFINITYGAIN)
      		sprintf(string, "%.1f", autoevent.selected->value);
      	else
      		sprintf(string, "oo");
      }
        
      if(y > row - BOXSIZE / 2 + 20)
        XDrawString(display, wavwin, gc, x, y - 3, string, strlen(string));
      else
        XDrawString(display, wavwin, gc, x, y + 20, string, strlen(string));

    do{
      XNextEvent(display, report);

      if(report->type == Expose) refreshdisplay();

      if(report->type == MotionNotify){
                                         /* hide old position */
        XSetFunction(display, gc, GXxor);
        if(autoevent.selectmute | autoevent.selectinvert)  /* auto is toggled */
          drawtoggleautos(0, row, startx, autoevent.firstautoselected,
            wavewindow.w);
        else                 /* auto is floating point */
          drawfloatautos(0, row, startx, autoevent.firstautoselected,
            wavewindow.w);
        XSetFunction(display, gc, GXcopy);

        x = report->xmotion.x;
        y = report->xmotion.y + BOXSIZE / 2;

        if(y < row + BOXSIZE) y = row + BOXSIZE;
        if(y > row + yscaletrack) y = row + yscaletrack;

        if(x > wavewindow.w){
          x = wavewindow.w;
          /* moveforward((x - wavewindow.w) *xscale); */
        }

        if(x < 0){
          x = 0;
          /* moveback((-x)*xscale); */
        }

                      /* set new position */
        if(autoevent.selected != autoevent.firstautoselected &&
          autoevent.selected->nextauto)
        {
          autoevent.selected->position = x * xscale + startx;
        }

                     /* set new value if shift is up */
        if(!(report->xmotion.state & ShiftMask)){
/* toggle auto */
          if(autoevent.selectmute | autoevent.selectinvert){
            if(y < row + yscaletrack / 2) autoevent.selected->value = 1;
            else autoevent.selected->value = 0;
          }else{
/* float auto */
            autoevent.selected->value =
              (float)(yscaletrack - BOXSIZE / 2 - (y - (row + BOXSIZE / 2))) / 
              	range + minfloat;
            if(autoevent.selected->value >= -0.2 && autoevent.selected->value <= 0.2) autoevent.selected->value = 0;
            autoevent.selected->value *= 10;
            autoevent.selected->value = (int)autoevent.selected->value;
            autoevent.selected->value /= 10;
          }
        }else{
          static struct Auto *prevauto, *nextauto;

                     /* recycle value if shift is down */
                     /* first auto */
          if(autoevent.selected == autoevent.firstautoselected){
            nextauto = autoevent.selected->nextauto;
            
            if(nextauto->value > minfloat + 0.1 && 
            	nextauto->value < maxfloat - 0.1)
	            autoevent.selected->value = nextauto->value;
	          else
	          
	          	autoevent.selected->value = 0;
          }else{
            prevauto = autoevent.selected->prevauto;
            nextauto = autoevent.selected->nextauto;
            
              /* take prevauto only if its value is in the middle */
            if(prevauto->value > minfloat + 0.1 && 
            	prevauto->value < maxfloat - 0.1)
              autoevent.selected->value = prevauto->value;


            else
              /* otherwise try nextauto's value if there is one and it's in the
                                          middle */
            if(nextauto && nextauto->value > minfloat + 0.1 && 
            	nextauto->value < maxfloat - 0.1)
              autoevent.selected->value = nextauto->value;
              /* otherwise put 0 */
              
            else
            	autoevent.selected->value = 0;
          }
        }

                                        /* draw new position */
        XSetFunction(display, gc, GXxor);
        if(autoevent.selectmute | autoevent.selectinvert)
          drawtoggleautos(0, row, startx, autoevent.firstautoselected,
            wavewindow.w);
        else
          drawfloatautos(0, row, startx, autoevent.firstautoselected,
            wavewindow.w);

        XSetFunction(display, gc, GXcopy);
        XCopyArea(display, wavpixmap, wavwin, gc, 0, row,
          wavewindow.w, yscaletrack, 0, row);
          
/* toggled auto */
        if(autoevent.selectmute | autoevent.selectinvert){
        	if(autoevent.selected->value) sprintf(string, "ON");
        	else sprintf(string, "OFF");
        }else{
/* floating point auto so print DB */
        	if(autoevent.selected->value >= 0) sprintf(string, "+%.1f", autoevent.selected->value);
        	else
       		if(autoevent.selected->value > INFINITYGAIN)
      			sprintf(string, "%.1f", autoevent.selected->value);
      		else
      			sprintf(string, "oo");
       }
        
        if(y > row + 20)
          XDrawString(display, wavwin, gc, x, y - 3, string, strlen(string));
        else
          XDrawString(display, wavwin, gc, x, y + 20, string, strlen(string));
      }
      while(XPending(display) > 1 && report->type != ButtonRelease)
        XNextEvent(display, report);
    }while(report->type != ButtonRelease);

                                          /* delete if out of bounds */
    if(autoevent.selected != autoevent.firstautoselected &&
      autoevent.selected->nextauto){
      if(autoevent.selected->position <
        autoevent.selected->prevauto->position |
        autoevent.selected->position >
        autoevent.selected->nextauto->position)
        deleteauto(autoevent.selected);
    }
                                           /* clear events */
    autoevent.selectmute = 0;
    autoevent.selectfade = 0;
    autoevent.selectinvert = 0;
    for(i = 0; i < outchannels; i++){
      autoevent.selectpan[i] = 0;
    }

    XSetFunction(display, gc, GXcopy);
    optimizeautomation(0);

    drawwave(0, row, wavewindow.w, row + yscaletrack, startx, tracknumber, NO_BLANK, FLASH,
      DRAWCURSOR);
  	if(playevent.playsound)
    	drawcursor(0, row, wavewindow.w, yscaletrack, startx, currentposition,
      	currentposition);
  }
  return result;
}

int automationisvisible(){
  static int result, i;

  result = 0;
  if(autoevent.showmute | autoevent.showfade | autoevent.showinvert) result = 1;

  for(i = 0; i < outchannels; i++){
    if(autoevent.showpan[i]) result = 1;
 	}
  return result;
}

int drawautomation(int column1, int row1, long column2, long row2, long startx,
  int starty, char flash)
{
  static struct Track *track;
  static int i, row_;

  track = trackof(starty);

  for(row_ = row1;
    row_ < row2 && track;
    row_ += yscaletrack, track = track->nexttrack)
  {
    if(autoevent.showmute){
      XSetForeground(display, gc, green.pixel);
      drawtoggleautos(column1, row_, startx, track->firstautomute, column2);
    }

    if(autoevent.showfade){
      XSetForeground(display, gc, black.pixel);
      drawfloatautos(column1, row_, startx, track->firstautofade, column2);
    }

    if(autoevent.showinvert){
      XSetForeground(display, gc, orange.pixel);
      drawtoggleautos(column1, row_, startx, track->firstautoinvert, column2);
    }

    for(i = 0; i < outchannels; i++){
      if(autoevent.showpan[i]){
        XSetForeground(display, gc, purple.pixel);
        drawfloatautos(column1, row_, startx, track->firstautopan[i], column2);
      }
    }
  }
}

/* check to see if track is past end of track at this point */
/* if automation check to see if automation is muted */
/* check for silence */
/* return 1 if current buffer is inaudible */
/* return 0 if current buffer can be heard */
int ismuted(struct Track *currenttrack, long startx, long end){
  static struct Auto *currentmute, *currentfade;
  static struct Edit *currentedit;
  static char result, result_;

  if(!currenttrack) return 1;
                            /* beyond end of track */
  if(!currenttrack->lastedit || 
  	startx > currenttrack->lastedit->start1 + currenttrack->lastedit->length)
    return 1;

                            /* silence spans selection */
  currentedit = segmentof(startx, currenttrack);
  if(!strcmp(currentedit->file, SILENCE) &&
    currentedit->start1 + currentedit->length > end) return 1;

                            /* no automation enabled */
  if(!autoevent.automation)
    return 0;

                                    /* check muting for 1 */
                                    /* first auto before startx */
  currentmute = autoof(startx, currenttrack->firstautomute);
  if(currentmute->value) result = 1;
  else result = 0;


                             /* check if muted all the way through */
  for(currentmute = currentmute->nextauto;
    currentmute && currentmute->position < end && result;
    currentmute = currentmute->nextauto)
    if(currentmute->value) result = 0;

  if(result) return result; /* if muted all the way through, quit now */

                                    /* check fading for 0 */
                                    /* first fade before startx */
  currentfade = autoof(startx, currenttrack->firstautofade);
  if(topower[(int)(currentfade->value * 10)] == 0) result = 1;
  else result = 0;

                              /* check if fade is 0 all the way through */
  for(currentfade = currentfade->nextauto;
    currentfade && currentfade->position < end && result;
    currentfade = currentfade->nextauto)
    if(topower[(int)(currentfade->value)]) result = 0;

                             /* check fade after end since it affects data
                                 before it */
  if(currentfade && topower[(int)(currentfade->value)]) result = 0;

  return result;
}

/* copy inbuffer to outbuffer for pans where inbuffer must be reused */
/* scale changes with each pan so must be calculated in here */
/* scale is passed as a power to factor in inverts */
void automatepan(struct Auto *firstauto, char *inbufferstart,
  long inbufferlength, char *outbufferstart, long startx, long endx, float scale)
{
  static struct Auto *currentauto, *nextauto;
  register TWO *inbuffer_, *inbuffer__, *outbuffer_, *outbuffer__;
  static char *inbufferend, *outbufferend;
  static long x;

  inbufferend = inbufferstart + inbufferlength;
  outbufferend = outbufferstart + inbufferlength;
  currentauto = autoof(startx, firstauto);              /* process float */
  nextauto = currentauto->nextauto;
  inbuffer_ = (TWO *)inbufferstart;
  outbuffer_ = (TWO *)outbufferstart;
  x = startx;

  while(nextauto && nextauto->position < endx && inbuffer_ < (TWO *)inbufferend)
  {
    /* buffer__ will never exceed bufferend because of loop condition */
    inbuffer__ = inbuffer_ + nextauto->position - x;
    outbuffer__ = outbuffer_ + nextauto->position - x;

    if(currentauto->value == nextauto->value){
      static float scale2;

      scale2 = topower[(int)(currentauto->value * 10)] * scale;

      if(scale2 == 0){           /* scale2 = 0 so fill with 0 */
      	outbuffer__ -= 4;
        while(outbuffer_ < outbuffer__){
          *outbuffer_++ = 0;
          *outbuffer_++ = 0;
          *outbuffer_++ = 0;
          *outbuffer_++ = 0;
        }

      	outbuffer__ += 4;
        while(outbuffer_ < outbuffer__){
          *outbuffer_++ = 0;
        }
        inbuffer_ = inbuffer__;
      }else
      if(scale2 == 1){           /* scale2 = 1 so copy directly */
      	inbuffer__ -= 4;
        while(inbuffer_ < inbuffer__){
          *outbuffer_++ = *inbuffer_++;
          *outbuffer_++ = *inbuffer_++;
          *outbuffer_++ = *inbuffer_++;
          *outbuffer_++ = *inbuffer_++;
        }
      	inbuffer__ += 4;
        while(inbuffer_ < inbuffer__){
          *outbuffer_++ = *inbuffer_++;
        }
      }else{
      	inbuffer__ -= 4;
      	while(inbuffer_ < inbuffer__){   /* scale2 is intermediate */
        	*outbuffer_++ = scale2 * *inbuffer_++;
        	*outbuffer_++ = scale2 * *inbuffer_++;
        	*outbuffer_++ = scale2 * *inbuffer_++;
        	*outbuffer_++ = scale2 * *inbuffer_++;
      	}

      	inbuffer__ += 4;
      	while(inbuffer_ < inbuffer__){   /* scale2 is intermediate */
        	*outbuffer_++ = scale2 * *inbuffer_++;
      	}
      }
    }else{                       /* scale2 changes during selection */
      static float slope;
      static long x1;
      static float scale2;

      slope = (nextauto->value - currentauto->value) /
        (nextauto->position - currentauto->position);
      x1 = x - currentauto->position;

			inbuffer__ -= 4;
      while(inbuffer_ < inbuffer__){
        scale2 = currentauto->value;
        scale2 += x1 * slope;
        scale2 *= scale;
        *outbuffer_++ = topower[(int)(scale2 * 10)] * *inbuffer_++;
        x1++;

        scale2 = currentauto->value;
        scale2 += x1 * slope;
        scale2 *= scale;
        *outbuffer_++ = topower[(int)(scale2 * 10)] * *inbuffer_++;
        x1++;

        scale2 = currentauto->value;
        scale2 += x1 * slope;
        scale2 *= scale;
        *outbuffer_++ = topower[(int)(scale2 * 10)] * *inbuffer_++;
        x1++;

        scale2 = currentauto->value;
        scale2 += x1 * slope;
        scale2 *= scale;
        *outbuffer_++ = topower[(int)(scale2 * 10)] * *inbuffer_++;
        x1++;
      }

			inbuffer__ += 4;
      while(inbuffer_ < inbuffer__){
        scale2 = currentauto->value;
        scale2 += x1 * slope;
        scale2 *= scale;

        *outbuffer_++ = topower[(int)(scale2 * 10)] * *inbuffer_++;
        x1++;
      }
    }

    x = nextauto->position;
    currentauto = nextauto;
    nextauto = currentauto->nextauto;
  }

                                       /* process auto beyond end */
  if(!nextauto || currentauto->value == nextauto->value){
    static float scale2;

    scale2 = topower[(int)(currentauto->value * 10)] * scale;

      if(scale2 == 0){           /* scale2 = 0 so fill with 0 */
      	outbufferend -= 8;
        while(outbuffer_ < (TWO *)outbufferend){
          *outbuffer_++ = 0;
          *outbuffer_++ = 0;
          *outbuffer_++ = 0;
          *outbuffer_++ = 0;
        }

      	outbufferend += 8;
        while(outbuffer_ < (TWO *)outbufferend){
          *outbuffer_++ = 0;
        }
        inbuffer_ = (TWO *)inbufferend;
      }else
      if(scale2 == 1){           /* scale2 = 1 so copy directly */
      	inbufferend -= 8;
        while(inbuffer_ < (TWO *)inbufferend){
          *outbuffer_++ = *inbuffer_++;
          *outbuffer_++ = *inbuffer_++;
          *outbuffer_++ = *inbuffer_++;
          *outbuffer_++ = *inbuffer_++;
        }
        
      	inbufferend += 8;
        while(inbuffer_ < (TWO *)inbufferend){
          *outbuffer_++ = *inbuffer_++;
        }
      }else{
      	inbufferend -= 8;
      	while(inbuffer_ < (TWO *)inbufferend){   /* scale2 is intermediate */
        	*outbuffer_++ = scale2 * *inbuffer_++;
        	*outbuffer_++ = scale2 * *inbuffer_++;
        	*outbuffer_++ = scale2 * *inbuffer_++;
        	*outbuffer_++ = scale2 * *inbuffer_++;
      	}

      	inbufferend += 8;
      	while(inbuffer_ < (TWO *)inbufferend){   /* scale2 is intermediate */
        	*outbuffer_++ = scale2 * *inbuffer_++;
      	}
      }
  }else{
    static float slope;                    /* scale2 changes in selection */
    static long x1;
    static float scale2;

    slope = (nextauto->value - currentauto->value) /
      (nextauto->position - currentauto->position);
    x1 = x - currentauto->position;

		inbufferend -= 8;
    while(inbuffer_ < (TWO *)inbufferend){
      scale2 = currentauto->value;
      scale2 += x1 * slope;
      scale2 *= scale;
      *outbuffer_++ = topower[(int)(scale2 * 10)] * *inbuffer_++;
      x1++;

      scale2 = currentauto->value;
      scale2 += x1 * slope;
      scale2 *= scale;
      *outbuffer_++ = topower[(int)(scale2 * 10)] * *inbuffer_++;
      x1++;

      scale2 = currentauto->value;
      scale2 += x1 * slope;
      scale2 *= scale;
      *outbuffer_++ = topower[(int)(scale2 * 10)] * *inbuffer_++;
      x1++;

      scale2 = currentauto->value;
      scale2 += x1 * slope;
      scale2 *= scale;
      *outbuffer_++ = topower[(int)(scale2 * 10)] * *inbuffer_++;
      x1++;
    }

		inbufferend += 8;
    while(inbuffer_ < (TWO *)inbufferend){
      scale2 = currentauto->value;
      scale2 += x1 * slope;
      scale2 *= scale;

      *outbuffer_++ = topower[(int)(scale2 * 10)] * *inbuffer_++;
      x1++;
    }
  }
}

/* alter inbuffer fade, mute, inverse, the factors that don't change
	 by channel */
void automatebuffer(struct Track *track, char *inbufferstart,
  long inbufferlength, long startx, long endx)
{
  static long x;
  static struct Auto *currentauto, *nextauto;
  register TWO *inbuffer_, *inbuffer__;
  static char *inbufferend;
  
  inbufferend = inbufferstart + inbufferlength;
  currentauto = autoof(startx, track->firstautomute);
  nextauto = currentauto->nextauto;
  inbuffer_ = (TWO *)inbufferstart;
  x = startx;

         /*======================= process mute =====================*/
  while(nextauto && nextauto->position < endx && inbuffer_ < (TWO *)inbufferend)
  {
    inbuffer__ = inbuffer_ + nextauto->position - x;
    if(currentauto->value){          /* current auto is muted */
    	inbuffer__ -= 4;
      while(inbuffer_ < inbuffer__){
        *inbuffer_++ = 0;
        *inbuffer_++ = 0;
        *inbuffer_++ = 0;
        *inbuffer_++ = 0;
      }
      
    	inbuffer__ += 4;
      while(inbuffer_ < inbuffer__){
        *inbuffer_++ = 0;
      }
    }else                                       /* otherwise skip */
      inbuffer_ = inbuffer__;

    x = nextauto->position;
    currentauto = nextauto;
    nextauto = currentauto->nextauto;
  }
                                              /* last auto is muted */
  if(currentauto->value){
  	inbufferend -= 8;
  	while(inbuffer_ < (TWO *)inbufferend){
    	*inbuffer_++ = 0;
    	*inbuffer_++ = 0;
    	*inbuffer_++ = 0;
    	*inbuffer_++ = 0;
    }

  	inbufferend += 8;
  	while(inbuffer_ < (TWO *)inbufferend){
    	*inbuffer_++ = 0;
    }
  }

/* ======================process invert================================ */
  currentauto = autoof(startx, track->firstautoinvert);
  nextauto = currentauto->nextauto;
  inbuffer_ = (TWO *)inbufferstart;
  x = startx;

  while(nextauto && nextauto->position < endx && inbuffer_ < (TWO *)inbufferend)
  {
    inbuffer__ = inbuffer_ + nextauto->position - x;
    if(currentauto->value){                     /* invert this segment */
    	inbuffer__ -= 4;
      while(inbuffer_ < inbuffer__){
        *inbuffer_++ *= -1;
        *inbuffer_++ *= -1;
        *inbuffer_++ *= -1;
        *inbuffer_++ *= -1;
      }

    	inbuffer__ += 4;
      while(inbuffer_ < inbuffer__){
        *inbuffer_++ *= -1;
      }
    }else                                 /* skip this segment */
      inbuffer_ = inbuffer__;

    x = nextauto->position;
    currentauto = nextauto;
    nextauto = currentauto->nextauto;
  }

  if(currentauto->value){
  	inbufferend -= 8;
  	while(inbuffer_ < (TWO *)inbufferend){
    	*inbuffer_++ *= -1;
    	*inbuffer_++ *= -1;
    	*inbuffer_++ *= -1;
    	*inbuffer_++ *= -1;
    }

  	inbufferend += 8;
  	while(inbuffer_ < (TWO *)inbufferend){
    	*inbuffer_++ *= -1;
    }
  }

/* ==============================process fade============================= */
  currentauto = autoof(startx, track->firstautofade);
  nextauto = currentauto->nextauto;
  inbuffer_ = (TWO *)inbufferstart;
  x = startx;

  while(nextauto && nextauto->position < endx && inbuffer_ < (TWO *)inbufferend)
  {
    /* buffer__ will never exceed bufferend because of loop condition */
    inbuffer__ = inbuffer_ + nextauto->position - x;
                           /* fade doesn't change in section */
    if(currentauto->value == nextauto->value){
      if(topower[(int)(currentauto->value * 10)] == 1)    /* fade is 1 so skip */
        inbuffer_ = inbuffer__;
      else
      if(topower[(int)(currentauto->value * 10)] == 0){   /* fade is 0 so fill with 0 */
      	inbuffer__ -= 4;
        while(inbuffer_ < inbuffer__){
          *inbuffer_++ = 0;
          *inbuffer_++ = 0;
          *inbuffer_++ = 0;
          *inbuffer_++ = 0;
        }

      	inbuffer__ += 4;
        while(inbuffer_ < inbuffer__){
          *inbuffer_++ = 0;
        }
      }else{                          /* fade requires scaling */
      	static float value;
      	
      	value = topower[(int)(currentauto->value * 10)];
      	inbuffer__ -= 4;
        while(inbuffer_ < inbuffer__){
          *inbuffer_++ *= value;
          *inbuffer_++ *= value;
          *inbuffer_++ *= value;
          *inbuffer_++ *= value;
        }

      	inbuffer__ += 4;
        while(inbuffer_ < inbuffer__){
          *inbuffer_++ *= value;
        }
      }
    }else{                        /* fade changes during selection */
      static float slope;
      static long x1;
      static float scale2;

      slope = (nextauto->value - currentauto->value) /
        (nextauto->position - currentauto->position);
      x1 = x - currentauto->position;

			inbuffer__ -= 4;
      while(inbuffer_ < inbuffer__){
        scale2 = currentauto->value;
        scale2 += x1 * slope;
        *inbuffer_++ *= topower[(int)(scale2 * 10)];
        x1++;

        scale2 = currentauto->value;
        scale2 += x1 * slope;
        *inbuffer_++ *= topower[(int)(scale2 * 10)];
        x1++;

        scale2 = currentauto->value;
        scale2 += x1 * slope;
        *inbuffer_++ *= topower[(int)(scale2 * 10)];
        x1++;

        scale2 = currentauto->value;
        scale2 += x1 * slope;
        *inbuffer_++ *= topower[(int)(scale2 * 10)];
        x1++;
      }

			inbuffer__ += 4;
      while(inbuffer_ < inbuffer__){
        scale2 = currentauto->value;
        scale2 += x1 * slope;

        *inbuffer_++ *= topower[(int)(scale2 * 10)];
        x1++;
      }
    }

    x = nextauto->position;
    currentauto = nextauto;
    nextauto = currentauto->nextauto;
  }

                /* next auto doesn't exist or is equal to this auto */
  if(!nextauto || currentauto->value == nextauto->value){
  	static float value;
  
  	value = topower[(int)(currentauto->value * 10)];
  	inbufferend -= 8;
    while(inbuffer_ < (TWO *)inbufferend){
      *inbuffer_++ *= value;
      *inbuffer_++ *= value;
      *inbuffer_++ *= value;
      *inbuffer_++ *= value;
    }

  	inbufferend += 8;
    while(inbuffer_ < (TWO *)inbufferend){
      *inbuffer_++ *= value;
    }
  }else{                        /* fade changes before end of selection */
    static float slope;
    static long x1;
    static float scale2;

    slope = (nextauto->value - currentauto->value) /
      (nextauto->position - currentauto->position);
    x1 = x - currentauto->position;

		inbufferend -= 4;
    while(inbuffer_ < (TWO *)inbufferend){
      scale2 = currentauto->value;
      scale2 += x1 * slope;
      *inbuffer_++ *= topower[(int)(scale2 * 10)];
      x1++;

      scale2 = currentauto->value;
      scale2 += x1 * slope;
      *inbuffer_++ *= topower[(int)(scale2 * 10)];
      x1++;

      scale2 = currentauto->value;
      scale2 += x1 * slope;
      *inbuffer_++ *= topower[(int)(scale2 * 10)];
      x1++;

      scale2 = currentauto->value;
      scale2 += x1 * slope;
      *inbuffer_++ *= topower[(int)(scale2 * 10)];
      x1++;
    }

		inbufferend += 4;
    while(inbuffer_ < (TWO *)inbufferend){
      scale2 = currentauto->value;
      scale2 += x1 * slope;

      *inbuffer_++ *= topower[(int)(scale2 * 10)];
      x1++;
    }
  }
}

int cutautomationonly(long selectionstart, long selectionend, char operation)
{
  if(selectionstart == selectionend) return 1;

                        /* don't update undo if being used as utility */
  if(operation == CUTAUTOS) updateundo(operation);
  copyautomationonly(selectionstart, selectionend, operation);
  clearautomationonly(selectionstart, selectionend, operation);
  if(operation == CUTAUTOS){
    drawwave(0, 0, wavewindow.w, wavewindow.h, startx, starty, BLANK, FLASH,
      DRAWCURSOR);
    drawlabels(startx);
  }
  drawwave(0, 0, wavewindow.w, wavewindow.h, startx, starty, BLANK, FLASH,
    DRAWCURSOR);
}

int copyautomationonly(long selectionstart, long selectionend, char operation)
{
  FILE *clipboard;

  if(selectionstart == selectionend) return 1;

  if(!(clipboard = fopen(AUTOCLIPBOARDFILENAME, "wb"))){
    printf("Can't open clipboard file.\n");
    return 1;
  }
  changesmade = 1;

  copyautomation(clipboard, selectionstart, selectionend);
  fclose(clipboard);
}

/* paste autos without shifting back */
int pasteautosonly(struct Auto *firstauto, struct Auto *lastauto, FILE *clipboard,
  long start)
{
  static struct Auto *frontauto, *currentauto, *backauto, *tempauto;
  static TWO newautos;
  static TWO i;
  static float value;
  static FOUR position, lastposition;

      /* can't paste after last automation point */
  frontauto = autoof(start, firstauto);
  if(!frontauto) return 1;

  backauto = frontauto->nextauto;

  newautos = gettwo(clipboard);
  position = 0;
  lastposition = 0;

     /* paste until end of clipboard or clipboard exceeds last automation */
  for(i = 0; i < newautos && position < lastauto->position; i++){
    position = getfour(clipboard) + start;
    value = getfloat(clipboard);

      /* if position and first position overlap, replace it */
    if(firstauto->position == position){
      firstauto->value = value;
    }else
      /* if position and last position overlap, replace it */
    if(lastauto->position == position){
      lastauto->value = value;
    }else{
      /* position and existing position don't overlap */
      if((position <= lastauto->position)){
        currentauto = createnewauto(frontauto, backauto);
        currentauto->position = position;
        currentauto->value = value;
        frontauto = currentauto;
        lastposition = position;
      }
    }
  }
  for(; i < newautos; i++){
    fseek(clipboard, 8, SEEK_CUR);
  }
}

/* paste automation clipboard without pushing anything back since the audio
   data wasn't edited */
int pasteautomationonly(long selectionstart, long selectionend, char operation)
{
  FILE *clipboard;
  struct Track *track;
  TWO newautopans, i;
  TWO newtracks;

                          /* don't update undo if being used as utility */
  if(operation == PASTEAUTOS) updateundo(operation);

  if(!(clipboard = fopen(AUTOCLIPBOARDFILENAME, "rb"))){
    printf("Can't open clipboard file.\n");
    return 1;
  }

            /* clear automation in the range selected */
  clearautomationonly(selectionstart, selectionend, operation);

             /* need to paste automation here because the range of the automation
                clipboard might be longer than the range of the selection */

  newautopans = gettwo(clipboard);                         /* new pans */
  newtracks = gettwo(clipboard);                           /* new tracks */

  for(track = firsttrack; track; track = track->nexttrack){
    if(track->record){
      pasteautosonly(track->firstautomute, track->lastautomute,
        clipboard, selectionstart);
      pasteautosonly(track->firstautofade, track->lastautofade,
        clipboard, selectionstart);
      pasteautosonly(track->firstautoinvert, track->lastautoinvert,
        clipboard, selectionstart);                /* paste pans */
      for(i = 0; i < outchannels && i < newautopans; i++)
        pasteautosonly(track->firstautopan[i], track->lastautopan[i],
          clipboard, selectionstart);
    }
  }
  optimizeautomation(operation);

  fclose(clipboard);
  changesmade = 1;
  drawwave(0, 0, wavewindow.w, wavewindow.h, startx, starty, BLANK, FLASH,
    DRAWCURSOR);
}

void clearautosonly(struct Auto *firstauto, struct Auto *lastauto,
  long start, long end)
{
  static struct Auto *currentauto, *temp;

          /* delete autos but don't delete the last auto or the first auto */
  for(currentauto = autoof(start, firstauto);
    currentauto && currentauto->position < end && currentauto != lastauto;
    currentauto = temp)
  {
    temp = currentauto->nextauto;
    if(currentauto != firstauto && currentauto->position >= start)
      deleteauto(currentauto);
  }
}

/* clear automation only, without shifting anything back since the audio data
   doesn't change length here */
int clearautomationonly(long selectionstart, long selectionend, char operation)
{
  struct Track *track;
  TWO i;

  if(selectionstart == selectionend) return 0;

  if(operation == CLEARAUTOS) updateundo(operation);
  changesmade = 1;

  for(track = firsttrack; track; track = track->nexttrack){
    if(track->record){
      clearautosonly(track->firstautomute, track->lastautomute, selectionstart, selectionend);
      clearautosonly(track->firstautofade, track->lastautofade, selectionstart, selectionend);
      clearautosonly(track->firstautoinvert, track->lastautoinvert, selectionstart,
        selectionend);
      for(i = 0; i < outchannels; i++){
        clearautosonly(track->firstautopan[i], track->lastautopan[i], selectionstart,
          selectionend);
      }
    }
  }
  optimizeautomation(operation);

  if(operation == CLEARAUTOS)
    drawwave(0, 0, wavewindow.w, wavewindow.h, startx, starty, BLANK, FLASH,
      DRAWCURSOR);
}
