#ifndef AUTOMATE_H
#define AUTOMATE_H

#include "defs.h"

struct autoevent_ {
  char play;                      /* automatic adjustment during playback */
  char automation;
  char showmute;
  char showfade;
  char *showpan;                  /* array of flags designating which pans are visible */
  char showinvert;

  struct Auto *selected;
  struct Auto *firstautoselected;
  char selectmute;
  char selectfade;
  char selectinvert;
  char *selectpan;
} autoevent;                      /* automation event flags */

/* position for drawing toggle automation points */
#define TOGGLEY           16
/* size of box to draw for automation points */
#define BOXSIZE           8

int ismuted(struct Track *currenttrack, long startx, long end);

/* initialization options */
void initautoevents();
void initautomation(int starttrack, long length);
void destroyautomation(int starttrack);
void loadshowpans(FILE *in, float value);
void saveshowpans(FILE *out);

/* file operations */
void readautomation(FILE *in, int firsttrack, int newtracks, int newoutchannels);
void writeautomation(FILE *out);
void writeeffectautomation(FILE *out, struct CastHeader *castheader);

/* edit operations in conjunction with audio editing */
void shiftautomation(struct Track *currenttrack, long start, long length);
void pasteautomation(FILE *clipboard, long start, long length, char operation);
void copyautomation(FILE *clipboard, long start, long end);
void clearautomation(long start, long end, char operation);
void changeautomationchannels(int newchannels);
int selectautomation(XEvent *report);
int automationisvisible();
int drawautomation(int column1, int row1, long column2, long row2, long startx,
  int starty, char flash);

/* playback operations */
void automatepan(struct Auto *firstauto, char *inbufferstart,
  long inbufferlength, char *outbufferstart, long startx, long endx, float scale);
void automatebuffer(struct Track *track, char *inbufferstart,
  long inbufferlength, long startx, long endx);

/* independant edit operations */
int cutautomationonly(long selectionstart, long selectionend, char operation);
int copyautomationonly(long selectionstart, long selectionend, char operation);
int pasteautomationonly(long selectionstart, long selectionend, char operation);
int clearautomationonly(long selectionstart, long selectionend, char operation);
void optimizeautomation(char operation);

#endif
