/*
  Broadcast 2.0 multitrack audio editing
  (c) 1997 Adam Williams

  This program is distributed with the intent that it will be useful, without
  any warranty.  The code is being updated and has many bugs which are
  constantly being elucidated so changes should not be made without notifying
  the author.
*/

/* bcast.c      initializers, main routines, event handlers */

#include "bcast.h"
#include "file.h"
#include "play.h"
#include "record.h"
#include "interfac.h"
#include "graphics.h"
#include "console.h"
#include "menu.h"
#include "movement.h"
#include "audio.h"
#include "automate.h"
#include "effects.h"
#include "cd2drive.h"
#include "compress.h"

#include <pwd.h>
#include <sys/types.h>
#include <unistd.h>

void wavscrollevent(XEvent *report){
  static long startx_, starty_;
  static long distance;
  static int i;

  if(wavscroll.x.event){
    switch(wavscroll.x.event){
      case SPAN:
        if(report->type == MotionNotify){
          startx_ = startx;
          xscrollevent(wavscroll.xpixmap, wavscroll.xwin,
            report, &wavscroll.x);
          if(wavscroll.x.position > startx_){
          	distance = wavscroll.x.position - startx_;
          	distance /= xscale;
          	distance *= xscale;
            moveforward(distance);
          }
          if(wavscroll.x.position < startx_){
          	distance = startx_ - wavscroll.x.position;
          	distance /= xscale;
          	distance *= xscale;
            moveback(distance);
          }
        }
        break;
      case ARROW1:
          moveback(wavewindow.w / 16 * xscale);
        break;
      case ARROW2:
          moveforward(wavewindow.w / 16 * xscale);
        break;
      case UPPAGE:
        moveback(wavewindow.w * xscale);
        break;
      case DNPAGE:
        moveforward(wavewindow.w * xscale);
        break;
    }
  }
  else
  if(wavscroll.y.event){
    switch(wavscroll.y.event){
      case SPAN:
        if(report->type == MotionNotify){
          starty_ = wavscroll.y.position;
          yscrollevent(wavscroll.ypixmap, wavscroll.ywin, report,
            &wavscroll.y);
          if(wavscroll.y.position > starty_)
            movedown(wavscroll.y.position - starty_);
          if(wavscroll.y.position < starty_)
            moveup(starty_ - wavscroll.y.position);
        }
        break;
      case ARROW1:
        moveup(1);
        break;
      case ARROW2:
        movedown(1);
        break;
      case UPPAGE:
        moveup(wavewindow.h / yscaletrack);
        break;
      case DNPAGE:
        movedown(wavewindow.h / yscaletrack);
        break;
    }
  }
  else
  if(report->xbutton.window == wavscroll.xwin)
    xscrollevent(wavscroll.xpixmap, wavscroll.xwin, report, &wavscroll.x);
  else
  if(report->xbutton.window == wavscroll.ywin)
    yscrollevent(wavscroll.ypixmap, wavscroll.ywin, report, &wavscroll.y);
}

void handleexpose(XEvent *report){
  if(report->xexpose.window == playwin)
    refreshplay();
  else
    if(report->xexpose.window == consolewin)
      refreshconsole();
    else
      refreshdisplay();
}

int mainbuttonrelease(XEvent *report){
  if(consoleevent.consoleadjust) consolebuttonrelease(report);
  if(patchevent.patchadjust) patchbuttonrelease(report);
  if(menuevent.baron | menuevent.menuon){
  	if(menuevent.submenuon) submenubuttonrelease(report);
  	else menubuttonrelease(report);
  }
  if(wavscroll.x.event) xscrollevent(wavscroll.xpixmap, wavscroll.xwin,
                          report, &wavscroll.x);
  if(wavscroll.y.event) yscrollevent(wavscroll.ypixmap, wavscroll.ywin,
                          report, &wavscroll.y);
  if(consolewindow.scroll.event) consolescrollevent(report);
}

int mainbuttonpress(XEvent *report){
  int x, y;

  if(report->xbutton.window == menuwin | menuevent.baron | menuevent.menuon){
    menubuttonpress(report);
    return 0;
  }

  if(report->xbutton.window == wavscroll.xwin |
    report->xbutton.window == wavscroll.ywin)
    wavscrollevent(report);

  if(report->xbutton.window == wavwin){
    selectwave(report);

    if(playevent.playsound){
      playevent.stopplay = 1;
      playevent.paused = 0;
    }
    return 0;
  }

  if(report->xbutton.window == patchwin){
    patchbuttonpress(report);
    return 0;
  }

  if(report->xbutton.window == labelwin){
    selectlabel(report);
    if(playevent.playsound){
      playevent.stopplay = 1;
      playevent.paused = 0;
    }
    return 0;
  }

  if(report->xbutton.window == consolewin){
    consolebuttonpress(report);
  }

  if(report->xbutton.window == playwin){
    playbuttonpress(report);
  }
}

void mainkeypress(XEvent *report){
  KeySym keysym;
  char keys_return[2];

  keys_return[0] = '\0';
  XLookupString ((XKeyEvent*)report, keys_return, 1, &keysym, 0);

  switch(keysym){
    case XK_Up:
      if(report->xkey.window == consolewin){
        consolemoveup(1);
      }else{
        if(report->xkey.state & ControlMask) zoomoutvert();
          else{
            if(!playevent.playsound)
              zoomout(selectionstart, selectionend);
            else{
              zoomout(currentposition, currentposition);
            }
          }
      }
      break;

    case XK_Down:
      if(report->xkey.window == consolewin){
        consolemovedown(1);
      }else{
        if(report->xkey.state & ControlMask) zoominvert();
          else{
            if(!playevent.playsound)
              zoomin(selectionstart, selectionend);
            else{
              zoomin(currentposition, currentposition);
            }
          }
      }
      break;

    case XK_Left:
      if(report->xkey.window == consolewin){
        consoledecreaselevel();
      }else{
        moveback(wavewindow.w/2*xscale);
      }
      break;
    case XK_Right:
      if(report->xkey.window == consolewin){
        consoleincreaselevel();
      }else{
        moveforward(wavewindow.w/2*xscale);
      }
      break;
    case XK_Next:
      if(report->xkey.window == consolewin){
        consolemovedown(consolewindow.h / consolewindow.moduleh);
      }else{
        if(report->xkey.state & ControlMask) zoominverttrack();
          else movedown(1);
      }
      break;
     case XK_Prior:
      if(report->xkey.window == consolewin){
        consolemoveup(consolewindow.h / consolewindow.moduleh);
      }else{
        if(report->xkey.state & ControlMask) zoomoutverttrack();
          else moveup(1);
      }
      break;
     case XK_BackSpace:
       if(!playevent.playsound){
         if((report->xkey.state & ShiftMask) | (report->xkey.state & LockMask))
           clearautomationonly(selectionstart, selectionend, CLEARAUTOS);
         else
           clear(selectionstart, selectionend, CLEAR);
       }
       break;

    default:
      switch(keys_return[0]){
        case 'p':
          if(!playevent.playsound){
            playevent.playsound = 1;
          	if(playevent.playsound == 1) playsound(selectionstart,
          		selectionend);
          }
          else
            playevent.pause = 1;
          break;
        case 'r':
          if(!playevent.playsound){
            DrawDnRecord(playpixmap, playwin, &playwindow.record);
            recordsound(selectionstart, selectionend);
            DrawUpRecord(playpixmap, playwin, &playwindow.record);
          }
          break;
        case 'l':
          if(!playevent.playsound){
            if(selectionstart == selectionend){
              togglelabel(selectionstart);
            }else{
              togglelabel(selectionstart);
              togglelabel(selectionend);
            }
          }else
            playlabel();
        break;
        case 'f':   fit(selectionstart, selectionend);                break;
        case 'n':
          if(!playevent.playsound)
            createnewfile();
          break;
        case 'o':
          if(!playevent.playsound)
            loadfile();
          break;
        case 'x':
          if(!playevent.playsound)
            cut(selectionstart, selectionend, CUT);
          break;
        case 'v':
          if(!playevent.playsound)
            paste(selectionstart, selectionend, PASTE);
          break;
        case ' ':
          if(!playevent.playsound)
            pastesilence(selectionstart, selectionend);
          break;
        case 'c':
          if(!playevent.playsound)
            copy(selectionstart, selectionend, COPY);
          break;
        case 'X':
          if(!playevent.playsound)
            cutautomationonly(selectionstart, selectionend, CUTAUTOS);
          break;
        case 'V':
          if(!playevent.playsound)
            pasteautomationonly(selectionstart, selectionend, PASTEAUTOS);
          break;
        case 'C':
          if(!playevent.playsound)
            copyautomationonly(selectionstart, selectionend, COPYAUTOS);
          break;

        case 'm':
          if(!playevent.playsound)
            mute(selectionstart, selectionend, MUTE);
          break;

        case 's':
          if(!playevent.playsound)
            saveasfile();
          else{
            playevent.stopplay = 1;
            playevent.paused = 0;
          }
          break;
        case 'i':
          if(!playevent.playsound)
            importfile();
          break;

        case 't':
          if(!playevent.playsound)
            addtrack();
          break;

        case 'd':
          if(!playevent.playsound)
            deletetrack();
          break;

        case 'a':
          toggleautomation();
          break;

        case 'L':
          toggleloop();
          break;

        case 'z':
          if(!playevent.playsound)
            undo();
          break;

        case 'Z':
          if(!playevent.playsound)
            redo();
          break;

        case 'q':
          if(!playevent.playsound)
            saveandexit();
          break;
      }
      break;
  }
}

void mainconfigurenotify(XEvent *report){
      if(report->xconfigure.window == win){
        if(menuevent.baron | menuevent.menuon) menubuttonpress(report);
        resizewindow(report);
      }

      if(report->xconfigure.window == consolewin) consoleresizewindow(report);
}

void mainhandleevent(XEvent *report){
  switch (report->type){
    case Expose:
      handleexpose(report);
    break;

    case ConfigureNotify:
      mainconfigurenotify(report);
    break;

    case MotionNotify:
      if(wavscroll.x.event |
        wavscroll.y.event) wavscrollevent(report);
      if(consolewindow.scroll.event) consolescrollevent(report);
      if(consoleevent.consoleadjust) consolemotion(report);
      if(patchevent.patchadjust) patchmotion(report);
      if(menuevent.baron) menubarmotion(report);
      if(menuevent.submenuon) submenumotion(report);
      else if(menuevent.menuon) menumotion(report);
      if(wavscroll.x.event == SPAN | wavscroll.y.event == SPAN)
        wavscrollevent(report);
    break;

    case KeyPress:
      mainkeypress(report);
    break;

    case ButtonPress:
      mainbuttonpress(report);
      if(wavscroll.x.event |
        wavscroll.y.event) wavscrollevent(report);
      if(consoleevent.consoleadjust) consolemotion(report);
    break;

    case ButtonRelease:
      mainbuttonrelease(report);
    break;
  }          /* end switch */
}

/* initialize general filenames for clipboard etc */
void initgeneralfiles(){
  uid_t uid;
  struct passwd *pw;

  uid = getuid();             /* get user id */
  pw = getpwuid(uid);         /* get info for user */

            /* put user directory before each file */
  sprintf(INITIALIZERFILE, "%s/%s", pw->pw_dir, _INITIALIZERFILE_);
  sprintf(CLIPBOARDFILENAME, "%s/%s", pw->pw_dir, _CLIPBOARDFILENAME_);
  sprintf(AUTOCLIPBOARDFILENAME, "%s/%s", pw->pw_dir, _AUTOCLIPBOARDFILENAME_);
  sprintf(UNDOFILENAME, "%s/%s", pw->pw_dir, _UNDOFILENAME_);
}

/* load initializer variables
   return 1 if failure return 0 if success */
int loadinitialvariables(){
  FILE *in;
  char field[256];
  long i;
  float value;

  if(!(in = fopen(INITIALIZERFILE, "r"))){
    printf("No %s file.  Using default settings.\n", INITIALIZERFILE);
    return 1;
  }

      /* read fields */
      /* can't initilize playback loop since there isn't a file to get
         begin and end points normally */
  while(!feof(in)){
    fscanf(in, "%s", field);
    fscanf(in, "%f", &value);

      /* load OUTCHANNELS first for memory allocation */
    if(!strcmp(field, "OUTCHANNELS"))
      {
        changeautomationchannels(value);  /* this changes
        																							outchannels itself */
      }

    if(!strcmp(field, "SAMPLERATE"))
      samplerate = value;
    if(!strcmp(field, "PLAYBUFFER"))
      PLAYBUFFER = value;
    if(!strcmp(field, "AUTOMATION"))
      autoevent.automation = value;

    if(!strcmp(field, "CONSOLEHEIGHT")){
      consolewindow.h = value;
      consolewindow.scroll.h = value;
    }
    if(!strcmp(field, "SCREENWIDTH"))
      screenwidth = value;
    if(!strcmp(field, "SCREENHEIGHT"))
      screenheight = value;
    if(!strcmp(field, "YSCALE"))
      yscale = value;
    if(!strcmp(field, "YSCALETRACK"))
      yscaletrack = value;

    if(!strcmp(field, "TITLES"))
      titles = value;
    if(!strcmp(field, "SHOWTRACKGROUPS"))
      showtrackgroups = value;
    if(!strcmp(field, "SHOWEDITS"))
      showedits = value;
    if(!strcmp(field, "HIGHQUALITYREFRESH"))
      highqualityrefresh = value;
    if(!strcmp(field, "SHOWMUTE"))
      autoevent.showmute = value;
    if(!strcmp(field, "SHOWFADE"))
      autoevent.showfade = value;
    if(!strcmp(field, "SHOWINVERT"))
      autoevent.showinvert = value;
          /* outchannels in this file must equal pans specified here */
    if(!strcmp(field, "SHOWPANS"))
      loadshowpans(in, value);
    if(!strcmp(field, "REVERBPARAMS"))
      loadreverbparams(in, value);
    if(!strcmp(field, "NORMALIZELEVEL"))
      loadnormalizelevel(in, value);
    if(!strcmp(field, "SHIFTLABELS"))
      shiftlabels = value;


    if(!strcmp(field, "COMPRESSDIR"))
    	loadinitcompressdir(in);
    if(!strcmp(field, "COMPRESSPARAMS"))
    	loadinitcompress(in, value);
    if(!strcmp(field, "SAVEDIR"))
      loadinitsavedir(in);
    if(!strcmp(field, "LOADDIR"))
      loadinitloaddir(in);
     if(!strcmp(field, "CD2DRIVEDATA"))
      loadcddrivedata(in, value);
       
    if(!strcmp(field, "LOADASFILETYPE"))
      loadinitloadtype(value);
    if(!strcmp(field, "PLAYASFILETYPE"))
      loadinitplaytype(value);
    if(!strcmp(field, "EFFECTDIR"))
      loadiniteffectdir(in);
    if(!strcmp(field, "EFFECTTYPE"))
      loadiniteffecttype(value);
    if(!strcmp(field, "INPUTDIR"))
      loadinitinputdir(in);
    if(!strcmp(field, "DUPLEXENABLE"))
      duplexenable = value;
    if(!strcmp(field, "RECORDDIR"))
      loadinitrecdir(in);
    if(!strcmp(field, "RECORDFILETYPE"))
      loadinitrectype(value);
    if(!strcmp(field, "RECORDINPUT"))
      recevent.audioinput = value;
    if(!strcmp(field, "DUPLEX"))
      recevent.duplex = value;
    if(!strcmp(field, "NFRAMES"))
      getnframes(value);
    if(!strcmp(field, "FRAMESIZE"))
      getframesize(value);
  }

  fclose(in);

  return 0;
}

/* save initializer variables
   return 1 if failure return 0 if success */
int saveinitialvariables(){
  FILE *out;

  if(!(out = fopen(INITIALIZERFILE, "w"))){
    printf("Can't create %s.  Defaults will be used on next launch.\n",
      INITIALIZERFILE);
    return 1;
  }

      /* can't initilize playback loop since there isn't a file to get
         begin and end points normally */
      /* save OUTCHANNELS first for memory allocation */
  fprintf(out, "OUTCHANNELS %d\n", outchannels);
  fprintf(out, "SAMPLERATE %ld\n", samplerate);
  fprintf(out, "PLAYBUFFER %ld\n", PLAYBUFFER);
  fprintf(out, "AUTOMATION %d\n", autoevent.automation);

  fprintf(out, "CONSOLEHEIGHT %d\n", consolewindow.h);
  fprintf(out, "SCREENWIDTH %d\n", screenwidth);
  fprintf(out, "SCREENHEIGHT %d\n", screenheight);
  fprintf(out, "YSCALE %d\n", yscale);
  fprintf(out, "YSCALETRACK %d\n", yscaletrack);

  fprintf(out, "TITLES %d\n", titles);
  fprintf(out, "SHOWTRACKGROUPS %d\n", showtrackgroups);
  fprintf(out, "SHOWEDITS %d\n", showedits);
  fprintf(out, "HIGHQUALITYREFRESH %d\n", highqualityrefresh);
  fprintf(out, "SHOWMUTE %d\n", autoevent.showmute);
  fprintf(out, "SHOWFADE %d\n", autoevent.showfade);
  fprintf(out, "SHOWINVERT %d\n", autoevent.showinvert);
  fprintf(out, "SHIFTLABELS %d\n", shiftlabels);
  fprintf(out, "DUPLEX %d\n", recevent.duplex);

  fprintf(out, "DUPLEXENABLE %ld\n", duplexenable);
  fprintf(out, "RECORDINPUT %ld\n", recevent.audioinput);
  fprintf(out, "RECORDFILETYPE %ld\n", saveinitrectype());
  fprintf(out, "PLAYASFILETYPE %ld\n", saveinitplaytype());
  fprintf(out, "LOADASFILETYPE %d\n", saveinitloadtype());
  fprintf(out, "CD2DRIVEDATA");
  savecd2drivedata(out);
  fprintf(out, "NFRAMES %d\n", putnframes());
  fprintf(out, "FRAMESIZE %d\n", putframesize());

  saveinitsavedir(out);
  saveinitloaddir(out);
  saveiniteffectdir(out);
  saveiniteffecttype(out);
  saveinitinputdir(out);
  saveinitrecdir(out);
  
  savenormalizelevel(out);
  saveinitcompressdir(out);
  saveinitcompress(out);


  saveshowpans(out);
  savereverbparams(out);

  fclose(out);
  return 0;
}

void initevents(){
  playevent.playsound = 0;
  playevent.loop = 0;
  consoleevent.consoleadjust = 0;
  consoleevent.panadjust = 0;
  consoleevent.leveladjust = 0;

  consoleevent.eqleveladjust = 0;
  consoleevent.eqfreqadjust = 0;
  consoleevent.eqwidthadjust = 0;

  patchevent.patchadjust = 0;
  patchevent.playpatchadjust = 0;
  patchevent.recpatchadjust = 0;

  menuevent.menuon = 0;                 /* menu events */
  menuevent.submenuon = 0;
  menuevent.baron = 0;
  recevent.channels = 0;

  initautoevents();
}

void initializevariables(){
  FOUR byteordertest;

  initgeneralfiles();

  byteordertest = 0x00000001;
  byteorder = *((unsigned char *)&byteordertest);
         /* this frequency table is replaced by one on disk if the resource
            file exists */
  initfreq();
  initdb();

  screenwidth = 820;
  screenheight = 330;
  startx = 0;               /* current position of view in virtual file */
  starty = 0;               /* current track at start of view */
  consolestarty = 0;             /* current track at start of console view */
  groupstarty = 0;
  xscale = 1;               /* horizontal scale of view */
  yscale = 128;             /* vertical scale of wave */
  yscaletrack = 128;
  selectionstart = 0;       /* current selection start */
  selectionend = 0;         /* current selection end */
  currentposition = 0;      /* current position for playback */
  outchannels = 2;           /* total output channels */
  changesmade = 0;
  shiftlabels = 0;          /* shift labels when cutting and pasting if 1 */
  titles = 1;               /* draw titles if 1 */
  showtrackgroups = 0;
  showedits = 0;
  highqualityrefresh = 1;
  samplerate = 44100;
  PLAYBUFFER = MAXPLAYBUFFER;
  RECBUFFER = MAXRECBUFFER;
  consolewindow.group.text = malloc(256);
  initautoevents();
  initevents();
  fittime();

  firsttrack = 0; /* pointer to first track in database */
  lasttrack = 0;  /* pointer to last track in database */
  firstlabel = 0;
  lastlabel = 0;
  firsttrackgroup = 0;
  lasttrackgroup = 0;
  cleardatabase();
  clearlabels();

    /* replace the customizable values if resource file exists */
  loadinitialvariables();
  loadmenusettings();         /* set menus according to values */
                              /* and ALLOCATE MEMORY for variable menu titles */
}

int main(int argc, char *argv[]){
  int i;
  char infilename[256], filetype[5];
  FILE *in;
  XEvent report, nextreport;

  printf("Broadcast 2.1 (c) 1997 Adam Williams\n");

  if(argc >= 2) strcpy(infilename, argv[1]);
  initializevariables();
  initializescreen();

  if(argc >= 2 && (in = fopen(infilename, "rb"))){
    for(i = 0; i < 4; i++){
      filetype[i] = fgetc(in);
    }
    filetype[i] = 0;
    fclose(in);
                      /* determine file type */
                      /* default to PCM if no file type can be determined */
    if(!(strcmp(filetype, "RIFF"))){
      if(loadwave(infilename, WAV, LOAD)) return 1;
    }else if(!(strcmp(filetype, "EDIT"))){
      if(loadedits(infilename, LOAD)) return 1;
      }else if(!(strcmp(filetype, "CAST"))){
        if(loadcast(infilename, LOAD)) return 1;
      }else{
        if(loadwave(infilename, saveinitloadtype(), LOAD)) return 1;
        }
  }else{
    drawwave(0, 0, wavewindow.w, wavewindow.h, startx, starty, NO_BLANK, FLASH,
      DRAWCURSOR);
  }

  drawpatches(starty);
  drawlabels(startx);
  drawtime(startx);
  drawmenubar(-1);
  drawplaycontrols();
  drawconsolecontrols();
  updatexscrollbar(wavscroll.xpixmap, wavscroll.xwin, startx,
    wavewindow.w * xscale, longesttrack(), &wavscroll.x);
  updateyscrollbar(wavscroll.ypixmap, wavscroll.ywin, starty,
    wavewindow.h / yscaletrack, tracks(), &wavscroll.y);

  while (1) {
    XNextEvent(display, &report);

    mainhandleevent(&report);
  }
}
