#ifdef _LINUX_

#include "defs.h"
#include "interfac.h"
#include "cd2drive.h"
#include "fileattr.h"
#include "record.h"
#include "edit.h"
#include "bcast.h"
#include <linux/cdrom.h>


int NFRAMES = 2;
int FRAMESIZE = 2352;

struct setupwin_ {
	int w, h;
	
	char *title;
	
	bctitle start;
	bctitle track1text, minute1text, second1text;
	bctitle colon1, colon2;
	bctextbox track1, minute1, second1;
	
	bctitle end;
	bctitle track2text, minute2text, second2text;
	bctitle colon3, colon4;
	bctextbox track2, minute2, second2;

	bcbutton back;
	bcbutton next;
	bcbutton cancel;
} setupwin;

struct setupwin_ setupwin = {
	325, 186,
	"Set up CD transfer",
	{5, 15, "Start:"}, 
	{5, 32, "Track"}, {75, 32, "Min"}, {145, 32, "Sec"},
	{60, 60, ":"}, {130, 60, ":"},
	{5, 35, 50, 30, ""}, {75, 35, 50, 30, ""}, {145, 35, 50, 30, ""},
	{5, 80, "End:"}, 
	{5, 97, "Track"}, {75, 97, "Min"}, {145, 97, "Sec"},
	{60, 120, ":"}, {130, 120, ":"},
	{5, 100, 50, 30, ""}, {75, 100, 50, 30, ""}, {145, 100, 50, 30, ""},
	{5, 140, 100, 30, "Back"}, {110, 140, 100, 30, "Do it"}, 
	{220, 140, 100, 30, "Cancel"}
};

struct setupevent_ {
	bctextbox *activetextbox;
} setupevent;

struct setupevent_ setupevent = {
	0
};

struct previewwin_ {
	int w, h;
	char *title;
	bctitle previewing;
	bcbutton back;
	bcbutton doit;
	bcbutton cancel;
} previewwin;

struct previewwin_ previewwin = {
	400, 400,
	"Preview recording",
	{5, 5, "Hit start to begin recording"},
	{5, 30, 100, 30, "Back"}, {210, 30, 100, 30, "Start"},
	{320, 30, 100, 30, "Cancel"}
};

char cdfilename[256] = "";
char cd2drivetype = WAV;
int track1 = 1, minute1 = 0, second1 = 0;
int track2 = 2, minute2 = 0, second2 = 0;
long startlba, endlba, buffersize;
int cdrom;
char *buffer;

void loadcddrivedata( FILE *in, int value){
	
  cd2drivetype = value;
	fscanf(in, "%s", cdfilename);
	fscanf(in, "%d", &track1);
	fscanf(in, "%d", &minute1);
	fscanf(in, "%d", &second1);
	fscanf(in, "%d", &track2);
	fscanf(in, "%d", &minute2);
	fscanf(in, "%d", &second2);
}

void getnframes(int value){
	NFRAMES = value;
}

void getframesize(int value){
	FRAMESIZE = value;
}	

int savecd2drivedata(FILE *out){
  fprintf(out, " %d\n", cd2drivetype);
	fprintf(out, "%s\n", cdfilename);
	fprintf(out, "%d", track1);
	fprintf(out, " %d", minute1);
	fprintf(out, " %d", second1);
	fprintf(out, " %d", track2);
	fprintf(out, " %d", minute2);
	fprintf(out, " %d\n", second2);
}

int putnframes(){
	return NFRAMES;
}

int putframesize(){
	return FRAMESIZE;
}

int doit(long start, long end){
	struct cdrom_read_audio arg;

	FILE *out;
	long i, tracks;
	long totallength, currentlength;   /* in buffersizes */
	char endofselection = 0;
	int attempts, previewing = 3;       /* bug in device driver */
	long startlba_;
	
	changesmade = 1;
  updateundo(RECORD);

	buffersize = SAVEBUFFER;
	buffersize /= NFRAMES * FRAMESIZE;
	buffersize *= NFRAMES * FRAMESIZE;

	totallength = endlba - startlba;
	totallength *= FRAMESIZE;
	totallength /= buffersize;
	totallength += previewing + 1;
	currentlength = 0;

	if(!(buffer = malloc(buffersize))){
		printf("Can't allocate recording buffer.\n");
		return 1;
	};
	
	if((out = fopen(recevent.file, "wb")) == 0){
		printf("Can't open output file.\n");
		free(buffer);
		return 1;
	}
	
	if(recevent.format == WAV){
		fwrite(buffer, WAVHEADERLENGTH, 1, out);
	}

  /*========================= read the data =======================*/
	initializeprogress("CD 2 Drive transfer");

             /* bug in device driver */
	startlba_ = startlba - buffersize * previewing / FRAMESIZE;
	arg.addr_format = CDROM_LBA;
	arg.nframes = NFRAMES;
	for(arg.addr.lba = startlba_; 
		arg.addr.lba < endlba && !endofselection;)
	{
		if(arg.addr.lba + buffersize / FRAMESIZE > endlba){
			buffersize = (endlba - arg.addr.lba) / NFRAMES;
			buffersize *= NFRAMES * FRAMESIZE;
			endofselection = 1;
		}
			
		for(i = 0; i < buffersize; 
			i += NFRAMES * FRAMESIZE,
			arg.addr.lba += NFRAMES)
		{
			arg.buf = &buffer[i];
			for(attempts = 0; attempts < 3; attempts++){
				if(!(ioctl(cdrom, CDROMREADAUDIO, &arg))){
					attempts = 3;
				}else
				if(attempts == 2 && !previewing) printf("Can't read CD audio.\n");
			}
		}

		if(arg.addr.lba > startlba)
  	if((recevent.format == WAV && !byteorder) ||
    	(recevent.format == PCMHILO && byteorder) ||
    	(recevent.format == PCMLOHI && !byteorder))
    	swapbytes(buffer, buffersize);

		if(arg.addr.lba > startlba)
		fwrite(buffer, buffersize, 1, out);

		currentlength++;
		if(updateprogress(currentlength, totallength)){
			destroyprogress();
			close(cdrom);
			free(buffer);
			return 2;
		}
	}

	if(ioctl(cdrom, CDROMSTOP) < 0){
		printf("Can't stop motor.\n");
	}
	
	recevent.length = ftell(out);
	reclabels = 0;
	if(recevent.format == WAV){
		fseek(out, 0, SEEK_SET);
		writeheader(out, recevent.length, recevent.channels);
		recevent.length -= WAVHEADERLENGTH;
	}
	recevent.length /= recevent.channels * 2;
	free(buffer);
	fclose(out);
	close(cdrom);
	destroyprogress();
	
  paste(start, end, RECORD);

  drawpatches(starty);
  drawtime(startx);
  drawplaycontrols();
  drawconsolecontrols();
}

int previewtransfer(){
  Visual *vis;
  XVisualInfo visinfo;
  XSizeHints size_hints;
  XSetWindowAttributes attr;
  Cursor arrow;
  unsigned long mask;
  XEvent report;
  KeySym keysym;
  XComposeStatus composestatus;
  char keys_return[2];
	Window win;
	Pixmap pixmap;

	int success = 0;
	struct cdrom_read_audio arg;
	int i, attempts;

  /*==============================initialize screen===========================*/
  arrow = XCreateFontCursor(display, XC_top_left_arrow);
  vis = DefaultVisual(display, screen);
  mask = CWEventMask | CWBackPixel | CWBorderPixel | CWColormap |
    CWCursor;
  attr.event_mask = ExposureMask | ButtonPressMask |
    ButtonReleaseMask | KeyPressMask | KeyReleaseMask | PointerMotionMask;
  attr.background_pixel = grey.pixel;
  attr.border_pixel = dkgrey.pixel;
  attr.colormap = cmap;
  attr.cursor = arrow;
  win = XCreateWindow(display, rootwin, 0, 0, previewwin.w, previewwin.h,
    0, depth, InputOutput, vis, mask, &attr);
  pixmap =   XCreatePixmap(display, win, previewwin.w, previewwin.h, depth);
  size_hints.flags = PSize;
  size_hints.width = previewwin.w;
  size_hints.height = previewwin.h;
  XSetStandardProperties(display, win, previewwin.title, previewwin.title, None,
    0, 0, &size_hints);
  XMapWindow(display, win);
  XSetForeground(display, gc, grey.pixel);
  XFillRectangle(display, pixmap, gc, 0, 0, previewwin.w, previewwin.h);

	DrawTitle(pixmap, win, &previewwin.previewing);
	DrawUpButton(pixmap, win, &previewwin.back);
	DrawUpButton(pixmap, win, &previewwin.doit);
	DrawUpButton(pixmap, win, &previewwin.cancel);

	openaudio();
	initplaybackaudio(2);
	buffersize = SAVEBUFFER / NFRAMES / FRAMESIZE;
	buffersize *= NFRAMES * FRAMESIZE;

	if(!(buffer = malloc(buffersize))){
		displayalert("Not enough memory to allocate preview buffer.\n");
		closeaudio();
		XDestroyWindow(display, win);
		XFreePixmap(display, pixmap);
		return 1;
	}
	
	arg.addr_format = CDROM_LBA;
	arg.nframes = NFRAMES;
	
	arg.nframes = buffersize / FRAMESIZE;
	do{
		arg.addr.lba = startlba;
		arg.buf = buffer;
		
		for(i = 0; i < buffersize; i += NFRAMES * FRAMESIZE){
			if(XPending(display)){
				flushevents();

				XNextEvent(display, &report);
				switch(report.type){
				}
			}
			for(attempts = 0; attempts < 3; attempts++){
				if(!(ioctl(cdrom, CDROMREADAUDIO, &arg))){
					attempts = 3;
				}
			}
		}
	}while(!success);
	
	closeaudio();
}

int setuptransfer(){
	int success = 0;
	int x, y;
	bctextbox *newactivebox;

  Visual *vis;
  XVisualInfo visinfo;
  XSizeHints size_hints;
  XSetWindowAttributes attr;
  Cursor arrow;
  unsigned long mask;
  XEvent report;
  KeySym keysym;
  XComposeStatus composestatus;
  char keys_return[2];
	Window win;
	Pixmap pixmap;

	struct cdrom_tochdr hdr;
	struct cdrom_tocentry entry[100];
	int i, tracks;

  /*==============================initialize screen===========================*/
  arrow = XCreateFontCursor(display, XC_top_left_arrow);
  
  vis = DefaultVisual(display, screen);
  mask = CWEventMask | CWBackPixel | CWBorderPixel | CWColormap |
    CWCursor;
  attr.event_mask = ExposureMask | ButtonPressMask |
    ButtonReleaseMask | KeyPressMask | KeyReleaseMask | PointerMotionMask;
  attr.background_pixel = grey.pixel;
  attr.border_pixel = dkgrey.pixel;
  attr.colormap = cmap;
  attr.cursor = arrow;
  win = XCreateWindow(display, rootwin, 0, 0, setupwin.w, setupwin.h,
    0, depth, InputOutput, vis, mask, &attr);
  pixmap =   XCreatePixmap(display, win, setupwin.w, setupwin.h, depth);
  size_hints.flags = PSize;
  size_hints.width = setupwin.w;
  size_hints.height = setupwin.h;
  XSetStandardProperties(display, win, setupwin.title, setupwin.title, None,
    0, 0, &size_hints);
  XMapWindow(display, win);
  XSetForeground(display, gc, grey.pixel);
  XFillRectangle(display, pixmap, gc, 0, 0, setupwin.w, setupwin.h);

	DrawUpButton(pixmap, win, &setupwin.back);
	DrawUpButton(pixmap, win, &setupwin.next);
	DrawUpButton(pixmap, win, &setupwin.cancel);
	create_text_box(pixmap, win, &setupwin.track1);
	create_text_box(pixmap, win, &setupwin.minute1);
	create_text_box(pixmap, win, &setupwin.second1);
	create_text_box(pixmap, win, &setupwin.track2);
	create_text_box(pixmap, win, &setupwin.minute2);
	create_text_box(pixmap, win, &setupwin.second2);
	
	DrawTitle(pixmap, win, &setupwin.start);
	DrawTitle(pixmap, win, &setupwin.track1text);
	DrawTitle(pixmap, win, &setupwin.minute1text);
	DrawTitle(pixmap, win, &setupwin.second1text);
	DrawTitle(pixmap, win, &setupwin.colon1);
	DrawTitle(pixmap, win, &setupwin.colon2);
	
	DrawTitle(pixmap, win, &setupwin.end);
	DrawTitle(pixmap, win, &setupwin.track2text);
	DrawTitle(pixmap, win, &setupwin.minute2text);
	DrawTitle(pixmap, win, &setupwin.second2text);
	DrawTitle(pixmap, win, &setupwin.colon3);
	DrawTitle(pixmap, win, &setupwin.colon4);

	setupevent.activetextbox = &setupwin.track1;
	activate_box(pixmap, win, setupevent.activetextbox);
	
	do{         /* while tracks are out of range */
		/*==================================events=======================*/
		do{
			XNextEvent(display, &report);
			switch(report.type){
				case Expose:
					if(report.xexpose.window == win)
          	XCopyArea(display, pixmap, win, gc, 0, 0,
            	setupwin.w, setupwin.h, 0, 0);
        	else handleexpose(&report); 
        	break;

      	case KeyPress:
        	XLookupString ((XKeyEvent *)&report, keys_return, 1, &keysym,
          	&composestatus);

					switch(keysym){
						case XK_Escape:
							XDestroyWindow(display, win);
							XFreePixmap(display, pixmap);
							return 2;
							break;

						case XK_Tab:
							if(!(report.xkey.state & ShiftMask)){
								deactivate_box(pixmap, win, setupevent.activetextbox);
								if(setupevent.activetextbox == &setupwin.track1)
									setupevent.activetextbox = &setupwin.minute1;
								else if(setupevent.activetextbox == &setupwin.minute1)
									setupevent.activetextbox = &setupwin.second1;
								else if(setupevent.activetextbox == &setupwin.second1)
									setupevent.activetextbox = &setupwin.track2;
								else if(setupevent.activetextbox == &setupwin.track2)
									setupevent.activetextbox = &setupwin.minute2;
								else if(setupevent.activetextbox == &setupwin.minute2)
									setupevent.activetextbox = &setupwin.second2;
								else if(setupevent.activetextbox == &setupwin.second2)
									setupevent.activetextbox = &setupwin.track1;
								activate_box(pixmap, win, setupevent.activetextbox);
							}else{
								deactivate_box(pixmap, win, setupevent.activetextbox);
								if(setupevent.activetextbox == &setupwin.track1)
									setupevent.activetextbox = &setupwin.second2;
								else if(setupevent.activetextbox == &setupwin.minute1)
									setupevent.activetextbox = &setupwin.track1;
								else if(setupevent.activetextbox == &setupwin.second1)
									setupevent.activetextbox = &setupwin.minute1;
								else if(setupevent.activetextbox == &setupwin.track2)
									setupevent.activetextbox = &setupwin.second1;
								else if(setupevent.activetextbox == &setupwin.minute2)
									setupevent.activetextbox = &setupwin.track2;
								else if(setupevent.activetextbox == &setupwin.second2)
									setupevent.activetextbox = &setupwin.minute2;
								activate_box(pixmap, win, setupevent.activetextbox);
 							}
							break;
						default:
							switch(keys_return[0]){
								case 13:
									success = 1;
									break;

								default:
									handle_text_event(pixmap, win, &report, setupevent.activetextbox);
									break;
							}
							break;
					}
      		break;

      	case ButtonPress:
        	x = report.xbutton.x;
        	y = report.xbutton.y;
        	newactivebox = 0;

        	if(buttonpressrelease(pixmap, win, x, y, &setupwin.next)){
        		success = 1;
        	}
        	if(buttonpressrelease(pixmap, win, x, y, &setupwin.cancel)){
						XDestroyWindow(display, win);
						XFreePixmap(display, pixmap);
						return 2;
        	}
        	if(buttonpressrelease(pixmap, win, x, y, &setupwin.back)){
						XDestroyWindow(display, win);
						XFreePixmap(display, pixmap);
						return 1;
        	}

        	if(testbox(x, y, (bcbox *)&setupwin.track1))
        		newactivebox = &setupwin.track1;
        	if(testbox(x, y, (bcbox *)&setupwin.minute1))
        		newactivebox = &setupwin.minute1;
        	if(testbox(x, y, (bcbox *)&setupwin.second1))
        		newactivebox = &setupwin.second1;
        	if(testbox(x, y, (bcbox *)&setupwin.track2))
        		newactivebox = &setupwin.track2;
        	if(testbox(x, y, (bcbox *)&setupwin.minute2))
        		newactivebox = &setupwin.minute2;
        	if(testbox(x, y, (bcbox *)&setupwin.second2))
        		newactivebox = &setupwin.second2;

        	if(newactivebox){
        		if(newactivebox != setupevent.activetextbox){
        			deactivate_box(pixmap, win, setupevent.activetextbox);
        			setupevent.activetextbox = newactivebox;
        			activate_box(pixmap, win, setupevent.activetextbox);
        		}else{
        			handle_text_event(pixmap, win, &report, setupevent.activetextbox);
        		}
        	}
      		break;
			}
		}while(!success);
		
		track1 = atol(setupwin.track1.text);
		minute1 = atol(setupwin.minute1.text);
		second1 = atol(setupwin.second1.text);
		track2 = atol(setupwin.track2.text);
		minute2 = atol(setupwin.minute2.text);
		second2 = atol(setupwin.second2.text);
		
	/*====================================get cdrom info===================================*/
		if((cdrom = open("/dev/cdrom", O_RDONLY)) < 0){
			displayalert("Can't open cdrom drive.");
			success = 0;
		}
		ioctl(cdrom, CDROMSTART);         /* start motor */
		if(ioctl(cdrom, CDROMREADTOCHDR, &hdr) < 0){
			close(cdrom);
  		displayalert("Can't get table of contents total.");
			success = 0;
  	}
  	for(i = 0; i < hdr.cdth_trk1; i++){
			entry[i].cdte_track = 1 + i;
			entry[i].cdte_format = CDROM_LBA;
			if(ioctl(cdrom, CDROMREADTOCENTRY, &entry[i]) < 0){
				ioctl(cdrom, CDROMSTOP);
				close(cdrom);
				success = 0;
				displayalert("Can't get table of contents entry.");
				break;
			}
  	}
  	entry[i].cdte_track = CDROM_LEADOUT;
  	entry[i].cdte_format = CDROM_LBA;
		if(ioctl(cdrom, CDROMREADTOCENTRY, &entry[i]) < 0){
			ioctl(cdrom, CDROMSTOP);
			close(cdrom);
			displayalert("Can't get table of contents leadout.");
			success = 0;
		}
  	tracks = hdr.cdth_trk1+1;

		if(track1 <= 0 || track1 > tracks){
			ioctl(cdrom, CDROMSTOP);
			close(cdrom);
			displayalert("Start track is out of range.");
			success = 0;
		}
		if(track2 < track1 || track2 <= 0 || track2 > tracks){
			ioctl(cdrom, CDROMSTOP);
			close(cdrom);
			displayalert("End track is out of range.");
			success = 0;
		}
		if(track1 == track2 && minute2 == 0 && second2 == 0){
			ioctl(cdrom, CDROMSTOP);
			close(cdrom);
			displayalert("End position is out of range.");
			success = 0;
		}
	}while(!success);

	startlba = entry[track1 - 1].cdte_addr.lba;
	startlba += (minute1 * 44100 * 4 * 60 + second1 * 44100 * 4) 
		/ FRAMESIZE;

	endlba = entry[track2 - 1].cdte_addr.lba;
	if(track2 < tracks){
		endlba += (minute2 * 44100 * 4 * 60 + second2 * 44100 * 4) 
			/ FRAMESIZE;
	}
	XDestroyWindow(display, win);
	XFreePixmap(display, pixmap);
	return 0;
}

int cd2drive(long start, long end){
	int result;
	FILE *out;
	char string[256];
	
                           /* get # of tracks with record permission */
  if(recordabletracks() == 0){
		displayalert("Define some tracks to record on.");
  	return 1;
	}
	recevent.channels = 2;   /* must be 2 because recording from a CD */

  strcpy(recevent.file, cdfilename);
  recevent.format = cd2drivetype;
  
  sprintf(setupwin.track1.text, "%d", track1);
  sprintf(setupwin.minute1.text, "%d", minute1);
  sprintf(setupwin.second1.text, "%d", second1);
  sprintf(setupwin.track2.text, "%d", track2);
  sprintf(setupwin.minute2.text, "%d", minute2);
  sprintf(setupwin.second2.text, "%d", second2);

checkpoint1:
  if(getfileattributes(recevent.file, &recevent.format, RECORD, "CD 2 Drive")){
    return 1;
  }
  cd2drivetype = recevent.format;
  strcpy(cdfilename, recevent.file);
  if(out = fopen(recevent.file, "rb")){
  	char string[256];
  	
    fclose(out);
    sprintf(string, "%s exists.  Record over it?", recevent.file);
    if(confirm(string, "CD 2 Drive")){
      return 1;
    }
                        /* delete index file if one exists */
  	sprintf(string, "%s%s", recevent.file, SCALEFILEEXT);
  	remove(string);
  }

checkpoint2:
	result = setuptransfer();
	if(result == 1) goto checkpoint1;    /* back button pressed */
	else if(result == 2) return 1;       /* cancel button pressed */
	
/* 	result = previewtransfer();
	if(result == 1) goto checkpoint2;   back button pressed
	else if(result == 2) return 1;   cancel button pressed
 */	

	result = doit(start, end);
	if(result == 1) goto checkpoint2;
	else if(result == 2) return 1;       /* cancel button pressed */
}
#else
#include "defs.h"

void loadcddrivedata( FILE *in, int value){
}

void getnframes(int value){
}

void getframesize(int value){
}

int savecd2drivedata(FILE *out){
}

int putnframes(){
}

int putframesize(){
}

int cd2drive(long selectionstart, long selectionend){
	displayalert("CD2Drive not available for your operating system.");
}

#endif
