/*
  Broadcast 2.0 multitrack audio editing
  (c) 1997 Adam Williams

  This program is distributed with the intent that it will be useful, without
  any warranty.  The code is being updated and has many bugs which are
  constantly being elucidated so changes should not be made without notifying
  the author.
*/

/* compress or expand */

#include "interfac.h"
#include "compress.h"
#include "effects.h"
#include "record.h"
#include "units.h"
#include <math.h>

#define MINSLOPE 0.0000001

struct compresswindow_ {
  int w, h;
  bcbutton ok;
  bcbutton cancel;
  bcbutton load;
  bcbutton save;
  bcbutton histogram;
  bcbox levels;
  bctitle attacktitle;
  bctextbox attack;
  bcbox ms;
  bcbox samples;
  bctitle mstitle;
  bctitle samplestitle;
  bcbutton reset;
} compresswindow;

struct compresswindow_ compresswindow = {
  520, 335,
  {10, 290, 100, 30, "OK"},
  {340, 290, 100, 30, "Cancel"},
  {120, 290, 70, 30, "Load"},
  {230, 290, 70, 30, "Save"},
  {400, 130, 100, 30, "Histogram"},
  {40, 15, 350, 250},           /* levels */
  {400, 20, "Attack"},
  {400, 25, 100, 30, ""},            /* attack */
  {400, 65, 15, 15},            /* ms */
  {400, 85, 15, 15},  /* samples */
  {420, 80, "ms"},            /* ms */
  {420, 100, "samples"},  /* samples */
  {400, 170, 100, 30, "Reset"}
};

struct compressevent_ {
  char activebox;
  char leveladjust;
  int whichlevel;
} compressevent;

const float INFINITYGAIN2 = -45;
struct compressevent_ compressevent = {0, 0, 0};
Window compresswin;
Pixmap compresspixmap;

static char compressfilename[256] = "";

static struct compresspoint_ {
	float inlevel;
	float outlevel;
} compresspoint[256];

static int totalpoints = 0;
static int attack = 0;
static int attacktype = MS;
static int histogrambuilt = 0;   /* is histogram built? */
long histogram[351];


void loadinitcompressdir(FILE *in){
  fscanf(in, "%s", compressfilename);
}

void saveinitcompressdir(FILE *out){
   char string[256];
/*
  cutdirectory(string, compressfilename);
 */
 
 	strcpy(string, compressfilename);
   /* only save if a string exists since fscanf would skip the next line */
  if(strlen(string)) fprintf(out, "COMPRESSDIR %s\n", string);
}

void loadinitcompress(FILE *in, int value){
  int i;


  totalpoints = value;
  for(i = 0; i < totalpoints; i++){
		fscanf(in, "%f", &compresspoint[i].inlevel);
		fscanf(in, "%f", &compresspoint[i].outlevel);
	}
  fscanf(in, " %d", &attack);
}

void saveinitcompress(FILE *out){
  int i;

  fprintf(out, "COMPRESSPARAMS");
  fprintf(out, " %d", totalpoints);
  for(i = 0; i < totalpoints; i++)
    fprintf(out, " %f %f", compresspoint[i].inlevel, compresspoint[i].outlevel);
  fprintf(out, " %d", attack);
  fprintf(out, "\n");
}

int loadcompress()
{
  FILE *in;
  int i;
  char string[256];

  if(getfileattributes(compressfilename, 0, INPUT, "Load Parameters"))
    return 1;

  if(!(in = fopen(compressfilename, "rb"))){
    displayalert("Can't open file");
    return 1;
  }

	fscanf(in, "%s", string);           /* load in file type */

  if(strcmp(string, "COMPRESSPARAMS")){
    displayalert("Incorrect file type.");
    fclose(in);
    return 1;
  }

	fscanf(in, "%d", &totalpoints);
	for(i = 0; i < totalpoints; i++){
		fscanf(in, "%f", &compresspoint[i].inlevel);
		fscanf(in, "%f", &compresspoint[i].outlevel);
	}
	fscanf(in, "%d", &attack);
  fclose(in);
}

int savecompress()
{
  FILE *out;
  char string[256], string_[256];
  int i;

  if(getfileattributes(compressfilename, 0, INPUT, "Save Parameters"))
    return 1;

  if(out = fopen(compressfilename, "rb")){                     /* filename exits */
    fclose(out);

    cutfilename(string_, compressfilename);
    sprintf(string, "%s exists.   Overwrite?", string_);
    if(confirm(string, "Effect file")) return 1;
  }
  if(!(out = fopen(compressfilename, "w"))){
    displayalert("Can't open file for output.");
    return 1;
  }

	fprintf(out, "COMPRESSPARAMS");
	fprintf(out, " %d", totalpoints);
  for(i = 0; i < totalpoints; i++){
		fprintf(out, " %f", compresspoint[i].inlevel);
		fprintf(out, " %f", compresspoint[i].outlevel);
  }
	fprintf(out, " %d\n", attack);
  fclose(out);
}

void drawattacktype(){
	switch(attacktype){
		case MS:
			DrawDnRadial(compresspixmap, (bcbutton *)&compresswindow.ms);
			DrawUpRadial(compresspixmap, (bcbutton *)&compresswindow.samples);
		break;
		
		case SAMPLES:
			DrawUpRadial(compresspixmap, (bcbutton *)&compresswindow.ms);
			DrawDnRadial(compresspixmap, (bcbutton *)&compresswindow.samples);
		break;
	}
	XCopyArea(display, compresspixmap, compresswin, gc, compresswindow.ms.x,
		compresswindow.ms.y, compresswindow.ms.w * 2, 60,
		compresswindow.ms.x, compresswindow.ms.y);
}


/* draw a handle in the specified box */
void drawcompresshandle(int x, int y, bcbox *box){
	const int w = 10, h = 10;
	static int maxx, maxy, minx, miny;
	
	minx = x - w / 2;
	miny = y - h / 2;
	maxx = x + w / 2;
	maxy = y + h / 2;
	
	if(maxx > box->w + box->x) maxx = box->w + box->x;
	if(maxy > box->h + box->y) maxy = box->h + box->y;
	if(minx < box->x) minx = box->x;
	if(miny < box->y) miny = box->y;

	XFillRectangle(display, compresspixmap, gc, minx, miny, 
		maxx - minx, maxy - miny);
}

void drawlevels(){
	int i;
	float x1, y1, x2, y2, x, y;
	float xs, ys;

	xs = (float)compresswindow.levels.w / -INFINITYGAIN2;
	ys = (float)compresswindow.levels.h / -(INFINITYGAIN2 - 3);
	x1 = compresspoint[0].inlevel;
	y1 = compresspoint[0].outlevel;

	x1 -= INFINITYGAIN2;
	y1 -= INFINITYGAIN2;
	x1 *= xs;
	y1 *= ys;
	x1 += compresswindow.levels.x;
	y1 = compresswindow.levels.y + compresswindow.levels.h - y1;

	XSetForeground(display, gc, white.pixel);
	XSetFunction(display, gc, GXxor);

						/* draw line */
	for(i = 1; i < totalpoints; i++){
		x2 = compresspoint[i].inlevel;
		y2 = compresspoint[i].outlevel;

		x2 -= INFINITYGAIN2;
		y2 -= INFINITYGAIN2;
		x2 *= xs;
		y2 *= ys;
		x2 += compresswindow.levels.x;
		y2 = compresswindow.levels.y + compresswindow.levels.h - y2;

		XDrawLine(display, compresspixmap, gc, x1, y1, x2, y2);
		x1 = x2;
		y1 = y2;
	}

						/* draw handles */
	for(i = 0; i < totalpoints; i++){
		x = compresspoint[i].inlevel;
		y = compresspoint[i].outlevel;

		x -= INFINITYGAIN2;
		y -= INFINITYGAIN2;
		x *= xs;
		y *= ys;
		x += compresswindow.levels.x;
		y = compresswindow.levels.y + compresswindow.levels.h - y;
		drawcompresshandle(x, y, &compresswindow.levels);
	}

  XSetFunction(display, gc, GXcopy);
	XSetForeground(display, gc, black.pixel);

  XCopyArea(display, compresspixmap, compresswin, gc, 
  	compresswindow.levels.x, compresswindow.levels.y,       /* flash on screen */
    compresswindow.levels.w+2, compresswindow.levels.h+2, 
    compresswindow.levels.x, compresswindow.levels.y);
}

void drawhistogram(){
	int i;
	
	if(histogrambuilt){
		XSetForeground(display, gc, white.pixel);        /* levels */
		XFillRectangle(display, compresspixmap, gc, compresswindow.levels.x,
			compresswindow.levels.y, 
			compresswindow.levels.w, compresswindow.levels.h);
		XSetForeground(display, gc, black.pixel);
		XDrawRectangle(display, compresspixmap, gc, compresswindow.levels.x-1, compresswindow.levels.y-1, 
			compresswindow.levels.w+2, compresswindow.levels.h+2);
		
		XSetForeground(display, gc, grey.pixel);
		for(i = 0; i < compresswindow.levels.w; i++){           /* draw graph */
			XDrawLine(display, compresspixmap, gc, 
				compresswindow.levels.x + i, 
				compresswindow.levels.y + compresswindow.levels.h,
				compresswindow.levels.x + i, 
				compresswindow.levels.y + compresswindow.levels.h - histogram[i]
			);
		}
		XSetForeground(display, gc, black.pixel);

		drawlevels();
 		XCopyArea(display, compresspixmap, compresswin, gc, 
 	 		compresswindow.levels.x, compresswindow.levels.y,       /* flash on screen */
    	compresswindow.levels.w, compresswindow.levels.h, 
 	  	compresswindow.levels.x, compresswindow.levels.y);
	}
}

int buildhistogram(long selectionstart, long selectionend){
  struct Track *currenttrack;
	TWO *inbuffer;
	register long i;
	long x, bufferlength;   /* bufferlength in samples */
	long *conversiontable_, *conversiontable;
	int division;
	long highvalue = 0;
	double xs, ys, j;

	if(!(inbuffer = malloc(SAVEBUFFER))){
		printf("Not enough memory to build histogram\n");
		return 1;
	}
	
	if(!(conversiontable = malloc(sizeof(long) * 65536))){
		printf("Not enough memory to build conversion table\n");
		free(inbuffer);
		return 1;
	}

	/* clear conversion table */
	for(i = 0; i < 65536;){
		conversiontable[i++] = 0;
		conversiontable[i++] = 0;
		conversiontable[i++] = 0;
		conversiontable[i++] = 0;
	}
	conversiontable_ = conversiontable + 32768;
	
	initializeprogress("Histogram build...");
	
  for(currenttrack = firsttrack; currenttrack;
    currenttrack = currenttrack->nexttrack)
  {
    if(currenttrack->record){
    	x = selectionstart;
    	bufferlength = SAVEBUFFER / 2;

      while(x < selectionend){
        if(selectionend - x < bufferlength)
          bufferlength = selectionend - x;

        if(readbuffer((char *)inbuffer, bufferlength * 2, currenttrack, x)){
          destroyprogress();
          free(inbuffer);
          free(conversiontable);
          return 1;
        }

        bufferlength -= 4;
        for(i = 0; i < bufferlength;){
          conversiontable_[inbuffer[i++]]++;
          conversiontable_[inbuffer[i++]]++;
          conversiontable_[inbuffer[i++]]++;
          conversiontable_[inbuffer[i++]]++;
        }
        
        bufferlength += 4;
        while(i < bufferlength){
          conversiontable_[inbuffer[i++]]++;
        }

        x += bufferlength;
        progress += bufferlength;
        if(updateprogress(progress, totallength)){
          destroyprogress();
          free(inbuffer);
          free(conversiontable);
          return 1;
        }
      }     /* end check track */
		}        /* end current track recordable */
	}         /* end check each track */

	for(i = 0; i < compresswindow.levels.w;){       /* clear histogram */
		histogram[i++] = 0;
	}

	histogram[0] = conversiontable[0];
	xs = (float)compresswindow.levels.w / INFINITYGAIN2;
	for(i = 1; i < 32768; i++){     /* add up positive values */
		j = log((float)i / 32768) / log(2) * 3;
		if(j < INFINITYGAIN2) j = INFINITYGAIN2;
		j *= xs;
		
		histogram[(int)(compresswindow.levels.w - j)] += conversiontable_[i];
	}

	for(i = 1; i > -32768; i--){     /* add up negative values */
		j = log((float)-i / 32768) / log(2) * 3;
		if(j < INFINITYGAIN2) j = INFINITYGAIN2;
		j *= xs;
		
		histogram[(int)(compresswindow.levels.w - j)] += conversiontable_[i];
	}

	highvalue = 0;
	for(i = 0; i < compresswindow.levels.w; i++){   /* normalize graph */
		if(histogram[i] > highvalue) highvalue = histogram[i];
	}

	ys = (float)highvalue / compresswindow.levels.h;
	for(i = 0; i < compresswindow.levels.w; i++){
		histogram[i] /= ys;
	}

	histogrambuilt = 1;
	drawhistogram();

	destroyprogress();
	free(inbuffer);
	free(conversiontable);
	return 0;
}

/* test a handle in the specified box */
/* return 1 if true */
int testcompresshandle(int x, int y, int hx, int hy, bcbox *box){
	const int w = 10, h = 10;
	static int minx, miny, maxx, maxy;
	
	minx = hx - w / 2;
	miny = hy - h / 2;
	maxx = hx + w / 2;
	maxy = hy + h / 2;
	
	if(maxx > box->w + box->x) maxx = box->w + box->x;
	if(maxy > box->h + box->y) maxy = box->h + box->y;
	if(minx < box->x) minx = box->x;
	if(miny < box->y) miny = box->y;

	if(x >= minx && x <= maxx && y >= miny && y <= maxy) return 1;
	else return 0;
}

int leveladjust(int x, int y, XEvent *report){
	int i, j, result;
	float x1, y1, x2, y2;
	float xs, ys, slope;
	int minx, miny, maxx, maxy;
	const int w = 10, h = 10;
	char string[32], string2[16];
	
	xs = (float)compresswindow.levels.w / -INFINITYGAIN2;
	ys = (float)compresswindow.levels.h / -(INFINITYGAIN2 - 3);

	/* test each handle */
	result = 0;
	for(i = 0; i < totalpoints && !result; i++){
		x1 = compresspoint[i].inlevel;
		y1 = compresspoint[i].outlevel;
	
		x1 -= INFINITYGAIN2;
		y1 -= INFINITYGAIN2;
		x1 *= xs;
		y1 *= ys;
		x1 += compresswindow.levels.x;
		y1 = compresswindow.levels.y + compresswindow.levels.h - y1;

		if(testcompresshandle(x, y, x1, y1, &compresswindow.levels)) result = 1;
	}
	i--;
	
	/* test each line */
	if(!result){
		for(i = 1; i <= totalpoints && !result; i++){
			x1 = compresspoint[i].inlevel;
			x1 -= INFINITYGAIN2;
			x1 *= xs;
			x1 += compresswindow.levels.x;
			if(x < x1) result = 1;
		}
		i--;
		
		if(result){         /* x between two points */
			result = 0;
			x1 = compresspoint[i-1].inlevel;
			y1 = compresspoint[i-1].outlevel;
		
			x1 -= INFINITYGAIN2;
			y1 -= INFINITYGAIN2;
			x1 *= xs;
			y1 *= ys;
			x1 += compresswindow.levels.x;
			y1 = compresswindow.levels.y + compresswindow.levels.h - y1;

			x2 = compresspoint[i].inlevel;
			y2 = compresspoint[i].outlevel;
		
			x2 -= INFINITYGAIN2;
			y2 -= INFINITYGAIN2;
			x2 *= xs;
			y2 *= ys;
			x2 += compresswindow.levels.x;
			y2 = compresswindow.levels.y + compresswindow.levels.h - y2;

			slope = (y2 - y1) / (x2 - x1);
			
			miny = (x - x1) * slope + y1 - h / 2;
			maxy = miny + h;

			if(y1 < compresswindow.levels.y) y1 = compresswindow.levels.y;
			if(y2 > compresswindow.levels.y + compresswindow.levels.h) y2 = compresswindow.levels.y + compresswindow.levels.h;

			if(y > miny && y < maxy){
				result = 1;
				drawlevels();       /* hide levels */
				             /* make new handle */
				for(j = totalpoints; j >= i; j--){
					compresspoint[j + 1].inlevel = compresspoint[j].inlevel;
					compresspoint[j + 1].outlevel = compresspoint[j].outlevel;
				}
				compresspoint[i].inlevel = x;
				compresspoint[i].inlevel -= compresswindow.levels.x;
				compresspoint[i].inlevel /= xs;
				compresspoint[i].inlevel += INFINITYGAIN2;
				compresspoint[i].outlevel = y;
				compresspoint[i].outlevel -= compresswindow.levels.y + compresswindow.levels.h;
				compresspoint[i].outlevel /= -ys;
				compresspoint[i].outlevel += INFINITYGAIN2;
				totalpoints++;
				drawlevels();
			}
		}    /* end between two points */
	}      /* end test each line */

	if(!result) return 0;       /* nothing selected */

	/* i specifies the handle selected */
	/* event handler */
	do{
		XNextEvent(display, report);

		switch(report->type){
			case Expose:
        if(report->xexpose.window == compresswin)
          XCopyArea(display, compresspixmap, compresswin, gc, 0, 0,
            compresswindow.w, compresswindow.h, 0, 0);
        else handleexpose(&report);
				break;

			case MotionNotify:
				drawlevels();
				x = report->xmotion.x;
				y = report->xmotion.y;
				
				if(x < compresswindow.levels.x) x = compresswindow.levels.x;
				if(y < compresswindow.levels.y) y = compresswindow.levels.y;
				if(x > compresswindow.levels.x + compresswindow.levels.w) x = compresswindow.levels.x +
				compresswindow.levels.w;
				if(y > compresswindow.levels.y + compresswindow.levels.h) y = compresswindow.levels.y +
				compresswindow.levels.h;

				if(i != 0 && i != totalpoints-1){
					compresspoint[i].inlevel = x;
					compresspoint[i].inlevel -= compresswindow.levels.x;
					compresspoint[i].inlevel /= xs;
					compresspoint[i].inlevel += INFINITYGAIN2;
				}
	
				if(report->xmotion.state & ShiftMask){   /* shift is down */
				                 /* first level selected */
					if(i == 0){
						if(compresspoint[i + 1].outlevel < 2.9 &&
							compresspoint[i + 1].outlevel > INFINITYGAIN2 + 0.1)
							compresspoint[i].outlevel = compresspoint[i + 1].outlevel;
						else
							compresspoint[i].outlevel = 0;
					}
					else
					if(i == totalpoints - 1){
						if(compresspoint[i - 1].outlevel < 2.9 &&
							compresspoint[i - 1].outlevel > INFINITYGAIN2 + 0.1)
							compresspoint[i].outlevel = compresspoint[i - 1].outlevel;
						else
							compresspoint[i].outlevel = 0;
					}
					else
					                  /* prev level is not extreme */
					if(compresspoint[i - 1].outlevel < 2.9 &&
						compresspoint[i - 1].outlevel > INFINITYGAIN2 + 0.1)
					{
							compresspoint[i].outlevel = compresspoint[i - 1].outlevel;
					}
					else       
					compresspoint[i].outlevel = 0;
				}
				else                 /* no shift so set outlevel */
				{
					compresspoint[i].outlevel = y;
					compresspoint[i].outlevel -= compresswindow.levels.y + compresswindow.levels.h;
					compresspoint[i].outlevel /= -ys;
					compresspoint[i].outlevel += INFINITYGAIN2;
				}
				
				drawlevels();
				
				if(compresspoint[i].inlevel == -20) sprintf(string, "oo, ");
				else
				if(compresspoint[i].inlevel < 0) sprintf(string, "%.1f, ", compresspoint[i].inlevel);
				else sprintf(string, "+%.1f, ", compresspoint[i].inlevel);
				
				
				if(compresspoint[i].outlevel == -20) sprintf(string2, "oo");
				else
				if(compresspoint[i].outlevel < 0) sprintf(string2, "%.1f", compresspoint[i].outlevel);
				else sprintf(string2, "+%.1f", compresspoint[i].outlevel);
				
				strcat(string, string2);
				if(x > compresswindow.levels.x + compresswindow.levels.w - 100)
					x = compresswindow.levels.x + compresswindow.levels.w - 100;
				if(y < compresswindow.levels.y + 30)
					y = compresswindow.levels.y + 30;
				XDrawString(display, compresswin, gc, x + 10,
					y - 10, string, strlen(string));
				break;

			default:
				break;
		}
	}while(report->type != ButtonRelease);
	
	          /* test compresspoint to see if a delete */
	if(i != 0 && i != totalpoints-1 &&
		((compresspoint[i].inlevel <= compresspoint[i-1].inlevel) ||
		(compresspoint[i].inlevel >= compresspoint[i+1].inlevel)))
	{
		drawlevels();
		while(i < totalpoints){    /* delete the point */
			compresspoint[i].inlevel = compresspoint[i+1].inlevel;
			compresspoint[i].outlevel = compresspoint[i+1].outlevel;
			i++;
		}
		totalpoints--;
		drawlevels();
	}
  XCopyArea(display, compresspixmap, compresswin, gc, compresswindow.levels.x, compresswindow.levels.y,
    compresswindow.levels.w + 2, compresswindow.levels.h + 2, compresswindow.levels.x, compresswindow.levels.y);
}

int killtable(float **conversiontable){
	free(*conversiontable);
}

int buildtable(float **conversiontable){
	int i;
	float in1, out1, in2, out2, j;
	float slope, dbin;

	if(!(*conversiontable = malloc(32768 * sizeof(float)))){
		printf("Not enough memory to build conversion table.\n");
		return 1;
	}

	in1 = compresspoint[0].inlevel;
	out1 = compresspoint[0].outlevel;

	if(in1 == INFINITYGAIN2) in1 = 0;
	else
	in1 = pow(2, in1 / 3);
	in1 *= 32768;

	if(out1 == INFINITYGAIN2) out1 = 0;
	else
	out1 = pow(2, out1 / 3);
	out1 *= 32768;

	for(i = 1; i < totalpoints; i++){
		in2 = compresspoint[i].inlevel;
		out2 = compresspoint[i].outlevel;
		
		if(in2 == INFINITYGAIN2) in2 = 0;
		else
		in2 = pow(2, in2 / 3);
		in2 *= 32768;

		if(out2 == INFINITYGAIN2) out2 = 0;
		else
		out2 = pow(2, out2 / 3);
		out2 *= 32768;
		
		slope = (out2 - out1) / (in2 - in1);

		for(j = in1; j <= in2; j++){             /* add segment to table */
			(*conversiontable)[(int)j] = out1 + (j - in1) * slope;
			
			if(j == 0)(*conversiontable)[(int)j] = INFINITYGAIN2;
			else{
				(*conversiontable)[(int)j] = (*conversiontable)[(int)j] / j;
		     /* table now contains an amplitude factor in % over input in
		            integer */
		    if((*conversiontable)[(int)j] == 0) (*conversiontable)[(int)j] = INFINITYGAIN2;
		    	else
				(*conversiontable)[(int)j] = 3 * log((*conversiontable)[(int)j]) /
					log(2);
			}	      
		}            /* table now contains an amplitude factor in DB over input in integer */
		in1 = in2;
		out1 = out2;
	}
	
	return 0;
}

int compress(long selectionstart, long selectionend){
  struct Track *currenttrack;
  char *inbufferstart;
  char string[256];
  float *conversiontable;
  long bufferlength;   /* length of buffer in bytes */

        /*=======================interface===================================*/
  XEvent report;                 /* configuration window */
  Visual *vis;
  XVisualInfo visinfo;
  XSizeHints size_hints;
  XSetWindowAttributes attr;
  Cursor arrow;
  unsigned long mask;
  KeySym keysym;
  XComposeStatus composestatus;
  char keys_return[2];
  int x, y, y_, i;
  int oldactivebox;
  int result;
	int attack_;

  if(initeffectfile()){
    return 1;
  }
  
                                   /* open compress window */
  arrow = XCreateFontCursor(display, XC_top_left_arrow);
  vis = DefaultVisual(display, screen);

  mask = CWEventMask | CWBackPixel | CWBorderPixel | CWColormap |
    CWCursor;
  attr.event_mask = ExposureMask | ButtonPressMask | PointerMotionMask |
    ButtonReleaseMask | KeyPressMask;
  attr.background_pixel = grey.pixel;
  attr.border_pixel = dkgrey.pixel;
  attr.colormap = cmap;
  attr.cursor = arrow;

  compresswin = XCreateWindow(display, rootwin, 0, 0, compresswindow.w, compresswindow.h,
    0, depth, InputOutput, vis, mask, &attr);

  compresspixmap = XCreatePixmap(display, compresswin, compresswindow.w,
    compresswindow.h, depth);

  size_hints.flags = PSize;
  size_hints.width = compresswindow.w;
  size_hints.height = compresswindow.h;

  XSetStandardProperties(display, compresswin, "Compress", "Compress", None, 0, 0,
    &size_hints);

  XMapWindow(display, compresswin);

  XSetForeground(display, gc, grey.pixel);
  XFillRectangle(display, compresspixmap, gc, 0, 0, compresswindow.w, compresswindow.h);

	                  /* initialize variables */
	if(totalpoints == 0){
		totalpoints = 2;
		compresspoint[0].inlevel = INFINITYGAIN2;
		compresspoint[0].outlevel = INFINITYGAIN2;
		compresspoint[1].inlevel = 0;
		compresspoint[1].outlevel = 0;
		attack = 1000;
	}
	sprintf(compresswindow.attack.text, "%d", attack);

                    /* initialize display */

	XSetForeground(display, gc, white.pixel);        /* levels */
	XFillRectangle(display, compresspixmap, gc, compresswindow.levels.x,
		compresswindow.levels.y, 
		compresswindow.levels.w, compresswindow.levels.h);
	XSetForeground(display, gc, black.pixel);
	XDrawRectangle(display, compresspixmap, gc, compresswindow.levels.x-1, compresswindow.levels.y-1, 
		compresswindow.levels.w+2, compresswindow.levels.h+2);
		
																				/* level text */
	sprintf(string, "oo", i);
	XDrawString(display, compresspixmap, gc, compresswindow.levels.x - 30,
			compresswindow.levels.y + compresswindow.levels.h + 15, string, strlen(string));

	for(i = -40, y = compresswindow.levels.h - compresswindow.levels.h / 5 + 40;
		i < 3; i += 8, y -= compresswindow.levels.h / 6){
		if(i >= 0) sprintf(string, "+%d", i);
		else sprintf(string, "%d", i);
		XDrawString(display, compresspixmap, gc, compresswindow.levels.x - 30,
			y , string, strlen(string));
	}

 	for(i = -40, x = compresswindow.levels.x + compresswindow.levels.w / 5 - 20;
		i <= 0; i += 8, x += compresswindow.levels.w / 6){
		if(i >= 0) sprintf(string, "+%d", i);
		else sprintf(string, "%d", i);
		XDrawString(display, compresspixmap, gc, x,
			compresswindow.levels.y + compresswindow.levels.h + 15 , string, strlen(string));
	}
	drawlevels();
	                                      /* buttons */
	DrawUpButton(compresspixmap, compresswin, &compresswindow.ok);
	DrawUpButton(compresspixmap, compresswin, &compresswindow.cancel);
	DrawUpButton(compresspixmap, compresswin, &compresswindow.load);
	DrawUpButton(compresspixmap, compresswin, &compresswindow.save);
	DrawUpButton(compresspixmap, compresswin, &compresswindow.histogram);
	DrawUpButton(compresspixmap, compresswin, &compresswindow.reset);
	drawattacktype();
	DrawTitle(compresspixmap, compresswin, &compresswindow.attacktitle);
	DrawTitle(compresspixmap, compresswin, &compresswindow.mstitle);
	DrawTitle(compresspixmap, compresswin, &compresswindow.samplestitle);
	XCopyArea(display, compresspixmap, compresswin, gc, 0, 0, compresswindow.w,
		compresswindow.h, 0, 0);

                               /* attack box */
	activate_box(compresspixmap, compresswin, &compresswindow.attack);
	drawhistogram();

	result = 0;
                    /* get parameters */
                                /* event handler */
  do{
    XNextEvent(display, &report);
    switch(report.type){
      case Expose:
        if(report.xexpose.window == compresswin)
          XCopyArea(display, compresspixmap, compresswin, gc, 0, 0,
            compresswindow.w, compresswindow.h, 0, 0);
        else handleexpose(&report);
        break;

      case KeyPress:
        keys_return[0] = '\0';
        XLookupString ((XKeyEvent *)&report, keys_return, 1, &keysym,
          &composestatus);

        switch(keysym){
          case XK_Escape:
            XDestroyWindow(display, compresswin);
            XFreePixmap(display, compresspixmap);
            killeffectfile();
            return 1;
            break;

          default:
            switch(keys_return[0]){
              case 13:
                XDestroyWindow(display, compresswin);
                XFreePixmap(display, compresspixmap);
								attack = atol(compresswindow.attack.text);
                result = 1;
                break;

              default:
                handle_text_event(compresspixmap, compresswin, &report,
                	&compresswindow.attack);
								attack = atol(compresswindow.attack.text);
                break;
            }
            break;
        }
      break;                                      /* end case keypress */

      case ButtonPress:
        x = report.xbutton.x;
        y = report.xbutton.y;
                                       /* ok button pressed */
        if(buttonpressrelease(compresspixmap, compresswin, x, y, &compresswindow.ok)){
          XDestroyWindow(display, compresswin);
          XFreePixmap(display, compresspixmap);
					attack = atol(compresswindow.attack.text);
          result = 1;
          break;
        }
                                       /* cancel button pressed */
        if(buttonpressrelease(compresspixmap, compresswin, x, y,
          &compresswindow.cancel))
        {
          XDestroyWindow(display, compresswin);
          XFreePixmap(display, compresspixmap);
          killeffectfile();
          return 1;
          break;
        }

                                     /* load button pressed */
        if(buttonpressrelease(compresspixmap, compresswin, x, y,
          &compresswindow.load))
        {
					drawlevels();
          loadcompress();
					drawlevels();
        	sprintf(compresswindow.attack.text, "%d", attack);
        	activate_box(compresspixmap, compresswin, &compresswindow.attack);
          break;
        }

                                     /* save button pressed */
        if(buttonpressrelease(compresspixmap, compresswin, x, y,
          &compresswindow.save))
        {
          savecompress();
          break;
        }

                             /* histogram */
        if(buttonpressrelease(compresspixmap, compresswin, x, y,
          &compresswindow.histogram))
        {
  				buildhistogram(selectionstart, selectionend);
     	  }


                                     /* reset button pressed */
        if(buttonpressrelease(compresspixmap, compresswin, x, y,
          &compresswindow.reset))
        {
					drawlevels();
					totalpoints = 2;
					compresspoint[0].inlevel = INFINITYGAIN2;
					compresspoint[0].outlevel = INFINITYGAIN2;
					compresspoint[1].inlevel = 0;
					compresspoint[1].outlevel = 0;
					drawlevels();
          break;
        }
        
                             /* ms button pressed */
        if(testbox(x, y, &compresswindow.ms))
        {
        	if(attacktype == SAMPLES){   /* convert samples to ms and redraw */
	        	attacktype = MS;
	        	drawattacktype();
	        	attack *= 1000;
	        	attack /= samplerate;
	        	sprintf(compresswindow.attack.text, "%ld", attack);
	        	activate_box(compresspixmap, compresswin, &compresswindow.attack);
	        	break;
        	}
        }

                             /* samples button pressed */
        if(testbox(x, y, &compresswindow.samples))
        {
        	if(attacktype == MS){   /* convert ms to samples and redraw */
        		attacktype = SAMPLES;
        		drawattacktype();
	        	attack *= samplerate;
	        	attack /= 1000;
	        	sprintf(compresswindow.attack.text, "%ld", attack);
	        	activate_box(compresspixmap, compresswin, &compresswindow.attack);
	        	break;
        	}
        }

        								/* attack box selected */
        if(testbox(x, y, (bcbox *)&compresswindow.attack))
        {
          handle_text_event(compresspixmap, compresswin, &report, &compresswindow.attack);
          break;
        }

												/* level adjust */
        if(testbox(x, y, (bcbox *)&compresswindow.levels))
        {
        	leveladjust(x, y, &report);
          break;
        }
        break;

      default:
        break;
    }                           /* end switch report type */
  }while(!result);   /* end event handler */

/*================================processor================================*/
	if(buildtable(&conversiontable)){
    killeffectfile();
		return 1;   /* build table of level conversions from points */
	}

	if(attack <= 0){
    killeffectfile();
    displayalert("Attack too small.");
    killtable(&conversiontable);
		return 1;   /* build table of level conversions from points */
	}
	
	/* ms to bytes */
	if(attacktype == MS){
		attack_ = attack * samplerate;
		attack_ /= 1000;
		attack_ *= 2;
	}      /* attack is now in bytes */
	else{
		attack_ = attack * 2;
	}
	
	bufferlength = SAVEBUFFER + attack_;
  if(!(inbufferstart = malloc(bufferlength))){
    printf("Not enough memory to compress.\n");
    killtable(&conversiontable);
    killeffectfile();
    return 1;
  }

  initializeprogress("Compress/expand...");

  for(currenttrack = firsttrack; currenttrack;
    currenttrack = currenttrack->nexttrack)
  {
    if(currenttrack->record){
      register TWO *inbuffer_, *readahead_;
      register TWO *readaheadend, *startofslope, *endofslope;
      static long x, i;
      register float currentvalue, highestvalue, startvalue;  /* used to predict changes */
      register float startscale, currentscale;       /* used to calculate scale */
      register double testslope, valueslope, scaleslope;

			bufferlength = SAVEBUFFER + attack_;
      x = selectionstart;
      currentvalue = 0;
      startvalue = 0;
      highestvalue = 0;
      valueslope = 0;  /* start at no change */
      startscale = 0;
      scaleslope = 0;
      startofslope = (TWO *)inbufferstart;
      endofslope = (TWO *)inbufferstart;
      inbuffer_ = (TWO *)inbufferstart;
      readahead_ = (TWO *)inbufferstart;
			readaheadend = (TWO *)inbufferstart + (bufferlength + attack_) / 2;

      while(x < selectionend){
                  /* first buffer encompasses bufferlength + attack */
     		if(x == selectionstart){
					bufferlength = SAVEBUFFER + attack_;  /* edit data */

      		if(selectionend - x < bufferlength / 2){  /* end of track reached */
      			bufferlength = (selectionend - x) * 2;
      			readaheadend = (TWO *)inbufferstart + bufferlength / 2;
      		}

        	if(readbuffer(inbufferstart, bufferlength, currenttrack, x)){
          	destroyprogress();
          	killeffectfile();
          	free(inbufferstart);
          	killtable(&conversiontable);
				   	return 1;
        	}
        }
        else   /* additional buffers encompass just bufferlength */
        {
					bufferlength = SAVEBUFFER;  /* edit data */

      		if(selectionend - x < bufferlength / 2){  /* end of track reached */
      			bufferlength = (selectionend - x) * 2;
      			readaheadend = (TWO *)inbufferstart + (attack_ + bufferlength) / 2;
      		}
                 /* add additional buffer after attack */
        	if(readbuffer(inbufferstart + attack_, bufferlength, currenttrack, x)){
          	destroyprogress();
          	killeffectfile();
          	free(inbufferstart);
          	killtable(&conversiontable);
          	return 1;
        	}
        }

				while(readahead_ < readaheadend){
              /* use highest value first and if the valueslope is negative */
          
          /* end of slope has been passed */
          if(inbuffer_ >= endofslope){
						startofslope = inbuffer_;
						endofslope = inbuffer_;
						valueslope = 0;  /* reset slope */
						scaleslope = 0;
						startscale = conversiontable[*inbuffer_];
						startvalue = *inbuffer_;
    				highestvalue = *inbuffer_;
    				readahead_ = inbuffer_;
    			}
    			
					                /* look at entire range for highest value */
					while(readahead_ < readaheadend 
						&& readahead_ < inbuffer_ + attack_ / 2 && valueslope <= 0)
					{
						if(readahead_ > startofslope && *readahead_ > highestvalue){
							highestvalue = *readahead_;
							valueslope = (*readahead_ - startvalue) / (readahead_ - startofslope);
							scaleslope = (conversiontable[*readahead_] - startscale) / (readahead_ - startofslope);
							if(abs(valueslope) < MINSLOPE) valueslope = 0;
							if(abs(scaleslope) < MINSLOPE) scaleslope = 0;
							endofslope = readahead_;
						}
						readahead_++;
					}
					if(valueslope <= 0) endofslope = readahead_;

            /* readahead_ is now attack_ distance from inbuffer_ or
            	 the valueslope turned positive before attack distance */
            	 
                   /* modify inbuffer if the valueslope is negative or zero */
					while(inbuffer_ < endofslope 
						&& readahead_ < readaheadend && valueslope <= 0)
					{
						currentscale = startscale + scaleslope * (inbuffer_ - startofslope);
						*inbuffer_ *= pow(2, currentscale / 3);
						if(*readahead_ > highestvalue){
							currentvalue = startvalue + valueslope * (inbuffer_ - startofslope);
							
							highestvalue = *readahead_;
							startvalue = currentvalue;
							startofslope = inbuffer_;
          		endofslope = readahead_;
          		startscale = currentscale;
							valueslope = (*readahead_ - startvalue) / (readahead_ - startofslope);
							scaleslope = (conversiontable[*readahead_] - currentscale) / (readahead_ - startofslope);
							if(abs(valueslope) < MINSLOPE) valueslope = 0;
							if(abs(scaleslope) < MINSLOPE) scaleslope = 0;
						}
						inbuffer_++;
						readahead_++;
					}


                /* use highest valueslope if the valueslope is positive */
          /* end of valueslope has been passed */
					if(inbuffer_ >= endofslope){
						startofslope = inbuffer_;
						endofslope = inbuffer_;
						startvalue = *inbuffer_;
						startscale = conversiontable[*inbuffer_];
						scaleslope = 0;
						valueslope = 0;
					}


							     /* look at remainder of range for highest valueslope */
					while(readahead_ < readaheadend 
						&& readahead_ < inbuffer_ + attack_ / 2 && valueslope >= 0)
					{
						if(readahead_ - startofslope > 0){
							testslope = (*readahead_ - startvalue) / (readahead_ - startofslope);
							if(abs(testslope) < MINSLOPE) testslope = 0;
							if(testslope > valueslope){
								valueslope = testslope;
								endofslope = readahead_;
								scaleslope = (conversiontable[*readahead_] - startscale) / (readahead_ - startofslope);
								if(abs(scaleslope) < MINSLOPE) scaleslope = 0;
							}
						}
						readahead_++;
					}

            /* readahead_ is now attack distance from inbuffer_ */
                   /* modify inbuffer until end of valueslope or
                   		end of buffer if the valueslope is positive */
					while(inbuffer_ < endofslope && readahead_ < readaheadend && valueslope >= 0){
						currentscale = startscale + scaleslope * (inbuffer_ - startofslope);
						*inbuffer_ *= pow(2, currentscale / 3);

						currentvalue = startvalue + valueslope * (inbuffer_ - startofslope);
						testslope = (*readahead_ - currentvalue) / (readahead_ - inbuffer_);
						if(abs(testslope) < MINSLOPE) testslope = 0;
						if(testslope > valueslope){
							endofslope = readahead_;
							startofslope = inbuffer_;
							startvalue = currentvalue;
							valueslope = testslope;
							startscale = currentscale;
							scaleslope = (conversiontable[*readahead_] - startscale) / (readahead_ - startofslope);
							if(abs(scaleslope) < MINSLOPE) scaleslope = 0;
						}
						inbuffer_++;
						readahead_++;
					}
					/* peak has been reached and time to get new valueslope for next range */
					/* or end of buffer has been reached */
				}

       progress += bufferlength / 2;
        if(x == selectionstart){
        	x += bufferlength / 2;
					bufferlength -= attack_;   /* for writebuffer */
				}else{
					x += bufferlength / 2;
				}

				if(x < selectionend){
        	if(writebuffer(inbufferstart, bufferlength)){
          	free(inbufferstart);         /* writebuffer failed */
          	killtable(&conversiontable);
          	return 1;
        	}

        	inbuffer_ -= bufferlength / 2;
        	readahead_ -= bufferlength / 2;
        	startofslope -= bufferlength / 2;
        	endofslope -= bufferlength / 2;

        	for(i = bufferlength; i < bufferlength + attack_; i++){
        		/* inbufferstart is a char so count in chars */
        		inbufferstart[i - bufferlength] = inbufferstart[i];
        	}
        }
      }           /* end x < selectionend */
      while(inbuffer_ < readaheadend && inbuffer_ < endofslope){
     		currentscale = startscale + valueslope * (inbuffer_ - startofslope);
				*inbuffer_++ *= pow(2, currentscale / 3);
      }
      
      while(inbuffer_ < readaheadend){
      	*inbuffer_++ *= pow(2, currentscale / 3);
      }
     
			if(writebuffer(inbufferstart, (char *)inbuffer_ - inbufferstart)){
        free(inbufferstart);         /* writebuffer failed */
        killtable(&conversiontable);
        return 1;
      }
       nextchannel();
    }
  }

  closeeffectfile();
  free(inbufferstart);
  killtable(&conversiontable);

  recevent.length = selectionend - selectionstart;
  strcpy(recevent.file, filename);
  recevent.channels = totaltracks;
  recevent.format = effectfiletype;
  reclabels = 0;
  paste(selectionstart, selectionend, EFFECT);
  return 0;
}
