/*
  Broadcast 2.0 multitrack audio editing
  (c) 1997 Adam Williams

  This program is distributed with the intent that it will be useful, without
  any warranty.  The code is being updated and has many bugs which are
  constantly being elucidated so changes should not be made without notifying
  the author.
*/

/* console.c      console interface and persistent storage   */

#include "defs.h"
#include "interfac.h"
#include "console.h"
#include "file.h"
#include "database.h"
#include "pictures.h"
#include "undo.h"
#include "units.h"
#include "play.h"

struct consolewindow_ consolewindow = {
  722, 120, "Console", 60, 709,

  {
    {                  /* eq1 */
      20, 0, 90, 10, 
      
      {20, 0, 15, 15, ""},         /* in */
      {40, 0, 15, 15, ""},         /* bp */
    	{25, 0, 30, 30, POT_DB, INFINITYGAIN, 15, 0, 0, 0, ""}, /* level pot */
    	{56, 0, 30, 30, POT_FREQ, 0, TOTALFREQS, 0, 0, 0, ""}, /* freq pot */
    	{87, 0, 30, 30, POT_INT, 2, 50, 0, 0, 0, ""}, /* width pot */
    },

    {                  /* eq2 */
      135, 0, 90, 10,

      {135, 0, 15, 15, ""},         /* in */
      {155, 0, 15, 15, ""},          /* bp */
    	{140, 0, 30, 30, POT_DB, INFINITYGAIN, 15, 0, 0, 0, ""}, /* level pot */
    	{171, 0, 30, 30, POT_FREQ, 0, TOTALFREQS, 0, 0, 0, ""}, /* freq pot */
    	{202, 0, 30, 30, POT_INT, 2, 50, 0, 0, 0, ""}, /* width pot */
    },

    {                  /* eq3 */
      250, 0, 90, 10, 

      {250, 0, 15, 15, ""},         /* in */
      {270, 0, 15, 15, ""},          /* bp */
    	{255, 0, 30, 30, POT_DB, INFINITYGAIN, 15, 0, 0, 0, ""}, /* level pot */
    	{286, 0, 30, 30, POT_FREQ, 0, TOTALFREQS, 0, 0, 0, ""}, /* freq pot */
    	{317, 0, 30, 30, POT_INT, 2, 50, 0, 0, 0, ""}, /* width pot */
    },
  },

  {510, 0, 190, 14, SLIDER_DB, INFINITYGAIN, 3,  0, 0, ""},         /* level */
  {400, 0, 100, 14, SLIDER_DB, INFINITYGAIN, 0, 0, 0, ""},          /* pan   */

  {506, 0, 200, 20, ""},             /* group */
  {370, 0, 15, 15},                  /* invert */
  {709, 0, 12, 119, 0, 0, 0, 0}      /* scroll bar */
};

static int totaltracks;

void drawinvert(bcbox *invert, char invertvalue, char flash){
  if(invertvalue) DrawDnBox(consolepixmap,
    invert->x, invert->y + 2, invert->w, invert->h);
  else DrawUpBox(consolepixmap,
    invert->x, invert->y + 2, invert->w, invert->h);
  if(flash) XCopyArea(display, consolepixmap, consolewin, gc,
    invert->x, invert->y, invert->w+1, invert->h+3,
    invert->x, invert->y);
}

/* draw a track module and don't flash on the screen */
void drawconsolecontrol(struct Track *currenttrack, int y, int track){
  static int i, y1;
  static char string[5];

  DrawUpBox(consolepixmap, 0, y, consolewindow.modulew-1, consolewindow.moduleh-1);

  draweqcontrol(consolepixmap, consolewin, currenttrack->eq, y);
/* draw pans */
  for(consolewindow.pan.y = y+2, i = 0;
    i < outchannels; consolewindow.pan.y += 20, i++)
  {
    consolewindow.pan.value = currenttrack->pan[i];
    
		initslider(consolewin, consolepixmap, &consolewindow.pan, 0);
  }

  consolewindow.level.y = y + 18;
  consolewindow.level.value = currenttrack->level;
  
/* draw fader */
	initslider(consolewin, consolepixmap, &consolewindow.level, 0);

/* draw INV toggle */
  consolewindow.invert.y = y + 2;
  drawinvert(&consolewindow.invert, currenttrack->invert, NO_FLASH);

/* draw descriptions */
  XSetForeground(display, gc, black.pixel);
  sprintf(string, "%d", track+1);
  XDrawString(display, consolepixmap, gc, 5, y + 7 + consolewindow.moduleh / 2,
    string, strlen(string));

  XSetForeground(display, gc, purple.pixel);
  sprintf(string, "PAN");
  XDrawString(display, consolepixmap, gc,
    consolewindow.pan.x + 40, consolewindow.pan.y + 10,
    string, strlen(string));

  XSetForeground(display, gc, black.pixel);
  sprintf(string, "FADE");
  XDrawString(display, consolepixmap, gc,
    consolewindow.level.x + 50, consolewindow.level.y + 28,
    string, strlen(string));

  XSetForeground(display, gc, yellow.pixel);
  sprintf(string, "INV");
  XDrawString(display, consolepixmap, gc, consolewindow.invert.x - 5, consolewindow.invert.y + 35,
    string, strlen(string));
}

/* print the tracks contained in this group to the button text */
void printtracks(struct TrackGroup *currentgroup, int totaltracks){
  static int i;
  static char string_[5];

  consolewindow.group.text[0] = 0;
  for(i = 0;
    i < totaltracks && strlen(consolewindow.group.text) < 25; i++)
  {
    if(currentgroup->tracks[i]){
      sprintf(string_, "%d ", i + 1);
      strcat(consolewindow.group.text, string_);
    }
  }
  if(i < totaltracks) strcat(consolewindow.group.text, "...");
}

/* draw a single track group and don't flash it on the screen */
void drawconsolegroup(struct TrackGroup *currentgroup, int y, int group){
  static int i, y1;
  static char string[256], string_[5];
  static int *track;
  static int totaltracks;

  totaltracks = tracks();
  DrawUpBox(consolepixmap, 0, y, consolewindow.modulew-1, consolewindow.moduleh-1);

  draweqcontrol(consolepixmap, consolewin, currentgroup->eq, y);
  
/* draw pans */
  for(consolewindow.pan.y = y+2, i = 0;
    i < outchannels; consolewindow.pan.y += 20, i++)
  {
    consolewindow.pan.value = currentgroup->pan[i];
    
		initslider(consolewin, consolepixmap, &consolewindow.pan, 0);
  }
  consolewindow.level.y = y + 2;
  consolewindow.level.value = currentgroup->level;
  
/* draw fade */
	initslider(consolewin, consolepixmap, &consolewindow.level, 0);
  
/* draw invert */
  consolewindow.invert.y = y + 2;
  drawinvert(&consolewindow.invert, currentgroup->invert, NO_FLASH);

  printtracks(currentgroup, totaltracks);

  consolewindow.group.y = y + consolewindow.moduleh - consolewindow.group.h-3;
  DrawUpButton(consolepixmap, consolewin, &consolewindow.group);

  XSetForeground(display, gc, black.pixel);

  sprintf(string, "PAN");
  XDrawString(display, consolepixmap, gc,
    consolewindow.pan.x + 40, consolewindow.pan.y + 10,
    string, strlen(string));
  sprintf(string, "FADE");
  XDrawString(display, consolepixmap, gc,
    consolewindow.level.x + 50, consolewindow.level.y + 28,
    string, strlen(string));
  sprintf(string, "INV");
  XDrawString(display, consolepixmap, gc, consolewindow.invert.x - 5, consolewindow.invert.y + 35,
    string, strlen(string));
}

/* draw the console and flash it on the screen */
void drawconsolecontrols(){
  static int track, y;

               /* set module height here because there are too many routines
                  which set the number of outchannels */
  consolewindow.moduleh = 20 * outchannels + 20;
               /* if 1 channel, fake module height */
  if(consolewindow.moduleh < 60) consolewindow.moduleh = 60;

  if(!showtrackgroups){
    static int track;
    struct Track *currenttrack;
    static int totaltracks;

    totaltracks = tracks();
    XSetForeground(display, gc, grey.pixel);
    XFillRectangle(display, consolepixmap, gc, 0, 0,
      consolewindow.modulew, consolewindow.h);
    currenttrack = trackof(consolestarty);

    for(y = 0, track = consolestarty; y < consolewindow.h, track < totaltracks;
      y += consolewindow.moduleh, track++, currenttrack = currenttrack->nexttrack)
    {
      drawconsolecontrol(currenttrack, y, track);
    }

    updateyscrollbar(consolepixmap, consolewin, consolestarty,
      consolewindow.h / consolewindow.moduleh, tracks(), &consolewindow.scroll);
  }else{
    static int group;
    struct TrackGroup *currentgroup;

    XSetForeground(display, gc, grey.pixel);
    XFillRectangle(display, consolepixmap, gc, 0, 0,
      consolewindow.modulew, consolewindow.h);
    currentgroup = groupof(groupstarty);

    for(y = 0, group = groupstarty;
      y < consolewindow.h && currentgroup;
      y += consolewindow.moduleh, group++, currentgroup = currentgroup->nextgroup)
    {
      drawconsolegroup(currentgroup, y, group);
    }
    updateyscrollbar(consolepixmap, consolewin, groupstarty,
      consolewindow.h / consolewindow.moduleh, groups(), &consolewindow.scroll);
  }
  refreshconsole();
}

void consolescrollevent(XEvent *report){
  static int starty_;

  switch(report->type){
    case ButtonPress:
      yscrollevent(consolepixmap, consolewin, report, &consolewindow.scroll);
      break;

    case ButtonRelease:
      yscrollevent(consolepixmap, consolewin, report,
        &consolewindow.scroll);
      drawconsolecontrols();
      break;
  }

  if(consolewindow.scroll.event){
    switch(consolewindow.scroll.event){
      case SPAN:
        if(report->type == MotionNotify){
          static FOUR position_;

          position_ = consolewindow.scroll.position;
          yscrollevent(consolepixmap, consolewin, report,
            &consolewindow.scroll);
          if(!showtrackgroups) consolestarty = consolewindow.scroll.position;
          else groupstarty = consolewindow.scroll.position;
          if(position_ != consolewindow.scroll.position) drawconsolecontrols();
        }
        break;

      case ARROW1:
        consolemoveup(1);
        break;

      case ARROW2:
        consolemovedown(1);
        break;

      case UPPAGE:
        consolemoveup(consolewindow.h / consolewindow.moduleh);
        break;

      case DNPAGE:
        consolemovedown(consolewindow.h / consolewindow.moduleh);
        break;
    }
  }
}

void consolebuttonpress(XEvent *report){
  static int y, y1, y2, x;

  totaltracks = tracks();
  x = report->xbutton.x;
  y = report->xbutton.y;

  consoleevent.panadjust = 0;
  consoleevent.leveladjust = 0;
  consoleevent.track = 0;

  if(testbox(x, y, (bcbox *)&consolewindow.scroll))
    consolescrollevent(report);
	else
  if(!showtrackgroups){
/*========================test a module========================*/
    static struct Track *currenttrack;
    static int track, pannum;

    track = y / consolewindow.moduleh + consolestarty;
    currenttrack = trackof(track);

    if(currenttrack){
      consolewindow.level.y = (track - consolestarty) * consolewindow.moduleh + 18;
      consolewindow.invert.y = (track - consolestarty) * consolewindow.moduleh + 2;

      if(testbox(x, y, &consolewindow.invert)){
        currenttrack->invert ^= 1;
        drawinvert(&consolewindow.invert, currenttrack->invert, FLASH);
        goto end;
      }
                                                           /* test fader */
      if(testbox(x, y, (bcbox *)&consolewindow.level)){
        consoleevent.consoleadjust = 1;
        consoleevent.track = track;
        consoleevent.currenttrack = currenttrack;

        consoleevent.leveladjust = 1;
        goto end;
      }                                                    /* test pans */
      for(consolewindow.pan.y = 
      	(track - consolestarty) * consolewindow.moduleh + 2, pannum = 0;
        pannum < outchannels;
        consolewindow.pan.y += 20, pannum++)
      {
        if(testbox(x, y, (bcbox *)&consolewindow.pan)){
          consoleevent.consoleadjust = 1;
          consoleevent.track = track;
          consoleevent.currenttrack = currenttrack;

          consoleevent.panadjust = 1;
          consoleevent.pan = pannum;
          goto end;
        }
      }
      testeq(x, y, track, currenttrack->eq);
    }
  }else{
/*======================test a group===========================*/
    static struct TrackGroup *currentgroup;
    static int group, pannum;
    static int totaltracks;

    group = y / consolewindow.moduleh + groupstarty;

    currentgroup = groupof(group);

    if(currentgroup){
      consolewindow.level.y = (group - groupstarty) * consolewindow.moduleh + 2;
      consolewindow.invert.y = (group - groupstarty) * consolewindow.moduleh + 2;

      if(testbox(x, y, &consolewindow.invert)){
        currentgroup->invert ^= 1;
        drawinvert(&consolewindow.invert, currentgroup->invert, FLASH);
        goto end;
      }
                                                           /* test fader */
      if(testbox(x, y, (bcbox *)&consolewindow.level)){
        consoleevent.consoleadjust = 1;
        consoleevent.group = group;
        consoleevent.currentgroup = currentgroup;

        consoleevent.leveladjust = 1;
        goto end;
      }

                                  /* test list button */
      consolewindow.group.y = (group - groupstarty + 1) * consolewindow.moduleh
        - consolewindow.group.h - 3;
      totaltracks = tracks();
      printtracks(currentgroup, totaltracks);
      if(!playevent.playsound &&
        buttonpressrelease(consolepixmap, consolewin, x, y, &consolewindow.group))
      {
        editconsolegroup(currentgroup);
        drawconsolecontrols();
        goto end;
      }
                                                          /* test pans */
      for(consolewindow.pan.y = 
      	(group - groupstarty) * consolewindow.moduleh + 2, pannum = 0;
        pannum < outchannels;
        consolewindow.pan.y += 20, pannum++)
      {
        if(testbox(x, y, (bcbox *)&consolewindow.pan)){
          consoleevent.consoleadjust = 1;
          consoleevent.group = group;
          consoleevent.currentgroup = currentgroup;

          consoleevent.panadjust = 1;
          consoleevent.pan = pannum;
          goto end;
        }
      }
      testeq(x, y, group, currentgroup->eq);
    }
  }
end:
  consolemotion(report);
}

void consolebuttonrelease(XEvent *report){
  consoleevent.consoleadjust = 0;
  consoleevent.eqleveladjust = 0;
  consoleevent.eqfreqadjust = 0;
  consoleevent.eqwidthadjust = 0;
  if(consolewindow.scroll.event) consolescrollevent(report);
}

void consolemotion(XEvent *report){
	updateundo(TWEEKCONSOLE);
	changesmade = 1;

  if(consoleevent.consoleadjust){
    if(consoleevent.panadjust){
      if(!showtrackgroups){        /* set the database value */
        consolewindow.pan.y = consoleevent.pan * 20
          + (consoleevent.track - consolestarty) * consolewindow.moduleh + 2;
				sliderevent(consolewin, consolepixmap, report, &consolewindow.pan, 1);

        consoleevent.currenttrack->pan[consoleevent.pan] =
          consolewindow.pan.value;
      }else{
        consolewindow.pan.y = consoleevent.pan * 20
          + (consoleevent.group - groupstarty) * consolewindow.moduleh + 2;
				sliderevent(consolewin, consolepixmap, report, &consolewindow.pan, 1);

        consoleevent.currentgroup->pan[consoleevent.pan] =
          consolewindow.pan.value;
      }
    }

    if(consoleevent.leveladjust){
      if(!showtrackgroups){
        consolewindow.level.y = (consoleevent.track - consolestarty) * consolewindow.moduleh + 18 ;
				sliderevent(consolewin, consolepixmap, report, &consolewindow.level, 1);

        consoleevent.currenttrack->level = consolewindow.level.value;
      }else{
        consolewindow.level.y = (consoleevent.track - consolestarty) * consolewindow.moduleh + 2;
				sliderevent(consolewin, consolepixmap, report, &consolewindow.level, 1);

        consoleevent.currentgroup->level = consolewindow.level.value;
      }
    }

    if(consoleevent.eqleveladjust) eqlevelmotion(report);
    if(consoleevent.eqfreqadjust) eqfreqmotion(report);
    if(consoleevent.eqwidthadjust) eqwidthmotion(report);
  }
}

void consoledecreaselevel(){
  if(!showtrackgroups){
    static struct Track *currenttrack;

    for(currenttrack = firsttrack; currenttrack; currenttrack = currenttrack->nexttrack){
      currenttrack->level -= 0.1;
      if(currenttrack->level < consolewindow.level.minvalue)
        currenttrack->level = consolewindow.level.minvalue;
    }
  }else{
    static struct TrackGroup *currentgroup;

    for(currentgroup = firsttrackgroup; currentgroup; currentgroup =
    currentgroup->nextgroup){
      currentgroup->level -= 0.1;
      if(currentgroup->level < consolewindow.level.minvalue)
      	currentgroup->level = consolewindow.level.minvalue;
    }
  }
  drawconsolecontrols(consolestarty);
}

void consoleincreaselevel(){
  if(!showtrackgroups){
    static struct Track *currenttrack;

    for(currenttrack = firsttrack; currenttrack; currenttrack =
      currenttrack->nexttrack)
    {
      currenttrack->level += 0.1;
      if(currenttrack->level > consolewindow.level.maxvalue)
        currenttrack->level = consolewindow.level.maxvalue;
    }
  }else{
    static struct TrackGroup *currentgroup;

    for(currentgroup = firsttrackgroup; currentgroup; currentgroup =
      currentgroup->nextgroup)
    {
      currentgroup->level += 0.1;
      if(currentgroup->level > consolewindow.level.maxvalue)
      	currentgroup->level = consolewindow.level.maxvalue;
    }
  }
  drawconsolecontrols(consolestarty);
}

int playpatchon(int track){
  static struct Track *currenttrack;
  currenttrack = trackof(track);

  currenttrack->play = 1;
  drawpatches(starty);
}

int recpatchon(int track){
  static struct Track *currenttrack;
  currenttrack = trackof(track);

  currenttrack->record = 1;
  drawpatches(starty);
}

int playpatchoff(int track){
  static struct Track *currenttrack;
  currenttrack = trackof(track);

  currenttrack->play = 0;
  drawpatches(starty);
}

int recpatchoff(int track){
  static struct Track *currenttrack;
  currenttrack = trackof(track);

  currenttrack->record = 0;
  drawpatches(starty);
}

void patchbuttonpress(XEvent *report){
  FOUR x, y;
  int track;
  int _y;
  struct Track *currenttrack;

  totaltracks = tracks();
  x = report->xbutton.x;
  y = report->xbutton.y;

  for(track = starty, _y = yscaletrack / 2 + patchwindow.y - 15;
    _y < patchwindow.h, track < totaltracks; track++, _y += yscaletrack)
  {
    if(y >= _y && y <= _y+20){ /* patch selected */
      if(x >= patchwindow.w / 2){     /* record selected */
        patchevent.patchadjust = 1;
        patchevent.recpatchadjust = 1;
        currenttrack = trackof(track);

        if(currenttrack->record) patchevent.on = 0;
          else patchevent.on = 1;
      }else{                   /* play selected */
        patchevent.patchadjust = 1;
        patchevent.playpatchadjust = 1;
        currenttrack = trackof(track);

        if(currenttrack->play) patchevent.on = 0;
          else patchevent.on = 1;
      }
    }
  }
  if(patchevent.patchadjust) patchmotion(report);
}

void patchmotion(XEvent *report){
  FOUR x, y;
  int track;
  int _y;

  x = report->xmotion.x;
  y = report->xmotion.y;

  for(track = starty, _y = yscaletrack / 2 + patchwindow.y - 15;
    _y < patchwindow.h, track < totaltracks; track++, _y += yscaletrack){
    if(y >= _y && y <= _y+20){ /* patch selected */
      if(patchevent.recpatchadjust){     /* record selected */
        if(patchevent.on) recpatchon(track);
          else recpatchoff(track);
      }else{                   /* play selected */
        if(patchevent.playpatchadjust){
          if(patchevent.on) playpatchon(track);
            else playpatchoff(track);
        }
      }
    }
  }
}

void patchbuttonrelease(XEvent *report){
  patchevent.patchadjust = 0;
  patchevent.playpatchadjust = 0;
  patchevent.recpatchadjust = 0;
}

void deletegroup(struct TrackGroup *currentgroup){
  static struct TrackGroup *frontgroup, *backgroup;

  free(currentgroup->tracks);
  free(currentgroup->pan);

  frontgroup = currentgroup->prevgroup;
  backgroup = currentgroup->nextgroup;

  if(currentgroup == firsttrackgroup) firsttrackgroup = backgroup;
  if(currentgroup == lasttrackgroup) lasttrackgroup = frontgroup;
  if(frontgroup) frontgroup->nextgroup = backgroup;
  if(backgroup) backgroup->prevgroup = frontgroup;
  free(currentgroup);
}

void addgroup(){
  struct TrackGroup *currentgroup;
  int totaltracks, i;

  if(!lasttrackgroup){
    currentgroup = lasttrackgroup = firsttrackgroup =
      malloc(sizeof(struct TrackGroup));
    currentgroup-> prevgroup = currentgroup->nextgroup = 0;
  }else{
    currentgroup = malloc(sizeof(struct TrackGroup));
    currentgroup->prevgroup = lasttrackgroup;
    lasttrackgroup->nextgroup = currentgroup;
    lasttrackgroup = currentgroup;
    lasttrackgroup->nextgroup = 0;
  }

  totaltracks = tracks();                                 /* allocate memory */
  currentgroup->tracks = malloc(totaltracks);
  currentgroup->pan = malloc(4 * outchannels);
                                                /* set defaults */
  for(i = 0; i < totaltracks; i++) currentgroup->tracks[i] = 0;
  for(i = 0; i < outchannels; i++) currentgroup->pan[i] = 0;
  currentgroup->level = 0;
  currentgroup->invert = 0;
  for(i = 0; i < TOTALEQS; i++){
    currentgroup->eq[i].level = 0;
    currentgroup->eq[i].freq = 0;
    currentgroup->eq[i].width = consolewindow.eq[i].widthpot.minvalue;
    currentgroup->eq[i].in = 0;
  }

  editconsolegroup(currentgroup);
  drawconsolecontrols();
}

void readgroups(FILE *in, int newchannels, int track1){
  int totalgroups, i, group;
  struct TrackGroup *currentgroup;
  int newtracks, totaltracks;
  char *newgrouptracks;

  totalgroups = gettwo(in);
  newtracks = gettwo(in);
  totaltracks = tracks();

  for(currentgroup = firsttrackgroup; currentgroup;
    currentgroup = currentgroup->nextgroup)        /* expand existing group tracks */
  {
    newgrouptracks = malloc(totaltracks);
    for(i = 0; i < track1; i++){
      newgrouptracks[i] = currentgroup->tracks[i];
    }
    for(; i < totaltracks; i++){
      newgrouptracks[i] = 0;                            /* null new tracks in old groups */
    }
    free(currentgroup->tracks);
    currentgroup->tracks = newgrouptracks;
  }

  for(group = 0; group < totalgroups; group++){          /* append groups to end of file */
    if(!lasttrackgroup){
      currentgroup = lasttrackgroup = firsttrackgroup =
        malloc(sizeof(struct TrackGroup));
      currentgroup-> prevgroup = currentgroup->nextgroup = 0;
    }else{
      currentgroup = malloc(sizeof(struct TrackGroup));
      currentgroup->prevgroup = lasttrackgroup;
      lasttrackgroup->nextgroup = currentgroup;
      lasttrackgroup = currentgroup;
      lasttrackgroup->nextgroup = 0;
    }

    currentgroup->tracks = malloc(totaltracks);
    currentgroup->pan = malloc(4 * outchannels);

    for(i = 0; i < track1; i++) currentgroup->tracks[i] = 0; /* null existing tracks */
                                                 /* read in new tracks */
    for(; i < totaltracks; i++) currentgroup->tracks[i] = fgetc(in);
                                                 /* load pans and fill extras */
    for(i = 0; i < newchannels && i < outchannels; i++){
      currentgroup->pan[i] = getfloat(in);
    }
                                         /* if too many newchannels, skip extras */
    if(i < newchannels) fseek(in, 4 * (newchannels - i), SEEK_CUR);
                                         /* if too few newchannels, fill outchannels */
    for(; i < outchannels; i++) currentgroup->pan[i] = 0;

    currentgroup->level = getfloat(in);
    currentgroup->invert = fgetc(in);
    for(i = 0; i < TOTALEQS; i++){
      currentgroup->eq[i].level = gettwo(in);
      currentgroup->eq[i].freq = gettwo(in);
      currentgroup->eq[i].width = gettwo(in);
      currentgroup->eq[i].in = fgetc(in);
    }
  }
}

void writegroups(FILE *out){
  struct TrackGroup *currentgroup;
  int totaltracks, i;

  puttwo(groups(), out);
  totaltracks = tracks();
  puttwo(totaltracks, out);

  for(currentgroup = firsttrackgroup;
    currentgroup;
    currentgroup = currentgroup->nextgroup)
  {
    for(i = 0; i < totaltracks; i++) fputc(currentgroup->tracks[i], out);
    for(i = 0; i < outchannels; i++){
      putfloat(currentgroup->pan[i], out);
    }

    putfloat(currentgroup->level, out);
    fputc(currentgroup->invert, out);
    for(i = 0; i < TOTALEQS; i++){
      puttwo(currentgroup->eq[i].level, out);
      puttwo(currentgroup->eq[i].freq, out);
      puttwo(currentgroup->eq[i].width, out);
      fputc(currentgroup->eq[i].in, out);
    }
  }
}

int editconsolegroup(struct TrackGroup *currentgroup){
  int i;
  int totaltracks;
  int result;

  totaltracks = tracks();
  result = editlist(currentgroup->tracks, totaltracks,
    "Select group members", EDITGROUP);

  if(result == 2) deletegroup(currentgroup);
}
