/*
  Broadcast 2.0 multitrack audio editing
  (c) 1997 Adam Williams

  This program is distributed with the intent that it will be useful, without
  any warranty.  The code is being updated and has many bugs which are
  constantly being elucidated so changes should not be made without notifying
  the author.
*/

/* data.c      the Nom-ROM audio powerplant, turns data into reality  */

#include "defs.h"
#include "data.h"
#include "file.h"
#include "database.h"
#include "automate.h"
#include "units.h"

static char *inbufferstart1, *autobufferstart;
static char *inbufferstart2;
static int channels;

static double *prevconsole;       /* previous values for console eq */
static double *prevgroup;         /* previous values for group eq */

                                /* swap bytes for WAV */
void swapbytes(char *buffer, long buffersize){
  char byte2;
  char byte1;
  char *bufferend;

  bufferend = buffer + buffersize;
  
  bufferend -= 8;
  while(buffer < bufferend){
      byte1 = *buffer++;
      byte2 = *buffer--;
      *buffer++ = byte2;
      *buffer++ = byte1;
      byte1 = *buffer++;
      byte2 = *buffer--;
      *buffer++ = byte2;
      *buffer++ = byte1;
      byte1 = *buffer++;
      byte2 = *buffer--;
      *buffer++ = byte2;
      *buffer++ = byte1;
      byte1 = *buffer++;
      byte2 = *buffer--;
      *buffer++ = byte2;
      *buffer++ = byte1;
  }
  
  bufferend += 8;
  while(buffer < bufferend){
      byte1 = *buffer++;
      byte2 = *buffer--;
      *buffer++ = byte2;
      *buffer++ = byte1;
  }
}

int startbuffering(){
  if(!(inbufferstart2 = malloc(SAVEBUFFER))){
    printf("Not enough memory to start input buffering.\n");
    return 1;
  }
  return 0;
}

void killbuffering(){
  free(inbufferstart2);
}

/* load single track into outbuffer */
/* return 1 if error                */
/* return 0 on success              */

int readbuffer(char *outbufferstart, long outbufferlength, struct Track *currenttrack,
  long startx)
{
  static long x;
  static long offset, remaining;
  static long inbufferlength;
  register TWO *_outbuffer, *_outbufferend, *_inbuffer;
  register char *outbufferend;
  register char *inbufferend;
  static int frame;
  static FILE *in;
  static struct Edit *currentedit;
  static long start;              /* start of track in BCAST file format */
  static int i;
  static char mustswapbytes;
  static int newoutchannels;

  x = startx;
  inbufferlength = SAVEBUFFER;

  _outbuffer = (TWO *)outbufferstart;
  outbufferend = outbufferstart + outbufferlength;

                                    /* find segment startx is in */
  currentedit = segmentof(startx, currenttrack);
  if(currentedit == 0){             /* selection is past end of track so clear */
  	outbufferend -= 8;
    while(_outbuffer < (TWO *)outbufferend){
      *_outbuffer++ = 0;
      *_outbuffer++ = 0;
      *_outbuffer++ = 0;
      *_outbuffer++ = 0;
    }

  	outbufferend += 8;
    while(_outbuffer < (TWO *)outbufferend){
      *_outbuffer++ = 0;
    }
  }
                                  /* read each segment */
  while(currentedit && (char *)_outbuffer < outbufferend){
                               /* remaining frames in segment */
    remaining = currentedit->start1 + currentedit->length - x;
    _outbufferend = _outbuffer + remaining;
    if(_outbufferend > (TWO *)outbufferend){
      _outbufferend = (TWO *)outbufferend;
      remaining = ((TWO *)outbufferend - _outbuffer);
    }
                                                     /* segment is silence */
    if(!strcmp(currentedit->file, SILENCE)){
    	_outbufferend -= 4;
      while(_outbuffer < _outbufferend){
        *_outbuffer++ = 0;
        *_outbuffer++ = 0;
        *_outbuffer++ = 0;
        *_outbuffer++ = 0;
      }
    	_outbufferend += 4;
      while(_outbuffer < _outbufferend){
        *_outbuffer++ = 0;
      }
    }else{                                           /* segment isn't silence */
      if(!(in = fopen(currentedit->file, "rb"))){
        printf("Can't open %s for playback.\n", currentedit->file);
        return 1;
      }

      switch(currentedit->format){                   /* find start of segment */
        case WAV:
          frame = currentedit->channels * 2;
          offset = WAVHEADERLENGTH;
          offset += (x - currentedit->start1 + currentedit->start2) * frame;
          offset += currentedit->track * 2;
          break;

        case PCMHILO:
          frame = currentedit->channels * 2;
          offset = 0;
          offset += (x - currentedit->start1 + currentedit->start2) * frame;
          offset += currentedit->track * 2;
          break;

        case PCMLOHI:
          frame = currentedit->channels * 2;
          offset = 0;
          offset += (x - currentedit->start1 + currentedit->start2) * frame;
          offset += currentedit->track * 2;
          break;

        case BCAST:
                          /* skip track information */
          fseek(in, 6, SEEK_SET);
          newoutchannels = gettwo(in);
          fseek(in,
            44 + currentedit->track * (15 + 4 * newoutchannels),
            SEEK_SET);
          start = getfour(in);       /* get start of this track */

          frame = 2;
          offset = start +
            (x - currentedit->start1 + currentedit->start2) * frame;
          break;
      }                       /* frame = # of bytes per sample in original file */

      fseek(in, offset, SEEK_SET);
      inbufferlength = SAVEBUFFER;
      inbufferlength /= frame;
      inbufferlength *= frame;
      mustswapbytes =
        ((currentedit->format == WAV && !byteorder) |
        (currentedit->format == PCMHILO && byteorder) |
        (currentedit->format == PCMLOHI && !byteorder));

                               /* load remaining samples in segment */
      if(currentedit->format == BCAST | (frame == 2 && !mustswapbytes)){
                               /* read directly if possible */
        inbufferlength = (_outbufferend - _outbuffer) * frame;
        fread(_outbuffer, inbufferlength, 1, in);
        _outbuffer += inbufferlength / 2;
      }else{
                               /* buffer if necessary */
        while(_outbuffer < _outbufferend){
          if(inbufferlength > (_outbufferend - _outbuffer) * frame)
            inbufferlength = (_outbufferend - _outbuffer) * frame;

          fread(inbufferstart2, inbufferlength, 1, in);
          if(mustswapbytes) swapbytes(inbufferstart2, inbufferlength);

          _inbuffer = (TWO *)inbufferstart2;
          inbufferend = inbufferstart2 + inbufferlength;
          
          inbufferend -= frame/2 * 8;
          while((char *)_inbuffer < inbufferend){
            *_outbuffer++ = *_inbuffer;
            _inbuffer += frame/2;
            *_outbuffer++ = *_inbuffer;
            _inbuffer += frame/2;
            *_outbuffer++ = *_inbuffer;
            _inbuffer += frame/2;
            *_outbuffer++ = *_inbuffer;
            _inbuffer += frame/2;
          }
          
          inbufferend += frame/2 * 8;
          while((char *)_inbuffer < inbufferend){
            *_outbuffer++ = *_inbuffer;
            _inbuffer += frame/2;
          }
        }
      }
      fclose(in);
    }                    /* end if segment is not silence */
                             /* move on to next segment */
    x += remaining;
    currentedit = currentedit->nextedit;
  }

					            /* clear remaining buffer */
	outbufferend -= 8;
  while(_outbuffer < (TWO *)outbufferend){
    *_outbuffer++ = 0;
    *_outbuffer++ = 0;
    *_outbuffer++ = 0;
    *_outbuffer++ = 0;
  }
  outbufferend += 8;
  while(_outbuffer < (TWO *)outbufferend){
    *_outbuffer++ = 0;
  }
  
  return 0;
}

int startmixdown(long outbufferlength, int channels_){
  int frame;
  long inbufferlength;
  int totaltracks;
  int i;

  channels = channels_;

  frame = channels * 2;
  inbufferlength = outbufferlength / frame * 2;

  if(!(inbufferstart1 = malloc(inbufferlength))){
    printf("Not enough memory to start powerplant.\n");
    return 1;
  }

  if(!(autobufferstart = malloc(inbufferlength))){
    printf("Not enough memory to start automation.\n");
    free(inbufferstart1);
    return 1;
  }

  totaltracks = tracks();
  if(!(prevconsole = malloc(totaltracks * sizeof(double) * TOTALEQS * 2))){
    printf("Not enough memory for console eq.\n");
    free(inbufferstart1);
    if(autoevent.automation) free(autobufferstart);
  }

  if(!(prevgroup = malloc(totaltracks * sizeof(double) * TOTALEQS * 2))){
    printf("Not enough memory for group eq.\n");
    free(inbufferstart1);
    free(prevconsole);
    if(autoevent.automation) free(autobufferstart);
  }

  if(startbuffering()){
    free(inbufferstart1);
    free(prevconsole);
    free(prevgroup);
    if(autoevent.automation) free(autobufferstart);
    return 1;
  }
                                               /* initialize eq buffers */
  for(i = 0; i < totaltracks * TOTALEQS * 2; i++){
    prevconsole[i] = 0;
    prevgroup[i] = 0;
  }

  return 0;
}

void killmixdown(){
  free(inbufferstart1);
  free(autobufferstart);
  free(prevconsole);
  free(prevgroup);
  killbuffering();
}
                           /* mix  down bufferlength to one buffer */
int mixdown(char *outbufferstart, long outbufferlength, long startx){
  static int outchannel, i, track;
  register char *outbufferend;
  register char *autobufferend;
  register TWO *_outbuffer, *_inbuffer;
  static float scale1, scale2;
  static struct Track *currenttrack;
  static struct TrackGroup *currentgroup;
  static char played, firstbp;
  static long endx;
  static long inbufferlength;
  static int frame;

  played = 0;
  
  frame = channels * 2;
  inbufferlength = outbufferlength / frame * 2;

  outbufferend = outbufferstart + outbufferlength;
  autobufferend = autobufferstart + inbufferlength;
  endx = startx + inbufferlength / 2;     /* end sample of current buffer */

                                               /* locate first track */
  for(currenttrack = firsttrack, track = 0;
    currenttrack && (!currenttrack->play ||
    ismuted(currenttrack, startx, endx));
    currenttrack = currenttrack->nexttrack, track++)
    ;

                                 /* set all the channels equal to the first track */
  if(currenttrack && currenttrack->play &&
    !ismuted(currenttrack, startx, endx))
  {
    played = 1;
	  firstbp = 1;   /* no bp has been done yet */
    readbuffer(inbufferstart1, inbufferlength, currenttrack, startx);

    /*====================== process reusable automation ========================*/
    if(autoevent.automation)
      automatebuffer(currenttrack, inbufferstart1, inbufferlength, startx, endx);

    /*===============process console fade, invert ======================*/
    scale1 = topower[(int)(currenttrack->level * 10)];                  /* process fade    */

    if(currenttrack->invert) scale1 *= -1;         /* process invert  */

    /*===============process group fades, inverts ======================*/
    for(currentgroup = firsttrackgroup;
      currentgroup;
      currentgroup = currentgroup->nextgroup)
    {
      if(currentgroup->tracks[track])         /* track is member of group */
      {
        scale1 *= topower[(int)(currentgroup->level * 10)];                /* process fade */

        if(currentgroup->invert) scale1 *= -1;        /* process invert */

                                                   /* process group eq */
        /* groupeq modifies applies scale1 and modifies it */
        processeq(inbufferstart1, inbufferlength, currentgroup->eq,
          &prevgroup[track * 2 * TOTALEQS], &scale1);
      }
    }

    /* process console eq after group fade/pan so scale1 will be calculated from
			the group eq */

    processeq(inbufferstart1, inbufferlength, currenttrack->eq,
      &prevconsole[track * 2 * TOTALEQS], &scale1);

    /*=====================process group band pass filters================================*/
    for(currentgroup = firsttrackgroup;
      currentgroup;
      currentgroup = currentgroup->nextgroup)
    {
      if(currentgroup->tracks[track])         /* track is member of group */
      {
                         /* process group bandpass */
                         /* use automation buffer as output */
				processbp(inbufferstart1, inbufferlength, autobufferstart, 
					currentgroup->eq, &prevgroup[track * 2 * TOTALEQS], &scale1, &firstbp);
      }
    }

                         /* process console bandpass */
                         /* use same automation buffer as output and firstsbp */
                         /* will be 0 if bandpass has already been done */
		processbp(inbufferstart1, inbufferlength, autobufferstart, 
			currenttrack->eq, &prevconsole[track * 2 * TOTALEQS], &scale1, &firstbp);

              /* must copy from autobuffer back to inbuffer or else automatepan
                 must automate autobuffer to inbuffer, a lethal condition */
		if(!firstbp){ /* only copy if bp was performed */
			_inbuffer = (TWO *)autobufferstart;
			_outbuffer = (TWO *)inbufferstart1;
			autobufferend -= 8;

			while((char *)_inbuffer < autobufferend){
				*_outbuffer++ = *_inbuffer++;
				*_outbuffer++ = *_inbuffer++;
				*_outbuffer++ = *_inbuffer++;
				*_outbuffer++ = *_inbuffer++;
			}

			autobufferend += 8;

			while((char *)_inbuffer < autobufferend){
				*_outbuffer++ = *_inbuffer++;
			}
		}
		
    /*================ process pan and mix track into all channels ===================*/
    for(outchannel = 0; outchannel < channels; outchannel++){
      /*==========================process console pan=====================*/
      scale2 = scale1 * topower[(int)(currenttrack->pan[outchannel] * 10)];

      /*==========================process group pans======================*/
      for(currentgroup = firsttrackgroup;
        currentgroup;
        currentgroup = currentgroup->nextgroup)     /* test each group */
      {
        if(currentgroup->tracks[track])         /* track is member of group */
        {
          scale2 *= topower[(int)(currentgroup->pan[outchannel] * 10)];      /* process pan */
        }
      }

      /*===========================process unreusable automation============*/
      if(autoevent.automation){                   /* copy inbuffer to autobuffer */
        automatepan(currenttrack->firstautopan[outchannel],
          inbufferstart1, inbufferlength, autobufferstart, startx, endx, scale2);

                  /* copy autobuffer, a sequence of one channel to
                  	outbuffer, a sequence of many channels */
        _inbuffer = (TWO *)autobufferstart;
        _outbuffer = (TWO *)outbufferstart + outchannel;
        outbufferend -= 8 * channels;

        while((char *)_outbuffer < outbufferend){
          *_outbuffer = *_inbuffer++;
          _outbuffer += channels;

          *_outbuffer = *_inbuffer++;
          _outbuffer += channels;

          *_outbuffer = *_inbuffer++;
          _outbuffer += channels;

          *_outbuffer = *_inbuffer++;
          _outbuffer += channels;
        }

        outbufferend += 8 * channels;
        while((char *)_outbuffer < outbufferend){
          *_outbuffer = *_inbuffer++;
          _outbuffer += channels;
        }
      /*===========================no automation=============================*/
      }else{
        static TWO sample;

                  /* copy autobuffer, a sequence of one channel to
                  	outbuffer, a sequence of many channels and factor in scaling */
        _inbuffer = (TWO *)inbufferstart1;
        _outbuffer = (TWO *)outbufferstart + outchannel;

        if(scale2 != 1){                 /* scaling hasn't been done */
        	outbufferend -= 8 * channels;
          while((char *)_outbuffer < outbufferend){
            sample = *_inbuffer++;
            sample *= scale2;
            *_outbuffer = sample;
            _outbuffer += channels;
          
            sample = *_inbuffer++;
            sample *= scale2;
            *_outbuffer = sample;
            _outbuffer += channels;
          
            sample = *_inbuffer++;
            sample *= scale2;
            *_outbuffer = sample;
            _outbuffer += channels;
          
            sample = *_inbuffer++;
            sample *= scale2;
            *_outbuffer = sample;
            _outbuffer += channels;
          }

        	outbufferend += 8 * channels;
          while((char *)_outbuffer < outbufferend){
            sample = *_inbuffer++;
            sample *= scale2;

            *_outbuffer = sample;
            _outbuffer += channels;
          }
        }else{                            /* scaling has been done */
        	outbufferend -= 8 * channels;
          while((char *)_outbuffer < outbufferend){
            *_outbuffer = *_inbuffer++;
            _outbuffer += channels;
            
            *_outbuffer = *_inbuffer++;
            _outbuffer += channels;
            
            *_outbuffer = *_inbuffer++;
            _outbuffer += channels;
            
            *_outbuffer = *_inbuffer++;
            _outbuffer += channels;
          }
          
        	outbufferend += 8 * channels;
          while((char *)_outbuffer < outbufferend){
            *_outbuffer = *_inbuffer++;
            _outbuffer += channels;
          }
      	} /* end scaling has been done */
   		} /* end no automation */
  	} /* end process pan and mix into all channels */
  } /* end set all channels equal to first track */

/*=================mix later tracks into all the channels================*/
  if(currenttrack){
    for(track++, currenttrack = currenttrack->nexttrack; currenttrack;
      currenttrack = currenttrack->nexttrack, track++)
    {
      if(currenttrack->play && !ismuted(currenttrack, startx, endx))
      {
	  		firstbp = 1;   /* no bp has been done yet */
        readbuffer(inbufferstart1, inbufferlength, currenttrack, startx);
        if(autoevent.automation)                    /* process reusable parameters */
          automatebuffer(currenttrack, inbufferstart1, inbufferlength, startx, endx);

        /*===============process console fade, invert ======================*/
        scale1 = topower[(int)(currenttrack->level * 10)];                  /* process fade    */

        if(currenttrack->invert) scale1 *= -1;         /* process invert  */

        /*===============process group fades, inverts ======================*/
        for(currentgroup = firsttrackgroup;
          currentgroup;
          currentgroup = currentgroup->nextgroup)
        {
          if(currentgroup->tracks[track])         /* track is member of group */
          {
            scale1 *= topower[(int)(currentgroup->level * 10)];           /* process fade    */

            if(currentgroup->invert) scale1 *= -1;   /* process invert  */

                                                     /* process eq      */
            processeq(inbufferstart1, inbufferlength, currentgroup->eq,
              &prevgroup[track * 2 * TOTALEQS], &scale1);
          }
        }

        /* process console eq after group fade/pan so scale1 will be calculated from
        	the group eq*/
        processeq(inbufferstart1, inbufferlength, currenttrack->eq,
          &prevconsole[track * 2 * TOTALEQS], &scale1);

		    /*=====================process band pass filters================================*/
		    for(currentgroup = firsttrackgroup;
		      currentgroup;
		      currentgroup = currentgroup->nextgroup)
		    {
		      if(currentgroup->tracks[track])         /* track is member of group */
		      {
		                         /* process group bandpass */
		                         /* use automation buffer as output */
						processbp(inbufferstart1, inbufferlength, autobufferstart, 
							currentgroup->eq, &prevgroup[track * 2 * TOTALEQS], &scale1, &firstbp);
		      }
		    }
		                         /* process console bandpass */
		                         /* use same automation buffer as output since firstsbp */
		                         /* will be 0 if bandpass has already been done */
				processbp(inbufferstart1, inbufferlength, autobufferstart, 
					currenttrack->eq, &prevconsole[track * 2 * TOTALEQS], &scale1, &firstbp);
					
		              /* must copy from autobuffer back to inbuffer or else automatepan
		                 must automate autobuffer to inbuffer, a lethal condition */
				if(!firstbp){ /* only copy if bp was performed */
					_inbuffer = (TWO *)autobufferstart;
					_outbuffer = (TWO *)inbufferstart1;
					autobufferend -= 8;

					while((char *)_inbuffer < autobufferend){
						*_outbuffer++ = *_inbuffer++;
						*_outbuffer++ = *_inbuffer++;
						*_outbuffer++ = *_inbuffer++;
						*_outbuffer++ = *_inbuffer++;
					}

					autobufferend += 8;

					while((char *)_inbuffer < autobufferend){
						*_outbuffer++ = *_inbuffer++;
					}
				}

   /*==================== process pan and mix track into all channels ==============*/
        for(outchannel = 0; outchannel < channels; outchannel++){
          /*==========================process console pan=====================*/
          scale2 = scale1 * topower[(int)(currenttrack->pan[outchannel] * 10)];

          /*==========================process group pans======================*/
          for(currentgroup = firsttrackgroup;
            currentgroup;
            currentgroup = currentgroup->nextgroup)     /* test each group */
          {
            if(currentgroup->tracks[track])         /* track is member of group */
            {
              scale2 *= topower[(int)(currentgroup->pan[outchannel] * 10)];      /* process pan */
            }
          }

          /*===========================process unreusable automation============*/
          if(autoevent.automation){           /* copy inbuffer to autobuffer */
            automatepan(currenttrack->firstautopan[outchannel],
              inbufferstart1, inbufferlength, autobufferstart, startx, endx, scale2);

            _inbuffer = (TWO *)autobufferstart;
            _outbuffer = (TWO *)outbufferstart + outchannel;

						outbufferend -= 8 * channels;
            while((char *)_outbuffer < outbufferend){
              *_outbuffer += *_inbuffer++;
              _outbuffer += channels;

              *_outbuffer += *_inbuffer++;
              _outbuffer += channels;

              *_outbuffer += *_inbuffer++;
              _outbuffer += channels;

              *_outbuffer += *_inbuffer++;
              _outbuffer += channels;
            }

						outbufferend += 8 * channels;
            while((char *)_outbuffer < outbufferend){
              *_outbuffer += *_inbuffer++;
              _outbuffer += channels;
            }
          }else{
            static TWO sample;

            _inbuffer = (TWO *)inbufferstart1;
            _outbuffer = (TWO *)outbufferstart + outchannel;

            if(scale2 != 1){                         /* scaling has been done */
            	outbufferend -= 8 * channels;
              while((char *)_outbuffer < outbufferend){
                sample = *_inbuffer++;
                sample *= scale2;
                *_outbuffer += sample;
                _outbuffer += channels;

                sample = *_inbuffer++;
                sample *= scale2;
                *_outbuffer += sample;
                _outbuffer += channels;

                sample = *_inbuffer++;
                sample *= scale2;
                *_outbuffer += sample;
                _outbuffer += channels;

                sample = *_inbuffer++;
                sample *= scale2;
                *_outbuffer += sample;
                _outbuffer += channels;
              }

            	outbufferend += 8 * channels;
              while((char *)_outbuffer < outbufferend){
                sample = *_inbuffer++;
                sample *= scale2;
                *_outbuffer += sample;
                _outbuffer += channels;
              }
            }else{                                   /* scaling has been done */
            	outbufferend -= 8 * channels;
              while((char *)_outbuffer < outbufferend){
                *_outbuffer += *_inbuffer++;
                _outbuffer += channels;

                *_outbuffer += *_inbuffer++;
                _outbuffer += channels;

                *_outbuffer += *_inbuffer++;
                _outbuffer += channels;

                *_outbuffer += *_inbuffer++;
                _outbuffer += channels;
              }

            	outbufferend += 8 * channels;
              while((char *)_outbuffer < outbufferend){
                *_outbuffer += *_inbuffer++;
                _outbuffer += channels;
              }
            } /* end scaling has been done */
          } /* end no automation */
        } /* end process pan and mix into all channels */
      } /* end track is audible */
    } /* end mix later tracks into all channels */
  } /* end there is a later track */

  if(!played){  /* if nothing audible, fill buffer with silence */
    _outbuffer = (TWO *)outbufferstart;
    
    outbufferend -= 8;
    while(_outbuffer < (TWO *)outbufferend){
      *_outbuffer++ = 0;
      *_outbuffer++ = 0;
      *_outbuffer++ = 0;
      *_outbuffer++ = 0;
    }

    outbufferend += 8;
    while(_outbuffer < (TWO *)outbufferend){
      *_outbuffer++ = 0;
    }
  }

  return 0;
}
