/*
  Broadcast 2.0 multitrack audio editing
  (c) 1997 Adam Williams

  This program is distributed with the intent that it will be useful, without
  any warranty.  The code is being updated and has many bugs which are
  constantly being elucidated so changes should not be made without notifying
  the author.
*/

/* database.c      RAM database handling and editing routines */

#include "defs.h"
#include "database.h"
#include "interfac.h"
#include "console.h"
#include "graphics.h"
#include "units.h"
#include "audio.h"

void putfourswap(FOUR number, FILE *file){
  fputc(number & 0xff, file);
  fputc((number & 0xff00) >> 8, file);
  fputc((number & 0xff0000) >> 16, file);
  fputc((number & 0xff000000) >> 24, file);
}

void putfour(FOUR number, FILE *file){
  fputc((number & 0xff000000) >> 24, file);
  fputc((number & 0xff0000) >> 16, file);
  fputc((number & 0xff00) >> 8, file);
  fputc(number & 0xff, file);
}

void putfloat(float number, FILE *file){
  fwrite(&number, 4, 1, file);
}

float getfloat(FILE *file){
  static float number;

  fread(&number, 4, 1, file);
  return number;
}

FOUR getfour(FILE *in){
  static FOUR number=0;

  number = (FOUR)fgetc(in) << 24;
  number += (FOUR)fgetc(in) << 16;
  number += (FOUR)fgetc(in) << 8;
  number += fgetc(in);
return number;
}

FOUR getfourswap(FILE *in){
  static FOUR number=0;

  number = (FOUR)fgetc(in);
  number += (FOUR)fgetc(in) << 8;
  number += (FOUR)fgetc(in) << 16;
  number += fgetc(in) << 24;
return number;
}

TWO gettwo(FILE *in){
  static TWO number=0;

  number = (FOUR)fgetc(in) << 8;
  number += fgetc(in);
return number;
}

void puttwo(TWO number, FILE *file){
  fputc((number & 0xff00) >> 8, file);
  fputc(number & 0xff, file);
}

void assigntext(char **location, char *filename){
  *location = malloc(strlen(filename)+1);
  strcpy(*location, filename);
}

void readtext(FILE *in, char **text){
  static int k;
  char newtext[256];

  for(k = 0; (newtext[k] = fgetc(in)) != 0; k++)
    ;
  *text = malloc(k+1);
  strcpy(*text, newtext);
}

void setpans(float **pan, int track){
  static int i;
  static float *_pan;
  int value;

  if(track % 2) value = INFINITYGAIN;
  else value = 0;

  _pan = *pan = malloc(sizeof(float) * outchannels);
  for(i = 0; i < outchannels; i++){
    *_pan++ = (float)value;
    value ^= INFINITYGAIN;
  }
}

void readpans(FILE *in, float *pan, int channels){
  free(pan);
  pan = malloc(sizeof(float) * channels);
  fread(pan, sizeof(float) * channels, 1, in);
}

void writepans(float *pans, FILE *out, int channels){
	int i;
	
	for(i = 0; i < channels; i++){
  	putfloat(pans[i], out);
  }
}

                           /* return length of longest track */
long longesttrack()
{
  static struct Track *track;
  static FOUR end;

  for(track = firsttrack, end = 0; track; track = track->nexttrack){
    if(track->firstedit){
      if(track->lastedit->start1 + track->lastedit->length > end)
        end = track->lastedit->start1 + track->lastedit->length;
    }
  }
  return end;
}

                           /* return length of longest playable track */
long longestplayabletrack()
{
  static struct Track *track;
  static FOUR end;

  for(track = firsttrack, end = 0; track; track = track->nexttrack)
  {
    if(track->firstedit)
    {
      if(track->lastedit->start1 + track->lastedit->length > end && track->play)
        end = track->lastedit->start1 + track->lastedit->length;
    }
  }
  return end;
}

int edits(struct Track *track){
  static int total;
  static struct Edit *edit;

  for(edit = track->firstedit, total = 0;
    edit;
    edit = edit->nextedit, total++)
  ;

  return total;
}

int tracks(){
  static int total;
  static struct Track *track;

  for(track = firsttrack, total = 0; track; track = track->nexttrack, total++)
    ;

  return total;
}

int recordabletracks(){
  static int total;
  static struct Track *track;

  for(track = firsttrack, total = 0; track; track = track->nexttrack)
    if(track->record) total++;

  return total;
}

int groups(){
  static int total;
  static struct TrackGroup *group;

  for(group = firsttrackgroup, total = 0; group; group = group->nextgroup, total++)
    ;

  return total;
}

struct Edit *segmentof(long x, struct Track *currenttrack){
  static struct Edit *currentedit;

  for(currentedit = currenttrack->firstedit;
    currentedit && (currentedit->start1 > x |
      currentedit->start1 + currentedit->length < x);
    currentedit = currentedit->nextedit)
      ;

    return currentedit;
}

struct TrackGroup *groupof(int starty){
  static struct TrackGroup *currentgroup;
  static int group;

  if(starty >= groups() | starty < 0) return 0;
  for(currentgroup = firsttrackgroup, group = 0;
    currentgroup != 0 && group < starty;
    currentgroup = currentgroup->nextgroup, group++){
  }
  return currentgroup;
}

struct Track *trackof(int starty){
  static struct Track *currenttrack;
  static int track;

  if(starty >= tracks() | starty < 0) return 0;
  for(currenttrack = firsttrack, track = 0;
    currenttrack && track < starty;
    currenttrack = currenttrack->nexttrack, track++){
  }
  return currenttrack;
}

struct Label *labelof(long x){
  static struct Label *currentlabel;

  for(currentlabel = firstlabel;
    currentlabel != 0 && currentlabel->position < x;
    currentlabel = currentlabel->nextlabel)
    {
      ;
    }
  return currentlabel;
}

long labels(){
  static struct Label *currentlabel;
  static long i;

  for(currentlabel = firstlabel, i=0;
    currentlabel != 0;
    currentlabel = currentlabel->nextlabel, i++)
    {
      ;
    }
    return i;
}

void insertlabel(long x){
  static struct Label *frontlabel, *backlabel;
  static struct Label *newlabel;

  backlabel = labelof(x);
  newlabel = malloc(sizeof(struct Label));
  frontlabel = 0;

  newlabel->position = x;
  newlabel->nextlabel = backlabel;
  if(backlabel){
    frontlabel = backlabel->prevlabel;
    newlabel->prevlabel = frontlabel;
    backlabel->prevlabel = newlabel;
  }else{
    newlabel->prevlabel = frontlabel = lastlabel;
    lastlabel = newlabel;
  }
  if(frontlabel) frontlabel->nextlabel = newlabel;
  else firstlabel = newlabel;
}

void deletelabel(long x){
  static struct Label *frontlabel, *backlabel, *currentlabel;

  currentlabel = labelof(x);
  frontlabel = currentlabel->prevlabel;
  backlabel = currentlabel->nextlabel;

  if(currentlabel == firstlabel) firstlabel = backlabel;
  if(currentlabel == lastlabel) lastlabel = frontlabel;
  if(frontlabel) frontlabel->nextlabel = backlabel;
  if(backlabel) backlabel->prevlabel = frontlabel;
  free(currentlabel);
}

void cleardatabase(){
  struct Track *currenttrack, *temptrack;
  struct Edit *currentedit, *tempedit;
  struct TrackGroup *currentgroup, *tempgroup;

  for(currenttrack = firsttrack; currenttrack;
    currenttrack = temptrack)
  {
    temptrack = currenttrack->nexttrack;
    for(currentedit = currenttrack->firstedit; currentedit != 0;
      currentedit = tempedit)
      {
        tempedit = currentedit->nextedit;
        free(currentedit->file);
        free(currentedit);
      }
    free(currenttrack->pan);
    free(currenttrack);
  }
  firsttrack = lasttrack = 0;

  for(currentgroup = firsttrackgroup; currentgroup;
    currentgroup = tempgroup)
  {
    tempgroup = currentgroup->nextgroup;
    free(currentgroup->tracks);
    free(currentgroup);
  }
  firsttrackgroup = lasttrackgroup = 0;
}

void clearlabels(){
  struct Label *currentlabel, *templabel;

  for(currentlabel = firstlabel; currentlabel != 0;
    currentlabel = templabel)
  {
    templabel = currentlabel->nextlabel;
    free(currentlabel);
  }
  firstlabel = lastlabel = 0;
}

int optimizeedits(){
  int i, j, k;
  struct Track *currenttrack;
  struct Edit *frontedit, *backedit;

  for(currenttrack = firsttrack; currenttrack; currenttrack = currenttrack->nexttrack){
    do{                                   /* there are still 0 length edits */
      k = 0;
      for(frontedit = currenttrack->firstedit;
        frontedit && !k;
        frontedit = frontedit->nextedit)
      {
        if(frontedit->length == 0){
          if(frontedit->prevedit != 0)
            frontedit->prevedit->nextedit = frontedit->nextedit;
          else
            currenttrack->firstedit = frontedit->nextedit;

          if(frontedit->nextedit != 0)
            frontedit->nextedit->prevedit = frontedit->prevedit;
          else
            currenttrack->lastedit = frontedit->prevedit;

          free(frontedit->file);
          free(frontedit);
          k = 1;
        }
      }
    }while(k);

    do{                                  /* there are still duplicates */
      k=0;                                     /* check for duplicates */
      if(currenttrack->firstedit){
        for(frontedit = currenttrack->firstedit,
          backedit = frontedit->nextedit;
          backedit && !k;
          frontedit = frontedit->nextedit, backedit = backedit->nextedit)
        {
                        /* if two consecutive segments make one real segment */
                        /* or if two consecutive segments are silence */
          if(!(strcmp(frontedit->file, backedit->file)))
            if(frontedit->start2 + frontedit->length == backedit->start2 |
              (strcmp(frontedit->file, SILENCE) == 0))
            {
              frontedit->length += backedit->length;
              frontedit->nextedit = backedit->nextedit;    /* link past offending edit */
              if(backedit->nextedit) backedit->nextedit->prevedit = frontedit;
                                   /* change lastedit if offending edit is lastedit */
              if(backedit == currenttrack->lastedit) currenttrack->lastedit = frontedit;
              free(backedit->file);                        /* remove offending edit */
              free(backedit);
              k = 1;
            }
        }
      }
    }while(k);

    do{                           /* if last edit is silence or has length 0 */
      k=0;
      if(currenttrack->lastedit){
        backedit = currenttrack->lastedit;
        if(!strcmp(backedit->file, SILENCE) | !backedit->length){
          if(backedit->prevedit){
            frontedit = backedit->prevedit;
            frontedit->nextedit = 0;
            currenttrack->lastedit = frontedit;
            k=1;
          }else{
            free(backedit);
            currenttrack->lastedit = currenttrack->firstedit = 0;
          }
        }
      }
    }while(k);
  }
}

int optimizelabels(){
  struct Label *frontlabel, *backlabel;
  int k;
  do{
    k=0;
    if(firstlabel && firstlabel->nextlabel){
      for(frontlabel = firstlabel, backlabel = frontlabel->nextlabel;
        backlabel && backlabel != frontlabel && !k;
        frontlabel = frontlabel->nextlabel, backlabel = backlabel->nextlabel){
        
                      /* if two consecutive labels are the same */
        if(frontlabel->position == backlabel -> position){
          deletelabel(frontlabel->position);
          k=1;
        }
                      /* if two consecutive labels are out of order */
        if(!k && frontlabel->position > backlabel -> position){
                          /* swap label positions */
					frontlabel->position ^= backlabel->position;
					backlabel->position ^= frontlabel->position;
					frontlabel->position ^= backlabel->position;
          k=1;
        }
      }
    }
  }while(k);
}

void dumpedits(){
  struct Track *currenttrack;
  struct Edit *currentedit;
  int track, i;
  float *pan;

  for(track = 0, currenttrack = firsttrack; currenttrack != 0;
    currenttrack = currenttrack->nexttrack, track++)
  {
    printf("track %d %ld: %ld %ld\n", track, currenttrack,
      currenttrack->firstedit, currenttrack->lastedit);
    for(currentedit = currenttrack->firstedit;
      currentedit; currentedit = currentedit->nextedit)
    {
      printf("             edit: %ld %ld %ld %ld %d %s %d %d ",
        currentedit,
        currentedit->start1,
        currentedit->start2,
        currentedit->length,
        currentedit->track,
        currentedit->file,
        currentedit->channels,
        currentedit->format);
      /*pan = currenttrack->pan;
      for(i = 0; i < outchannels; i++){
        printf("%.2f ", *pan++);
      }*/
      printf("\n");
    }
  }
}

int dumplabels(){
  static struct Label *currentlabel;

  for(currentlabel = firstlabel;
    currentlabel != 0;
    currentlabel = currentlabel->nextlabel)
    {
      printf("%ld\n", currentlabel->position);
    }
    return 0;
}


void deltrack(struct Track *track){
  struct Edit *currentedit, *tempedit;

  for(currentedit = track->firstedit; currentedit;
    currentedit = tempedit)
    {
      tempedit = currentedit->nextedit;
      free(currentedit->file);
      free(currentedit);
    }
  free(track->pan);
  free(track);
}

int addtrack(){
  int track;
  struct Track *fronttrack, *backtrack, *currenttrack;
  struct TrackGroup *currentgroup;
  int newblock, i, totaltracks;
  char *newtracks;

  changesmade = 1;
  updateundo(ADD);

  totaltracks = track = tracks();
  if(!lasttrack){                                 /* add track */
    currenttrack = lasttrack = firsttrack = malloc(sizeof(struct Track));
    currenttrack-> prevtrack = currenttrack->nexttrack = 0;
  }else{
    currenttrack = malloc(sizeof(struct Track));
    currenttrack->prevtrack = lasttrack;
    lasttrack->nexttrack = currenttrack;
    lasttrack = currenttrack;
    lasttrack->nexttrack = 0;
  }

  totaltracks++;                        /* set track parameters */
  currenttrack->firstedit = currenttrack->lastedit = 0; 
               /* first and last edit are 0 */
  currenttrack->record = 1;
  currenttrack->play = 1;
  currenttrack->level = 0;
  setpans(&currenttrack->pan, track);
  currenttrack->invert = 0;

  for(i = 0; i < TOTALEQS; i++){                   /* set eq */
    currenttrack->eq[i].level = 0;
    currenttrack->eq[i].freq = 0;
    currenttrack->eq[i].width = consolewindow.eq[i].widthpot.minvalue;
    currenttrack->eq[i].in = 0;
  }

  for(currentgroup = firsttrackgroup; currentgroup; /* update group tracks field */
    currentgroup = currentgroup->nextgroup)
  {
    newtracks = malloc(totaltracks);
    for(i = 0; i < totaltracks - 1; i++)
      newtracks[i] = currentgroup->tracks[i];
    newtracks[i] = 0;
    free(currentgroup->tracks);
    currentgroup->tracks = newtracks;
  }

  initautomation(track, 0);
  drawpatches(starty);
  drawconsolecontrols();

  newblock = (track - starty) * yscaletrack;
  drawwave(0, newblock, wavewindow.w, wavewindow.h, startx, track, NO_BLANK, FLASH, DRAWCURSOR);
  updateyscrollbar(wavscroll.ypixmap, wavscroll.ywin, starty,
    wavewindow.h / yscaletrack, tracks(), &wavscroll.y);
}

int deletetrack(){
  int track;
  struct Track *fronttrack, *backtrack, *currenttrack;
  struct TrackGroup *currentgroup;
  int newblock, i, totaltracks;
  char *newtracks;
  
  totaltracks = track = tracks() - 1;

  changesmade = 1;
  updateundo(REMOVE);

  if(lasttrack != 0){
    currenttrack = lasttrack->prevtrack;
    deltrack(lasttrack);
    if(currenttrack != 0){
      currenttrack->nexttrack = 0;
    }
    if(firsttrack == lasttrack) firsttrack = currenttrack;
    lasttrack = currenttrack;
  }
  totaltracks--;

  for(currentgroup = firsttrackgroup; currentgroup;
    currentgroup = currentgroup->nextgroup)
  {
    newtracks = malloc(totaltracks + 1);
    for(i = 0; i < totaltracks + 1; i++){
      newtracks[i] = currentgroup->tracks[i];
    }

    free(currentgroup->tracks);
    currentgroup->tracks = newtracks;
  }

  destroyautomation(track);

  drawpatches(starty);
  drawconsolecontrols();
  newblock = (track - starty) * yscaletrack;
  drawwave(0, newblock, wavewindow.w, wavewindow.h, startx, track, NO_BLANK, FLASH,
    DRAWCURSOR);
  updateyscrollbar(wavscroll.ypixmap, wavscroll.ywin, starty,
    wavewindow.h / yscaletrack, tracks(), &wavscroll.y);
}

int changechannels(){
  int newchannels = outchannels, track, group;
  float *newpans;
  int value, i;
  struct Track *currenttrack;
  struct TrackGroup *currentgroup;
  int maxvalue = 0;
  char *caption[] = {"Enter the number of output channels to use"};
  bctextbox textbox[1];
  
  sprintf(textbox[0].text, "%d", newchannels);

  if(getnumericinput(1, textbox, 0, caption, "Channels", 0))
    return 1;
  
  newchannels = atol(textbox[0].text);
  if(newchannels < 1) return 1;
  if(newchannels < outchannels){
    if(confirm("Delete excess channels?", "Channels")) return 1;
  }

  changesmade = 1;

  for(track = 0, currenttrack = firsttrack; currenttrack != 0;
    currenttrack = currenttrack->nexttrack, track++)
  {
    newpans = malloc(sizeof(float) * newchannels);

    if(newchannels < outchannels) outchannels = newchannels;
    for(i = 0; i < outchannels && i < newchannels; i++){
      newpans[i] = currenttrack->pan[i];
    }
    free(currenttrack->pan);
    
    currenttrack->pan = newpans;
    if(i % 2) value = INFINITYGAIN; else value = 0;
    if(track % 2) value ^= INFINITYGAIN;
    for(; i < newchannels; i++){
      currenttrack->pan[i] = (float)value;
      value ^= INFINITYGAIN;
    }
  }

  for(group = 0, currentgroup = firsttrackgroup; currentgroup != 0;
    currentgroup = currentgroup->nextgroup, group++)
  {
    newpans = malloc(sizeof(float) * newchannels);

    for(i = 0; i < outchannels && i < newchannels; i++){
      newpans[i] = currentgroup->pan[i];
    }
    free(currentgroup->pan);
    
    currentgroup->pan = newpans;
    if(i % 2) value = INFINITYGAIN; else value = 0;
    if(track % 2) value ^= INFINITYGAIN;
    for(; i < newchannels; i++){
      currenttrack->pan[i] = (float)value;
      value ^= INFINITYGAIN;
    }
  }

  changeautomationchannels(newchannels);  /* this changes
  																								outchannels itself */

  drawconsolecontrols();
  return 0;
}

int setplaybuffer(){
  int buffersize = PLAYBUFFER / 1024;
  char *caption[] = {"Enter playback buffer size * 1024."};
  bctextbox textbox[1];
  bcslider slider[1] = {0, 0, 0, 0, SLIDER_INT, 0, MAXPLAYBUFFER / 1024, buffersize, 0, ""};
  
  sprintf(textbox[0].text, "%d", buffersize);

  if(getnumericinput(1, textbox, slider, caption, "Play Buffer", 0))
    return 1;
	buffersize = slider[0].value;
  PLAYBUFFER = buffersize * 1024;
}

int setsamplerate(){
  char *caption[] = {"Enter the sample rate."};
  bctextbox textbox[1];
  
  sprintf(textbox[0].text, "%d", samplerate);

  if(getnumericinput(1, textbox, 0, caption, "Sample Rate", 0))
    return 1;

	samplerate = atol(textbox[0].text);

  fittime();
  drawtime(startx);
  
  return 0;
}

int dumpclipboard(){
  TWO j, k;
  FILE *clipboard;
  char string[256];
  FOUR clipboardlength, totaltracks, track;
  TWO labelentries, label, clipboardentries;

  if(!(clipboard = fopen(CLIPBOARDFILENAME, "rb"))){
    printf("Can't open clipboard file.\n");
    return 1;
  }

  printf("totaltracks: %ld \n", totaltracks = getfour(clipboard));
  for(track = 0; track < totaltracks; track++){
                       /* clipboard track exists */
    clipboardlength = getfour(clipboard);
    printf("  length %ld ", clipboardlength);
                       /* get total edits in clipboard track */
    clipboardentries = gettwo(clipboard);
    printf("%ld edits %d\n", ftell(clipboard), clipboardentries);

    for(j = 0; j < clipboardentries; j++){
      printf("    start %ld ", getfour(clipboard));       /* start in real file
      */
      printf("length %ld ", getfour(clipboard));       /* length in real file */
      printf("channel %d ", gettwo(clipboard));       /* channel of real file */
      for(k = 0; (string[k] = fgetc(clipboard)) != 0; k++)
        ;
      printf("%s ", string);                      /* name of real file */
      printf("channels %d ", gettwo(clipboard));       /* channels in real file
      */
      printf("format %d\n", fgetc(clipboard));           /* format of real file
      */
    }
  }
  printf("labelentries: %ld\n", labelentries = getfour(clipboard));
  for(label = 0; label < labelentries; label++){
    printf("    %ld\n", getfour(clipboard));
  }
  fclose(clipboard);
}

void togglelabel(long x){
  struct Label *currentlabel;

  currentlabel = labelof(x);
  if(currentlabel){
    if(currentlabel->position == x) deletelabel(x);
    else insertlabel(x);
  }else{
    insertlabel(x);
  }

  optimizelabels();
  drawlabels(startx);
  changesmade = 1;
}

int selectlabel(XEvent *report){
  static long x, y, temp;
  static long endx;
  static struct Label *currentlabel;

  endx = startx + wavewindow.w*xscale;
  x = report->xmotion.x;
  x = x * xscale + startx;
  y = report->xmotion.y;

  for(currentlabel = lastlabel; currentlabel; currentlabel = currentlabel->prevlabel){
    if((x > currentlabel->position - 15 * xscale) &&
      (x < currentlabel->position + 15 * xscale))
    {
      x = currentlabel->position;

      if(report->xmotion.state & ShiftMask){   /* shift down */
                              /* hide original cursor */
        drawcursor(0, 0, wavewindow.w, wavewindow.h, startx, selectionstart, selectionend);

        if(x > (selectionstart + selectionend)/2)
          selectionend = x;
        else
          selectionstart = x;
                              /* draw new cursor */
        drawcursor(0, 0, wavewindow.w, wavewindow.h, startx, selectionstart, selectionend);
        refreshwav();
      }else{                         /* shift up */
                              /* hide original cursor */
        drawcursor(0, 0, wavewindow.w, wavewindow.h, startx, selectionstart, selectionend);
        selectionstart = x;
        selectionend = x;
                              /* draw new cursor */
        drawcursor(0, 0, wavewindow.w, wavewindow.h, startx, selectionstart, selectionend);
        refreshwav();
      }

      drawlabels(startx);
      drawplayselection();
      while(report->type != ButtonRelease){
        XNextEvent(display, report);
        if(report->type == Expose){
          refreshdisplay();
        }
      }               /* end wait for button release */
      return 0;
    }                   /* end test to see if label is selected */
  }
  return 1;
}

/* select an edit */
int selectedits(XEvent *report){
	struct Track *currenttrack;
	struct Edit *currentedit, *prevedit;
	char result = 0, endselected = 0, startselected = 0;
	long x, y, row, column, newcolumn;
	const int editheight = 16, editwidth = 8;
	long screenend;
	int totaltracks;

	screenend = startx + xscale * wavewindow.w;
	x = report->xbutton.x;
	y = report->xbutton.y;

	/*=============== test all visible edits for selection==================== */
	for(currenttrack = trackof(starty), row = yscaletrack/2 - editheight/2; 
		currenttrack && !result && row < wavewindow.h;
		currenttrack = currenttrack->nexttrack, row += yscaletrack)
	{
	            /* cursor in track */
		if(y > row && y < row + editheight){
			for(currentedit = segmentof(startx, currenttrack);
				!result && currentedit && currentedit->start1 <= screenend;
				currentedit = currentedit->nextedit)
			{
			          /* test previous edit endpoint */
			          /* in case this is the first edit in the view */
				if(currentedit->prevedit){
					prevedit = currentedit->prevedit;
					column = prevedit->start1 + prevedit->length;

					column = (column - startx) / xscale;
					if(x > column - editwidth && x < column){
						result = 1;
						endselected = 1;
						currentedit = prevedit;
						column = prevedit->start1 + prevedit->length;
					}
				}
				        /* test current edit startpoint */
				if(!result){
					column = (currentedit->start1 - startx) / xscale;
					if(x > column && x < column + editwidth){
						result = 1;
						startselected = 1;
						column = currentedit->start1;
					}
				}
				        /* test current edit endpoint */
				if(!result){
					column = currentedit->start1 + currentedit->length;
					
					column = (column - startx) / xscale;
					if(x > column - editwidth && x < column){
						result = 1;
						endselected = 1;
						column = currentedit->start1 + currentedit->length;
					}
				}
			}
		}
	}

	row -= yscaletrack;

	/*=========================== edit selected============================== */
	if(result){
	  if(report->xmotion.state & ShiftMask){   /* shift down */
	  	long newstart;
	  	
	  	newstart = column;

                            /* hide original cursor */
      drawcursor(0, 0, wavewindow.w, wavewindow.h, startx, selectionstart, selectionend);

      if(newstart > (selectionstart + selectionend) / 2)
        selectionend = newstart;
      else
        selectionstart = newstart;
      
                            /* draw new cursor */
      drawcursor(0, 0, wavewindow.w, wavewindow.h, startx, selectionstart, selectionend);
      refreshwav();
      drawplayselection();
      return 1;             /* can't edit anything if shift down */
	  }else{                    /* shift up */
	  	long newstart;
    	int selectionstart_, selectionend_;
    	
                                   /* hide old cursor */
    	drawcursor(0, 0, wavewindow.w, wavewindow.h, startx, selectionstart,
    		selectionend);

			newstart = column;
			selectionstart = selectionend = newstart;

                                   /* draw new cursor */
    	drawcursor(0, 0, wavewindow.w, wavewindow.h, startx, selectionstart,
    		selectionend);
		  drawplayselection();

  		newcolumn = column;   /* convert column from sample to pixels */
  		newcolumn -= startx;
  		newcolumn /= xscale;
  		
      refreshwav();
                     /* force movement outside editwidth */
    	while(report->type != ButtonRelease && abs(x - newcolumn) < editwidth){
    		XNextEvent(display, report);

		    if(report->type == Expose) refreshdisplay();

		    if(report->type == MotionNotify){
		      x = report->xmotion.x;
		      y = report->xmotion.y;
		    }
    	} /* end force movement */
    	
    	  /* if didn't move out of boundary, change cursor position and exit */
    	if(report->type == ButtonRelease){
    		drawplayselection();
				return 1;
     	}

                     /* draw new edit point */
    	if(startselected) DrawRightEditCursor(wavpixmap, newcolumn, row);
    	if(endselected) DrawLeftEditCursor(wavpixmap, newcolumn, row);
      refreshwav();
             /* temporary storage so playselection can be updated */
      selectionstart_ = selectionstart;
      selectionend_ = selectionend;

      /*========================== move edit point around =======================*/
    	while(report->type != ButtonRelease){
    		XNextEvent(display, report);

		    if(report->type == Expose) refreshdisplay();

		    if(report->type == MotionNotify){
		                     /* hide old edit point */
		    	if(startselected) DrawRightEditCursor(wavpixmap, newcolumn, row);
		    	if(endselected) DrawLeftEditCursor(wavpixmap, newcolumn, row);

		      x = report->xmotion.x;
		      y = report->xmotion.y;

		      if(x > wavewindow.w){
		        moveforward((x - wavewindow.w) * xscale);
		      }
		      if(x < 0){
		        moveback((-x) * xscale);
		      }
		      if(y < 0){
		        moveup(wavewindow.h / yscaletrack / 2 + 1);
		      }
		      if(y > wavewindow.h){
		        movedown(wavewindow.h / yscaletrack / 2 + 1);
		      }

		      newcolumn = x;
		      if(newcolumn > wavewindow.w) newcolumn = wavewindow.w;

		                     /* draw new edit point */
		    	if(startselected) DrawRightEditCursor(wavpixmap, newcolumn, row);
		    	if(endselected) DrawLeftEditCursor(wavpixmap, newcolumn, row);
		    	refreshwav();
		    	selectionstart = selectionend = newcolumn * xscale + startx;
		    	drawplayselection();
		    }
    		flushevents();
    	} /* end move edit point around */
    	        /* playselection won't be updated anymore */
    	selectionstart = selectionstart_;
    	selectionend = selectionend_;
    	
  		                     /* hide old edit point */
    	if(startselected) DrawRightEditCursor(wavpixmap, newcolumn, row);
    	if(endselected) DrawLeftEditCursor(wavpixmap, newcolumn, row);

			if(newcolumn != (column - startx) / xscale){ /* new column is different than old column */
				long mincolumn, maxcolumn;/* minimum and maximum column based on file boundaries */
				long length;

				updateundo(TWEEKEDITS);
				changesmade = 1;

				newcolumn *= xscale;    /* convert newcolumn to samples */
				newcolumn += startx;

				if(newcolumn < column){  /* set minimum and maximum boundaries */
					mincolumn = newcolumn;
					maxcolumn = column;
				}
				
				if(newcolumn > column){
					mincolumn = column;
					maxcolumn = newcolumn;
				}

     /*=================== test all edits for column boundaries =====================*/
				for(currenttrack = firsttrack; 
					currenttrack;
					currenttrack = currenttrack->nexttrack)
				{
					if(currenttrack->record){
						result = 0; /* only need one edit for a track */
						
						for(currentedit = currenttrack->firstedit;
							/* need to test boundaries so don't use segmentof */
							currentedit && !result;)
						{
						          /* test start of edit */
							if(startselected){
								if(currentedit->start1 == column) result = 1;
							}
							        /* test end of edit */
							if(endselected){
								if(currentedit->start1 + currentedit->length == column)
									result = 1;
							} /* end test */
							if(!result) currentedit = currentedit->nextedit;
						} /* end test edits */
						
						              /* track has edit on column so test position */
						if(result){
							FILE *in;
							long infilelength;
							int frame;

								          /* end selected specific condition */
							if(endselected){
					          /* mincolumn can't be before segment start */
					          /* otherwise resulting edit would end before it began */
								if(mincolumn < currentedit->start1)
								  mincolumn = currentedit->start1;
							}

								          /* start selected specific condition */
							if(startselected){
					          /* maxcolumn can't be after segment end */
					          /* otherwise resulting edit would end before it began */
								if(maxcolumn > currentedit->start1 + currentedit->length)
								  maxcolumn = currentedit->start1 + currentedit->length;
							}

								
						                /* segment isn't silence so has boundaries */
							if(strcmp(currentedit->file, SILENCE)){
								static int newoutchannels;

								        /* mincolumn can't be before beginning of file */
								if(column - mincolumn > 
									column - currentedit->start1 + currentedit->start2)
								{
								            /* put mincolumn at start of file if it is */
									mincolumn = 
									column - (column - currentedit->start1 + currentedit->start2);
								}

								            /* test for end of file */
							  if(!(in = fopen(currentedit->file, "rb"))){
							    printf("Can't open %s for getting boundaries.\n", currentedit->file);
							    return 0;
							  }

								switch(currentedit->format){
							    case WAV:
							      fseek(in, 0, SEEK_END);
							      infilelength = ftell(in) - WAVHEADERLENGTH;
							      frame = currentedit->channels * 2;
							      infilelength /= frame;
							      break;

							    case PCMHILO:
							      fseek(in, 0, SEEK_END);
							      infilelength = ftell(in);
							      frame = currentedit->channels * 2;
							      infilelength /= frame;
							      break;

							    case PCMLOHI:         /* same as PCMHILO */
							      fseek(in, 0, SEEK_END);
							      infilelength = ftell(in);
							      frame = currentedit->channels * 2;
							      infilelength /= frame;
							      break;

							    case BCAST:                               /* load header from bcast */
							      frame = 2;
							      fseek(in, 6, SEEK_SET);
							      newoutchannels = gettwo(in);     /* need outchannels used in file */

							      fseek(in,
							      	CASTHEADLEN               /* headerlength */
							      	+ currentedit->track
							      	* (15 + 4 * newoutchannels)/* track number in real file */
							      	+ 4                    /* start of track */
							      	, SEEK_SET);

							      infilelength = getfour(in); /* get track length */

							      infilelength /= 2; /* frame always 2 */
							      break;
							  }
							  fclose(in);

							  if(maxcolumn - currentedit->start1 + currentedit->start2 
							  	> infilelength)
							  	maxcolumn = 
							  	infilelength - currentedit->start2 + currentedit->start1;

							  if(mincolumn > maxcolumn) return 0; /* out of range */
							} /* end current edit isn't silence */
						} /* end test position of edit in track */
					} /* end track is recordable */
				} /* end test tracks */

				if(newcolumn > maxcolumn) newcolumn = maxcolumn;
				if(newcolumn < mincolumn) newcolumn = mincolumn;
				length = newcolumn - column;

     /*================ change all edits with right column boundaries ============*/
				for(currenttrack = firsttrack; 
					currenttrack;
					currenttrack = currenttrack->nexttrack)
				{
					if(currenttrack->record){
						result = 0; /* only change one edit for a track */
						
						for(currentedit = currenttrack->firstedit;
							/* need to test boundaries so don't use segmentof */
							currentedit && !result;
							)
						{
						          /* test start of edit */
							if(startselected){
								if(currentedit->start1 == column) result = 1;
							}
							        /* test end of edit */
							if(endselected){
								if(currentedit->start1 + currentedit->length == column)
									result = 1;
							} /* end test */
							if(!result) currentedit = currentedit->nextedit;
						} /* end test edits */

						              /* track has edit on column so change it */
						if(result){
							if(startselected){
								currentedit->start2 += length;   /* shift real start */
								currentedit->length -= length;   /* take out length */
								
								for(currentedit = currentedit->nextedit;
									currentedit; 
									currentedit = currentedit->nextedit)
								{
									currentedit->start1 -= length; /* shift later edits */
								}
	
							               /* shift automation */
							  shiftautomation(currenttrack, mincolumn, -length);
							}
							
							if(endselected){
								currentedit->length += length;  /* shift length */
								
								for(currentedit = currentedit->nextedit;
									currentedit; 
									currentedit = currentedit->nextedit)
								{
									currentedit->start1 += length; /* shift later edits */
								}
	
							               /* shift automation */
							  shiftautomation(currenttrack, mincolumn, length);
							}
						} /* end alter track */
					} /* end track has record */
				} /* end check each track */

                 /* shift labels only once */
				if(shiftlabels){
					struct Label *currentlabel;
					
					for(currentlabel = labelof(column);
						currentlabel;
						currentlabel = currentlabel->nextlabel)
					{
						if(startselected)
							currentlabel->position -= length;
						if(endselected)
							currentlabel->position += length;
						
						if(currentlabel->position < 0) deletelabel(currentlabel->position);
					}
				}
				optimizelabels();
				optimizeedits();
				      /* set selection to reflect change if end is selected */
				if(endselected) selectionstart = selectionend = newcolumn;

		  	drawwave(0, 0, wavewindow.w, wavewindow.h, startx, starty, NO_BLANK,
		  		FLASH, DRAWCURSOR);
		  	drawlabels(startx);
		  	updatexscrollbar(wavscroll.xpixmap, wavscroll.xwin, startx,
		    	wavewindow.w * xscale, longesttrack(), &wavscroll.x);
			} /* end new column is different than old column */

			refreshwav();
			drawplayselection();
			return 1;
	  } /* end shift is up */
	} /* end edit is selected */

	return 0;
}

/* clear an edit */
int clearedit(long start, long end, char operation){
	struct Track *currenttrack;
	struct Edit *currentedit, *nextedit;
	char result = 0, undoupdated = 0;
	long length;
	struct Label *currentlabel;
	int i;
	
	            /* more than one point selected */
	if(start != end) return 0;
	
	            /* no edits visible */
	if(!showedits) return 0;
	
	            /* test all edits for selection */
	for(currenttrack = firsttrack; 
		currenttrack;
		currenttrack = currenttrack->nexttrack)
	{
		if(currenttrack->record){
			for(currentedit = currenttrack->firstedit;
			                  /* can't delete end of last edit */
				currentedit != currenttrack->lastedit;
				currentedit = currentedit->nextedit)
			{
				   /* can't delete start of first edit */
				   /* end of an edit selected */
				if(currentedit->start1 + currentedit->length == start){
					nextedit = currentedit->nextedit;
                           /* next edit must be in same file as first edit */
					if(!strcmp(nextedit->file, currentedit->file)){
						result = 1;
  					changesmade = 1;
						if(!undoupdated){          /* update undo only once */
							undoupdated = 1;
							updateundo(CLEAR);
						}
						length = -currentedit->length;
						             /* extend first edit to match second edit */
						currentedit->length = nextedit->start2 - currentedit->start2;
						length += currentedit->length;    /* get the extra length */
						
						             /* shift everything back */
						for(;nextedit; nextedit = nextedit->nextedit){
							nextedit->start1 += length;
						}
	
					               /* shift automation */
					  shiftautomation(currenttrack, start, length);
					} /* end edit is in same file */
				} /* end end of an edit selected */
			} /* end check each edit */
		} /* end recordable track */
	} /* end check each track */
	
	if(result){
                 /* shift labels only once */
                 /* take the length of the last edit shifted */
		if(shiftlabels){
			for(currentlabel = labelof(start);
				currentlabel;
				currentlabel = currentlabel->nextlabel)
			{
				currentlabel->position += length;
				if(currentlabel->position < 0) deletelabel(currentlabel->position);
			}
		}
		optimizelabels();
		optimizeedits();
		
		drawwave(0, 0, wavewindow.w, wavewindow.h, startx, starty,
			NO_BLANK, FLASH, DRAWCURSOR);
		drawlabels(startx);
		updatexscrollbar(wavscroll.xpixmap, wavscroll.xwin, startx,
			wavewindow.w * xscale, longesttrack(), &wavscroll.x);
		drawplayselection();
	}
	return result;
}
