#ifndef DEFS_H
#define DEFS_H

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <fcntl.h>
#include <unistd.h>

#include <X11/Xlib.h>
#include <X11/Xutil.h>
#include <X11/keysym.h>
#include <X11/cursorfont.h>

#define TWO short int
#define FOUR int
#define FLOAT float

#define MAXLABELS 128
#define MAXRECLABELS 32

                    /* names of the general files */
                    /* user's directory is added on to each */
#define _INITIALIZERFILE_ ".bcastrc"
#define _CLIPBOARDFILENAME_ ".bcclip"
#define _AUTOCLIPBOARDFILENAME_ ".bcautoclip"
#define _UNDOFILENAME_ ".bcundo"

#define SCALEFILEEXT ".idx"
#define WAVHEADERLENGTH 44
#define CASTHEADLEN  44
#define SAVEBUFFER 1000000
#define INDEXFILESIZE 1000000
#define TOTALEQS 3  /* total eqs in console window */

/*===============================operation descriptions================*/
#define CUT                    1
#define COPY                   2
#define CLEAR                  3
#define PASTE                  4
#define RECORD                 5
#define ADD                    6
#define REMOVE                 7
#define IMPORT                 8
#define MUTE                   9
#define EFFECT                 15
#define PLAYASFILE             10
#define SAVEASFILE             14
#define LOAD                   11
#define INPUT                  25
#define UNDO                   12
#define PLAY                   13
#define EDITGROUP              16
#define SELECTPANS             17
#define REVERB                 15
#define CUTAUTOS               18
#define COPYAUTOS              19
#define PASTEAUTOS             20
#define CLEARAUTOS             21
#define TWEEKAUTOS             22
#define TWEEKEDITS             23
#define TWEEKCONSOLE           24

/*============================file types==========================*/
#define PCMHILO         0
#define WAV             1
#define EDITS           2
#define BCAST           3
#define PCMLOHI         4

/*=======================audio input types========================*/
#define MIC             0
#define LINE            1

#define UPDATE_UNDO     1
#define NO_UPDATE_UNDO  0
#define DISPLAY         1
#define NO_DISPLAY      0

#define NO_UNDO         0
#define NO_RECORD       0
#define NO_PLAY         0

#define BLANK           1
#define NO_BLANK        0
#define FLASH           1
#define NO_FLASH        0
#define DRAWCURSOR      1
#define NO_DRAWCURSOR   0

              /* actual strings to contain general files */
extern char INITIALIZERFILE[256];
extern char CLIPBOARDFILENAME[256];
extern char UNDOCLIPFILENAME[256];
extern char AUTOCLIPBOARDFILENAME[256];
extern char UNDOAUTOCLIPFILENAME[256];
extern char UNDOFILENAME[256];
extern char UNDOTEMPFILE[256];

extern FOUR startx;            /* current position of view in virtual file */
extern FOUR starty;            /* current track at start of view */
extern FOUR consolestarty;          /* current track at start of console */
extern FOUR groupstarty;       /* current group at start of groups */
extern FOUR timescale;         /* horizontal scale of times */
extern FOUR xscale;            /* horizontal scale of view */
extern FOUR yscale;            /* vertical scale of wave */
extern FOUR yscaletrack;       /* vertical scale of track */
extern FOUR selectionstart;       /* current selection start */
extern FOUR selectionend;         /* current selection end */
extern FOUR currentposition;         /* current position for playback */
extern TWO screenwidth, screenheight;
extern char shiftlabels;
extern char titles;              /* draw titles if 1 */
extern char showtrackgroups, showedits;      /* show groups if 1, console if 0 */
extern char byteorder;           /* 1 if intel  0 if normal */
extern FOUR PLAYBUFFER, RECBUFFER;
extern char highqualityrefresh;

extern int screen;
extern Display *display;
extern Window patchwin, wavwin, rootwin, win;
extern Window labelwin, timewin, menuwin, popupwin, popupsubwin;
extern Window playwin, consolewin;
extern Pixmap patchpixmap, wavpixmap, playpixmap, consolepixmap;
extern Pixmap labelpixmap, timepixmap, menupixmap, popuppixmap, popupsubpixmap;
extern GC gc;
extern int depth;

extern XFontStruct *menufont, *smallfont;

extern Colormap cmap;
extern XColor orange, green, red, ltgrey, grey, migrey;
extern XColor dkgrey, ltcyan, cyan, micyan, dkcyan, pink;
extern XColor black, white, yellow, purple, blue;

/*=========================== main window sizes ======================*/
struct WaveWindow {
  int x, y, w, h;
} wavewindow;

struct PatchWindow {
  int x, y, w, h;
} patchwindow;

struct LabelWindow {
  int x, y, w, h;
} labelwindow;

struct MenuWindow {
  int x, y, w, h;
} menuwindow;

struct TimeWindow {
  int x, y, w, h;
} timewindow;


#define SILENCE "silence"
struct Edit {
  struct Edit *prevedit;     /* pointer to previous edit */
  FOUR start1;        /* start of segment in the virtual file */
  FOUR start2;        /* start of segment in the real file */
  FOUR length;        /* length of segment */
  TWO track;          /* track of segment in real file*/
  char *file;         /* real file in which the segment is in */
  TWO channels;       /* # of channels in real file or tracks in a BCAST file */
  char format;        /* data format of file */
  struct Edit *nextedit;     /* pointer to next edit */
};                 /* single edit in edit list of a track */

struct Auto {
  struct Auto *prevauto;
  FOUR position;
  float value;
  struct Auto *nextauto;
};

struct Eq {
  float level;
  int freq;
  int width;
  char in;  /* flag for eq and bandpass 1 if eq 2 if bp */
};

struct Track {
  struct Track *prevtrack;     /* pointer to previous track */

  /*=======================edit data=============================*/
  struct Edit *firstedit;     /* pointer to first Edit in track */
  struct Edit *lastedit;      /* pointer to last Edit in track */

  /*=======================console data===============================*/
  char record;         /* modify permission for track */
  char play;           /* play permission for track */
  float level;         /* main level for track */
  float *pan;          /* array of pans for track */
  char invert;         /* invert track output */

  struct Eq eq[TOTALEQS];     /* equalization */

  /*=======================automation data=======================*/
  struct Auto *firstautomute;
  struct Auto *lastautomute;
  struct Auto *firstautofade;
  struct Auto *lastautofade;
  struct Auto **firstautopan;
  struct Auto **lastautopan;
  struct Auto *firstautoinvert;
  struct Auto *lastautoinvert;

  struct Track *nexttrack;     /* pointer to next track */
};

struct Label {
  struct Label *prevlabel;
  struct Label *nextlabel;
  FOUR position;
};

struct TrackGroup {
  struct TrackGroup *prevgroup;     /* pointer to previous track */
  char *tracks;         /* list of tracks with members tagged */
  float level;         /* main level for track */
  float *pan;          /* array of pans for track */
  char invert;         /* invert track output */

  struct Eq eq[TOTALEQS];     /* equalization */

  struct TrackGroup *nextgroup;     /* pointer to next track */
};

struct CastHeader {
  FOUR start;
  FOUR length;
};

/*=====================================database==========================*/
struct Track *firsttrack; /* pointer to first track in database */
struct Track *lasttrack;  /* pointer to last track in database */

struct Label *firstlabel;
struct Label *lastlabel;

struct TrackGroup *firsttrackgroup;
struct TrackGroup *lasttrackgroup;

extern float mainlevel;
extern FOUR samplerate;
extern TWO outchannels;
extern char changesmade;

#endif
