/*
  Broadcast 2.0 multitrack audio editing
  (c) 1997 Adam Williams

  This program is distributed with the intent that it will be useful, without
  any warranty.  The code is being updated and has many bugs which are
  constantly being elucidated so changes should not be made without notifying
  the author.
*/

/* edit.c      editing for audio data  */

#include "defs.h"
#include "interfac.h"
#include "graphics.h"
#include "movement.h"
#include "edit.h"
#include "file.h"
#include "database.h"
#include "audio.h"
#include "automate.h"
#include "undo.h"
#include "record.h"

/* mute a selection */
int mute(long start, long end, char operation){
  struct Track *currenttrack;

	if(start == end) return 1;
  changesmade = 1;
  if(operation == MUTE) updateundo(MUTE);

  recevent.channels = 0;
  for(currenttrack = firsttrack; currenttrack != 0;
    currenttrack = currenttrack->nexttrack)
  {
    if(currenttrack->record) recevent.channels++;
  }

  recevent.length = end - start;
  strcpy(recevent.file, SILENCE);


  paste(start, end, EFFECT);

  /* want to treat labels and automation like an effect */
}

/* paste a region from clipboard or record structure */

int paste(long start, long end, char operation){
  FILE *clipboard;
  long x;
  long _start;                /* start of real segment in clipboard */
  long _length;               /* length of real segment in clipboard */
  int i, j, k;
  TWO labelentries;            /* number of label entries in clipboard */
  FOUR labelstart;            /* start of labels in file */
  char filename[32];            /* name of file real segment is in */
  int track;
  TWO clipboardentries;   /* number of edits in clipboard track */
  FOUR totaltracks;        /* number of new tracks */
  FOUR clipboardlength;   /* length of clipboard track */
  FOUR tracklength;       /* length of destination track */
  struct Track *currenttrack;
  struct Edit *currentedit, *frontedit, *backedit;
  struct Label *currentlabel;

  changesmade = 1;
  
                        /* don't update undo if being used as utility */
  if(operation == PASTE) updateundo(operation);

/*dumpclipboard();*/

  clear(start, end, operation);

  if(operation== PASTE){
    if(!(clipboard = fopen(CLIPBOARDFILENAME, "rb"))){
      printf("Can't open clipboard file.\n");
      return 1;
    }

    totaltracks = getfour(clipboard);
    labelstart = getfour(clipboard);
    if(totaltracks == 0){           /* no tracks in clipboard */
      fclose(clipboard);
      return 1;
    }
  }
  if(operation == RECORD | operation == EFFECT) totaltracks = recevent.channels;

                          /* scan each destination track */
  for(currenttrack = firsttrack, track = 0;
    currenttrack && track < totaltracks;
    currenttrack = currenttrack->nexttrack)
  {
                            /* destination track is recordable */
    if(currenttrack->record){
      if(operation == PASTE){
                                  /* get total length of clipboard track */
        clipboardlength = getfour(clipboard);
                                  /* get total edits in clipboard track */
        clipboardentries = gettwo(clipboard);
      }
      if(operation == RECORD | operation == EFFECT){
        clipboardlength = recevent.length;
        clipboardentries = 1;
      }

              /* copy ensures that every clipboard track is the same length */
       frontedit = segmentof(start, currenttrack);
       if(frontedit == 0){
                                                                /* track is empty */
         if(currenttrack->firstedit == 0 && currenttrack->lastedit == 0){
                                                                 /* create filler */
          frontedit = currenttrack->firstedit =
             currenttrack->lastedit = malloc(sizeof(struct Edit));
           frontedit->prevedit = frontedit->nextedit = 0;

          frontedit->start1 = 0;
          frontedit->length = start;
          assigntext(&frontedit->file, SILENCE);
        }else{                                            /* track isn't empty */
           frontedit = currenttrack->lastedit->nextedit = malloc(sizeof(struct Edit));
                                                          /* create filler */
          frontedit->start1 = currenttrack->lastedit->start1 +
            currenttrack->lastedit->length;
          frontedit->prevedit = currenttrack->lastedit;
          frontedit->nextedit = 0;
          currenttrack->lastedit = frontedit;

          frontedit->length = start - frontedit->start1;
          assigntext(&frontedit->file, SILENCE);
        }
        if(operation == PASTE){
          x = start;
          for(i = 0; i < clipboardentries; i++){
            currentedit = frontedit->nextedit = currenttrack->lastedit =
              malloc(sizeof(struct Edit));            /* create new edit */
            currentedit->prevedit = frontedit;        /* link to previous edit */
            currentedit->nextedit = 0;

            currentedit->start1 = x;
            currentedit->start2 = getfour(clipboard);
            currentedit->length = getfour(clipboard);
            currentedit->track  = gettwo(clipboard);
            readtext(clipboard, &currentedit->file);
            currentedit->channels = gettwo(clipboard);
            currentedit->format = fgetc(clipboard);

            frontedit = currentedit;
            x += currentedit->length;
          }
        }
        if(operation == RECORD | operation == EFFECT){
          currentedit = frontedit->nextedit = currenttrack->lastedit =
            malloc(sizeof(struct Edit));
          currentedit->prevedit = frontedit;
          currentedit->nextedit = 0;

          currentedit->start1 = start;
          currentedit->start2 = 0;
          currentedit->length = recevent.length;
          currentedit->track = track;
          assigntext(&currentedit->file, recevent.file);
          currentedit->channels = recevent.channels;
          currentedit->format = recevent.format;
        }
      }else{                                        /* start is inside track */
        backedit = malloc(sizeof(struct Edit));/* copy old edit into new edit */
        backedit->length = frontedit->length;
        frontedit->length = start - frontedit->start1;
        backedit->start1 = frontedit->start1 + frontedit->length;
        backedit->start2 = frontedit->start2 + frontedit->length;
        backedit->length -= frontedit->length;
        backedit->track = frontedit->track;
        assigntext(&backedit->file, frontedit->file);
        backedit->channels = frontedit->channels;
        backedit->format = frontedit->format;
        backedit->nextedit = frontedit->nextedit;

        for(currentedit = backedit; currentedit; currentedit =
          currentedit->nextedit)                   /* shift all segments back */
        {
          currentedit->start1 += clipboardlength;
        }
        if(currenttrack->lastedit == frontedit) currenttrack->lastedit = backedit;

                                                      /* read clipboard track */
        if(operation == PASTE){
          x = start;
          for(i = 0; i < clipboardentries; i++){
            currentedit = frontedit->nextedit = malloc(sizeof(struct Edit));
                                                        /* create new edit */
            currentedit->prevedit = frontedit;       /* link to previous edit */

            currentedit->start1 = x;
            currentedit->start2 = getfour(clipboard);
            currentedit->length = getfour(clipboard);
            currentedit->track  = gettwo(clipboard);
            readtext(clipboard, &currentedit->file);
            currentedit->channels = gettwo(clipboard);
            currentedit->format = fgetc(clipboard);

            frontedit = currentedit;
            x += currentedit->length;
          }
          frontedit->nextedit = backedit;
          backedit->prevedit = frontedit;
        }
         if(operation == RECORD | operation == EFFECT){
           currentedit = frontedit->nextedit = backedit->prevedit =
             malloc(sizeof(struct Edit));                  /* create new edit */
          currentedit->prevedit = frontedit;              /* and link it in */
          currentedit->nextedit = backedit;

          currentedit->start1 = start;
          currentedit->start2 = 0;
          currentedit->length = recevent.length;
          currentedit->track = track;
          assigntext(&currentedit->file, recevent.file);
          currentedit->channels = recevent.channels;
          currentedit->format = recevent.format;
        }
      }                       /* end start is inside track */
      track++;                /* increase track counter */
    }                         /* end if recordable track */
  }                           /* end check each track */

  optimizeedits();
                            /* shift labels to right if shiftlabels is 1
                               and operation is not an effect */
  if(shiftlabels && operation != EFFECT){
    for(currentlabel = labelof(start);
      currentlabel;
      currentlabel = currentlabel->nextlabel)
    {
      currentlabel->position += clipboardlength;
    }
  }
                            /* insert labels */
  if(operation == PASTE){     /* paste operation */
    fseek(clipboard, labelstart, SEEK_SET);
    labelentries = getfour(clipboard);
    for(j = 0; j < labelentries; j++){
      insertlabel(getfour(clipboard) + start);
    }
  }else{                      /* record or effect operation */
    for(j = 0; j < reclabels; j++){
      insertlabel(reclabel[j] + start);
    }
  }

  optimizelabels();

  pasteautomation(clipboard, start, clipboardlength, operation);

  if(operation == PASTE) fclose(clipboard);
  selectionstart = start;
  selectionend = start + clipboardlength;

/* uncomment to fit the selection automatically */
/*
  if(selectionend - selectionstart > wavewindow.w * xscale)
    fit(selectionstart, selectionend);
  else{
*/
    drawwave(0, 0, wavewindow.w, wavewindow.h, startx, starty, NO_BLANK, FLASH,
      DRAWCURSOR);
    updatexscrollbar(wavscroll.xpixmap, wavscroll.xwin, startx,
      wavewindow.w * xscale, longesttrack(), &wavscroll.x);
/*
  }
*/

  drawlabels(startx);
  drawplayselection();

  return 0;
}

/* insert silence of the length of the selected region */
int pastesilence(long start, long end){
  int totaltracks;

  totaltracks = recordabletracks();
  if(!totaltracks) return 1;

  updateundo(RECORD);
  changesmade = 1;

  recevent.length = end - start;
  strcpy(recevent.file, SILENCE);
  recevent.channels = recordabletracks();
  recevent.format = 0;
  reclabels = 0;
  paste(selectionstart, selectionstart, RECORD);
}

/* copy a region to the clipboard */
int copy(long start, long end, char operation){
  FILE *clipboard;
  TWO labelentries;
  TWO clipboardentries;
  FOUR totaltracks, fileposition, fileposition_, clipboardlength;
  FOUR tracklength;
  int track;
  TWO segments;                /* # of segments in clipboard track */
  struct Track *currenttrack;
  struct Edit *frontedit, *backedit;
  struct Label *currentlabel;

  if(start == end) return 1;

  if(!(clipboard = fopen(CLIPBOARDFILENAME, "wb"))){
    printf("Can't open clipboard file.\n");
    return 1;
  }

  putfour(0, clipboard);       /* skip total tracks */
  putfour(0, clipboard);       /* skip label start */

                           /* scan each track for record permission */
  totaltracks = 0;
  for(currenttrack = firsttrack; currenttrack != 0;
    currenttrack = currenttrack->nexttrack)
  {
                                        /* track has record permission */
    if(currenttrack->record){
                           /* find segment containing start and end of selection */
      frontedit = segmentof(start, currenttrack);
      backedit = segmentof(end, currenttrack);
      putfour(end - start, clipboard);           /* store length */
      totaltracks++;

      if(frontedit != 0){                           /* start is inside track */
        if(frontedit == backedit){                 /* start and end in same segment */
          puttwo(1, clipboard); /* clipboard entries */
                                                              /* start of real file */
          putfour(frontedit->start2 + start - frontedit->start1, clipboard);
          putfour(end - start, clipboard);                 /* length of selection */
          puttwo(frontedit->track, clipboard); /* track in real file */
          fprintf(clipboard, "%s%c", frontedit->file, 0);          /* real filename */
          puttwo(frontedit->channels, clipboard);            /* channels in real file */
          fputc(frontedit->format, clipboard);               /* format of real file */
        }else{                           /* start and end are in different segments */
          clipboardentries = 0;
          fileposition = ftell(clipboard);

          fputc(0, clipboard);                                 /* skip clipboardentries */
          fputc(0, clipboard);
          putfour(frontedit->start2 + start - frontedit->start1, clipboard);
          putfour(frontedit->length - (start - frontedit->start1), clipboard);
          puttwo(frontedit->track, clipboard);         /* write first segment */
          fprintf(clipboard, "%s%c", frontedit->file, 0);
          puttwo(frontedit->channels, clipboard);
          fputc(frontedit->format, clipboard);
          clipboardentries++;

          for(frontedit = frontedit->nextedit; frontedit != backedit && frontedit != 0;
            frontedit = frontedit->nextedit)
          {
            putfour(frontedit->start2, clipboard); /* write additional segments */
            putfour(frontedit->length, clipboard);
            puttwo(frontedit->track, clipboard);
            fprintf(clipboard, "%s%c", frontedit->file, 0);
            puttwo(frontedit->channels, clipboard);
            fputc(frontedit->format, clipboard);
            clipboardentries++;
          }

          if(backedit != 0){                        /* if backedit is inside track */
            putfour(backedit->start2, clipboard);          /* write last segment */
            putfour(end - backedit->start1, clipboard);
            puttwo(backedit->track, clipboard);
            fprintf(clipboard, "%s%c", backedit->file, 0);
            puttwo(backedit->channels, clipboard);
            fputc(backedit->format, clipboard);
            clipboardentries++;
          }else{                                   /* if backedit is outside track */
            frontedit = currenttrack->lastedit;

            putfour(0, clipboard);
            putfour(end - frontedit->start1 - frontedit->length, clipboard);
            puttwo(0, clipboard);
            fprintf(clipboard, "%s%c", SILENCE, 0);
            puttwo(0, clipboard);
            fputc(0, clipboard);
            clipboardentries++;
          }
          fileposition_ = ftell(clipboard);       /* store # of edits */
          fseek(clipboard, fileposition, SEEK_SET);
          puttwo(clipboardentries, clipboard);
          fseek(clipboard, fileposition_, SEEK_SET);
        }                       /* end start and end in different segments */
      }else{                              /* start is outside track */
        puttwo(1, clipboard);             /* store # of edits */
        putfour(0, clipboard);
        putfour(end - start, clipboard);
        puttwo(0, clipboard);
        fprintf(clipboard, "%s%c", SILENCE, 0);
        puttwo(0, clipboard);
        fputc(0, clipboard);
      }
    }                      /* end current track has record permission */
  }                        /* end search each track for record permission */

  fileposition = ftell(clipboard);
  fseek(clipboard, 0, SEEK_SET);
  putfour(totaltracks, clipboard);     /* store totaltracks */
  putfour(fileposition, clipboard);     /* store label start */
  fseek(clipboard, fileposition, SEEK_SET);
                             /* store labels */
  fputc(0, clipboard);       /* skip labelentries */
  fputc(0, clipboard);
  fputc(0, clipboard);
  fputc(0, clipboard);

  labelentries = 0;
  for(currentlabel = labelof(start);
    currentlabel && currentlabel->position <= end;
    currentlabel = currentlabel->nextlabel)
  {
    putfour(currentlabel->position - start, clipboard);
    labelentries++;
  }
                           /*  store labelentries */
  fileposition_ = ftell(clipboard);
  fseek(clipboard, fileposition, SEEK_SET);
  putfour(labelentries, clipboard);
  fseek(clipboard, fileposition_, SEEK_SET);

  copyautomation(clipboard, start, end);      /* store automation */
  fclose(clipboard);

  /*dumpclipboard();*/
  return 0;
}

/* clear region from start to end */

int clear(long start, long end1, char operation){
  int i, j;
  long length;          /* length of segment being deleted */
  long end = end1;
  long tracklength;
  int track;
  struct Track *currenttrack;
  struct Edit *currentedit, *tempedit, *frontedit, *backedit;
  struct Label *currentlabel, *nextlabel;

	if(operation == CLEAR && clearedit(start, end1, operation)) return 0; 

  if(start == end1) return 0;

  changesmade = 1;
  if(operation == CLEAR) updateundo(operation);

                                  /* process each track */
  for(currenttrack = firsttrack; currenttrack != 0;
    currenttrack = currenttrack->nexttrack)
  {
    end = end1;        /* end to be modified if the current track ends before it */
    length = end - start;
    if(currenttrack->record){            /* track has record permission */
      frontedit = segmentof(start, currenttrack);
      backedit = segmentof(end, currenttrack);
      if(frontedit == 0){    /* start is outside of track */
        continue;
      }
      if(frontedit == backedit | frontedit == currenttrack->lastedit){
                                       /* start and end are in same segment */
        if(backedit == 0){                          /* end is outside track */
          end = frontedit->start1 + frontedit->length;
          length = end - start;
        }
        backedit = malloc(sizeof(struct Edit));          /* create new edit */
        backedit->length = frontedit->length;
        frontedit->length = start - frontedit->start1;

        if(frontedit == currenttrack->lastedit)
          currenttrack->lastedit = backedit;  /* change lastedit if frontedit was last edit */

        backedit->nextedit = frontedit->nextedit; /* link back edit to next edit */
        frontedit->nextedit = backedit;           /* link front edit to back edit */
        backedit->prevedit = frontedit;           /* link back edit to front edit */
        backedit->start1 = frontedit->start1 + frontedit->length;
        backedit->start2 = frontedit->start2 + frontedit->length + length;
        backedit->length -= frontedit->length + length;
        backedit->track = frontedit->track;
        assigntext(&backedit->file, frontedit->file);
        backedit->channels = frontedit->channels;
        backedit->format = frontedit->format;

        for(currentedit = backedit->nextedit; currentedit != 0;
          currentedit = currentedit->nextedit)
        {
          currentedit->start1 -= length;           /* shift edits foreward */
        }
      }else{                        /* start and end are in different segments */
        if(backedit == 0){        /* end is outside track */
          backedit = currenttrack->lastedit;
          end = backedit->start1 + backedit->length;
          length = end - start;
        }
        frontedit->length = start - frontedit->start1; /* change front edit length */
        tempedit = frontedit->nextedit;
        frontedit->nextedit = backedit;             /* link front to back edit */
        backedit->prevedit = frontedit;             /* link back to front edit */
        backedit->length -= end - backedit->start1; /* change back edit length */
        backedit->start2 += end - backedit->start1; /* change back edit real start */
        backedit->start1 = frontedit->start1 + frontedit->length; /* change back edit virtual start */
        for(frontedit = tempedit; frontedit != backedit;
          frontedit = currentedit)
        {                                           /* delete edits */
          currentedit = frontedit->nextedit;
          free(frontedit->file);
          free(frontedit);
        }
        for(currentedit = backedit->nextedit; currentedit != 0;
          currentedit = currentedit->nextedit)
        {
          currentedit->start1 -= length;    /* shift segments foreward */
        }
      }                     /* end start and end in different segments */
    }                       /* end track has record permission */
  }                         /* end process each track */

  optimizeedits();
  							/* shift labels left and delete labels if shiftlabels
  													 is 1 and this is not an effect */

  if(shiftlabels && operation != EFFECT){

    for(currentlabel = labelof(start);
      currentlabel && currentlabel->position < end1;
      currentlabel = nextlabel)
    {
    	nextlabel = currentlabel->nextlabel;
      deletelabel(currentlabel->position);
    }

    for(currentlabel = labelof(end); currentlabel; currentlabel = currentlabel->nextlabel){
      currentlabel->position -= length;
    }

    optimizelabels();
  }

  clearautomation(start, end1, operation);

  selectionend = selectionstart;
  if(operation == CLEAR){
    drawwave(0, 0, wavewindow.w, wavewindow.h, startx, starty, NO_BLANK, FLASH,
      DRAWCURSOR);
    drawlabels(startx);
    updatexscrollbar(wavscroll.xpixmap, wavscroll.xwin, startx,
      wavewindow.w * xscale, longesttrack(), &wavscroll.x);
    drawplayselection();
  }

  return 0;
}

                                    /* cut from start to end */
int cut(long start, long end, char operation){
  if(start == end) return 1;
                        /* don't update undo if being used as utility */
  if(operation == CUT) updateundo(CUT);
  copy(start, end, operation);
  clear(start, end, operation);
  if(operation == CUT){
    drawwave(0, 0, wavewindow.w, wavewindow.h, startx, starty, NO_BLANK, FLASH,
      DRAWCURSOR);
    drawlabels(startx);
  }

  drawplayselection();
  return 0;
}

int selectwave(XEvent *report){
  FOUR *edgeofselect;           /* pointer to selectionend or selectionstart, whichever
                                   is the edge of the selection being modified */
  XEvent nextreport;
  long x, y, temp;
  int i, outofbounds = 0;
                             /* test the edits for selection */
  if(showedits && selectedits(report)) return 0;
  
                             /* test the automation for selection */
  if(automationisvisible() && selectautomation(report)) return 0;

  x = report->xmotion.x * xscale + startx;
  y = report->xmotion.y;
  if(x < 0) x = 0;


  if(report->xmotion.state & ShiftMask){   /* shift down */
                                   /* select edge of selection reselected */
    if(x > (selectionstart + selectionend)/2) edgeofselect = &selectionend;
    else  edgeofselect = &selectionstart;
  }else{                                   /* shift up */
                                   /* hide old cursor */
    drawcursor(0, 0, wavewindow.w, wavewindow.h, startx, selectionstart, selectionend);
    selectionstart = x;
    selectionend = x;
                                   /* show new cursor */
    drawcursor(0, 0, wavewindow.w, wavewindow.h, startx, selectionstart, selectionend);
    refreshwav();
    edgeofselect = &selectionend;
  }

  drawplayselection();
  while(report->type != ButtonRelease){
    XNextEvent(display, report);

    if(report->type == Expose) refreshdisplay();

    if(report->type == MotionNotify){
      x = report->xmotion.x;
      y = report->xmotion.y;

      if(x > wavewindow.w){
        moveforward((x - wavewindow.w) *xscale);
      }
      if(x < 0){
        moveback((-x)*xscale);
      }
      if(y < 0){
        moveup(1);
      }
      if(y > wavewindow.h){
        movedown(1);
      }

      drawcursor(0, 0, wavewindow.w, wavewindow.h, startx,
        selectionstart, selectionend);

      *edgeofselect = x*xscale + startx;
      if(*edgeofselect < 0) *edgeofselect = 0;
      if(selectionend < selectionstart){
        temp = selectionend;
        selectionend = selectionstart;
        selectionstart = temp;
        if(edgeofselect == &selectionend)
          edgeofselect = &selectionstart;
        else
          edgeofselect = &selectionend;
      }

      drawcursor(0, 0, wavewindow.w, wavewindow.h, startx, selectionstart, selectionend);
      drawplayselection();
      refreshwav();
    }
    flushevents();
  }
  drawlabels(startx);
  return 0;
}
