/*
  Broadcast 2.0 multitrack audio editing
  (c) 1997 Adam Williams

  This program is distributed with the intent that it will be useful, without
  any warranty.  The code is being updated and has many bugs which are
  constantly being elucidated so changes should not be made without notifying
  the author.
*/

/* effects.c      effects processors  */

#include <math.h>

#include "interfac.h"
#include "effects.h"
#include "graphics.h"
#include "undo.h"
#include "edit.h"
#include "data.h"
#include "database.h"
#include "units.h"
#include "console.h"
#include "record.h"

static int writechannel;            /* channel for writebuffer and nextchannel */
char effectfiletype = BCAST;         /* filetype for effects */
static struct CastHeader *castheader;
static FILE *out;
long totallength, progress;
char *outbufferstart_;
int totaltracks, frame;
char filename[256] = "";

void loadiniteffectdir(FILE *in){
  fscanf(in, "%s", filename);
}

void loadiniteffecttype(int value){
	effectfiletype = value;
}

void saveiniteffecttype(FILE *out){
  fprintf(out, "EFFECTTYPE %d\n", effectfiletype);
}

void saveiniteffectdir(FILE *out){
  char string[256];

  /* cutdirectory(string, filename); */
	strcpy(string, filename);
  /* only save if a string exists since fscanf would skip the next line */
  if(strlen(string)) fprintf(out, "EFFECTDIR %s\n", string);
}

/* open output file for effect
   return 1 on failure else return 0 */
int initeffectfile(){
  TWO i = 0;
  struct Track *currenttrack;
  struct Edit *currentedit;
  char *_filename;
  char string[256];
  char string_[256];

                  /* get total length */
                 /* uncomment to generate default filename */
/*
  totallength = 0;
  for(totaltracks = 0, currenttrack = firsttrack;
    currenttrack; currenttrack = currenttrack->nexttrack)
  {
    if(currenttrack->record){
      if(totaltracks == 0){
        currentedit = segmentof(selectionstart, currenttrack);
        strcpy(filename, currentedit->file);
      }


      totaltracks++;
      totallength += selectionend - selectionstart;
    }
  }
*/

  /* for no default filenames */
  totaltracks = recordabletracks();
  totallength = totaltracks * (selectionend - selectionstart);

  if(totallength == 0) return 1;
  frame = totaltracks * 2;
  writechannel = 0;
           /* uncomment to generate default filename */
  /* test for existing effect files up to file # 256 */
  /* create file one higher than last existing effect file */
/*
  do{
    for(_filename = filename + strlen(filename);
      _filename >= filename && *_filename != '/';
      _filename--)
      ;

    _filename++;
    sprintf(_filename, "effect_%d", i);
    if(out = fopen(filename, "rb"))
      fclose(out);
    i++;
  }while(out && i < 256);
*/
  if(getfileattributes(filename, &effectfiletype, EFFECT, "Effect file"))
    return 1;

  if(out = fopen(filename, "rb")){                     /* filename exits */
    fclose(out);

    cutfilename(string_, filename);
    sprintf(string, "%s exists.   Overwrite?", string_);
    if(confirm(string, "Effect file")) return 1;
  }
  sprintf(string, "%s%s", filename, SCALEFILEEXT); /* remove scale file */
  if(out = fopen(string, "rb")){
    fclose(out);
    remove(string);
  }

  if(out = fopen(filename, "rb")){          /* remove data file */
    fclose(out);
    remove(filename);
  }

  if(i == 256 | !(out = fopen(filename, "w+b"))){  /* open effect file */
    printf("Can't create effect file.\n");
    return 1;
  }

  if(startbuffering()){       /* initialize input buffers */
    fclose(out);
    return 1;
  }

  changesmade = 1;
  updateundo(EFFECT);

  progress = 0;

  switch(effectfiletype){
    case BCAST:                              /* initialize header */
      if(!(castheader = malloc(sizeof(struct CastHeader) * (totaltracks + 1)))){
        printf("Not enough memory to save as BCAST\n");
        fclose(out);
        killbuffering();
        return 1;
      }
                                                    /* write file description */
      fprintf(out, "CAST");
                                                            /* write # tracks */
      puttwo(totaltracks, out);
                                              /* # of output channels in file */
      puttwo(outchannels, out);
                                                          /* write main level */
      putfloat(mainlevel, out);
                                                         /* write sample rate */
      putfour(samplerate, out);
                                                         /* write # of labels */
      putfour(0, out);

      for(i = 0; i < 4; i++) fputc(0, out);               /* skip label start */

      for(i = 0; i < 20; i++) fputc(0, out);     /* write 20 bytes of padding */

      for(i = 0; i < totaltracks * (15 + 4 * outchannels); i++)
        fputc(0, out);                         /* skip track list */
      castheader[writechannel].start = ftell(out);    /* store start of first track */
      break;

    case   WAV:
      if(!(outbufferstart_ = malloc(SAVEBUFFER))){    /* initialize output buffer */
        printf("Not enough memory to save as WAV.\n");
        fclose(out);
        killbuffering();
        return 1;
      }

      for(i = 0; i < WAVHEADERLENGTH; i++) fputc(0, out);
      break;

    case PCMHILO:
      if(!(outbufferstart_ = malloc(SAVEBUFFER))){    /* initialize output buffer */
        printf("Not enough memory to save as WAV.\n");
        fclose(out);
        killbuffering();
        return 1;
      }
     
    case PCMLOHI:            /* same as PCMHILO */
      if(!(outbufferstart_ = malloc(SAVEBUFFER))){    /* initialize output buffer */
        printf("Not enough memory to save as WAV.\n");
        fclose(out);
        killbuffering();
        return 1;
      }
      break;
  }
  return 0;
}

void killeffectfile(){
  killbuffering();
  if(effectfiletype == BCAST) free(castheader);
  else free(outbufferstart_);
  fclose(out);
  remove(filename);
}

/* increment output channel for writing */
/* output channels must be written consecutively */
void nextchannel(){
  switch(effectfiletype){
    case BCAST:
      castheader[writechannel].length =
        ftell(out) - castheader[writechannel].start;
      writechannel++;
      castheader[writechannel].start = ftell(out);
      break;

    case WAV:
      writechannel++;
      fseek(out, WAVHEADERLENGTH + writechannel * 2, SEEK_SET);
      break;

    case PCMHILO:
      writechannel++;
      fseek(out, writechannel * 2, SEEK_SET);
      break;

    case PCMLOHI:
      writechannel++;
      fseek(out, writechannel * 2, SEEK_SET);
      break;
  }
}

/* write buffer to output file */
int writebuffer(char *bufferstart, long buffersize){
  static TWO *outbuffer_, *buffer_;
  static char *bufferend, *outbufferend_;
  static long outbuffersize, fileposition;

  if(effectfiletype == BCAST){
    if(!(fwrite(bufferstart, buffersize, 1, out))){
        printf("Disk error\n");
        destroyprogress();
        killeffectfile();
        return 1;
    }
  }else{
    outbuffersize = SAVEBUFFER;
    outbuffersize /= frame;
    outbuffersize *= frame;

    buffer_ = (TWO *)bufferstart;
    bufferend = bufferstart + buffersize;

    if((!byteorder && effectfiletype == WAV) ||
    	(byteorder && effectfiletype == PCMHILO) ||
    	(!byteorder && effectfiletype == PCMLOHI)) swapbytes(bufferstart, buffersize);

    while(buffer_ < (TWO *)bufferend){
      if(bufferend - (char *)buffer_ < outbuffersize / totaltracks)
        outbuffersize = (bufferend - (char *)buffer_) * totaltracks;
      outbufferend_ = outbufferstart_ + outbuffersize;

      if(writechannel > 0){          /* if higher channel read in old data */
        fileposition = ftell(out);

        fread(outbufferstart_, outbuffersize, 1, out);
        fseek(out, fileposition, SEEK_SET);
      }

                             /* overlay next channel */
      for(outbuffer_ = (TWO *)outbufferstart_;
        outbuffer_ < (TWO *)outbufferend_;
        outbuffer_ += totaltracks)
      {
        *outbuffer_ = *buffer_++;
      }

      if(!fwrite(outbufferstart_, outbuffersize, 1, out)){
        printf("Disk error\n");
        destroyprogress();
        killeffectfile();
        return 1;
      }
      fflush(out);
    }
  }

  if(updateprogress(progress, totallength)){
    destroyprogress();
    killeffectfile();
    return 1;
  }
  return 0;
}

int closeeffectfile(){
  struct Track *currenttrack;
  FOUR fileposition;
  TWO i;

  fflush(out);

  switch(effectfiletype){
    case BCAST:
      fileposition = ftell(out);   /* write labels */
      fseek(out, 20, SEEK_SET);
      putfour(fileposition, out);
      fseek(out, fileposition, SEEK_SET);

      puttwo(0, out);              /* write groups */
      puttwo(0, out);

      writeeffectautomation(out, castheader);
                                   /* write eq */
      puttwo(totaltracks, out);

      for(currenttrack = firsttrack;
        currenttrack;
        currenttrack = currenttrack->nexttrack)
      {
        if(currenttrack->record){
          for(i = 0; i < TOTALEQS; i++){
            putfloat(currenttrack->eq[i].level, out);
            puttwo(currenttrack->eq[i].freq, out);
            puttwo(currenttrack->eq[i].width, out);
            fputc(currenttrack->eq[i].in, out);
          }
        }
      }

      i = 0;
      fseek(out, 44, SEEK_SET);                            /* write header */
      for(currenttrack = firsttrack;
        currenttrack;
        currenttrack = currenttrack->nexttrack)
      {
        if(currenttrack->record){
          /* must use fwrite because readbuffer uses int in */
          putfour(castheader[i].start, out);       /* put start of track */
          putfour(castheader[i].length, out);     /* put length of track */
          fputc(currenttrack->record, out);         /* write record permission */
          fputc(currenttrack->play, out);             /* write play permission */
          putfloat(currenttrack->level, out);              /* write level */
          writepans(currenttrack->pan, out, outchannels);          /* write pans */
          fputc(currenttrack->invert, out);
          i++;
        }
      }

      free(castheader);
      break;

    case WAV:
    	fseek(out, 0, SEEK_END);
      fileposition = ftell(out);
      free(outbufferstart_);
      fseek(out, 0, SEEK_SET);
      writeheader(out, (fileposition - WAVHEADERLENGTH) / totaltracks / 2, writechannel);
      break;

    case PCMHILO:
      free(outbufferstart_);
      break;

    case PCMLOHI:
      free(outbufferstart_);
      break;
  }

  killbuffering();
  destroyprogress();
  refreshdisplay();
  fclose(out);
}

int crossfade(long selectionstart, long selectionend){
  struct Track *currenttrack;
  struct Edit *currentedit;
  char *inbufferstart, *outbufferstart;
  long selectionpre;
  register float scale;
  register TWO sample;

  if(!(inbufferstart = malloc(SAVEBUFFER))){
    printf("Not enough memory to crossfade.\n");
    return 1;
  }

  if(!(outbufferstart = malloc(SAVEBUFFER))){
    printf("Not enough memory to crossfade.\n");
    free(inbufferstart);
    return 1;
  }
                                     /* determine start of crossfade */
  selectionpre = selectionstart - (selectionend - selectionstart);
                                     /* truncate selection if near beginning */
  if(selectionpre < 0){
    selectionpre = 0;
    selectionend = selectionstart * 2;
  }

  if(initeffectfile()){
    free(inbufferstart);
    free(outbufferstart);
    return 1;
  }

  initializeprogress("Crossfade...");
  for(currenttrack = firsttrack; currenttrack;
    currenttrack = currenttrack->nexttrack)
  {                                         /* process one channel at a time */
    if(currenttrack->record){
      register TWO *inbuffer_, *outbuffer_;
      register char *inbufferend;
      register long x1_, x1, x2_, x2;
      static long selection;
      static long bufferlength;

      bufferlength = SAVEBUFFER;
      x1 = selectionstart;
      x2 = selectionpre;
      selection = selectionend - selectionstart;
      x1_ = 0;
      x2_ = selection;

      while(x1 < selectionend){
        if(selectionend - x1 < bufferlength / 2)
          bufferlength = (selectionend - x1) * 2;
        if(readbuffer(inbufferstart, bufferlength, currenttrack, x1)){
          destroyprogress();
          free(inbufferstart);
          free(outbufferstart);
          killeffectfile();
          return 1;
        }

        inbuffer_ = (TWO *)inbufferstart;
        outbuffer_ = (TWO *)outbufferstart;
        inbufferend = inbufferstart + bufferlength;
        while(inbuffer_ < (TWO *)inbufferend){
          scale = x1_;
          scale /= selection;             /* scale is now percentage of selection */
          *outbuffer_ = *inbuffer_++;
          *outbuffer_++ *= scale;
          x1_++;
        }

        if(readbuffer(inbufferstart, bufferlength, currenttrack, x2)){
          destroyprogress();
          free(inbufferstart);
          free(outbufferstart);
          killeffectfile();
          return 1;
        }

        inbuffer_ = (TWO *)inbufferstart;
        outbuffer_ = (TWO *)outbufferstart;
        inbufferend = inbufferstart + bufferlength;
        while(inbuffer_ < (TWO *)inbufferend){
          scale = x2_;
          scale /= selection;
          sample = *inbuffer_++;
          sample *= scale;
          *outbuffer_++ += sample;
          x2_--;
        }

        progress += bufferlength / 2;
        if(writebuffer(outbufferstart, bufferlength)){
          free(inbufferstart);
          free(outbufferstart);
          return 1;
        }

        x1 += bufferlength / 2;
        x2 += bufferlength / 2;
      }
      nextchannel();                   /* end of track */
    }
  }

  closeeffectfile();
  free(inbufferstart);
  free(outbufferstart);

  recevent.length = selectionstart - selectionpre;
  strcpy(recevent.file, filename);
  recevent.channels = totaltracks;
  recevent.format = effectfiletype;
  reclabels = 0;
  paste(selectionpre, selectionend, EFFECT);
  return 0;
}

static float normalizelevel = 0;

void loadnormalizelevel(FILE *in, float value){
  normalizelevel = value;
}

void savenormalizelevel(FILE *out){
  fprintf(out, "NORMALIZELEVEL %f\n", normalizelevel);
}

int normalize(long selectionstart, long selectionend){
  struct Track *currenttrack;
  char *inbufferstart, *outbufferstart;
  char *captions[] = {
    "Set the normalization level"
  };
  bctextbox textbox[1];
  bcslider slider[1] = {0, 0, 0, 0, SLIDER_DB, INFINITYGAIN, 6, 0, 0, ""};
  float normalizelevel_;
  
  slider[0].value = normalizelevel;
  slidertotext(&textbox[0], &slider[0]);

  if(!(inbufferstart = malloc(SAVEBUFFER))){
    printf("Not enough memory to normalize.\n");
    return 1;
  }

  if(!(outbufferstart = malloc(SAVEBUFFER))){
    printf("Not enough memory to normalize.\n");
    free(inbufferstart);
    return 1;
  }

  if(initeffectfile()){
    free(inbufferstart);
    free(outbufferstart);
    return 1;
  }

  if(getnumericinput(1, textbox, slider, captions, "Normalize", 0))
  {
    killeffectfile();
    free(inbufferstart);
    free(outbufferstart);
    return 1;
  }
  
  normalizelevel = slider[0].value;
  normalizelevel_ = topower[(int)(slider[0].value * 10)];

  initializeprogress("Normalize...");

  for(currenttrack = firsttrack; currenttrack;
    currenttrack = currenttrack->nexttrack)
  {
    if(currenttrack->record){
      static TWO *inbuffer_, *outbuffer_;
      static char *inbufferend;
      static long x;
      static long bufferlength;
      static long highsample;
      static float scale;
      static long sample;

      bufferlength = SAVEBUFFER;
      x = selectionstart;
      highsample = 0;

      while(x < selectionend){                  /* get highest sample */
        if(selectionend - x < bufferlength / 2)
          bufferlength = (selectionend - x) * 2;
        if(readbuffer(inbufferstart, bufferlength, currenttrack, x)){
          destroyprogress();
          free(inbufferstart);
          free(outbufferstart);
          killeffectfile();
          return 1;
        }

        inbuffer_ = (TWO *)inbufferstart;
        inbufferend = inbufferstart + bufferlength;

        inbufferend -= 8;
        while(inbuffer_ < (TWO *)inbufferend){
          if(abs(*inbuffer_) > highsample)
            highsample = abs(*inbuffer_);
          inbuffer_++;

          if(abs(*inbuffer_) > highsample)
            highsample = abs(*inbuffer_);
          inbuffer_++;

          if(abs(*inbuffer_) > highsample)
            highsample = abs(*inbuffer_);
          inbuffer_++;

          if(abs(*inbuffer_) > highsample)
            highsample = abs(*inbuffer_);
          inbuffer_++;
        }

        inbufferend += 8;
        while(inbuffer_ < (TWO *)inbufferend){
          if(abs(*inbuffer_) > highsample)
            highsample = abs(*inbuffer_);
          inbuffer_++;
        }

        x += bufferlength / 2;
        progress += bufferlength / 4;
        if(updateprogress(progress, totallength)){
          free(inbufferstart);
          free(outbufferstart);
          destroyprogress();
          killeffectfile();
          return 1;
        }
      }

      bufferlength = SAVEBUFFER;
      x = selectionstart;
      if(highsample != 0){
        scale = normalizelevel_ * 32767;
        scale /= highsample;
      }
      else scale = 1;

      while(x < selectionend){                  /* amplify */
        if(selectionend - x < bufferlength / 2)
          bufferlength = (selectionend - x) * 2;

        if(readbuffer(inbufferstart, bufferlength, currenttrack, x)){
          destroyprogress();
          killeffectfile();
          free(inbufferstart);
          free(outbufferstart);
          return 1;
        }

        inbuffer_ = (TWO *)inbufferstart;
        outbuffer_ = (TWO *)outbufferstart;
        inbufferend = inbufferstart + bufferlength;

                          /* don't worry about clipping */
        if(normalizelevel_ <= 1){
        	inbufferend -= 8;
          while(inbuffer_ < (TWO *)inbufferend){
            *outbuffer_ = *inbuffer_++;
            *outbuffer_++ *= scale;

            *outbuffer_ = *inbuffer_++;
            *outbuffer_++ *= scale;

            *outbuffer_ = *inbuffer_++;
            *outbuffer_++ *= scale;

            *outbuffer_ = *inbuffer_++;
            *outbuffer_++ *= scale;
          }
          
        	inbufferend += 8;
          while(inbuffer_ < (TWO *)inbufferend){
            *outbuffer_ = *inbuffer_++;
            *outbuffer_++ *= scale;
          }
        }else{
                          /* worry about clipping */
        	inbufferend -= 8;
          while(inbuffer_ < (TWO *)inbufferend){
            sample = *inbuffer_++;
            sample *= scale;
            if(sample > 32767) *outbuffer_++ = 32767;
            else if(sample < -32768) *outbuffer_++ = -32768;
            else *outbuffer_++ = sample;

            sample = *inbuffer_++;
            sample *= scale;
            if(sample > 32767) *outbuffer_++ = 32767;
            else if(sample < -32768) *outbuffer_++ = -32768;
            else *outbuffer_++ = sample;

            sample = *inbuffer_++;
            sample *= scale;
            if(sample > 32767) *outbuffer_++ = 32767;
            else if(sample < -32768) *outbuffer_++ = -32768;
            else *outbuffer_++ = sample;

            sample = *inbuffer_++;
            sample *= scale;
            if(sample > 32767) *outbuffer_++ = 32767;
            else if(sample < -32768) *outbuffer_++ = -32768;
            else *outbuffer_++ = sample;
          }
          
        	inbufferend += 8;
          while(inbuffer_ < (TWO *)inbufferend){
            sample = *inbuffer_++;
            sample *= scale;
            if(sample > 32767) *outbuffer_++ = 32767;
            else if(sample < -32768) *outbuffer_++ = -32768;
            else *outbuffer_++ = sample;
          }
        }

        progress += bufferlength / 4;
        if(writebuffer(outbufferstart, bufferlength)){
          free(inbufferstart);
          free(outbufferstart);
          return 1;
        }

        x += bufferlength / 2;
      }

      nextchannel();
    }
  }

  closeeffectfile();
  free(inbufferstart);
  free(outbufferstart);
  recevent.length = selectionend - selectionstart;
  strcpy(recevent.file, filename);
  recevent.channels = totaltracks;
  recevent.format = effectfiletype;
  reclabels = 0;
  paste(selectionstart, selectionend, EFFECT);
  return 0;
}

int balance(long selectionstart, long selectionend){
  struct Track *currenttrack;
  struct Edit *currentedit;
  char *inbufferstart, *outbufferstart;

  if(!(inbufferstart = malloc(SAVEBUFFER))){
    printf("Not enough memory to balance.\n");
    return 1;
  }

  if(!(outbufferstart = malloc(SAVEBUFFER))){
    printf("Not enough memory to balance.\n");
    free(inbufferstart);
    return 1;
  }

  if(initeffectfile()){
    free(inbufferstart);
    free(outbufferstart);
    return 1;
  }

  initializeprogress("Balance...");

  for(currenttrack = firsttrack; currenttrack;
    currenttrack = currenttrack->nexttrack)
  {
    if(currenttrack->record){
      static TWO *inbuffer_, *outbuffer_;
      static char *inbufferend;
      static long x;
      static long bufferlength;
      static TWO avgsample;
      static long double totalsample;
      static float scale;

      bufferlength = SAVEBUFFER;
      x = (selectionend - selectionstart) / 2 + selectionstart;
      totalsample = 0;

                                                         /* get average sample */
      if(selectionend - x < bufferlength / 2)
        bufferlength = (selectionend - x) * 2;
      if(readbuffer(inbufferstart, bufferlength, currenttrack, x)){
        destroyprogress();
        free(inbufferstart);
        free(outbufferstart);
        killeffectfile();
        return 1;
      }

      inbuffer_ = (TWO *)inbufferstart;
      inbufferend = inbufferstart + bufferlength;
      
      inbufferend -= 8;
      while(inbuffer_ < (TWO *)inbufferend){
        totalsample += *inbuffer_++;
        totalsample += *inbuffer_++;
        totalsample += *inbuffer_++;
        totalsample += *inbuffer_++;
      }
      
      inbufferend += 8;
      while(inbuffer_ < (TWO *)inbufferend){
        totalsample += *inbuffer_++;
      }

      avgsample = totalsample / (bufferlength / 2);
      bufferlength = SAVEBUFFER;
      x = selectionstart;

      while(x < selectionend){                  /* shift values */
        if(selectionend - x < bufferlength / 2)
          bufferlength = (selectionend - x) * 2;

        if(readbuffer(inbufferstart, bufferlength, currenttrack, x)){
          destroyprogress();
          killeffectfile();
          free(inbufferstart);
          free(outbufferstart);
          return 1;
        }

        inbuffer_ = (TWO *)inbufferstart;
        outbuffer_ = (TWO *)outbufferstart;
        inbufferend = inbufferstart + bufferlength;

				inbufferend -= 8;
        while(inbuffer_ < (TWO *)inbufferend){
          *outbuffer_ = *inbuffer_++;
          *outbuffer_++ -= avgsample;

          *outbuffer_ = *inbuffer_++;
          *outbuffer_++ -= avgsample;

          *outbuffer_ = *inbuffer_++;
          *outbuffer_++ -= avgsample;

          *outbuffer_ = *inbuffer_++;
          *outbuffer_++ -= avgsample;
        }

				inbufferend += 8;
        while(inbuffer_ < (TWO *)inbufferend){
          *outbuffer_ = *inbuffer_++;
          *outbuffer_++ -= avgsample;
        }

        progress += bufferlength / 2;

        if(writebuffer(outbufferstart, bufferlength)){
          free(inbufferstart);
          free(outbufferstart);
          return 1;
        }

        x += bufferlength / 2;
      }

      nextchannel();
    }
  }

  closeeffectfile();
  free(inbufferstart);
  free(outbufferstart);

  recevent.length = selectionend - selectionstart;
  strcpy(recevent.file, filename);
  recevent.channels = totaltracks;
  recevent.format = effectfiletype;
  reclabels = 0;
  paste(selectionstart, selectionend, EFFECT);
  return 0;
}

/*=============================reverb=======================================*/
/* set initial reverb values */

static float directlevel = 0;      /* initial sound level */
static int firstdelay = 500;     /* delay before first reflection */
static float latelevel1 = -3;      /* first late reflection level */
static float latelevel2 = -INFINITYGAIN;        /* last late reflection level  */
static int latereflections = 80;       /* # late reflections */
static int lateduration = 2000;     /* milliseconds for late reflections */
static int lowpass1 = 80;      /* initial band for low pass filter */
static int lowpass2 = 0;        /* final band for low pass filter */

void loadreverbparams(FILE *in, long value){
  int i;

  directlevel = value;
  fscanf(in, "%d", &firstdelay);
  fscanf(in, "%f", &latelevel1);
  fscanf(in, "%f", &latelevel2);
  fscanf(in, "%d", &latereflections);
  fscanf(in, "%d", &lateduration);
  fscanf(in, "%d", &lowpass1);
  fscanf(in, "%d", &lowpass2);
}

void savereverbparams(FILE *out){
  int i;

  fprintf(out, "REVERBPARAMS");
	fprintf(out, " %f", directlevel);
	fprintf(out, " %d", firstdelay);
	fprintf(out, " %f", latelevel1);
	fprintf(out, " %f", latelevel2);
	fprintf(out, " %d", latereflections);
	fprintf(out, " %d", lateduration);
	fprintf(out, " %d", lowpass1);
	fprintf(out, " %d", lowpass2);
  fprintf(out, "\n");
}

/* concert hall reverb effect using 32 bit processing */

/*
 * Sound Tools Bandpass effect file.
 *
 * Algorithm:  2nd order recursive filter.
 * Formula stolen from MUSIC56K, a toolkit of 56000 assembler stuff.
 * Quote:
 *   This is a 2nd order recursive band pass filter of the form.
 *   y(n)= a * x(n) - b * y(n-1) - c * y(n-2)
 *   where :
 *        x(n) = "IN"
 *        "OUT" = y(n)
 *        c = EXP(-2*pi*cBW/S_RATE)
 *        b = -4*c/(1+c)*COS(2*pi*cCF/S_RATE)
 *   if cSCL=2 (i.e. noise input)
 *        a = SQT(((1+c)*(1+c)-b*b)*(1-c)/(1+c))
 *   else
 *        a = SQT(1-b*b/(4*c))*(1-c)
 *   endif
 *   note :     cCF is the center frequency in Hertz
 *        cBW is the band width in Hertz
 *        cSCL is a scale factor, use 1 for pitched sounds
 *   use 2 for noise.
 */

int reverb(long selectionstart, long selectionend){
  struct Track *currenttrack;
  struct Edit *currentedit;
  char *inbufferstart, *outbufferstart, *finalbufferstart;
      /* inbufferstart - TWO byte input buffer
         outbufferstart - FLOAT processing buffer
         finalbufferstart - TWO byte output buffer */
  char *inbufferend, *outbufferend, *finalbufferend;
  long overlength, bufferlength;  /* length of overrun for delayed buffers */
                                  /* length of buffer to read */
  long x;                         /* position in source track */
  register TWO sample;         /* temporary storage for new sample to increase speed */
  int tracknumber;    /* number of recordable track being processed */
  float MINSCALE = 0.0001;
  int i;
  register float directlevel_;

/* set dialog */
  char *captions[] = {
    "Direct sound level",
    "Millisecond delay before reflections",
    "First late reflection level",
    "Last late reflection level",
    "# late reflections",
    "Milliseconds for late reflections",
    "Initial band for low pass filter",
    "Final band for low pass filter"
  };

	bctextbox textboxes[8];
	bcslider sliders[] = {
		{0, 0, 0, 0, SLIDER_DB, INFINITYGAIN, 3, 0, 0, ""},
		{0, 0, 0, 0, SLIDER_INT, 0, 1000, 0, 0, ""},
		{0, 0, 0, 0, SLIDER_DB, INFINITYGAIN, 3, 0, 0, ""},
	  {0, 0, 0, 0, SLIDER_DB, INFINITYGAIN, 3, 0, 0, ""},
	  {0, 0, 0, 0, SLIDER_INT, 0, 200, 0, 0, ""},
	  {0, 0, 0, 0, SLIDER_INT, 0, 30000, 0, 0, ""},
	  {0, 0, 0, 0, SLIDER_FREQ, 0, TOTALFREQS, 0, 0, ""},
	  {0, 0, 0, 0, SLIDER_FREQ, 0, TOTALFREQS, 0, 0, ""}
	};

	sliders[0].value = directlevel;
	sliders[1].value = firstdelay;
	sliders[2].value = latelevel1;
	sliders[3].value = latelevel2;
	sliders[4].value = latereflections;
	sliders[5].value = lateduration;
	sliders[6].value = lowpass1;
	sliders[7].value = lowpass2;

	for(i = 0; i < 8; i++){
		slidertotext(&textboxes[i], &sliders[i]);
	}
	
  if(initeffectfile())    /* get filename */
  {
    return 1;
  }

  if(getnumericinput(7, textboxes, sliders, captions, "Reverb Params", REVERB))
  {
    killeffectfile();
    return 1;
  }

	directlevel = sliders[0].value;
	firstdelay = sliders[1].value;
	latelevel1 = sliders[2].value;
	latelevel2 = sliders[3].value;
	latereflections = sliders[4].value;
	lateduration = sliders[5].value;
	lowpass1 = sliders[6].value;
	lowpass2 = sliders[7].value;

  overlength = firstdelay + lateduration + 1;   /* get length of reverberation in bytes */
  overlength *= (float)samplerate / 1000;
  overlength *= 2;

  if(!(inbufferstart = malloc(SAVEBUFFER))){
    killeffectfile();
    printf("Not enough memory to reverb.\n");
    return 1;
  }

  if(!(finalbufferstart = malloc(SAVEBUFFER))){
    killeffectfile();
    printf("Not enough memory to reverb output.\n");
    free(inbufferstart);
    return 1;
  }

  if(!(outbufferstart = malloc((SAVEBUFFER + overlength) * 2))){
    killeffectfile();
    printf("Not enough memory to reverb process.\n");
    free(inbufferstart);
    free(finalbufferstart);
    return 1;
  }

  initializeprogress("Reverb...");
  srand(1);
  tracknumber = 0;
  for(currenttrack = firsttrack; currenttrack; currenttrack = currenttrack->nexttrack)
  {
    if(currenttrack->record){
      register TWO *inbuffer_, *finalbuffer_;   /* current buffer positions */
      register FLOAT *outbuffer_;
      static struct Track *currenttrack_; /* track being overlayed */
      static FOUR *latestart; /* start points for reflections */
      static long division;     /* time between reflections */
      static long overlay;         /* counter for overlay */
      register double *out1, *out2;    /* previous output for low pass filter */
      static float lowpslope;      /* slope of low pass in late reflections */
      static float scaleslope;  /* slope of scale in late reflections */
      static float firstscale, lastscale; /* first and last scales for late
                                             reflections */

      bufferlength = SAVEBUFFER;
      outbuffer_ = (FLOAT *)outbufferstart;  /* clear initial overbuffer + buffer */
      outbufferend = outbufferstart + (bufferlength + overlength) * 2;
      while(outbuffer_ < (FLOAT *)outbufferend){
        *outbuffer_++ = 0;
      }

                                     /* get late reflection starts */
      latestart = malloc(4 * latereflections);
      if(latereflections){
        division = lateduration /
          latereflections;
        division *= samplerate / 1000;

                                   /* offset in units of samples */
        for(overlay = 0; overlay < latereflections; overlay++){
                                   /* first late reflection */
          latestart[overlay] = firstdelay * samplerate / 1000 
          	+ division * (overlay + 1);
          latestart[overlay] -= rand() % (int)(division / 1.33);
             /* give randomness of up to three fourths of a division */
        }
      }
                                     /* initialize low pass filter */
      out1 = malloc(sizeof(double) * latereflections);
      out2 = malloc(sizeof(double) * latereflections);
      for(overlay = 0; overlay < latereflections; overlay++){
        out1[overlay] = 0;
        out2[overlay] = 0;
      }

                             /* initialize late reflection levels */
      currenttrack_ = currenttrack;
      lowpslope = lowpass1 - lowpass2;
      lowpslope /= latereflections;
      firstscale = latelevel1;
      lastscale = latelevel2;

      if(firstscale == INFINITYGAIN) firstscale = INFINITYGAIN + 1;
      if(lastscale == INFINITYGAIN) lastscale = INFINITYGAIN + 1;
      scaleslope = (lastscale - firstscale) / latereflections;

                              /* reverb output to each track seperately */
                              /* reverb input from every track simultaneously */
      x = selectionstart;
      while(x < selectionend){       /* process track */
        static FLOAT *outbuffer__;     /* destination for overbuffer copy */
        static float scale;          /* scale for overlay */
        static long firstoverlay;    /* first overlay for overlaying all the occurences
                                        of a track */
        static int track;                /* number of track being overlayed */

        outbuffer_ = (FLOAT *)outbufferstart + bufferlength/2;
        outbuffer__ = (FLOAT *)outbufferstart;
        outbufferend = outbufferstart + (bufferlength + overlength) * 2;

                                     /* copy overbuffer to start of buffer */
        while(outbuffer_ < (FLOAT *)outbufferend){
          *outbuffer__++ = *outbuffer_++;
        }
                                     /* reset new overbuffer */
        while(outbuffer__ < (FLOAT *)outbufferend){
          *outbuffer__++ = 0;
        }

                                     /* read current track's data */
        if(selectionend - x < bufferlength / 2)
          bufferlength = (selectionend - x) * 2;
        if(readbuffer(inbufferstart, bufferlength, currenttrack, x)){
          destroyprogress();
          free(inbufferstart);
          free(outbufferstart);
          free(finalbufferstart);
          free(latestart);
          free(out1);
          free(out2);
          killeffectfile();
          return 1;
        }

                                     /* overlay direct sound */
				directlevel_ = topower[(int)(directlevel * 10)];  /* directlevel_ now
					equals a power */
        outbuffer_ = (FLOAT *)outbufferstart;
        inbuffer_ = (TWO *)inbufferstart;
        inbufferend = inbufferstart + bufferlength;

        while(inbuffer_ < (TWO *)inbufferend){
          sample = *inbuffer_++;
          sample *= directlevel_;
          *outbuffer_++ += sample;
        }

                                          /* overlay late reflections */
        for(firstoverlay = 0; firstoverlay < totaltracks;
          firstoverlay++)
        {
                                   /* load currenttrack_ */
          if(readbuffer(inbufferstart, bufferlength, currenttrack_, x)){
            destroyprogress();
            free(inbufferstart);
            free(outbufferstart);
            free(finalbufferstart);
            free(latestart);
            killeffectfile();
            free(out1);
            free(out2);
            return 1;
          }

                         /* overlay all reflections involving currenttrack_ */
          for(overlay = firstoverlay; overlay < latereflections;
            overlay += totaltracks)
          {
            register double a, b, c;    /* constants for filter */
            register double d, *out1_, *out2_; /* pointers to previous input */
            static float cutoff;  /* cutoff frequency */
            static int frequency; /* freqtable entry of cutoff frequency */
            static float scalereduction;
                 /* reduction factor for high overdub count */

                                   /* calculate low pass values */
            out1_ = &out1[overlay];
            out2_ = &out2[overlay];
            frequency = lowpass1 - overlay * lowpslope;
            cutoff = tofreq(frequency);

            if(cutoff < 20) cutoff = 20;  /* cut lowpass off at 30 */

                                   /* calculate level */
            scale = firstscale + scaleslope * overlay;
            scale = topower[(int)(scale * 10)];

                                  /* compensate for high number of overlays */
            scalereduction = latereflections / 30;
            if(scalereduction > 1) scale /= scalereduction;


            if(scale < MINSCALE) scale = MINSCALE;  /* cut scale off at MINSCALE */

            c = exp(-4 * 3.14159265 * cutoff / samplerate);
            b = -4 * c / (1 + c);
            a = sqrt(1 - b * b / (4 * c)) * (1 - c);
            a *= scale;

            outbuffer_ = (FLOAT *)outbufferstart + latestart[overlay];
            inbuffer_ = (TWO *)inbufferstart;
            inbufferend = inbufferstart + bufferlength;

						inbufferend -= 8;
						
#ifndef FLOAT_BUG
            while(inbuffer_ < (TWO *)inbufferend){
                                   /* process low pass filter */
              d = *inbuffer_++;
              d = a * d - b * *out1_ - c * *out2_;
              *out2_ = *out1_;
              *out1_ = d;
                                   /* overlay */
              *outbuffer_++ += d;
              
              d = *inbuffer_++;
              d = a * d - b * *out1_ - c * *out2_;
              *out2_ = *out1_;
              *out1_ = d;
              *outbuffer_++ += d;
              
              d = *inbuffer_++;
              d = a * d - b * *out1_ - c * *out2_;
              *out2_ = *out1_;
              *out1_ = d;
              *outbuffer_++ += d;
              
              d = *inbuffer_++;
              d = a * d - b * *out1_ - c * *out2_;
              *out2_ = *out1_;
              *out1_ = d;
              *outbuffer_++ += d;
            }
#else
            while(inbuffer_ < (TWO *)inbufferend){
                                   /* process low pass filter */
              d = *inbuffer_++;
                           /* check to see if 0 to avoid slowing down */
                           /* on HP eqing 0 slowed it down sometimes */
          		if(d) d *= a;
          		if(*out1_) d -= b * *out1_ + c * *out2_;
              *out2_ = *out1_;
              *out1_ = d;
                                   /* overlay */
              *outbuffer_++ += d;

              d = *inbuffer_++;
          		if(d) d *= a;
          		if(*out1_) d -= b * *out1_ + c * *out2_;
              *out2_ = *out1_;
              *out1_ = d;
              *outbuffer_++ += d;

              d = *inbuffer_++;
          		if(d) d *= a;
          		if(*out1_) d -= b * *out1_ + c * *out2_;
              *out2_ = *out1_;
              *out1_ = d;
              *outbuffer_++ += d;

              d = *inbuffer_++;
          		if(d) d *= a;
          		if(*out1_) d -= b * *out1_ + c * *out2_;
              *out2_ = *out1_;
              *out1_ = d;
              *outbuffer_++ += d;
            }
#endif

						inbufferend += 8;						
            while(inbuffer_ < (TWO *)inbufferend){
                                   /* process low pass filter */
              d = *inbuffer_++;

#ifndef FLOAT_BUG
              d = a * d - b * *out1_ - c * *out2_;

#else
                           /* check to see if 0 to avoid slowing down */
                           /* on HP eqing 0 slowed it down sometimes */
          		if(d) d *= a;
          		if(*out1_) d -= b * *out1_ + c * *out2_;
#endif

              *out2_ = *out1_;
              *out1_ = d;
                                   /* overlay */
              *outbuffer_++ += d;
            }

            progress += bufferlength / 2 / latereflections;
            if(updateprogress(progress, totallength)){
              destroyprogress();
              killeffectfile();
              free(inbufferstart);
              free(outbufferstart);
              free(finalbufferstart);
              free(latestart);
              free(out1);
              free(out2);
              return 1;
            }
          }  /* get next overlay */

          do{                  /* get next recordable track to overlay */
            currenttrack_ = currenttrack_->nexttrack;
            if(currenttrack_ == 0) currenttrack_ = firsttrack;
          }while(!currenttrack_->record);
        }

                             /* copy FLOAT buffer to final TWO buffer */
        outbuffer_ = (FLOAT *)outbufferstart;
        outbufferend = outbufferstart + bufferlength * 2;
        finalbuffer_ = (TWO *)finalbufferstart;

        while(outbuffer_ < (FLOAT *)outbufferend){
          if(*outbuffer_ > 32767) *finalbuffer_++ = 32767;
          else if(*outbuffer_ < -32768) *finalbuffer_++ = -32768;
          else *finalbuffer_++ = *outbuffer_;
          outbuffer_++;
        }

                                   /* write data to disk */
        if(writebuffer(finalbufferstart, bufferlength)){
          free(inbufferstart);
          free(outbufferstart);
          free(finalbufferstart);
          free(latestart);
          free(out1);
          free(out2);
          return 1;
        }

        x += bufferlength / 2;
      }
      nextchannel();
      tracknumber++;
      free(latestart);
      free(out1);
      free(out2);
    }
  }

  closeeffectfile();
  free(inbufferstart);
  free(outbufferstart);
  free(finalbufferstart);

  recevent.length = selectionend - selectionstart;
  strcpy(recevent.file, filename);
  recevent.channels = totaltracks;
  recevent.format = effectfiletype;
  reclabels = 0;
  paste(selectionstart, selectionend, EFFECT);
  return 0;
}

/* reverse direction */
int backwards(long selectionstart, long selectionend){
  struct Track *currenttrack;
  struct Edit *currentedit;
  char *inbufferstart, *outbufferstart;

  if(!(inbufferstart = malloc(SAVEBUFFER))){
    printf("Not enough memory to balance.\n");
    return 1;
  }

  if(!(outbufferstart = malloc(SAVEBUFFER))){
    printf("Not enough memory to balance.\n");
    free(inbufferstart);
    return 1;
  }

  if(initeffectfile()){    /* initialize effect file */
    free(inbufferstart);
    free(outbufferstart);
    return 1;
  }

  initializeprogress("Backwards...");

  for(currenttrack = firsttrack; currenttrack;
    currenttrack = currenttrack->nexttrack)
  {
    if(currenttrack->record){
      static TWO *inbuffer_, *outbuffer_;
      static char *inbufferend;
      static long x, i;
      static long bufferlength;

      bufferlength = SAVEBUFFER;
      x = selectionend;
      
      while(x > selectionstart){         /* copy buffers from end to start */
        x -= bufferlength / 2;
        if(readbuffer(inbufferstart, bufferlength, currenttrack, x)){
          destroyprogress();
          killeffectfile();
          free(inbufferstart);
          free(outbufferstart);
          return 1;
        }

        inbuffer_ = (TWO *)inbufferstart;
        outbuffer_ = (TWO *)outbufferstart + bufferlength / 2;
        inbufferend = inbufferstart + bufferlength;

				inbufferend -= 8;
        while(inbuffer_ < (TWO *)inbufferend){      /* reverse buffers */
          *outbuffer_-- = *inbuffer_++;
          *outbuffer_-- = *inbuffer_++;
          *outbuffer_-- = *inbuffer_++;
          *outbuffer_-- = *inbuffer_++;
        }

				inbufferend += 8;
        while(inbuffer_ < (TWO *)inbufferend){
          *outbuffer_-- = *inbuffer_++;
          *outbuffer_-- = *inbuffer_++;
          *outbuffer_-- = *inbuffer_++;
          *outbuffer_-- = *inbuffer_++;
        }

        progress += bufferlength / 2;

        if(writebuffer(outbufferstart, bufferlength)){
          free(inbufferstart);         /* writebuffer failed */
          free(outbufferstart);
          return 1;
        }

        if(x - bufferlength / 2 < selectionstart)
          bufferlength = (x - selectionstart) * 2;
      }

      nextchannel();
    }
  }

  closeeffectfile();
  free(inbufferstart);
  free(outbufferstart);

  recevent.length = selectionend - selectionstart;
  strcpy(recevent.file, filename);
  recevent.channels = totaltracks;
  recevent.format = effectfiletype;
  reclabels = 0;
  paste(selectionstart, selectionend, EFFECT);
  return 0;
}
