/*
  Broadcast 2.0 multitrack audio editing
  (c) 1997 Adam Williams

  This program is distributed with the intent that it will be useful, without
  any warranty.  The code is being updated and has many bugs which are
  constantly being elucidated so changes should not be made without notifying
  the author.
*/

/* file.c      persistent storage handlers  */

#include "defs.h"
#include "interfac.h"
#include "data.h"
#include "file.h"
#include "edit.h"
#include "graphics.h"
#include "bcast.h"
#include "database.h"
#include "automate.h"

static char outfilename[256] = "";
static char infilename[256] = "";
static char playasfiletype = WAV;
static char loadasfiletype = PCMLOHI;

void locatefilenames(){
  struct Track *track, *track_;
  struct Edit *edit, *edit_;
  FILE *in;
  char string[256], filename[256], oldfilename[256];
  char newfilename[256] = "";

  for(track = firsttrack; track; track = track->nexttrack){
    for(edit = track->firstedit; edit; edit = edit->nextedit){
      if(strcmp(edit->file, SILENCE)){
      	if(!(in = fopen(edit->file, "rb"))){
        	cutfilename(filename, edit->file);
        	sprintf(string, "Where is %s", filename);
        	strcpy(oldfilename, edit->file);

                	 /* if no filename is given, replace it with silence */
        	if(getfileattributes(newfilename, 0, INPUT, string)){
          	strcpy(newfilename, SILENCE);
        	}

              	 /* replace every occurrance of this file */
        	for(track_ = firsttrack; track_; track_ = track_->nexttrack){
          	for(edit_ = track_->firstedit; edit_; edit_ = edit_->nextedit){
            	if(!strcmp(edit_->file, oldfilename)){
              	free(edit_->file);
              	edit_->file = malloc(strlen(newfilename) + 1);
              	strcpy(edit_->file, newfilename);
            	}
          	}
        	}
      	}else fclose(in);
      }
    }
  }
}

/* load waveform as wave of pcm file */
int loadwave(char filename[], char filetype, char operation){
  FILE *in;
  long filelength;
  static long newtracks = 2;          /* prompts for new tracks so static */
  char keys_return[5];
  int track, track1, i;
  struct Track *currenttrack;
  struct Edit *currentedit;
  int outchannel;
  int frame, totaltracks;
  struct TrackGroup *currentgroup;
  char *newgroup;

  if(!(in = fopen(filename, "rb"))){
    displayalert("Can't open input file.");
    return 1;
  }

  fseek(in, 0, SEEK_END);
  filelength = ftell(in);
  if(filetype == WAV){
    filelength -= WAVHEADERLENGTH;
    fseek(in, 22, SEEK_SET);
    newtracks = fgetc(in);
    fseek(in, 24, SEEK_SET);
    samplerate = getfourswap(in);
  }
  
  if(filetype == PCMHILO || filetype == PCMLOHI){
	  char *caption[] = {"Enter the number of channels."};
	  bctextbox textbox[1];
	  
	  sprintf(textbox[0].text, "%d", newtracks);
    if(getnumericinput(1, textbox, 0, caption, "Load PCM", 0))
      return 1;
      
    newtracks = atol(textbox[0].text);
    if(newtracks < 1) return 1;
  }
  
  frame = 2 * newtracks;
  filelength /= frame;
                                   /* set starting track */
  if(operation == LOAD){
    cleardatabase();
    clearlabels();
    destroyautomation(0);
  }
  
 	track1 = totaltracks = tracks();

  for(track = 0; track < newtracks; track++, totaltracks++){
    if(totaltracks == 0){
      currenttrack = firsttrack = lasttrack = malloc(sizeof(struct Track));
      currenttrack->prevtrack = currenttrack->nexttrack = 0;
    }else{
      currenttrack = lasttrack->nexttrack = malloc(sizeof(struct Track));
      currenttrack->prevtrack = lasttrack;
      lasttrack = currenttrack;
    }
                             /* set up edit data */
    currentedit = malloc(sizeof(struct Edit));
    currentedit->prevedit = currentedit->nextedit = 0;
    currentedit->start1 = 0;
    currentedit->start2 = 0;
    currentedit->length = filelength;
    currentedit->track = track;
    assigntext(&currentedit->file, filename);
    currentedit->channels = newtracks;
    currentedit->format = filetype;

                             /* set up track data */
    currenttrack->firstedit = currenttrack->lastedit = currentedit;
    currenttrack->record = 0;
    currenttrack->play = 1;
    currenttrack->level = 0;
    setpans(&currenttrack->pan, totaltracks);
    currenttrack->invert = 0;
  }
  lasttrack->nexttrack = firsttrack->prevtrack = 0;

  if(operation == LOAD) clearlabels();
  fclose(in);

  for(currentgroup = firsttrackgroup;                 /* update groups */
    currentgroup;
    currentgroup = currentgroup->nextgroup)
  {
    newgroup = malloc(totaltracks + newtracks);
    for(i = 0; i < totaltracks; i++)
      newgroup[i] = currentgroup->tracks[i]; /* copy old tracks */
    for(; i < totaltracks + newtracks; i++)
      newgroup[i] = 0;                       /* null new tracks */
    free(currentgroup->tracks);
    currentgroup->tracks = newgroup;
  }

  initautomation(track1, filelength);
  initeq(track1, newtracks);
  if(operation == LOAD){
    updatemaintitle(filename);
    changesmade = 0;
  }
  filelength = longesttrack();
  fit(0, filelength);

  return 0;
}

int loadcast(char filename[], char operation){
  int newtracks, newlabels, newchannels;
  int track, totaltracks, track1, i;
  FILE *in;
  long headerlength, filelength;
  struct Track *currenttrack;
  struct Edit *currentedit;
  long labelstart;
  struct TrackGroup *currentgroup;
  char *newgroup;

  if(!(in = fopen(filename, "rb"))){
    displayalert("Can't open input file.");
    return 1;
  }

  if(operation == LOAD){
    cleardatabase();
    clearlabels();
    destroyautomation(0);
  }
                                                /* skip file description */
  fseek(in, 4, SEEK_SET);
                                                         /* get # tracks */
  newtracks = gettwo(in);
                                         /* # of output channels in file */
  newchannels = gettwo(in);
  if(operation != IMPORT) outchannels = newchannels;

                                                      /* get main level */
  if(operation != IMPORT) mainlevel = getfloat(in);
    else fseek(in, 4, SEEK_CUR);
                                                    /* get sample rate */
  if(operation == LOAD) samplerate = getfour(in);
    else fseek(in, 4, SEEK_CUR);
                                                    /* get # of labels */
  newlabels = getfour(in);
                                                    /* get label start */
  labelstart = getfour(in);

  fseek(in, 20, SEEK_CUR);                         /* skip 20 bytes of padding */

  track1 = totaltracks = tracks();
  headerlength = CASTHEADLEN +
    newchannels * (14 + 4 * outchannels);

  for(track = 0; track < newtracks; track++, totaltracks++){
    if(totaltracks == 0){
      currenttrack = firsttrack = lasttrack = malloc(sizeof(struct Track));
      firsttrack->prevtrack = firsttrack->nexttrack = 0;
    }else{
      currenttrack = lasttrack->nexttrack = malloc(sizeof(struct Track));
      currenttrack->prevtrack = lasttrack;
      lasttrack = currenttrack;
    }
                             /* set up edit data */
    currentedit = malloc(sizeof(struct Edit));
    currentedit->prevedit = currentedit->nextedit = 0;
    currentedit->start1 = 0;
    currentedit->start2 = 0;
    fseek(in, 4, SEEK_CUR);                 /* start of track not used here */
    currentedit->length = getfour(in) / 2;
    currentedit->track = track;
    assigntext(&currentedit->file, filename);
    currentedit->channels = newtracks;
    currentedit->format = BCAST;

                             /* set up track data */
    currenttrack->record = fgetc(in);
    currenttrack->play = fgetc(in);
    currenttrack->level = getfloat(in);
    currenttrack->pan = malloc(4 * outchannels);

    for(i = 0; i < newchannels && i < outchannels; i++){      /* read new pans */
      currenttrack->pan[i] = getfloat(in);
    }
    fseek(in, (newchannels - i) * 4, SEEK_CUR);       /* skip extra newchannels */
    for(; i < outchannels; i++){                      /* fill extra outchannels */
      currenttrack->pan[i] = 0;
    }

    currenttrack->invert = fgetc(in);
    currenttrack->firstedit = currenttrack->lastedit = currentedit;
  }
  lasttrack->nexttrack = firsttrack->prevtrack = 0;

  if(operation == LOAD) clearlabels();
  fseek(in, labelstart, SEEK_SET);
                                         /* read in labels for file */
  for(i = 0; i < newlabels; i++){
    insertlabel(getfour(in));
  }
  readgroups(in, newchannels, track1);

  readautomation(in, track1, newtracks, newchannels);
  readeq(in, track1);

  optimizelabels();
  fclose(in);

  if(operation == LOAD){
    updatemaintitle(filename);
    changesmade = 0;
  }
  filelength = longesttrack();
  fit(0, filelength);

  return 0;
}

/* load waveform as a list of segments */

int loadedits(char *filename, char operation){
  FILE *in;
  int i;
  long filelength, x;
  int track, newtracks, totaltracks, track1, edit, newedits;
  int outchannel, newchannels;
  int newlabels;
  struct Edit *currentedit;
  struct Track *currenttrack;

  if(!(in = fopen(filename, "rb"))){
    displayalert("Can't open input file.");
    return 1;
  }

  if(operation == LOAD) strcpy(outfilename, filename);
  if(operation == UNDO || operation == LOAD){
    cleardatabase();
    clearlabels();
    destroyautomation(0);
  }


                                                /* file format description */
  fseek(in, 4, SEEK_SET);
                                                    /* # of tracks in file */
  newtracks = gettwo(in);
                                           /* # of output channels in file */
  newchannels = gettwo(in);
  if(operation != IMPORT) outchannels = newchannels;
                                 /* read in main level and sample rate */
  if(operation == LOAD | operation == UNDO){
    mainlevel = getfloat(in);
    samplerate = getfour(in);
  }else
    fseek(in, 8, SEEK_CUR);
                                                    /* # of labels in file */
  newlabels = getfour(in);

  fseek(in, 20, SEEK_CUR);                                 /* skip padding */

  track1 = totaltracks = tracks();

                                                        /* read in a track */
  for(track = track1; track < track1 + newtracks; track++, totaltracks++){
    if(track == 0){
      currenttrack = firsttrack = lasttrack =
        malloc(sizeof(struct Track));
      currenttrack->prevtrack = currenttrack->nexttrack = 0;
    }else{
      currenttrack = lasttrack->nexttrack = malloc(sizeof(struct Track));
      currenttrack->prevtrack = lasttrack;
      lasttrack = currenttrack;
      currenttrack->nexttrack = 0;
    }

                             /* set up edit data */

    newedits = gettwo(in);

    currenttrack->record = fgetc(in);
    currenttrack->play = fgetc(in);
    currenttrack->level = getfloat(in);

    currenttrack->pan = malloc(4 * outchannels);
    for(i = 0; i < outchannels && i < newchannels; i++){/* load newpans */
      currenttrack->pan[i] = getfloat(in);
    }
    fseek(in, (newchannels - i) * 4, SEEK_CUR);       /* skip extra newchannels */
    for(; i < outchannels; i++){                     /* fill extra outchannels */
      currenttrack->pan[i] = 0;
    }

    currenttrack->invert = fgetc(in);

    if(newedits) currentedit = currenttrack->firstedit =
      currenttrack->lastedit = malloc(sizeof(struct Edit));
    else
      currentedit = currenttrack->firstedit = currenttrack->lastedit = 0;
                              /* read in edits */
    for(edit = 0, x = 0; edit < newedits; edit++){
      if(edit > 0){
        currentedit = currenttrack->lastedit->nextedit =
          malloc(sizeof(struct Edit));
        currentedit->prevedit = currenttrack->lastedit;
        currenttrack->lastedit = currentedit;
      }

      currentedit->start1 = x;
      currentedit->start2 = getfour(in);
      currentedit->length = getfour(in);
      currentedit->track = gettwo(in);
      readtext(in, &currentedit->file);
      currentedit->channels = gettwo(in);
      currentedit->format = fgetc(in);
      x += currentedit->length;
    }
    if(newedits) currenttrack->lastedit->nextedit =
      currenttrack->firstedit->prevedit = 0;
  }

                           /* read in labels for file */

  for(i = 0; i < newlabels; i++){
    insertlabel(getfour(in));
  }

  readgroups(in, newchannels, track1);

  readautomation(in, track1, newtracks, newchannels);

  readeq(in, track1);
  fclose(in);

  optimizelabels();

  optimizeautomation(0);

  locatefilenames();
  if(operation == IMPORT || operation == LOAD){
    fit(0, longesttrack());
  }
                 /* optimize for failed file locations */
  optimizeedits();
  optimizeautomation(0);

  if(operation == LOAD){
    updatemaintitle(filename);
    changesmade = 0;
  }

  return 0;
}

/* save waveform as a list of segments */

int saveasedits(char filename[]){
  FILE *out;
  int i;
  XEvent report;
  int track, edit;
  int outchannel;
  struct Edit *currentedit;
  struct Track *currenttrack;
  struct Label *currentlabel;

  if(!(out = fopen(filename, "wb"))){
    displayalert("Can't open output file.");
    return 1;
  }
                                        /* write file description */
  fprintf(out, "EDIT");
                                                /* write # tracks */
  puttwo(tracks(), out);
                                  /* # of output channels in file */
  puttwo(outchannels, out);
                                              /* write main level */
  putfloat(mainlevel, out);
                                              /* write sample rate */
  putfour(samplerate, out);
                                             /* write # of labels */
  putfour(labels(), out);

  for(i = 0; i < 20; i++){
    fputc(0, out);             /* write 20 bytes of padding */
  }

  currenttrack = firsttrack;
                                  /* write each track */
  for(currenttrack = firsttrack; currenttrack != 0; currenttrack =
    currenttrack->nexttrack)
  {
                                  /* write # of edits in this track */
    puttwo(edits(currenttrack), out);
    fputc(currenttrack->record, out);             /* write record permission */
    fputc(currenttrack->play, out);               /* write play permission */
    putfloat(currenttrack->level, out); /* write level */
    for(i = 0; i < outchannels; i++)
      putfloat(currenttrack->pan[i], out);
                                                         /* write pans */
    fputc(currenttrack->invert, out);
                                  /* write edits */
    for(currentedit = currenttrack->firstedit;
      currentedit != 0; currentedit = currentedit->nextedit)
    {
      putfour(currentedit->start2, out);
      putfour(currentedit->length, out);
      puttwo(currentedit->track, out);
      fprintf(out, "%s%c", currentedit->file, 0);
      puttwo(currentedit->channels, out);
      fputc(currentedit->format, out);
    }
  }

                                  /* write labels for file */
  for(currentlabel = firstlabel; currentlabel; currentlabel = currentlabel->nextlabel){
    putfour(currentlabel->position, out);
  }
  writegroups(out);
  writeautomation(out);
  writeeq(out);

  fclose(out);
  return 0;
}

                              /* write header for a WAV file */
void writeheader(FILE *out, long filelength, char channels){
  char frame = channels * 2;

  fprintf(out, "RIFF");
  putfourswap(filelength, out);
  fprintf(out, "WAVEfmt ");
  fputc(16, out);
  fputc(0, out);
  fputc(0, out);
  fputc(0, out);
  fputc(1, out);
  fputc(0, out);
  fputc(channels, out);
  fputc(0, out);
  putfourswap(samplerate, out);
  putfourswap(samplerate * frame, out);
  fputc(frame, out);
  fputc(0, out);
  fputc(16, out);
  fputc(0, out);
  fprintf(out, "data");
  putfourswap(filelength * frame - WAVHEADERLENGTH, out);
}

/* play waveform as a wav */
int playaswave(char filename[], char filetype){
  FILE *out;
  long filelength;
  char *buffer, indexfile[256], string[256], string_[256];
  long x;
  long bufferlength = SAVEBUFFER;
  long frame;

  frame = outchannels * 2;
  bufferlength /= frame;
  bufferlength *= frame;

  if(!(buffer = malloc(bufferlength))){
    displayalert("Not enough memory to open save buffer.");
    return 1;
  }
  if(!(out = fopen(filename, "wb"))){
    displayalert("Can't open output file saving.");
    free(buffer);
    return 1;
  }

  if(startmixdown(bufferlength, outchannels)){
    free(buffer);
    fclose(out);
    return 1;
  }

  sprintf(indexfile, "%s%s", filename, ".idx");
  remove(indexfile);

  cutfilename(string_, filename);
  sprintf(string, "Saving %s", string_);
  initializeprogress(string);
  filelength = longestplayabletrack();
  if(filetype == WAV) writeheader(out, filelength, outchannels);

  for(x=0; x < filelength; x += bufferlength/frame){
    if(filelength - x < bufferlength / frame)
      bufferlength = (filelength - x) * frame;
    if(mixdown(buffer, bufferlength, x)){
      free(buffer);
      fclose(out);
      destroyprogress();
      killmixdown();
      return 1;
    }

    if((filetype == WAV && !byteorder) ||      /* swap bytes for output format */
      (filetype == PCMHILO && byteorder) ||
      (filetype == PCMLOHI && !byteorder))
      swapbytes(buffer, bufferlength);

    if(!fwrite(buffer, bufferlength, 1, out)){
      printf("Disk error.\n");
      free(buffer);
      fclose(out);
      destroyprogress();
      killmixdown();
      return 1;
    }
    if(updateprogress(x, filelength)){
      free(buffer);
      fclose(out);
      destroyprogress();
      killmixdown();
      return 1;
    }
  }

  destroyprogress();
  free(buffer);
  fclose(out);
  killmixdown();
  return 0;
}

/* save as a BCAST file format */
int saveascast(char filename[]){
  FILE *out;
  char *buffer, indexfile[256], string[256], string_[256];
  long x, totalx = 0, currentlength, totallength = 0;
  long bufferlength = SAVEBUFFER, bufferlength_;
  struct Track *currenttrack;
  struct Label *currentlabel;
  int i, totaltracks, track;
  struct CastHeader *castheader;
  long labelstart;

  bufferlength /= 2;
  bufferlength *= 2;

  if(!(buffer = malloc(bufferlength))){
    displayalert("Not enough memory to save.");
    return 1;
  }

  if(!(out = fopen(filename, "wb"))){
    displayalert("Can't open output file for saving.");
    free(buffer);
    return 1;
  }

  if(startbuffering()){
    free(buffer);
    fclose(out);
    return 1;
  }

  sprintf(indexfile, "%s%s", filename, ".idx");
  remove(indexfile);

  cutfilename(string_, filename);
  sprintf(string, "Saving %s", string_);
  initializeprogress(string);

  totaltracks = tracks();
                                                /* write file description */
  fprintf(out, "CAST");
                                                        /* write # tracks */
  puttwo(totaltracks, out);
                                          /* # of output channels in file */
  puttwo(outchannels, out);
                                                      /* write main level */
  putfloat(mainlevel, out);
                                                     /* write sample rate */
  putfour(samplerate, out);
                                                     /* write # of labels */
  putfour(labels(), out);

  for(i = 0; i < 4; i++) fputc(0, out);               /* skip label start */

  for(i = 0; i < 20; i++) fputc(0, out);     /* write 20 bytes of padding */
  for(i = 0; i < totaltracks * (15 + 4 * outchannels); i++)
    fputc(0, out);                         /* skip track list */

  castheader = malloc(sizeof(struct CastHeader) * totaltracks);

                                                 /* get the total length */
  for(currenttrack = firsttrack; currenttrack; currenttrack =
    currenttrack->nexttrack)
  {
    totallength += currenttrack->lastedit->start1 +
      currenttrack->lastedit->length;
  }
                                                  /* write one track at a time */
  for(currenttrack = firsttrack, track = 0;
    currenttrack; currenttrack = currenttrack->nexttrack, track++)
  {
    castheader[track].start = ftell(out);
    currentlength = (currenttrack->lastedit->start1 + currenttrack->lastedit->length);
    bufferlength_ = bufferlength;

    for(x = 0; x < currentlength;
      x += bufferlength_ / 2, totalx += bufferlength_ / 2)
    {
      if(x + bufferlength_ / 2 > currentlength)
        bufferlength_ = (currentlength - x) * 2;

      readbuffer(buffer, bufferlength_, currenttrack, x);
      if(!fwrite(buffer, bufferlength_, 1, out)){
        printf("Disk error.\n");
        free(castheader);
        free(buffer);
        fclose(out);
        destroyprogress();
        killbuffering();
        return 1;
      }
      if(updateprogress(totalx, totallength)){
        free(castheader);
        free(buffer);
        fclose(out);
        remove(filename);
        destroyprogress();
        killbuffering();
        return 1;
      }
    }
    castheader[track].length = ftell(out) - castheader[track].start;
  }
  fflush(out);
  labelstart = ftell(out);

                                  /* write labels for file */
  for(currentlabel = firstlabel; currentlabel; currentlabel = currentlabel->nextlabel){
    putfour(currentlabel->position, out);
  }

  writegroups(out);               /* write track groups */
  writeautomation(out);           /* write automation */
  writeeq(out);                   /* write eq */

  fseek(out, 20, SEEK_SET);                            /* write header */
  putfour(labelstart, out);
  fseek(out, 44, SEEK_SET);                            /* write header */
  for(currenttrack = firsttrack, track = 0;
    track < totaltracks && currenttrack;
    track++, currenttrack = currenttrack->nexttrack)
  {
    putfour(castheader[track].start, out);                /* put start of track */
    putfour(castheader[track].length, out);              /* put length of track */

    fputc(currenttrack->record, out);                     /* write record permission */
    fputc(currenttrack->play, out);                         /* write play permission */
    putfloat(currenttrack->level, out);              /* write level */
    writepans(currenttrack->pan, out, outchannels);
    fputc(currenttrack->invert, out);
  }
  destroyprogress();
  free(castheader);
  free(buffer);
  fclose(out);
  killbuffering();
  return 0;
}

int playasfile(){
  char string[256];
  char string_[256];
  XEvent report;
  FILE *out;

  if(getfileattributes(outfilename, &playasfiletype, PLAYASFILE, "Play as file")) return 1;

  if(out = fopen(outfilename, "rb")){
    fclose(out);
      cutfilename(string_, outfilename);
      sprintf(string, "%s exists.  Overwrite?", string_);
      if(confirm(string, "Save")) return 1;
  }

  switch(playasfiletype){
    case PCMHILO:
      if(playaswave(outfilename, PCMHILO)) return 1;
    break;

    case PCMLOHI:
      if(playaswave(outfilename, PCMLOHI)) return 1;
    break;

    case WAV:
      if(playaswave(outfilename, WAV)) return 1;
    break;
  }

  /* changesmade = 0; */
  updatemaintitle(outfilename);
  return 0;
}


int saveasfile(){
  char string[256];
  char string_[256];
  static char filetype = EDITS;
  XEvent report;
  FILE *out;

  if(getfileattributes(outfilename, &filetype, SAVEASFILE, "Save as file")) return 1;

  if(out = fopen(outfilename, "rb")){
    fclose(out);
    cutfilename(string_, outfilename);
    sprintf(string, "%s exists.  Overwrite?", string_);
    if(confirm(string, "Save")) return 1;
  }

  switch(filetype){
    case BCAST:
      if(saveascast(outfilename)) return 1;
    break;

    case EDITS:
      if(saveasedits(outfilename)) return 1;
    break;
  }

  changesmade = 0;
  updatemaintitle(outfilename);
  refreshdisplay();
  return 0;
}

int saveandexit(){
  if(changesmade)
    if(confirmdel("Save edit list before exiting ( Answering ""No"" will destroy changes ) ?"))
      return 1;

  saveinitialvariables();
  exit(0);
}

int createnewfile(){
  if(changesmade)
    if(confirmdel("Save edit list before deleting ( Answering ""No"" will destroy changes ) ?"))
      return 1;

  destroyautomation(0);
  cleardatabase();
  clearlabels();
  updatemaintitle(" ");
  mainlevel = 1;
  startx = 0;
  starty = 0;
  consolestarty = 0;
  xscale = 1;
  selectionstart = 0;
  selectionend = 0;
  changesmade = 0;
  fittime();

  drawwave(0, 0, wavewindow.w, wavewindow.h, startx, starty, NO_BLANK, FLASH,
    DRAWCURSOR);
  drawpatches(starty);
  drawlabels(startx);
  drawtime(startx);
  drawmenubar(-1);
  drawplaycontrols();
  drawconsolecontrols(consolestarty);
  updatexscrollbar(wavscroll.xpixmap, wavscroll.xwin, startx,
    wavewindow.w * xscale, longesttrack(), &wavscroll.x);
  updateyscrollbar(wavscroll.ypixmap, wavscroll.ywin, starty,
    wavewindow.h / yscaletrack, tracks(), &wavscroll.y);
  return 0;
}

/* import a file */
int importfile(){
  char filetype[5];
  FILE *in;
  int i;

  if(getfileattributes(infilename, &loadasfiletype, LOAD, "Import File")) return
  1;

  if(in = fopen(infilename, "rb")){
  	updateundo(LOAD);
  	changesmade = 1;
  	
    for(i = 0; i < 4; i++){
      filetype[i] = fgetc(in);
    }
    filetype[i] = 0;
    fclose(in);
                      /* determine file type */
                      /* default to PCM if no file type can be determined */
    if(!(strcmp(filetype, "RIFF"))){
      if(loadwave(infilename, WAV, IMPORT)) return 1;
    }else if(!(strcmp(filetype, "EDIT"))){
      if(loadedits(infilename, IMPORT)) return 1;
      }else if(!(strcmp(filetype, "CAST"))){
        if(loadcast(infilename, IMPORT)) return 1;
      }else{
        if(loadwave(infilename, loadasfiletype, IMPORT)) return 1;
        }
  }
  drawpatches(starty);
  drawlabels(startx);
  drawtime(startx);
  drawplaycontrols();
  drawconsolecontrols();
  updatexscrollbar(wavscroll.xpixmap, wavscroll.xwin, startx,
    wavewindow.w * xscale, longesttrack(), &wavscroll.x);
  updateyscrollbar(wavscroll.ypixmap, wavscroll.ywin, starty,
    wavewindow.h / yscaletrack, tracks(), &wavscroll.y);
  changesmade = 1;
}

int loadfile(){
  char filetype[5];
  FILE *in;
  int i;

  if(getfileattributes(infilename, &loadasfiletype, LOAD, "Load File")) return 1;

  if(createnewfile()) return 1;

  if(in = fopen(infilename, "rb")){
  	updateundo(LOAD);
    for(i = 0; i < 4; i++){
      filetype[i] = fgetc(in);
    }
    filetype[i] = 0;
    fclose(in);
                      /* determine file type */
                      /* default to PCM if no file type can be determined */
    if(!(strcmp(filetype, "RIFF"))){
      if(loadwave(infilename, WAV, LOAD)) return 1;
    }else if(!(strcmp(filetype, "EDIT"))){
      if(loadedits(infilename, LOAD)) return 1;
      }else if(!(strcmp(filetype, "CAST"))){
        if(loadcast(infilename, LOAD)) return 1;
      }else{
        if(loadwave(infilename, loadasfiletype, LOAD)) return 1;
        }
  }

  drawpatches(starty);
  drawlabels(startx);
  drawtime(startx);
  drawplaycontrols();
  drawconsolecontrols(consolestarty);
  updatexscrollbar(wavscroll.xpixmap, wavscroll.xwin, startx,
    wavewindow.w * xscale, longesttrack(), &wavscroll.x);
  updateyscrollbar(wavscroll.ypixmap, wavscroll.ywin, starty,
    wavewindow.h / yscaletrack, tracks(), &wavscroll.y);

  changesmade = 0;
}

void loadinitloaddir(FILE *in){
  fscanf(in, "%s", infilename);
}

void loadinitsavedir(FILE *in){
  fscanf(in, "%s", outfilename);
}

void loadinitplaytype(long value){
  playasfiletype = value;
}

void loadinitloadtype(long value){
  loadasfiletype = value;
}

void saveinitloaddir(FILE *out){
  char string[256];

  cutdirectory(string, infilename);
  if(strlen(string)) fprintf(out, "LOADDIR %s\n", string);
}

void saveinitsavedir(FILE *out){
  char string[256];

  cutdirectory(string, outfilename);
  /* only save if a string exists since fscanf would skip the next line */
  if(strlen(string)) fprintf(out, "SAVEDIR %s\n", string);
}

long saveinitplaytype(){
  return playasfiletype;
}

long saveinitloadtype(){
  return loadasfiletype;
}
