/*
  Broadcast 2.0 multitrack audio editing
  (c) 1997 Adam Williams

  This program is distributed with the intent that it will be useful, without
  any warranty.  The code is being updated and has many bugs which are
  constantly being elucidated so changes should not be made without notifying
  the author.
*/

/* fileattr.c      interface for getting filename from user  */

#include <pwd.h>
#include <stddef.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <dirent.h>
#include <unistd.h>

#include "interfac.h"
#include "fileattr.h"
#include "graphics.h"
#include "record.h"

struct DirEntry {
  char *name;
  int directory;
};

bctextbox filenamebox = {
  5, 310, 350, 30,
  ""
};

/* 1024 possible filenames in a directory */
struct DirEntry dirlist[1024];
int totaldirentries;
int selectedfile;
static char workdir[256] = "";

Window filewin;
Pixmap filepixmap;

struct filewindow_{
  int w;
  int h;
  char *title;
  bcbutton mic;
  bcbutton line;
  bcbutton wave;
  bcbutton pcmhilo;
  bcbutton pcmlohi;
  bcbutton edits;
  bcbutton bcast;
  bcbutton ok;
  bcbutton cancel;
} filewindow;

struct filewindow_ filewindow = {
  418, 390, "File Properties",
  {330, 40, 16, 16, "MIC"},
  {330, 80, 16, 16, "LINE"},
  {330, 150, 16, 16, "WAV"},
  {330, 180, 16, 16, "PCMHILO"},
  {330, 210, 16, 16, "PCMLOHI"},
  {330, 240, 16, 16, "EDITS"},
  {330, 270, 16, 16, "BCAST"},
  {10, 350, 100, 30, "OK"},
  {300, 350, 100, 30, "Cancel"}
};

bcscroll dirscroll = {295, 25, 12, 265, 0, 0, 0, 0};    /* dirscroll     */
bcbox dirwindow = {5, 25, 290, 265};              /* directory     */

/* draw the input source for record operations */
void inputselect(){
  char *string;

  string = "Input";
  XSetForeground(display, gc, black.pixel);
  XDrawString(display, filepixmap, gc, filewindow.mic.x, filewindow.mic.y - 20, string,
  strlen(string));
  XDrawString(display, filepixmap, gc, filewindow.mic.x, filewindow.mic.y,
    filewindow.mic.text, strlen(filewindow.mic.text));
  XDrawString(display, filepixmap, gc, filewindow.line.x, filewindow.line.y,
    filewindow.line.text, strlen(filewindow.line.text));
  DrawUpRadial(filepixmap, &filewindow.mic);
  DrawUpRadial(filepixmap, &filewindow.line);
  switch(recevent.audioinput){
    case MIC:           DrawDnRadial(filepixmap, &filewindow.mic);       break;
    case LINE:          DrawDnRadial(filepixmap, &filewindow.line);      break;
  }
  XCopyArea(display, filepixmap, filewin, gc, filewindow.mic.x, filewindow.mic.y-40, 100,
    150, filewindow.mic.x, filewindow.mic.y-40);
  XFlush(display);
}

/* draw the file format selection for save operations */
void fileformat(char filetype, char operation){
  char *string;

                            /* file format area */
   XSetForeground(display, gc, black.pixel);
  string = "File format";
  XDrawString(display, filepixmap, gc, filewindow.wave.x, filewindow.wave.y - 20,
  string, strlen(string));

  switch(operation){
    case LOAD:
      XDrawString(display, filepixmap, gc, filewindow.pcmhilo.x, filewindow.pcmhilo.y,
        filewindow.pcmhilo.text, strlen(filewindow.pcmhilo.text));
      XDrawString(display, filepixmap, gc, filewindow.pcmlohi.x, filewindow.pcmlohi.y,
        filewindow.pcmlohi.text, strlen(filewindow.pcmlohi.text));
      DrawUpRadial(filepixmap, &filewindow.pcmhilo);
      DrawUpRadial(filepixmap, &filewindow.pcmlohi);
      break;

    case PLAYASFILE:
      XDrawString(display, filepixmap, gc, filewindow.wave.x, filewindow.wave.y,
        filewindow.wave.text, strlen(filewindow.wave.text));
      XDrawString(display, filepixmap, gc, filewindow.pcmhilo.x, filewindow.pcmhilo.y,
        filewindow.pcmhilo.text, strlen(filewindow.pcmhilo.text));
      XDrawString(display, filepixmap, gc, filewindow.pcmlohi.x, filewindow.pcmlohi.y,
        filewindow.pcmlohi.text, strlen(filewindow.pcmlohi.text));
      DrawUpRadial(filepixmap, &filewindow.wave);
      DrawUpRadial(filepixmap, &filewindow.pcmhilo);
      DrawUpRadial(filepixmap, &filewindow.pcmlohi);
      break;

    case SAVEASFILE:
      XDrawString(display, filepixmap, gc, filewindow.edits.x, filewindow.edits.y,
        filewindow.edits.text, strlen(filewindow.edits.text));
      XDrawString(display, filepixmap, gc, filewindow.bcast.x, filewindow.bcast.y,
        filewindow.bcast.text, strlen(filewindow.bcast.text));
      DrawUpRadial(filepixmap, &filewindow.edits);
      DrawUpRadial(filepixmap, &filewindow.bcast);
      break;

    case RECORD:
      XDrawString(display, filepixmap, gc, filewindow.wave.x, filewindow.wave.y,
        filewindow.wave.text, strlen(filewindow.wave.text));
      XDrawString(display, filepixmap, gc, filewindow.pcmhilo.x, filewindow.pcmhilo.y,
        filewindow.pcmhilo.text, strlen(filewindow.pcmhilo.text));
      XDrawString(display, filepixmap, gc, filewindow.pcmlohi.x, filewindow.pcmlohi.y,
        filewindow.pcmlohi.text, strlen(filewindow.pcmlohi.text));
      DrawUpRadial(filepixmap, &filewindow.wave);
      DrawUpRadial(filepixmap, &filewindow.pcmhilo);
      DrawUpRadial(filepixmap, &filewindow.pcmlohi);
      break;

    case EFFECT:
      XDrawString(display, filepixmap, gc, filewindow.wave.x, filewindow.wave.y,
        filewindow.wave.text, strlen(filewindow.wave.text));
      XDrawString(display, filepixmap, gc, filewindow.pcmhilo.x, filewindow.pcmhilo.y,
        filewindow.pcmhilo.text, strlen(filewindow.pcmhilo.text));
      XDrawString(display, filepixmap, gc, filewindow.pcmlohi.x, filewindow.pcmlohi.y,
        filewindow.pcmlohi.text, strlen(filewindow.pcmlohi.text));
      XDrawString(display, filepixmap, gc, filewindow.bcast.x, filewindow.bcast.y,
        filewindow.bcast.text, strlen(filewindow.bcast.text));
      DrawUpRadial(filepixmap, &filewindow.wave);
      DrawUpRadial(filepixmap, &filewindow.pcmhilo);
      DrawUpRadial(filepixmap, &filewindow.pcmlohi);
      DrawUpRadial(filepixmap, &filewindow.bcast);
      break;
  }

  switch(filetype){
    case WAV:           DrawDnRadial(filepixmap, &filewindow.wave);       break;
    case PCMHILO:       DrawDnRadial(filepixmap, &filewindow.pcmhilo);    break;
    case PCMLOHI:       DrawDnRadial(filepixmap, &filewindow.pcmlohi);    break;
    case EDITS:         DrawDnRadial(filepixmap, &filewindow.edits);      break;
    case BCAST:         DrawDnRadial(filepixmap, &filewindow.bcast);      break;
  }
  XCopyArea(display, filepixmap, filewin, gc, 0, 0,
    filewindow.w, filewindow.h, 0, 0);
  XFlush(display);
}

/* return 1 if filename is a directory or 0 if not */
int isdirectory(char *filename){
  struct stat ostat;

  if(!stat(filename, &ostat))
    if(S_ISDIR(ostat.st_mode)){
      return 1;
    }
  return 0;
}

/* store filename part of filename in output */
int cutfilename(char *output, char *input){
  struct stat ostat;
  static char *input_;

  if(!strlen(input)){           /* if null filename */
    strcpy(output, "");
    return 0;
  }

                     /* is filename itself a directory */
  if(!stat(input, &ostat)){
    if(S_ISDIR(ostat.st_mode)){
      strcpy(output, "");
      return 0;
    }
  }

                 /* if not, cut the part that isn't */
  input_ = input + strlen(input);
  while(input_ >= input && *input_-- != '/')
    ;
  input_++;

  if(*input_ == '/') input_++;
  strcpy(output, input_);
}

/* store the directory part of a filename in directory */
int cutdirectory(char *directory, char *filename){
  char *filename_, *filename__, *directory_;
  struct stat ostat;
  int len;

  if(!strlen(filename)){           /* if null filename */
    strcpy(directory, "");
    return 0;
  }

                          /* see if filename itself is the directory */
  len = strlen(filename) - 1;
  if(!stat(filename, &ostat))
    if(S_ISDIR(ostat.st_mode)){
      strcpy(directory, filename);
      if(directory[len] != '/') strcat(directory, "/");
      return 0;
    }

  filename_ = &filename[len];
  while(filename_ >= filename && *filename_-- != '/')
    ;
  filename_++;
                             /* copy directory out of filename */
  directory_ = directory;
  filename__ = filename;
  while(filename__ != filename_)
    *directory_++ = *filename__++;
  *directory_ = 0;

  len = strlen(directory);
  if(len && directory[len] != '/') strcat(directory, "/");
}

void destroydirectory(){
  int i;

  for(i = 0; i < totaldirentries; i++){
    free(dirlist[i].name);
  }

  totaldirentries = 0;
}

void cleardirbox(int flash){
                                               /* clear area */
  XSetForeground(display, gc, grey.pixel);
  XFillRectangle(display, filepixmap, gc, dirwindow.x, dirwindow.y - 20,
    300, 20);
                                               /* clear directory box */
  XSetForeground(display, gc, white.pixel);
  XFillRectangle(display, filepixmap, gc, dirwindow.x, dirwindow.y,
    dirwindow.w, dirwindow.h);
  XSetForeground(display, gc, black.pixel);
  XDrawRectangle(display, filepixmap, gc, dirwindow.x, dirwindow.y,
    dirwindow.w, dirwindow.h);

  if(flash){
    XCopyArea(display, filepixmap, filewin, gc, dirwindow.x, dirwindow.y - 20,
      dirwindow.w, dirwindow.h + 20, dirwindow.x, dirwindow.y - 20);
    XFlush(display);
  }
}

void drawdirectory(){
  static int i, j, y;
  static char string[256];

  cleardirbox(0);
                                               /* filenames */
  for(y = dirwindow.y + 17, i = dirscroll.position;
    i < totaldirentries && y < dirwindow.y + dirwindow.h; i++, y += 20)
  {
    XDrawString(display, filepixmap, gc, dirwindow.x + 5, y,
      dirlist[i].name, strlen(dirlist[i].name));
    if(i == selectedfile){
      XSetForeground(display, gc, white.pixel);
      XSetFunction(display, gc, GXxor);
      XFillRectangle(display, filepixmap, gc, dirwindow.x, y - 16,
        dirwindow.w, 20);
      XSetFunction(display, gc, GXcopy);
      XSetForeground(display, gc, black.pixel);
    }
  }
                                               /* working directory */
  XSetForeground(display, gc, black.pixel);

  truncatefilename(string, workdir);
  XDrawString(display, filepixmap, gc, dirwindow.x + 5, dirwindow.y - 4,
    string, strlen(string));

  XCopyArea(display, filepixmap, filewin, gc, dirwindow.x, dirwindow.y - 20,
    dirwindow.w, dirwindow.h + 20, dirwindow.x, dirwindow.y - 20);
  XFlush(display);
}

int comparedir(const struct DirEntry *c1, const struct DirEntry *c2)
{
  return strcmp (c1->name, c2->name);
}

/* return 1 if the filename is hidden */
int ishidden(char *filename){
  if((strcmp(filename, "..") && strcmp(filename, ".")) | !strcmp(workdir, "/")){
    if(filename[0] == '.') return 1;
  }
  return 0;
}

int loaddirectory(){
  DIR *dirstream;
  struct dirent *filename_;
  struct stat ostat;
  int i;
  char string[256];
  cleardirbox(1);
                                              /* erase old directory */
  for(i = 0; i < totaldirentries; i++){
    free(dirlist[i].name);
  }

  dirstream = opendir(workdir);                     /* read directory */
  if (! dirstream){
    printf("Couldn't open the directory.\n");
    return 1;
  }

  totaldirentries = 0;               /* get totaldirentries and filenames */
  while (filename_ = readdir (dirstream)){
    if(!ishidden(filename_->d_name)){/* add 2 in case it is a directory */
      dirlist[totaldirentries].name = malloc(strlen(filename_->d_name) + 2);
      strcpy(dirlist[totaldirentries].name, filename_->d_name);
      strcpy(string, workdir);
      strcat(string, dirlist[totaldirentries].name);

      if(!stat(string, &ostat))
        dirlist[totaldirentries].directory = S_ISDIR(ostat.st_mode);
      totaldirentries++;
    }
  }

  closedir(dirstream);

  qsort(dirlist, totaldirentries, sizeof(struct DirEntry), (void *)comparedir);

  for(i = 0; i < totaldirentries; i++){              /* print directory */
    if(dirlist[i].directory) strcat(dirlist[i].name, "/");
  }

  updateyscrollbar(filepixmap, filewin, 0, dirwindow.h / 20, totaldirentries,
    &dirscroll);
  selectedfile = -1;
  drawdirectory();
  return 0;
}

void directoryevent(XEvent *report)
{
  static int y;

  y = report->xbutton.y;
                                               /* locate specific file */
  selectedfile = dirscroll.position + (y - dirwindow.y) / 20;
  if(selectedfile >= totaldirentries) selectedfile = totaldirentries-1;
  strcpy(filenamebox.text, dirlist[selectedfile].name);
  drawdirectory();

  do{
    XNextEvent(display, report);
  }while(report->type != ButtonRelease);

  activate_box(filepixmap, filewin, &filenamebox);
}

void directorysearch(char *filename){
  int i;

  for(i = 0; i < totaldirentries; i++){
    if(strcmp(filename, dirlist[i].name) > 0){
      selectedfile = i+1;
    }
  }

  updateyscrollbar(filepixmap, filewin, selectedfile, dirwindow.h / 20,
    totaldirentries, &dirscroll);
  drawdirectory();
}

/* test filename to see if it's a directory */
/* return 1 if filename is a directory */
int directorychange(){
  int len;
  struct stat ostat;
  char string[256];
  uid_t uid;
  struct passwd *pw;
  int i;

  len = strlen(filenamebox.text);
  
  if(len == 0) return 0;       /* filename is blank */  

  for(i = 0; i < len; i++){    /* parse tildas */
    if(filenamebox.text[i] == '~'){
      uid = getuid();             /* get user id */
      pw = getpwuid(uid);         /* get info for user */
      strcpy(string, &filenamebox.text[i+1]); /* print user directory at location */
      sprintf(&filenamebox.text[i], "%s%s", pw->pw_dir, string);
      activate_box(filepixmap, filewin, &filenamebox);
      break;
    }
  }
  len = strlen(filenamebox.text) - 1;

  if(filenamebox.text[0] == '/'){        /* filename starts with root directory */
    if(!isdirectory(filenamebox.text)) return 0;
    if(filenamebox.text[len] != '/') strcat(filenamebox.text, "/");
    strcpy(workdir, filenamebox.text);
    destroydirectory();
    loaddirectory();
    filenamebox.text[0] = '\0';
    activate_box(filepixmap, filewin, &filenamebox);
    return 1;
  }

  if(!strcmp(filenamebox.text, "../") || !strcmp(filenamebox.text, "..")){  /* filename is previous directory */
    if(strcmp(workdir, "/")){   /* not in root directory so move up */
                                /* by truncating directory */
      char *workdir_ = workdir + strlen(workdir) - 2;
      while(workdir_ != workdir && *workdir_ != '/')
        *workdir_-- = '\0';

      destroydirectory();
      loaddirectory();
      filenamebox.text[0] = '\0';
      deactivate_box(filepixmap, filewin, &filenamebox);
      activate_box(filepixmap, filewin, &filenamebox);
      return 1;
    }else{
      strcpy(filenamebox.text, "");
      activate_box(filepixmap, filewin, &filenamebox);
      return 1;
    }
  }

  if(!strcmp(filenamebox.text, "./")){
    return 1;
  }

  sprintf(string, "%s%s", workdir, filenamebox.text);

  if(isdirectory(string)){    /* filename is a branch of current directory */
    strcat(workdir, filenamebox.text);
    if(filenamebox.text[len] != '/') strcat(workdir, "/");

    destroydirectory();
    loaddirectory();
    filenamebox.text[0] = '\0';
    activate_box(filepixmap, filewin, &filenamebox);
    return 1;
  }
  return 0;
}

/* prepare filename for output */
/* return 1 if no filename was given */
/* return 0 if a filename was given */
int endfileattributes(char *oldfilename)
{
  destroydirectory();
  XDestroyWindow(display, filewin);
  XFreePixmap(display, filepixmap);

                      /* filename is given */
  if(strlen(filenamebox.text)){
                      /* if filename doesn't start at root */
                      /* append filename to working directory */
    if(filenamebox.text[0] != '/'){
      sprintf(oldfilename, "%s%s", workdir, filenamebox.text);
    }else
      strcpy(oldfilename, filenamebox.text);
    return 0;
  }
  return 1;       /* no filename was given */
}

void dirmoveup(int number){
  dirscroll.position -= number;
  if(dirscroll.position < 0) dirscroll.position = 0;
  updateyscrollbar(filepixmap, filewin, dirscroll.position,
    dirwindow.h / 20, totaldirentries, &dirscroll);
  drawdirectory();
}

void dirmovedown(int number){
  dirscroll.position += number;

  if(dirscroll.position > totaldirentries - dirscroll.span)
    dirscroll.position = totaldirentries - dirscroll.span;
  updateyscrollbar(filepixmap, filewin, dirscroll.position,
    dirwindow.h / 20, totaldirentries, &dirscroll);
  drawdirectory();
}

void dirscrollevent(XEvent *report){
  switch(dirscroll.event){
    case SPAN:
      yscrollevent(filepixmap, filewin, report, &dirscroll);
      drawdirectory();
      break;
    case ARROW1:
      dirmoveup(1);
      break;
    case ARROW2:
      dirmovedown(1);
      break;
    case UPPAGE:
      dirmoveup(dirwindow.h / 20);

      break;
    case DNPAGE:
      dirmovedown(dirwindow.h / 20);

      break;
  }
}

/* get file attributes for all file management operations */
/* return 1 if cancel */
int getfileattributes(char *oldfilename, char *filetype,
  char operation, char *title)
{
  XEvent report, nextreport;
  Visual *vis;
  XVisualInfo visinfo;
  XSetWindowAttributes attr;
  Cursor arrow;
  unsigned long mask;
  KeySym keysym;
  XComposeStatus composestatus;
  XSizeHints size_hints;
  char keys_return[2];
  long time_ = 0;       /*time of last buttonpress */
  char string[256], filename[256];

  int x, y;

                                   /* open save window */
  arrow = XCreateFontCursor(display, XC_top_left_arrow);
  vis = DefaultVisual(display, screen);

  mask = CWEventMask | CWBackPixel | CWBorderPixel | CWColormap |
    CWCursor;
  attr.event_mask = ExposureMask | ButtonPressMask |
    ButtonReleaseMask | KeyPressMask | PointerMotionMask;
  attr.background_pixel = grey.pixel;
  attr.border_pixel = dkgrey.pixel;
  attr.colormap = cmap;
  attr.cursor = arrow;

  filewin = XCreateWindow(display, rootwin, 0, 0, filewindow.w, filewindow.h,
    0, depth, InputOutput, vis, mask, &attr);

  filepixmap =   XCreatePixmap(display, filewin,
    filewindow.w, filewindow.h, depth);

  size_hints.flags = PSize;
  size_hints.width = filewindow.w;
  size_hints.height = filewindow.h;

  XSetStandardProperties(display, filewin, "File", "File", None, 0, 0,
    &size_hints);

  XMapWindow(display, filewin);

  XSetForeground(display, gc, grey.pixel);
  XFillRectangle(display, filepixmap, gc, 0, 0, filewindow.w, filewindow.h);

                                /* draw file format */
  if(operation != INPUT) fileformat(*filetype, operation);
    
#ifndef _LINUX_                  /* recommend third party mixer */
  if(operation == RECORD){      /* draw input select */
    inputselect();
  }
#endif

  strcpy(filename, oldfilename);
                  /* get working directory from filename if */
                  /* filename is not a ../ or ./            */
                  /* filename has a length                  */
                  /* filename has a working directory or is a working directory */
  if(strlen(filename) | strcmp("../", filename) | strcmp("./", filename))
    cutdirectory(workdir, filename);
                  /* otherwise use existing working directory */

                  /* if existing working directory doesn't exist or
                     it isn't a directory, get working directory */
  if(!strlen(workdir) | !isdirectory(workdir)){
    getcwd(workdir, 256);
    strcat(workdir, "/");
  }

  totaldirentries = 0;          /* load starting directory */
  loaddirectory(workdir);

  cutfilename(filenamebox.text, filename);
  activate_box(filepixmap, filewin, &filenamebox);       /* draw filename box */

                                /* buttons */
  DrawUpButton(filepixmap, filewin, &filewindow.ok);
  DrawUpButton(filepixmap, filewin, &filewindow.cancel);

  dirscroll.event = 0;           /* reset events */

  XDrawString(display, filepixmap, gc, filenamebox.x + 5, filenamebox.y - 2,
    title, strlen(title));

  XCopyArea(display, filepixmap, filewin, gc, 0, 0, filewindow.w,
  filewindow.h, 0, 0);
  XFlush(display);
                                /* event handler */
  do{
                     /* this event reader works best */
    do{
      XNextEvent(display, &report);
      if(XPending(display)) XPeekEvent(display, &nextreport);
      else nextreport.type = 0;
    }while(
      (nextreport.type == MotionNotify && report.type == MotionNotify)
    );

    switch(report.type){
      case Expose:
        if(report.xexpose.window == filewin)
          XCopyArea(display, filepixmap, filewin, gc, 0, 0, filewindow.w,
          filewindow.h, 0, 0);
        else handleexpose(&report);
        break;

      case KeyPress:
        keys_return[0] = '\0';
        XLookupString ((XKeyEvent *)&report, keys_return, 1, &keysym,
          &composestatus);

        switch(keysym){
          case XK_Up:
            selectedfile--;
            if(selectedfile < 0) selectedfile = 0;
            strcpy(filenamebox.text, dirlist[selectedfile].name);
            activate_box(filepixmap, filewin, &filenamebox);
            dirmoveup(1);
            break;

          case XK_Down:
            selectedfile++;
            if(selectedfile >= totaldirentries) 
            	selectedfile = totaldirentries - 1;
            strcpy(filenamebox.text, dirlist[selectedfile].name);
            activate_box(filepixmap, filewin, &filenamebox);
            if(selectedfile > dirscroll.position) dirmovedown(1);
            else
            	drawdirectory();
            break;

          case XK_Prior:
            selectedfile -= dirwindow.h / 20;
            if(selectedfile < 0) selectedfile = 0;
            strcpy(filenamebox.text, dirlist[selectedfile].name);
            activate_box(filepixmap, filewin, &filenamebox);
            dirmoveup(dirwindow.h / 20);
            break;

          case XK_Next:
            selectedfile += dirwindow.h / 20;
            if(selectedfile >= totaldirentries) 
            	selectedfile = totaldirentries - 1;
            strcpy(filenamebox.text, dirlist[selectedfile].name);
            activate_box(filepixmap, filewin, &filenamebox);
            if(selectedfile > dirscroll.position + dirscroll.span)
	            dirmovedown(dirwindow.h / 20);
  					else
  						dirmovedown(1);
            break;

          case XK_Escape:
            destroydirectory();
            XDestroyWindow(display, filewin);
            XFreePixmap(display, filepixmap);
            return 1;
            break;

          default:
            if(keys_return[0] == 13){
              if(!directorychange()){
                return endfileattributes(oldfilename);
              }
            }else
              handle_text_event(filepixmap, filewin, &report,
                &filenamebox);
              directorysearch(filenamebox.text);
            break;
        }
      break; /* end case keypress */

      case MotionNotify:
        if(dirscroll.event){
          dirscrollevent(&report);
        }
        break;

      case ButtonRelease:
        if(dirscroll.event){
          yscrollevent(filepixmap, filewin, &report, &dirscroll);
        }
        break;

      case ButtonPress:
        x = report.xbutton.x;
        y = report.xbutton.y;

        if(testbox(x, y, (bcbox *)&filenamebox))
        {
          handle_text_event(filepixmap, filewin, &report,
            &filenamebox);
        }else
        if(testbox(x, y, (bcbox *)&dirscroll))
        {
          yscrollevent(filepixmap, filewin, &report, &dirscroll);
          if(dirscroll.event){
            dirscrollevent(&report);
          }
          goto endbuttonpress;
        }else
        if(testbox(x, y, &dirwindow))
        {
          if(report.xbutton.time - time_ < 300){
            if(!directorychange()){
              return endfileattributes(oldfilename);
            }
          }else{
            time_ = report.xbutton.time;
            directoryevent(&report);
          }
          goto endbuttonpress;
        }else
        if(buttonpressrelease(filepixmap, filewin, x, y, &filewindow.ok)){
          if(!directorychange()){
            return endfileattributes(oldfilename);
          }
        }else
        if(buttonpressrelease(filepixmap, filewin, x, y, &filewindow.cancel)){
          destroydirectory();
          XDestroyWindow(display, filewin);
          XFreePixmap(display, filepixmap);
          return 1;
        }

        switch(operation){
          case LOAD:
            if(testbutton(x, y, &filewindow.pcmhilo))
            {
              *filetype = PCMHILO;
              fileformat(*filetype, operation);
            }
            if(testbutton(x, y, &filewindow.pcmlohi))
            {
              *filetype = PCMLOHI;
              fileformat(*filetype, operation);
            }
            break;

          case PLAYASFILE:
            if(testbutton(x, y, &filewindow.wave))
            {
              *filetype = WAV;
              fileformat(*filetype, operation);
            }
            if(testbutton(x, y, &filewindow.pcmhilo))
            {
              *filetype = PCMHILO;
              fileformat(*filetype, operation);
            }
            if(testbutton(x, y, &filewindow.pcmlohi))
            {
              *filetype = PCMLOHI;
              fileformat(*filetype, operation);
            }
            break;

          case SAVEASFILE:
            if(testbutton(x, y, &filewindow.bcast))
            {
              *filetype = BCAST;
              fileformat(*filetype, operation);
            }
            if(testbutton(x, y, &filewindow.edits))
            {
              *filetype = EDITS;
              fileformat(*filetype, operation);
            }
            break;

          case RECORD:
            if(testbutton(x, y, &filewindow.wave))
            {
              *filetype = WAV;
              fileformat( *filetype, operation);
            }
            if(testbutton(x, y, &filewindow.pcmhilo))
            {
              *filetype = PCMHILO;
              fileformat(*filetype, operation);
            }
            if(testbutton(x, y, &filewindow.pcmlohi))
            {
              *filetype = PCMLOHI;
              fileformat(*filetype, operation);
            }
            
#ifndef _LINUX_                  /* recommend third party mixer */
            if(testbutton(x, y, &filewindow.mic)){
              recevent.audioinput = MIC;
              inputselect();
            }
            if(testbutton(x, y, &filewindow.line)){
              recevent.audioinput = LINE;
              inputselect();
            }
#endif
            break;

          case EFFECT:
            if(testbutton(x, y, &filewindow.wave))
            {
              *filetype = WAV;
              fileformat( *filetype, operation);
            }
            if(testbutton(x, y, &filewindow.pcmhilo))
            {
              *filetype = PCMHILO;
              fileformat(*filetype, operation);
            }
            if(testbutton(x, y, &filewindow.pcmlohi))
            {
              *filetype = PCMLOHI;
              fileformat(*filetype, operation);
            }
            if(testbutton(x, y, &filewindow.bcast))
            {
              *filetype = BCAST;
              fileformat(*filetype, operation);
            }
            break;
        }
endbuttonpress:
        break;

      default:
        break;
    }                           /* end switch report type */
  }while(1);                       /* end event handler */
}
