/*
  Broadcast 2.0 multitrack audio editing
  (c) 1997 Adam Williams

  This program is distributed with the intent that it will be useful, without
  any warranty.  The code is being updated and has many bugs which are
  constantly being elucidated so changes should not be made without notifying
  the author.
*/

/* graphics.c      main graphics routines, high speed wave drawing routine  */


#include "defs.h"
#include "interfac.h"
#include "graphics.h"
#include "file.h"
#include "edit.h"
#include "database.h"
#include "automate.h"
#include "console.h"
#include "play.h"

struct wavscroll_ wavscroll = {
  0, 0,                      /* x, y window */
  0, 0,                      /* x, y pixmap */
  0, 0, 0, 0,                  /* x and y positions of windows */
  {0, 0, 0, 12, 0, 0, 0, 0},
  {0, 0, 12, 0, 0, 0, 0, 0}  /* x, y scrollbar */
};

struct wavdogear_ wavdogear = {
  0, 0,
  0, 0, 15, 35,
  {0, 0, 15, 35}
};

void updatemaintitle(char *filename){
  char window_name[64];
  char newfilename[256];
  static char oldfilename[256] = "";

            /* if no filename given, use old one */
  if(strlen(filename)) cutfilename(oldfilename, filename);
  strcpy(newfilename, oldfilename);

  sprintf(window_name, "Broadcast 2.1 %s", newfilename);

  XSetStandardProperties(display, win, window_name, window_name, None, 0, 0,
    0);

  if(showtrackgroups) sprintf(window_name, "Groups %s", newfilename);
  else sprintf(window_name, "Console %s", newfilename);
  XSetStandardProperties(display, consolewin, window_name, window_name, None, 0,
    0, 0);

  sprintf(window_name, "Play %s", newfilename);
  XSetStandardProperties(display, playwin, window_name, window_name, None, 0,
    0, 0);

  XFlush(display);
}

void allocatecolor(int red, int green, int blue, XColor *Col){
    Col->flags = DoRed | DoGreen | DoBlue;

    Col->red   = red<<8   |   red;
    Col->green = green<<8 |   green;
    Col->blue  = blue<<8  |   blue;
    if( !XAllocColor(display, cmap, Col) )
    {   cmap = XCopyColormapAndFree(display, cmap);
        XAllocColor(display, cmap, Col);
    }
}

void allocatecolormap(){
  allocatecolor(75, 75, 75, &dkgrey);
  allocatecolor(125, 125, 125, &migrey);
  allocatecolor(150, 150, 150, &grey);
  allocatecolor(200, 200, 200, &ltgrey);
  allocatecolor(0, 75, 75, &dkcyan);
  allocatecolor(0, 125, 125, &micyan);
  allocatecolor(0, 150, 150, &cyan);
  allocatecolor(0, 200, 200, &ltcyan);
  allocatecolor(0, 255, 0, &green);
  allocatecolor(255, 0, 0, &red);
  allocatecolor(255, 150, 150, &pink);
  allocatecolor(0, 0, 0, &black);
  allocatecolor(255, 255, 255, &white);
  allocatecolor(255, 255, 0, &yellow);
  allocatecolor(255, 150, 0, &orange);
  allocatecolor(140, 0, 255, &purple);
  allocatecolor(0, 0, 255, &blue);
}

void loadfont(){
  char *menufontname = "-*-helvetica-bold-o-normal-*-14-*";
  char *smallfontname = "-*-helvetica-medium-r-normal-*-10-*";

  if ((menufont = XLoadQueryFont(display, menufontname)) == NULL) {
    (void) fprintf(stderr,"Sorry, can't find menu font.  Using fixed instead.\n");
    menufontname="fixed";
    menufont = XLoadQueryFont(display, menufontname);
  }
  
  if ((smallfont = XLoadQueryFont(display, smallfontname)) == NULL) {
    (void) fprintf(stderr,"Sorry, can't find small font.  Using fixed instead.\n");
    smallfontname="fixed";
    smallfont = XLoadQueryFont(display, smallfontname);
  }
}

void sizewindow(){
  menuwindow.x = 0;
  menuwindow.y = 0;
  menuwindow.w = screenwidth;
  menuwindow.h = 25;

  patchwindow.x = 0;
  patchwindow.y = menuwindow.h;
  patchwindow.w = 35;
  patchwindow.h = screenheight - patchwindow.y;

  labelwindow.x = patchwindow.w;
  labelwindow.y = menuwindow.h;
  labelwindow.w = screenwidth - labelwindow.x;
  labelwindow.h = 20;

  wavewindow.x = labelwindow.x;
  wavewindow.y = labelwindow.y + labelwindow.h;
  wavewindow.w = labelwindow.w - wavscroll.y.w;
  wavewindow.h = screenheight - wavewindow.y - wavscroll.x.h - 15;

  timewindow.x = wavewindow.x;
  timewindow.y = wavewindow.y + wavewindow.h + wavscroll.x.h;
  timewindow.w = wavewindow.w;
  timewindow.h = 15;

  wavscroll.y.h = wavewindow.h;
  wavscroll.x.w = wavewindow.w;
  wavscroll.xscrollx = wavewindow.x;
  wavscroll.xscrolly = wavewindow.y + wavewindow.h;
  wavscroll.yscrollx = wavewindow.x + wavewindow.w;
  wavscroll.yscrolly = wavewindow.y;

  wavdogear.x = wavscroll.yscrollx;
  wavdogear.y = wavscroll.xscrolly;
}

void resizewindow(XEvent *report){
  if(screenwidth != report->xconfigure.width |
    screenheight != report->xconfigure.height)
  {
    screenwidth = report->xconfigure.width;
    screenheight = report->xconfigure.height;

    XFreePixmap(display, wavpixmap);
    XFreePixmap(display, menupixmap);
    XFreePixmap(display, labelpixmap);
    XFreePixmap(display, patchpixmap);
    XFreePixmap(display, timepixmap);

    XFreePixmap(display, wavscroll.xpixmap);
    XFreePixmap(display, wavscroll.ypixmap);

    XFreePixmap(display, wavdogear.pixmap);

    sizewindow();

    XResizeWindow(display, menuwin, menuwindow.w, menuwindow.h);
    XResizeWindow(display, wavwin, wavewindow.w, wavewindow.h);
    XResizeWindow(display, timewin, timewindow.w, timewindow.h);
    XResizeWindow(display, patchwin, patchwindow.w, patchwindow.h);
    XResizeWindow(display, labelwin, labelwindow.w, labelwindow.h);

    XResizeWindow(display, wavscroll.ywin, wavscroll.y.w, wavscroll.y.h);
    XResizeWindow(display, wavscroll.xwin, wavscroll.x.w, wavscroll.x.h);

    XMoveWindow(display, menuwin, menuwindow.x, menuwindow.y);
    XMoveWindow(display, wavwin, wavewindow.x, wavewindow.y);
    XMoveWindow(display, labelwin, labelwindow.x, labelwindow.y);
    XMoveWindow(display, timewin, timewindow.x, timewindow.y);
    XMoveWindow(display, patchwin, patchwindow.x, patchwindow.y);

    XMoveWindow(display, wavscroll.xwin, wavscroll.xscrollx, wavscroll.xscrolly);
    XMoveWindow(display, wavscroll.ywin, wavscroll.yscrollx, wavscroll.yscrolly);

    XMoveWindow(display, wavdogear.win, wavdogear.x, wavdogear.y);

     menupixmap = XCreatePixmap(display, menuwin, menuwindow.w, menuwindow.h, depth);
     wavpixmap = XCreatePixmap(display, wavwin, wavewindow.w, wavewindow.h, depth);
     labelpixmap = XCreatePixmap(display, labelwin, labelwindow.w, labelwindow.h, depth);
     patchpixmap = XCreatePixmap(display, patchwin, patchwindow.w, patchwindow.h, depth);
     timepixmap = XCreatePixmap(display, timewin, timewindow.w, timewindow.h, depth);

    wavscroll.xpixmap = XCreatePixmap(display, wavscroll.xwin,
      wavscroll.x.w, wavscroll.x.h, depth);
    wavscroll.ypixmap = XCreatePixmap(display, wavscroll.ywin,
      wavscroll.y.w, wavscroll.y.h, depth);

     wavdogear.pixmap = XCreatePixmap(display, wavdogear.win,
       wavdogear.w, wavdogear.h, depth);

    drawwave(0, 0, wavewindow.w, wavewindow.h, startx, starty, NO_BLANK, FLASH,
      DRAWCURSOR);
    drawpatches(starty);
    drawlabels(startx);
    drawtime(startx);
    drawmenubar(-1);
    updatexscrollbar(wavscroll.xpixmap, wavscroll.xwin, startx,
      wavewindow.w * xscale, longesttrack(), &wavscroll.x);
    updateyscrollbar(wavscroll.ypixmap, wavscroll.ywin, starty,
      wavewindow.h / yscaletrack, tracks(), &wavscroll.y);
    DrawUpDogEar(wavdogear.pixmap, wavdogear.win, &wavdogear.box);
  }
}

void consoleresizewindow(XEvent *report){
  if(consolewindow.w != report->xconfigure.width |
    consolewindow.h != report->xconfigure.height)
  {
    consolewindow.w = report->xconfigure.width;
    consolewindow.h = report->xconfigure.height;
    consolewindow.scroll.h = report->xconfigure.height-1;
    XFreePixmap(display, consolepixmap);
     consolepixmap = XCreatePixmap(display, consolewin, consolewindow.w, consolewindow.h, depth);
    drawconsolecontrols();
  }
}

int initializescreen(){
  XGCValues gcvalues;
  unsigned long gcmask;
  Visual *vis;
  char *display_name = NULL;
  XSizeHints size_hints;
  XSetWindowAttributes attr;
  Cursor arrow;
  unsigned long mask;
  int x, y;
  Window temp;

  if ( (display=XOpenDisplay(display_name)) == NULL ) {
      fprintf( stderr, "cannot connect to X server");
      if (getenv("DISPLAY") == NULL)
          fprintf( stderr, ", 'DISPLAY' environment variable not set.\n");
      else
          fprintf( stderr, " %s\n", XDisplayName(display_name));
      exit ( -1 );
  }

  sizewindow();
  screen = DefaultScreen(display);
  rootwin = RootWindow(display, screen);
  vis = DefaultVisual(display, screen);
	depth=DefaultDepth(display, screen);

  /* cmap = XCreateColormap(display, rootwin, vis, AllocNone); */

	cmap = DefaultColormap(display, screen);

  allocatecolormap();
  XInstallColormap(display, cmap);
  loadfont();

  arrow = XCreateFontCursor(display, XC_top_left_arrow);

  mask = CWEventMask | CWBackPixel | CWBorderPixel | CWColormap |
    CWCursor | CWOverrideRedirect | CWSaveUnder;
  attr.event_mask = ExposureMask | ButtonPressMask |
    ButtonReleaseMask | StructureNotifyMask | KeyPressMask |
    PointerMotionMask;
  attr.background_pixel = grey.pixel;
  attr.border_pixel = white.pixel;
  attr.colormap = cmap;
  attr.cursor = arrow;
  attr.override_redirect = False;
  attr.save_under = False;

  attr.background_pixel = white.pixel;
               /* main window */
  win = XCreateWindow(display, rootwin, 0, 0, screenwidth, screenheight,
    0, depth, InputOutput, vis, mask, &attr);

  wavwin =   XCreateWindow(display, win, wavewindow.x,   wavewindow.y,
    wavewindow.w,   wavewindow.h, 0, depth, InputOutput, vis, mask, &attr);
    
  attr.background_pixel = grey.pixel;
  patchwin = XCreateWindow(display, win, patchwindow.x, patchwindow.y,
    patchwindow.w, patchwindow.h, 0, depth, InputOutput, vis, mask, &attr);
    
  labelwin = XCreateWindow(display, win, labelwindow.x, labelwindow.y,
    labelwindow.w, labelwindow.h, 0, depth, InputOutput, vis, mask, &attr);
    
  timewin =  XCreateWindow(display, win, timewindow.x,  timewindow.y,
    timewindow.w,  timewindow.h, 0, depth, InputOutput, vis, mask, &attr);
    
  wavscroll.xwin = XCreateWindow(display, win, wavscroll.xscrollx,
    wavscroll.xscrolly, wavscroll.x.w,  wavscroll.x.h, 0, depth,
    InputOutput, vis, mask, &attr);
    
  wavscroll.ywin = XCreateWindow(display, win, wavscroll.yscrollx,
    wavscroll.yscrolly, wavscroll.y.w,  wavscroll.y.h, 0, depth,
    InputOutput, vis, mask, &attr);
    
  wavdogear.win = XCreateWindow(display, win, wavdogear.x,
    wavdogear.y, wavdogear.w,  wavdogear.h, 0, depth,
    InputOutput, vis, mask, &attr);

  attr.background_pixel = cyan.pixel;
  menuwin =  XCreateWindow(display, win, menuwindow.x, menuwindow.y,
    menuwindow.w, menuwindow.h, 0, depth, InputOutput, vis, mask, &attr);

  wavpixmap = XCreatePixmap(display, wavwin,   wavewindow.w,   wavewindow.h, depth);
  patchpixmap = XCreatePixmap(display, patchwin, patchwindow.w, patchwindow.h,
    depth);
  labelpixmap = XCreatePixmap(display, labelwin, labelwindow.w, labelwindow.h,
    depth);
  timepixmap = XCreatePixmap(display, timewin,  timewindow.w,  timewindow.h, depth);
  menupixmap = XCreatePixmap(display, menuwin,  menuwindow.w,  menuwindow.h, depth);

  wavscroll.xpixmap = XCreatePixmap(display, wavscroll.xwin,
    wavscroll.x.w, wavscroll.x.h, depth);
  wavscroll.ypixmap = XCreatePixmap(display, wavscroll.ywin,
    wavscroll.y.w, wavscroll.y.h, depth);

  wavdogear.pixmap = XCreatePixmap(display, wavdogear.win,
    wavdogear.w, wavdogear.h, depth);

  XMapWindow(display, win);
  XMapSubwindows(display, win);

  XTranslateCoordinates(display, win, rootwin, 0, screenheight,
    &x, &y, &temp);

  attr.background_pixel = grey.pixel;
  playwin = XCreateWindow(display, rootwin, x, y, playwindow.w, playwindow.h,
    0, depth, InputOutput, vis, mask, &attr);
  consolewin = XCreateWindow(display, rootwin, x+playwindow.w, y, consolewindow.w, consolewindow.h,
    0, depth, InputOutput, vis, mask, &attr);

  playpixmap = XCreatePixmap(display, playwin, playwindow.w,
    playwindow.h, depth);
  consolepixmap = XCreatePixmap(display, consolewin, consolewindow.w, consolewindow.h, depth);

  attr.save_under = True;
  attr.override_redirect = True;
  attr.background_pixel = cyan.pixel;
  popupwin =  XCreateWindow(display, rootwin, 100,  100, 100,  100,
    0, depth, InputOutput, vis, mask, &attr);

  popupsubwin =  XCreateWindow(display, rootwin, 100,  100, 100,  100,
    0, depth, InputOutput, vis, mask, &attr);

  size_hints.flags = PSize;
  size_hints.width = screenwidth;
  size_hints.height = screenheight;

  XSetStandardProperties(display, win, "", "", None, 0, 0,
    &size_hints);

  size_hints.flags = PSize | PMinSize | PMaxSize;
  size_hints.width = playwindow.w;
  size_hints.height = playwindow.h;
  size_hints.min_width = playwindow.w;
  size_hints.max_width = playwindow.w;
  size_hints.min_height = playwindow.h;
  size_hints.max_height = playwindow.h;

  XSetStandardProperties(display, playwin, "", "", None, 0, 0,
    &size_hints);

  size_hints.flags = PSize | PMinSize | PMaxSize;
  size_hints.width = consolewindow.w;
  size_hints.height = consolewindow.h;
  size_hints.min_width = consolewindow.w;
  size_hints.max_width = consolewindow.w;
  size_hints.min_height = 10;
  size_hints.max_height = 991;

  XSetStandardProperties(display, consolewin, "", "", None, 0, 0,
    &size_hints);

  updatemaintitle("");

  XMapWindow(display, consolewin);
  XMapWindow(display, playwin);

  gcmask = GCFont;
  gcvalues.font = menufont->fid;
  gc = XCreateGC(display, rootwin, gcmask, &gcvalues);

  DrawUpDogEar(wavdogear.pixmap, wavdogear.win, &wavdogear.box);
}

void refreshwav(){
  XCopyArea(display, wavpixmap, wavwin, gc, 0, 0, wavewindow.w, wavewindow.h, 0, 0);
  XFlush(display);
}

void refreshplay(){
  XCopyArea(display, playpixmap, playwin, gc, 0, 0,
  playwindow.w, playwindow.h, 0, 0);
  XFlush(display);
}

void refreshconsole(){
  XCopyArea(display, consolepixmap, consolewin, gc, 0, 0,
  consolewindow.w, consolewindow.h, 0, 0);
  XFlush(display);
}

void refreshpatches(){
  XCopyArea(display, patchpixmap, patchwin, gc, 0, 0, patchwindow.w,
    patchwindow.h, 0, 0);
  XFlush(display);
}

void refreshxscroll(){
  XCopyArea(display, wavscroll.xpixmap, wavscroll.xwin, gc, 0, 0,
    wavscroll.x.w, wavscroll.x.h, 0, 0);
}

void refreshyscroll(){
  XCopyArea(display, wavscroll.ypixmap, wavscroll.ywin, gc, 0, 0,
    wavscroll.y.w, wavscroll.y.h, 0, 0);
}

void refreshwavdogear(){
  XCopyArea(display, wavdogear.pixmap, wavdogear.win, gc,
    wavdogear.box.x, wavdogear.box.y,
    wavdogear.box.w + 1, wavdogear.box.h + 1,
    wavdogear.box.x, wavdogear.box.y);
}

void refreshdisplay(){
  refreshwav();
  refreshmenubar();
  refreshpatches();
  XCopyArea(display, labelpixmap, labelwin, gc, 0, 0, labelwindow.w,
    labelwindow.h, 0, 0);
  XCopyArea(display, timepixmap, timewin, gc, 0, 0, timewindow.w, timewindow.h,
    0, 0);
  refreshxscroll();
  refreshyscroll();
  refreshwavdogear();
  XFlush(display);
}

void refreshmenubar(){
  XCopyArea(display, menupixmap, menuwin, gc, 0, 0, menuwindow.w, menuwindow.h,
    0, 0);
  XFlush(display);
}

                              /* draw column from xscale file */
void drawcolumn1(int column, int centery, int yscale,
  TWO highsample, TWO lowsample)
{
  static int y1, y2;

  y1 = highsample / yscale;
  y2 = lowsample / yscale;

  y1 += centery;
  y2 += centery;

  XDrawLine(display, wavpixmap, gc, column, y1, column, y2);
}

                             /* draw column from sample */
void drawcolumn2(int _column, int column, int yscale, TWO sample,
  int centery)
{

  static int y1;
  static int y2;

  y1 = sample / yscale;

  y1 += centery;

  if(_column >= column){
    XDrawLine(display, wavpixmap, gc, _column - 1, y2, _column, y1);
  }

  y2 = y1;
}

/* display time */
int drawtime(long startx){
  static long x;
  static char timestring[15];
  static int column;

  x = startx;
  x /= timescale;
  x *= timescale;

  DrawUpBox(timepixmap, 0, 0, timewindow.w-1, timewindow.h-1);
  XSetForeground(display, gc, black.pixel);
  for(column = (x - startx) / xscale;
    column < timewindow.w; column += timescale / xscale, x += timescale)
  {
    gettime(x, timestring);
    XDrawString(display, timepixmap, gc, column+2, timewindow.h-2, timestring, strlen(timestring));
    XDrawLine(display, timepixmap, gc, column, 0, column, timewindow.h);
  }
  XCopyArea(display, timepixmap, timewin, gc, 0, 0, timewindow.w, timewindow.h, 0, 0);
  XFlush(display);
}

/* display labels */

int drawlabels(long x){
  static int i;
  static char labelstring[5];
  static struct Label *currentlabel;
  static long screenstart, screenend;

  screenstart = startx;
  screenend = startx + labelwindow.w  * xscale;

  DrawUpBox(labelpixmap, 0, 0, labelwindow.w-1, labelwindow.h-1);
  for(i = 0, currentlabel = firstlabel;
    currentlabel && currentlabel->position <= screenend;
    currentlabel = currentlabel->nextlabel, i++)
  {
    if(currentlabel->position >= screenstart &&
      currentlabel->position <= screenend)
    {
      if(selectionstart == currentlabel->position |
        selectionend == currentlabel->position)
      {
        DrawDnBox(labelpixmap,
          (currentlabel->position - x) / xscale - 15, 2,
          30, labelwindow.h-4);
      }
      else
      {
        DrawUpBox(labelpixmap,
          (currentlabel->position - x) / xscale - 15, 2,
          30, labelwindow.h-4);
      }

      sprintf(labelstring, "%d", i);
      XSetForeground(display, gc, black.pixel);
      XDrawString(display, labelpixmap, gc,
        (currentlabel->position - x) / xscale - 10,
        labelwindow.h - 4, labelstring, strlen(labelstring));
    }
  }
  XCopyArea(display, labelpixmap, labelwin, gc, 0, 0, labelwindow.w, labelwindow.h, 0, 0);
  XFlush(display);
}

/* displaypatches */

void drawpatches(int starty){
  static int y, x;
  static int track;
  static struct Track *currenttrack;
  static int totaltracks;
  
  x = patchwindow.x + 3;

  totaltracks = tracks();
  DrawUpBox(patchpixmap, 0, 0, patchwindow.w-1, patchwindow.h-1);\
  currenttrack = trackof(starty);

  for(y = yscaletrack/2, track = starty;
    y < patchwindow.y + patchwindow.h, track < totaltracks;
    y += yscaletrack, track++, currenttrack = currenttrack->nexttrack)
  {
    DrawPatch(patchpixmap, x, y, currenttrack, track);
  }
  refreshpatches();
  XFlush(display);
}

int drawtitles(int column1, int row1, long column2, long row2, long startx,
  int starty)
{
  static struct Track *currenttrack;
  static struct Edit *currentedit, *prevedit;
  static char *title;
  static int _row, x, y;
  static long screenend;

  if(titles){
    screenend = column2 * xscale + startx;
    XSetForeground(display, gc, pink.pixel);
    for(currenttrack = trackof(starty), _row = row1;
      _row < row2 && currenttrack;
      currenttrack = currenttrack->nexttrack, _row += yscaletrack)
    {
      y = _row + yscaletrack/2;
      currentedit = segmentof(startx, currenttrack);
      if(currentedit) prevedit = currentedit->prevedit;
        else prevedit = 0;
      for(; currentedit && currentedit->start1 < screenend;
        currentedit = currentedit->nextedit)
      {
        if((!prevedit && strcmp(currentedit->file, SILENCE)) |
          (prevedit && strcmp(prevedit->file, currentedit->file) &&
          strcmp(currentedit->file, SILENCE)))
        {
          title = currentedit->file + strlen(currentedit->file);
          while(title != currentedit->file && *title != '/')
            title--;
          if(*title == '/') title++;

          if((currentedit->start1 - startx) / xscale >= column1 - column2){
            x = (currentedit->start1 - startx) / xscale + column1;
            XDrawString(display, wavpixmap, gc, x, y,
              title, strlen(title));
          }
        }
        prevedit = currentedit;
      }
    }
    XSetForeground(display, gc, black.pixel);
  }
}

/* display edits */
int drawedits(int column1, int row1, int column2, int row2, long startx,
  int starty)
{
  static struct Track *currenttrack;
  static struct Edit *currentedit;
  static long _row, x, y;
  static long screenend;

  if(showedits){
    screenend = column2 * xscale + startx;
                                              /* draw track */
    for(currenttrack = trackof(starty), _row = row1;
      _row < row2 && currenttrack;
      currenttrack = currenttrack->nexttrack, _row += yscaletrack)
    {
      y = _row + yscaletrack/2;
      currentedit = segmentof(startx, currenttrack);
      if(currentedit && currentedit->prevedit) currentedit = currentedit->prevedit;

                                              /* draw edits */
      for(; currentedit && currentedit->start1 < screenend;
        currentedit = currentedit->nextedit)
      {
      	if(currentedit->start1 + currentedit->length >= startx &&
      		currentedit->start1 + currentedit->length <= screenend)
      	{
      		x = currentedit->start1 + currentedit->length;
        	x -= startx;
        	x /= xscale;
        	x += column1;

                                    	 /* draw left edit */
        	DrawLeftEdit(wavpixmap, x, y);
        }
        
      	if(currentedit->start1 >= startx &&
      		currentedit->start1 <= screenend)
      	{
      		x = currentedit->start1;
        	x -= startx;
        	x /= xscale;
        	x += column1;
                                    	 /* draw right edit */
        	DrawRightEdit(wavpixmap, x, y);
        }
      }
    }
  }
}

/* display cursor */
int drawcursor(int column1, int row1, int column2, int row2, long startx,
  long selectionstart, long selectionend)
{
  static long _column1;
  static long _column2;
  static int width;
  static int height;
  static long viewend;

  width = column2 - column1;
  height = row2 - row1;
  viewend = startx + (long)xscale * (long)width;

  if(selectionstart >= startx)
    _column1 = (selectionstart - startx) / xscale + column1;
  else
    _column1 = column1;

  if(selectionend <= viewend)
    _column2 = (selectionend - startx) / xscale + 1;
  else
    _column2 = column2;

  if((selectionstart >= startx && selectionstart <=viewend) |
   (selectionend <= viewend && selectionend >= startx) |
   (selectionstart <= startx && selectionend >= viewend)){
    XSetFunction(display, gc, GXxor);
    XSetForeground(display, gc, white.pixel);
    XFillRectangle(display, wavpixmap, gc, _column1, row1, _column2-_column1, height);
    XSetForeground(display, gc, black.pixel);
    XSetFunction(display, gc, GXcopy);
  }
}

long createindexfile(char *scalefile, struct Edit *currentedit){
  FILE *out, *in;
  long inbufferlength, outbufferlength;
  long infilelength;
  FOUR scaled;
  TWO sample1, sample2;
  register TWO *highsample, *lowsample;
  char *outbufferstart, *inbufferstart, *inbufferend;
  register TWO *_inbuffer, *_inbufferend;
  unsigned char byte1;
  char frame, channel;
  TWO byte2;
  char string[256], string_[256];
  XEvent report;
  int track, i, cancel;
  struct CastHeader *indexheader;
  /* in indexheader, the length is really the end in the file of the track */
  struct CastHeader *castheader;
  long castheaderlength;
  long trackend;
  TWO newoutchannels;

  scaled = 1;
  indexheader = 0;
  castheader = 0;

  if(!(in = fopen(currentedit->file, "rb"))){
    printf("Can't open %s for creating index file.\n", scalefile);
    return 0;
  }

  switch(currentedit->format){
    case WAV:
      fseek(in, 0, SEEK_END);
      infilelength = ftell(in) - WAVHEADERLENGTH;
      frame = currentedit->channels * 2;
      fseek(in, WAVHEADERLENGTH, SEEK_SET);
      while((infilelength - WAVHEADERLENGTH) / frame / scaled > INDEXFILESIZE / (frame*2)){
        scaled *= 2;
      }
      break;

    case PCMHILO:
      fseek(in, 0, SEEK_END);
      infilelength = ftell(in);
      frame = currentedit->channels * 2;
      fseek(in, 0, SEEK_SET);
      while(infilelength / frame / scaled > INDEXFILESIZE / (frame*2)){
        scaled *= 2;
      }
      break;

    case PCMLOHI:          /* same as PCMHILO */
      fseek(in, 0, SEEK_END);
      infilelength = ftell(in);
      frame = currentedit->channels * 2;
      fseek(in, 0, SEEK_SET);
      while(infilelength / frame / scaled > INDEXFILESIZE / (frame*2)){
        scaled *= 2;
      }
      break;

    case BCAST:                               /* load header from bcast */
      frame = 2;
      fseek(in, 6, SEEK_SET);
      newoutchannels = gettwo(in);     /* get outchannels used in file */

      fseek(in, CASTHEADLEN, SEEK_SET);          /* get track locations in file */
      castheader = malloc(currentedit->channels * sizeof(struct CastHeader));
      for(i = 0; i < currentedit->channels; i++){
        castheader[i].start = getfour(in);
        castheader[i].length = getfour(in);
        fseek(in, 7 + 4 * newoutchannels, SEEK_CUR);
                                       /* skip track information */
      }
      castheaderlength = ftell(in);
      fseek(in, 0, SEEK_END);
      infilelength = ftell(in) - castheaderlength;
      while((infilelength - castheaderlength) / frame / scaled > INDEXFILESIZE / (frame*2)){
        scaled *= 2;
      }
      break;
  }

  if(scaled < 8){
    free(castheader);
    fclose(in);
    return 0;
  }

  if(!(out = fopen(scalefile, "wb"))){
    printf("Can't create index file.\n");
    free(castheader);
    fclose(in);
    return 0;
  }

  inbufferlength = SAVEBUFFER / 2 / scaled / frame;
  inbufferlength *= scaled * frame * 2;
  if(!(inbufferstart = malloc(inbufferlength))){
    printf("Not enough memory to create index file.\n");
    free(castheader);
    fclose(in);
    fclose(out);
    remove(scalefile);
    return 0;
  }

  outbufferlength = inbufferlength / currentedit->channels / scaled * 2;
  if(currentedit->format == BCAST) outbufferlength *= currentedit->channels;
  if(!(outbufferstart = malloc(outbufferlength))){
    printf("Not enough memory to create index file.\n");
    free(castheader);
    fclose(in);
    fclose(out);
    free(inbufferstart);
    remove(scalefile);
    return 0;
  }

  cutfilename(string_, scalefile);
  sprintf(string, "Creating index file %s", string_);
  initializeprogress(string);
  putfour(scaled, out);                                 /* store scale factor */
  for(i=0; i<20; i++) fputc(0, out);             /* 20 byte padding */
  for(track = 0; track < 8 * currentedit->channels; track++) fputc(0, out);
  indexheader = malloc(sizeof(struct CastHeader) * currentedit->channels);
                                                                 /* skip header */

                                               /* process each track seperately */
  for(track = 0; track < currentedit->channels; track++){
    indexheader[track].start = ftell(out);
                                    /* seek to start of channel in real file */
    switch(currentedit->format){
      case WAV:
        trackend = infilelength - currentedit->channels * 2;
        fseek(in, WAVHEADERLENGTH + track*2, SEEK_SET);
        break;

      case PCMHILO:
        trackend = infilelength - currentedit->channels * 2;
        fseek(in, track*2, SEEK_SET);
        break;

      case PCMLOHI:         /* same as PCMHILO */
        trackend = infilelength - currentedit->channels * 2;
        fseek(in, track*2, SEEK_SET);
        break;

      case BCAST:
        fseek(in, castheader[track].start, SEEK_SET);
        trackend = castheader[track].start + castheader[track].length;
        break;
    }

    inbufferlength = SAVEBUFFER / 2 / scaled / frame;
    inbufferlength *= scaled * frame * 2;
    while(ftell(in) < trackend){
        /* inbufferlength will never exceed end of file because division here
           rounds down */
      if(trackend - ftell(in) < inbufferlength){
        inbufferlength = trackend - ftell(in);
        inbufferlength /= scaled*frame;
        inbufferlength *= scaled*frame;
        inbufferlength += scaled*frame;
      }
                                  /* load buffer  */
      fread(inbufferstart, inbufferlength, 1, in);
      if((currentedit->format == WAV && !byteorder) ||
        (currentedit->format == PCMHILO && byteorder) ||
        (currentedit->format == PCMLOHI && !byteorder))
        swapbytes(inbufferstart, inbufferlength);

      cancel = 0;
      if(currentedit->format == BCAST)
        cancel = updateprogress(ftell(in), infilelength);
      else
         cancel = updateprogress((trackend * track + ftell(in)) /
           currentedit->channels, infilelength);
      if(cancel)
      {
        fclose(in);
        fclose(out);
        free(castheader);
        free(indexheader);
        free(inbufferstart);
        free(outbufferstart);
        remove(scalefile);
        destroyprogress();
        return 0;
      }

      _inbuffer = (TWO *)inbufferstart;
      inbufferend = inbufferstart + inbufferlength;
      highsample = (TWO *)outbufferstart;
      lowsample = highsample + 1;
                                                        /* calculate elements */
      while((char *)_inbuffer < inbufferend){
        _inbufferend = _inbuffer + scaled * frame / 2;
                                                /* calculate a single element */
        if(_inbuffer < _inbufferend){
          *lowsample = *highsample = *_inbuffer;
          _inbuffer += frame / 2;
        }
        
        _inbufferend -= 4 * frame / 2;
        while(_inbuffer < _inbufferend){
          if(*_inbuffer > *highsample) *highsample = *_inbuffer;
          else if(*_inbuffer < *lowsample) *lowsample = *_inbuffer;
          _inbuffer += frame / 2;

          if(*_inbuffer > *highsample) *highsample = *_inbuffer;
          else if(*_inbuffer < *lowsample) *lowsample = *_inbuffer;
          _inbuffer += frame / 2;

          if(*_inbuffer > *highsample) *highsample = *_inbuffer;
          else if(*_inbuffer < *lowsample) *lowsample = *_inbuffer;
          _inbuffer += frame / 2;

          if(*_inbuffer > *highsample) *highsample = *_inbuffer;
          else if(*_inbuffer < *lowsample) *lowsample = *_inbuffer;
          _inbuffer += frame / 2;
        }
        
        _inbufferend += 4 * frame / 2;
        while(_inbuffer < _inbufferend){
          if(*_inbuffer > *highsample) *highsample = *_inbuffer;
          else if(*_inbuffer < *lowsample) *lowsample = *_inbuffer;
          _inbuffer += frame / 2;
        }

        highsample += 2;
        lowsample += 2;
      }

                                        /* in case of full hard drive, abort */
      if(!(fwrite(outbufferstart, outbufferlength, 1, out))){
        fclose(in);
        fclose(out);
        free(castheader);
        free(indexheader);
        free(inbufferstart);
        free(outbufferstart);
        remove(scalefile);
        destroyprogress();
        return 0;
      }
    }
    indexheader[track].length = ftell(out);
  }

  fseek(out, 24, SEEK_SET);                   /* write index track positions */
  for(track = 0; track < currentedit->channels; track++){
    putfour(indexheader[track].start, out);
    putfour(indexheader[track].length, out);
  }

  fclose(in);
  fclose(out);
  free(castheader);
  free(indexheader);
  free(inbufferstart);
  free(outbufferstart);
  destroyprogress();
  return scaled;
}

int drawwave(int column1, int row1, long column2, long row2, long startx,
  int starty, char blank, char flash, char cursor)
{
  static FILE *in;
  static unsigned char byte1;
  static TWO byte2;
  static char scalefile[64];
  static char *inbufferstart, *inbufferend;
  static TWO *_inbuffer, *_inbufferend;
  static int _column, _row;
  static TWO sample, highsample, lowsample;
  static long x;                          /* x position in virtual file in samples */
  static int track, i; /* track in virtual file */ 
  static TWO newoutchannels;   /* outchannels for bcast file */
  static FOUR offset;                      /* offset of x in segment of real file */
  static FOUR scaled, element, bufferlength, indexend;
  static FOUR segmentend, windowend, segmentlength;
  static char frame;
  static XEvent report;
  static int centery;
  static int _yscale;
  static struct Track *currenttrack;
  static struct Edit *currentedit;
  static char automationvisible;
  static FOUR start;          /* start of track in BCAST file */
  static long initsavebuffer;

             /* debug option */
  /* printf("\n");
  dumpedits(); */

                   /*  blank the screen while drawing if blank */
  if(blank) XClearArea(display, wavwin, column1, row1,
    column2 - column1, row2 - row1, 0);

  XFlush(display);
  XSetForeground(display, gc, white.pixel);
  XFillRectangle(display, wavpixmap, gc, column1, row1,
    column2 - column1, row2 - row1);
  XSetForeground(display, gc, black.pixel);
  currenttrack = trackof(starty);
  automationvisible = automationisvisible();

  _yscale = 65536 / yscale;
  windowend = startx + (column2 - column1) * xscale;
  track = starty;

                                               /* draw a track one at a time */
  for(_row = row1;
    _row < row2 && currenttrack;
    track++, _row += yscaletrack, currenttrack = currenttrack->nexttrack)
  {
                                          /* find segment containing startx */
    centery = _row + yscaletrack / 2;
    _column = column1;
    x = startx;
    currentedit = segmentof(startx, currenttrack);

                                                          /* draw zero line */
    if(!automationvisible){
      XSetForeground(display, gc, green.pixel);
      XDrawLine(display, wavpixmap, gc, column1, centery, column2, centery);
      XSetForeground(display, gc, black.pixel);
    }else{
                      /* draw automation boundaries if visible */
      XSetForeground(display, gc, black.pixel);
                                 /* lower line */
      XDrawLine(display, wavpixmap, gc, column1, _row + yscaletrack - 1, column2,
        _row + yscaletrack - 1);
                             /* upper line */
      XDrawLine(display, wavpixmap, gc, column1, _row, column2,
        _row);
      XSetForeground(display, gc, grey.pixel);
    }

                                      /* display each segment one at a time */
    while(currentedit != 0 && _column < column2){
      static char issilence;

      issilence = 0;          /* if a segment is silence or a file can't
                                 be opened, turn silence on */
      /* subtract xscale from segmentend to keep end of track */
      segmentend = currentedit->start1 + currentedit->length - xscale;
      if(segmentend > windowend) segmentend = windowend;
      segmentlength = segmentend - x;

                                               /* real file is silence */
      if(!strcmp(currentedit->file, SILENCE)){
        issilence = 1;
        goto drawsilence;               /* skip to drawing silence */
      }
                                     /* real file isn't silence */
      if(!issilence){
                             /* attempt to open a scaled version of the file
                                          and read the xscale of the file */
        sprintf(scalefile, "%s%s", currentedit->file, SCALEFILEEXT);
        if(!(in = fopen(scalefile, "rb"))){
                          /* try creating an index file */
          scaled = createindexfile(scalefile, currentedit);
                        /* scale file was created successfully */
          if(scaled){
                          /* can't open the new scale file */
            if(!(in = fopen(scalefile, "rb"))){
              printf("Can't open index file %s for display.\n", scalefile);
              scaled = 1;
            }else{
              scaled = getfour(in);
            }
          }
        }else{
          scaled = getfour(in);
        }

        if(automationvisible) XSetForeground(display, gc, grey.pixel);
        else XSetForeground(display, gc, black.pixel);

                             /* if scaled version is impossible to create or
                                xscale is smaller than scaled version, read segment
                                from original file */
                             /* go to start of segment in file */
        if(!(scaled) | xscale < scaled){
          fclose(in);
          scaled = 1;
          if(x == startx && x > xscale){
            x -= xscale;
            _column--;
          }
                         /* if file can't be opened draw silence */
          if(!(in = fopen(currentedit->file, "rb"))){
            printf("Can't open %s for display.\n", currentedit->file);
            issilence = 1;
            goto drawsilence;
          }

          switch(currentedit->format){
            case WAV:
              frame = currentedit->channels * 2;
              offset = WAVHEADERLENGTH;
              element = frame * xscale;                        /* # of bytes in element */
              offset += (x - currentedit->start1 + currentedit->start2) * frame;
              offset += currentedit->track * 2;
              initsavebuffer = SAVEBUFFER / element;
              initsavebuffer *= element;
              bufferlength = initsavebuffer;
              break;

            case PCMHILO:
              frame = currentedit->channels * 2;
              offset = 0;
              element = frame * xscale;     /* # of bytes in element */
              offset += (x - currentedit->start1 + currentedit->start2) * frame;
              offset += currentedit->track * 2;
              break;

            case PCMLOHI:           /*  same as PCMHILO */
              frame = currentedit->channels * 2;
              offset = 0;
              element = frame * xscale;        /* # of bytes in element */
              offset += (x - currentedit->start1 + currentedit->start2) * frame;
              offset += currentedit->track * 2;
              break;

            case BCAST:
            	fseek(in, 6, SEEK_SET);
            	newoutchannels = gettwo(in);
                                               /* skip track information */
              fseek(in,
                44 + currentedit->track * (15 + 4 * newoutchannels),
                SEEK_SET);

              start = getfour(in);     /* load essential information */

              frame = 2;
              element = frame * xscale;                        /* # of bytes in element */
              offset = start +
                (x - currentedit->start1 + currentedit->start2) * frame;
              break;
          }                       /* frame = # of bytes per sample in original file */

          if(highqualityrefresh && xscale > 1){
                                /* allocate high quality refresh buffer */
            initsavebuffer = SAVEBUFFER / element;
            initsavebuffer *= element;
            bufferlength = initsavebuffer;
            if(!(inbufferstart = malloc(bufferlength + element))){
              printf("Not enough memory for high quality refresh.\n");
              fclose(in);
              return 1;
            }
          }
        }else{                    /* scale exists */
                           /* get scale file parameters*/
          fseek(in, 24 + currentedit->track * 8, SEEK_SET);
          offset = getfour(in);
          indexend = getfour(in);

          frame = 4;
          element = frame * xscale / scaled;          /* # of bytes in element */
          offset += (x - currentedit->start1 + currentedit->start2) / scaled * frame;
          initsavebuffer = SAVEBUFFER / element;
          initsavebuffer *= element;
          bufferlength = initsavebuffer;

          if(!(inbufferstart = malloc(bufferlength + element))){
            printf("Not enough memory to display wave.\n");
            fclose(in);
            return 1;
          }
        }

        fseek(in, offset, SEEK_SET);

                    /* draw columns for this segment if it isn't scaled */
        if(scaled == 1){     /*********** low quality refresh ****************/
          if(!highqualityrefresh | xscale == 1){
            switch(currentedit->format){
              case WAV:
                while(_column <= column2 && x < segmentend){
                  byte1 = fgetc(in);
                  byte2 = fgetc(in) << 8;
                  sample = byte1 | byte2;
                  drawcolumn2(_column, column1, _yscale, sample, centery);
                  _column++;
                  x += xscale;
                  fseek(in, element-2, SEEK_CUR);
                }
                break;

              case PCMHILO:
                while(_column <= column2 && x < segmentend){
                  byte2 = fgetc(in) << 8;
                  byte1 = fgetc(in);
                  sample = byte1 | byte2;
                  drawcolumn2(_column, column1, _yscale, sample, centery);
                  _column++;
                  x += xscale;
                  fseek(in, element-2, SEEK_CUR);
                }
                break;

              case PCMLOHI:
                while(_column <= column2 && x < segmentend){
                  byte1 = fgetc(in);
                  byte2 = fgetc(in) << 8;
                  sample = byte1 | byte2;
                  drawcolumn2(_column, column1, _yscale, sample, centery);
                  _column++;
                  x += xscale;
                  fseek(in, element-2, SEEK_CUR);
                }
                break;

              case BCAST:
                while(_column <= column2 && x < segmentend){
                  fseek(in, element-2, SEEK_CUR);
                  fread(&sample, 2, 1, in);   /* has to match byte order */
                  drawcolumn2(_column, column1, _yscale, sample, centery);
                  _column++;
                  x += xscale;
                }
                break;
            }
          }else{        /***************** high quality refresh **********/
            while(_column <= column2 && x < segmentend){
              if(x + bufferlength / frame > segmentend)
                bufferlength = (segmentend - x) * frame;

              fread(inbufferstart, bufferlength, 1, in);
              if((currentedit->format == WAV && !byteorder) ||
                (currentedit->format == PCMHILO && byteorder) ||
                (currentedit->format == PCMLOHI && !byteorder))
                swapbytes(inbufferstart, bufferlength);

                               /* clear extra segment for flush edges */
              /* this doesn't work for scaled drawing for some reason */
              if(bufferlength < initsavebuffer){
                _inbuffer = (TWO *)inbufferstart + bufferlength / 2;
                inbufferend = (char *)_inbuffer + element;
                if(inbufferend > inbufferstart + initsavebuffer);
                  inbufferend = inbufferstart + initsavebuffer;

                while(_inbuffer < (TWO *)inbufferend)
                  *_inbuffer++ = 0;
              }

              _inbuffer = (TWO *)inbufferstart;
              inbufferend = inbufferstart + bufferlength;
                                   /* draw columns for the segment */
              while((char *)_inbuffer < inbufferend){
                                   /* draw a column which is an element */
                _inbufferend = _inbuffer + element/2;

                if(_inbuffer < _inbufferend){
                  highsample = lowsample = *_inbuffer;
                }
                _inbuffer += frame / 2;
                
                _inbufferend -= 4 * frame / 2;
                while(_inbuffer < _inbufferend){
                  if(*_inbuffer > highsample) highsample = *_inbuffer;
                  else
                  if(*_inbuffer < lowsample) lowsample = *_inbuffer;
                  _inbuffer += frame / 2;

                  if(*_inbuffer > highsample) highsample = *_inbuffer;
                  else
                  if(*_inbuffer < lowsample) lowsample = *_inbuffer;
                  _inbuffer += frame / 2;

                  if(*_inbuffer > highsample) highsample = *_inbuffer;
                  else
                  if(*_inbuffer < lowsample) lowsample = *_inbuffer;
                  _inbuffer += frame / 2;

                  if(*_inbuffer > highsample) highsample = *_inbuffer;
                  else
                  if(*_inbuffer < lowsample) lowsample = *_inbuffer;
                  _inbuffer += frame / 2;
                }

                _inbufferend += 4 * frame / 2;
                while(_inbuffer < _inbufferend){
                  if(*_inbuffer > highsample) highsample = *_inbuffer;
                  else
                  if(*_inbuffer < lowsample) lowsample = *_inbuffer;
                  _inbuffer += frame / 2;
                }

                drawcolumn1(_column, centery, _yscale, highsample, lowsample);
                _column++;
                x += xscale;
              }                             /* end draw columns for the buffer */
            }
            free(inbufferstart);
          }       /********************end high quality refresh**************/
          fclose(in);
        }else{
                     /*****draw columns for this segment if it is scaled********/
          while(_column <= column2 && x < segmentend){

            if(x + bufferlength / frame * scaled > segmentend){
              bufferlength = ((segmentend - x + xscale) / xscale);
              bufferlength *= element;
            }
                                                /* # of bytes */

            fread(inbufferstart, bufferlength, 1, in);

            _inbuffer = (TWO *)inbufferstart;
            inbufferend = inbufferstart + bufferlength;
                                               /* draw columns for the segment */
            while((char *)_inbuffer < inbufferend){
                                          /* draw a column which is an element */
              _inbufferend = _inbuffer + element/2;

              if(_inbuffer < _inbufferend){
                highsample = *_inbuffer++;
                lowsample = *_inbuffer++;
              }
              while(_inbuffer < _inbufferend){
                if(*_inbuffer > highsample) highsample = *_inbuffer;
                _inbuffer++;
                if(*_inbuffer < lowsample) lowsample = *_inbuffer;
                _inbuffer++;
              }

              drawcolumn1(_column, centery, _yscale, highsample, lowsample);
              _column++;
              x += xscale;
            }                             /* end draw columns for the buffer */
          }                             /* end read buffers for this segment */

          free(inbufferstart);
          fclose(in);
        }                                        /* end if segment is scaled */
      }                        /* end segment isn't silence */

drawsilence:
      if(issilence){                /* segment is silence */
        {
          static int _column = -1, column1 = 0;
          sample = 0;
          drawcolumn2(_column, column1, _yscale, sample, centery);
        }
                                /* if segment is silence, increment pointers */
        _column += segmentlength  / xscale;
        x += segmentlength;
      }
      currentedit = currentedit->nextedit;
    }                   /* end display each segment one at a time */

                        /* draw automation one track at a time */
    if(automationvisible)
      drawautomation(column1, _row, column2, _row + yscaletrack, startx, track, 0);
    if(flash){
      XCopyArea(display, wavpixmap, wavwin, gc, column1, _row,
        column2 - column1, yscaletrack, column1, _row);
      XFlush(display);
    }
  }                   /* end draw one track at a time */

  drawtitles(column1, row1, column2, row2, startx, starty);
  drawlooppoints(column1, row1, column2, row2, startx);
  if(cursor){
  	drawcursor(column1, row1, column2, row2, startx, selectionstart, selectionend);
  	drawedits(column1, row1, column2, row2, startx, starty);
  }
  
  if(flash){
    XCopyArea(display, wavpixmap, wavwin, gc, column1, row1,
      column2 - column1, row2 - row1, column1, row1);
    XFlush(display);
  }
  return 0;
}
