/*
  Broadcast 2.0 multitrack audio editing
  (c) 1997 Adam Williams

  This program is distributed with the intent that it will be useful, without
  any warranty.  The code is being updated and has many bugs which are
  constantly being elucidated so changes should not be made without notifying
  the author.
*/

/* input.c    get numeric input from user   */


#include "interfac.h"
#include "input.h"
#include "eq.h"

struct inputwindow_ {
  int w, h;
  bcbutton ok;
  bcbutton cancel;
  bcbutton load;
  bcbutton save;
} inputwindow;

struct inputwindow_ inputwindow = {
  418, 160,
  {10, 0, 100, 30, "OK"},
  {300, 0, 100, 30, "Cancel"},
  {120, 0, 70, 30, "Load"},
  {200, 0, 70, 30, "Save"}
};

struct inputevent_ {
  char activebox;          /* # of textbox and slider being adjusted */
  char slideradjust;       /* slider adjustment */
} inputevent;

struct inputevent_ inputevent = {0, 0};
Window inputwin;
Pixmap inputpixmap;

static char inputfilename[256] = "";

void loadinitinputdir(FILE *in){
  fscanf(in, "%s", inputfilename);
}

void saveinitinputdir(FILE *out){
  char string[256];

  cutdirectory(string, inputfilename);
  /* only save if a string exists since fscanf would skip the next line */
  if(strlen(string)) fprintf(out, "INPUTDIR %s\n", string);
}

int loadnumbers(int totalnumbers, char loadsave, char *inputfilename, 
	bctextbox *textboxes, bcslider *sliders)
{
  FILE *in;
  int i;

  if(getfileattributes(inputfilename, 0, INPUT, "Load Parameters"))
    return 1;

  if(!(in = fopen(inputfilename, "rb"))){
    printf("Can't open %s\n", inputfilename);
  }

  if(fgetc(in) != loadsave){
    printf("Incorrect file type.\n");
    fclose(in);
    return 1;
  }

  for(i = 0; i < totalnumbers; i++){
		fscanf(in, "%s", textboxes[i].text);       /* load into textboxes */
  }
  if(sliders){
  	for(i = 0; i < totalnumbers; i++){
  		fscanf(in, "%f", &sliders[i].value);
  	}                               /* load into sliders */
  }
  fclose(in);
}

int savenumbers(int totalnumbers, char loadsave, 
	char *inputfilename, bctextbox *textboxes, bcslider *sliders)
{
  FILE *out;
  char string[256], string_[256];
  int i;

  if(getfileattributes(inputfilename, 0, INPUT, "Save Parameters"))
    return 1;

  if(out = fopen(inputfilename, "rb")){                     /* filename exits */
    fclose(out);

    cutfilename(string_, inputfilename);
    sprintf(string, "%s exists.   Overwrite?", string_);
    if(confirm(string, "Effect file")) return 1;
  }

  if(!(out = fopen(inputfilename, "wb"))){
    printf("Can't open %s for output.\n", inputfilename);
    return 1;
  }

  fputc(loadsave, out);
  for(i = 0; i < totalnumbers; i++){
    fprintf(out, "%s\n", textboxes[i].text);
  }
  if(sliders){
  	for(i = 0; i < totalnumbers; i++){
  		fprintf(out, "%f\n", sliders[i].value);
  	}
  }
  fclose(out);
}

void drawnumericinput(int totalnumbers, char *titles[], 
	bctextbox *textboxes, bcslider *sliders)
{
  int i;
	char string[256];     /* titles */

                  /* text boxes and titles */
  for(i = 0; i < totalnumbers; i++){
    XSetForeground(display, gc, black.pixel);
    XDrawString(display, inputpixmap, gc,
      textboxes[i].x + 5, textboxes[i].y - 4,
      titles[i], strlen(titles[i]));

    if(i == inputevent.activebox)
                               /* activate box */
      activate_box(inputpixmap, inputwin, &textboxes[i]);

    else
                                  /* input box */
      create_text_box(inputpixmap, inputwin, &textboxes[i]);
	}
	
                  /* sliders */
	if(sliders){
		for(i = 0; i < totalnumbers; i++){
	                            /* draw linear sliders if variable is nonzero */
	      initslider(inputwin, inputpixmap, &sliders[i], 1);
    }
  }

  XCopyArea(display, inputpixmap, inputwin, gc, 0, 0,       /* flash on screen */
    inputwindow.w, inputwindow.h, 0, 0);
}

/* edit numeric fields */
/* return 0 on success  return 1 on cancel */
/* provide values and data types */
/* point sliders to 0 for no sliders */
/* loadsave is 0 for no disk io  magic number for disk io */

int getnumericinput(int totalnumbers, 
	bctextbox *textboxes, bcslider *sliders, 
	char *titles[], char *windowtitle, char loadsave)
{
  XEvent report;
  Visual *vis;
  XVisualInfo visinfo;
  XSizeHints size_hints;
  XSetWindowAttributes attr;
  Cursor arrow;
  unsigned long mask;
  KeySym keysym;
  XComposeStatus composestatus;
  char keys_return[2];
  int x, y, y_, i;

  inputwindow.h = totalnumbers * 50 + 50;
  inputevent.activebox = 0;       	/* initialize events */
  inputevent.slideradjust = 0;      /* initialize events */

  /*=========================== open input window ===============================*/
  arrow = XCreateFontCursor(display, XC_top_left_arrow);
  vis = DefaultVisual(display, screen);

  mask = CWEventMask | CWBackPixel | CWBorderPixel | CWColormap |
    CWCursor;
  attr.event_mask = ExposureMask | ButtonPressMask | PointerMotionMask |
    ButtonReleaseMask | KeyPressMask;
  attr.background_pixel = grey.pixel;
  attr.border_pixel = dkgrey.pixel;
  attr.colormap = cmap;
  attr.cursor = arrow;

  inputwin = XCreateWindow(display, rootwin, 0, 0, inputwindow.w, inputwindow.h,
    0, depth, InputOutput, vis, mask, &attr);

  inputpixmap = XCreatePixmap(display, inputwin, inputwindow.w,
    inputwindow.h, depth);

  size_hints.flags = PSize;
  size_hints.width = inputwindow.w;
  size_hints.height = inputwindow.h;

  XSetStandardProperties(display, inputwin, windowtitle, windowtitle, None, 0, 0,
    &size_hints);

  XMapWindow(display, inputwin);

  XSetForeground(display, gc, grey.pixel);
  XFillRectangle(display, inputpixmap, gc, 0, 0, inputwindow.w, inputwindow.h);

  /*================================ buttons =================================*/
  inputwindow.ok.y = inputwindow.h - inputwindow.ok.h - 10;
  inputwindow.cancel.y = inputwindow.h - inputwindow.cancel.h - 10;
  DrawUpButton(inputpixmap, inputwin, &inputwindow.ok);
  DrawUpButton(inputpixmap, inputwin, &inputwindow.cancel);

                        /* load and save buttons */
  if(loadsave){
    inputwindow.load.y = inputwindow.h - inputwindow.load.h - 10;
    inputwindow.save.y = inputwindow.h - inputwindow.save.h - 10;
    DrawUpButton(inputpixmap, inputwin, &inputwindow.load);
    DrawUpButton(inputpixmap, inputwin, &inputwindow.save);
  }

                 /* set up text box locations */
  for(i = 0, y = 20; i < totalnumbers; i++, y += 50){
    textboxes[i].x = 10;
    textboxes[i].y = y;
    textboxes[i].w = 100;
    textboxes[i].h = 30;
  }

                 /* set up slider locations */
  if(sliders){
		for(i = 0, y = 20; i < totalnumbers; i++, y += 50){
			sliders[i].x = 120;
			sliders[i].y = y;
			sliders[i].w = 250;
			sliders[i].h = 16;
		}
	}

        /* initialize active box and display */
  drawnumericinput(totalnumbers, titles, textboxes, sliders);

                                /* event handler */
  do{
    XNextEvent(display, &report);
    switch(report.type){
      case Expose:
        if(report.xexpose.window == inputwin)
          XCopyArea(display, inputpixmap, inputwin, gc, 0, 0,
            inputwindow.w, inputwindow.h, 0, 0);
        else handleexpose(&report); 
        break;

      case KeyPress:
        keys_return[0] = '\0';
        XLookupString ((XKeyEvent *)&report, keys_return, 1, &keysym,
          &composestatus);

        switch(keysym){
          case XK_Escape:
            XDestroyWindow(display, inputwin);
            XFreePixmap(display, inputpixmap);
            return 1;
            break;

          default:
            switch(keys_return[0]){
              case 13:          /* successful completion */
                XDestroyWindow(display, inputwin);
                XFreePixmap(display, inputpixmap);
                return 0;
                break;

              default:          /* keyed data */
                if(inputevent.activebox < totalnumbers){
                  handle_text_event(inputpixmap, inputwin,
                    &report, &textboxes[inputevent.activebox]);
                  if(sliders){
                  	texttoslider(&textboxes[inputevent.activebox],
                  		&sliders[inputevent.activebox]);
                  }
                }
                break;
            }
            break;
        }
      break;                                      /* end case keypress */

      case ButtonPress:
        x = report.xbutton.x;
        y = report.xbutton.y;
                                       /* ok button pressed */
        if(buttonpressrelease(inputpixmap, inputwin, x, y, &inputwindow.ok)){
          XDestroyWindow(display, inputwin);
          XFreePixmap(display, inputpixmap);
          return 0;
        }
                                       /* cancel button pressed */
        if(buttonpressrelease(inputpixmap, inputwin, x, y,
          &inputwindow.cancel))
        {
          XDestroyWindow(display, inputwin);
          XFreePixmap(display, inputpixmap);
          return 1;
        }
                                       /* load or save button pressed */
        if(loadsave){
          if(buttonpressrelease(inputpixmap, inputwin, x, y,
            &inputwindow.load))
          {
            loadnumbers(totalnumbers, loadsave, inputfilename, textboxes, sliders);
            drawnumericinput(totalnumbers, titles, textboxes, sliders);
            break;
          }
          if(buttonpressrelease(inputpixmap, inputwin, x, y,
            &inputwindow.save))
          {
            savenumbers(totalnumbers, loadsave, inputfilename, textboxes, sliders);
            break;
          }
        }

          /* test sliders and text boxes */
        /* window has sliders */
        if(sliders){
	        for(i = 0; i < totalnumbers; i++){
	                                                          /* test slider */
	          if(testbox(x, y, (bcbox *)&sliders[i])){
	                                                       /* activate slider */
	            inputevent.slideradjust = 1;

	                                     /* deactivate old text box */
	            deactivate_box(inputpixmap, inputwin, &textboxes[inputevent.activebox]);
	            inputevent.activebox = i;
	            sliderevent(inputwin, inputpixmap, &report, &sliders[i], 1);
	                                     /* activate new text box */
	            strcpy(textboxes[i].text, sliders[i].text);
	            activate_box(inputpixmap, inputwin, &textboxes[inputevent.activebox]);
	          }
        	}
        }
        
        /* test text boxes */
       	for(i = 0; i < totalnumbers; i++){
          if(i < totalnumbers && testbox(x, y, (bcbox *)&textboxes[i])){
            if(inputevent.activebox == i){     /* old box editing */
              handle_text_event(inputpixmap, inputwin, &report,
                &textboxes[inputevent.activebox]);
            }else{                     /* new box selected */
                                       /* deactivate old text box */
              deactivate_box(inputpixmap, inputwin, &textboxes[inputevent.activebox]);
                                       /* activate new text box */
              activate_box(inputpixmap, inputwin, &textboxes[i]);
              inputevent.activebox = i;
              inputevent.slideradjust = 0;
            }
            i = totalnumbers;
          }
        }
        break;

      case ButtonRelease:
        if(inputevent.slideradjust) inputevent.slideradjust = 0;
        break;

      case MotionNotify:
        if(inputevent.slideradjust){
          sliderevent(inputwin, inputpixmap, &report, 
          	&sliders[inputevent.activebox], 1);
 	        strcpy(textboxes[inputevent.activebox].text, sliders[inputevent.activebox].text);
          activate_box(inputpixmap, inputwin, &textboxes[inputevent.activebox]);
        }
        break;

      default:
        break;
    }                           /* end switch report type */
  }while(1);                       /* end event handler */
}
