/*
  Broadcast 2.0 multitrack audio editing
  (c) 1997 Adam Williams

  This program is distributed with the intent that it will be useful, without
  any warranty.  The code is being updated and has many bugs which are
  constantly being elucidated so changes should not be made without notifying
  the author.
*/

/* interfac.c      simple interfaces, progress box  */


#include "bcast.h"
#include "interfac.h"
#include <sys/time.h>

struct DelWindow delwindow = {
  530, 90, "Save?",
  {10, 40, 100, 30, "Yes"},
  {200, 40, 100, 30, "No"},
  {350, 40, 100, 30, "Cancel"}
};

struct ConfirmWindow confirmwindow = {
  325, 90, "",
  {10, 40, 100, 30, "Yes"},
  {200, 40, 100, 30, "No"}
};

struct MessageWindow messagewindow = {
  530, 90, "Message",
  {10, 40, 100, 30, "OK"}
};

unsigned long timer = 0;

int testbox(int x, int y, bcbox *box){
  if(x >= box->x && x <= box->x + box->w){
    if(y >= box->y && y <= box->y + box->h){
      return 1;
    }
  }
  return 0;
}

int testbutton(int x, int y, bcbutton *button){
  if(x > button->x && x < button->x + button->w){
    if(y > button->y && y < button->y + button->h){
      return 1;
    }
  }
  return 0;
}

int buttonpressrelease(Drawable wdw, Window win, int x, int y, bcbutton *button){
  static XEvent report;
  static int x1, y1;

  if(x > button->x && x < button->x + button->w &&
    y > button->y && y < button->y + button->h)
  {
    DrawDnButton(wdw, win, button);
    do{
      XNextEvent(display, &report);
    }while(report.type != ButtonRelease);
    x1 = report.xbutton.x;
    y1 = report.xbutton.y;

    DrawUpButton(wdw, win, button);
    if(x1 > button->x && x1 < button->x + button->w &&
      y1 > button->y && y1 < button->y + button->h)
    {
        return 1;
    }else
      return 0;
  }
  return 0;
}

             /* use this during slow routines to reduce event queue to 
                button release or single motion events */
void flushevents(){
	static XEvent report;
	static int motion, button, x, y;
	
	motion = 0;
	button = 0;

  do{
    if(XPending(display)) XNextEvent(display, &report);
    else report.type = 0;
    if(report.type == MotionNotify){
    	motion = 1;
    	x = report.xmotion.x;
    	y = report.xmotion.y;
    }
    if(report.type == ButtonRelease) button = 1;
  }while(XPending(display));
  
  if(motion){
  	report.type = MotionNotify;
  	report.xmotion.x = x;
  	report.xmotion.y = y;
  	XPutBackEvent(display, &report);
	}
	
  if(button){
  	report.type = ButtonRelease;
  	XPutBackEvent(display, &report);
	}
}

/* return 1 if # of milliseconds have passed since last check */

int checktimer(){
/*  static unsigned long timer_;
  static struct timeval tp;
  static char result;

  result = 0;
  gettimeofday(&tp, 0);
  timer_ = tp.tv_usec;
  result = (timer_ - timer > 30000);
  timer = timer_;
  return result;
*/
}


void truncatefilename(char *output, char *input){
  static char *input_;
  static int i;

  input_ = &input[strlen(input) - 1];
  for(i = 0; input_ > input && i < 25; i++){
    input_--;
  }
  if(input_ > input) sprintf(output, "...%s", input_);
  else strcpy(output, input);
}

int confirmdel(char *string){
  XEvent report;
  Visual *vis;
  XVisualInfo visinfo;
  XSizeHints size_hints;
  XSetWindowAttributes attr;
  Cursor arrow;
  unsigned long mask;
  Window delwin;
  Pixmap delpixmap;
  KeySym keysym;
  XComposeStatus composestatus;
  char keys_return[2];
  int x, y;

                                   /* open delete window */
  arrow = XCreateFontCursor(display, XC_top_left_arrow);
  vis = DefaultVisual(display, screen);

  mask = CWEventMask | CWBackPixel | CWBorderPixel | CWColormap |
    CWCursor;
  attr.event_mask = ExposureMask | ButtonPressMask |
    ButtonReleaseMask | KeyPressMask;
  attr.background_pixel = grey.pixel;
  attr.border_pixel = dkgrey.pixel;
  attr.colormap = cmap;
  attr.cursor = arrow;

  delwin = XCreateWindow(display, rootwin, 0, 0, delwindow.w, delwindow.h,
    0, depth, InputOutput, vis, mask, &attr);

  delpixmap =   XCreatePixmap(display, delwin, delwindow.w, delwindow.h, depth);

  size_hints.flags = PSize;
  size_hints.width = delwindow.w;
  size_hints.height = delwindow.h;

  XSetStandardProperties(display, delwin, delwindow.title, "Save?", None, 0, 0,
    &size_hints);

  XMapWindow(display, delwin);

  XSetForeground(display, gc, grey.pixel);
  XFillRectangle(display, delpixmap, gc, 0, 0, delwindow.w, delwindow.h);
  XSetForeground(display, gc, black.pixel);
  XDrawString(display, delpixmap, gc, 10, 30, string, strlen(string));

                                /* buttons */
  DrawUpButton(delpixmap, delwin, &delwindow.yes);
  DrawUpButton(delpixmap, delwin, &delwindow.no);
  DrawUpButton(delpixmap, delwin, &delwindow.cancel);
  XCopyArea(display, delpixmap, delwin, gc, 0, 0, delwindow.w, delwindow.h, 0, 0);

                                /* event handler */
  do{
    XNextEvent(display, &report);
    switch(report.type){
      case Expose:
        if(report.xexpose.window == delwin)
          XCopyArea(display, delpixmap, delwin, gc, 0, 0, delwindow.w, delwindow.h, 0, 0);
        else handleexpose(&report);
        break;

      case KeyPress:
        keys_return[0] = '\0';
        XLookupString ((XKeyEvent *)&report, keys_return, 1, &keysym, &composestatus);
        switch(keysym){
          case XK_Escape:
            XDestroyWindow(display, delwin);
            XFreePixmap(display, delpixmap);
            return 1;
            break;

          default:
            switch(keys_return[0]){
              case 'n':
                XDestroyWindow(display, delwin);
                XFreePixmap(display, delpixmap);
                return 0;
                break;

              case 'y':
                XDestroyWindow(display, delwin);
                XFreePixmap(display, delpixmap);
                if(saveasfile()) return 1;
                return 0;
                break;
            }
            break;
        }
        break;

      case ButtonPress:
        x = report.xbutton.x;
        y = report.xbutton.y;

        if(buttonpressrelease(delpixmap, delwin, x, y, &delwindow.yes)){
          XDestroyWindow(display, delwin);
          XFreePixmap(display, delpixmap);
          if(saveasfile()) return 1;
          return 0;
        }
        if(buttonpressrelease(delpixmap, delwin, x, y, &delwindow.no)){
          XDestroyWindow(display, delwin);
          XFreePixmap(display, delpixmap);
          return 0;
        }
        if(buttonpressrelease(delpixmap, delwin, x, y, &delwindow.cancel)){
          XDestroyWindow(display, delwin);
          XFreePixmap(display, delpixmap);
          return 1;
        }
        break;

      default:
        break;
    }                           /* end switch report type */
  }while(1);                             /* end event handler */
}

int confirm(char *string, char *title){
  XEvent report;
  Visual *vis;
  XVisualInfo visinfo;
  XSizeHints size_hints;
  XSetWindowAttributes attr;
  Cursor arrow;
  unsigned long mask;
  Window confirmwin;
  Pixmap confirmpixmap;
  KeySym keysym;
  XComposeStatus composestatus;
  char keys_return[2];
  int x, y;

                                   /* open confirm window */
  arrow = XCreateFontCursor(display, XC_top_left_arrow);
  vis = DefaultVisual(display, screen);

  mask = CWEventMask | CWBackPixel | CWBorderPixel | CWColormap |
    CWCursor;
  attr.event_mask = ExposureMask | ButtonPressMask |
    ButtonReleaseMask | KeyPressMask | KeyReleaseMask;
  attr.background_pixel = grey.pixel;
  attr.border_pixel = dkgrey.pixel;
  attr.colormap = cmap;
  attr.cursor = arrow;

  confirmwin = XCreateWindow(display, rootwin, 0, 0, confirmwindow.w, confirmwindow.h,
    0, depth, InputOutput, vis, mask, &attr);

  confirmpixmap =   XCreatePixmap(display, confirmwin, confirmwindow.w, confirmwindow.h, depth);

  size_hints.flags = PSize;
  size_hints.width = confirmwindow.w;
  size_hints.height = confirmwindow.h;

  XSetStandardProperties(display, confirmwin, title, "Confirm", None, 0, 0,
    &size_hints);

  XMapWindow(display, confirmwin);

  XSetForeground(display, gc, grey.pixel);
  XFillRectangle(display, confirmpixmap, gc, 0, 0, confirmwindow.w, confirmwindow.h);
  XSetForeground(display, gc, black.pixel);
  XDrawString(display, confirmpixmap, gc, 10, 30, string, strlen(string));

                                /* buttons */
  DrawUpButton(confirmpixmap, confirmwin, &confirmwindow.yes);
  DrawUpButton(confirmpixmap, confirmwin, &confirmwindow.no);
  XCopyArea(display, confirmpixmap, confirmwin, gc, 0, 0,
    confirmwindow.w, confirmwindow.h, 0, 0);

                                /* event handler */
  do{
    XNextEvent(display, &report);
    switch(report.type){
      case Expose:
        XCopyArea(display, confirmpixmap, confirmwin, gc, 0, 0, confirmwindow.w, confirmwindow.h, 0, 0);
        break;

      case KeyPress:
        keys_return[0] = '\0';
        XLookupString ((XKeyEvent *)&report, keys_return, 1, &keysym, &composestatus);
        switch(keys_return[0]){
          case 'n':
            XDestroyWindow(display, confirmwin);
            XFreePixmap(display, confirmpixmap);
            return 1;
            break;

          case 'y':
            XDestroyWindow(display, confirmwin);
            XFreePixmap(display, confirmpixmap);
            return 0;
            break;

          case 13:
            XDestroyWindow(display, confirmwin);
            XFreePixmap(display, confirmpixmap);
            return 0;
            break;
        }
        break;

      case ButtonPress:
        x = report.xbutton.x;
        y = report.xbutton.y;

        if(buttonpressrelease(confirmpixmap, confirmwin, x, y, &confirmwindow.yes)){
          XDestroyWindow(display, confirmwin);
          XFreePixmap(display, confirmpixmap);
          return 0;
        }
        if(buttonpressrelease(confirmpixmap, confirmwin, x, y, &confirmwindow.no)){
          XDestroyWindow(display, confirmwin);
          XFreePixmap(display, confirmpixmap);
          return 1;
        }
        break;

      default:
        break;
    }                           /* end switch report type */
  }while(1);                             /* end event handler */
}

int displayalert(char *string){
  XEvent report;
  Visual *vis;
  XVisualInfo visinfo;
  XSizeHints size_hints;
  XSetWindowAttributes attr;
  Cursor arrow;
  unsigned long mask;
  Window messagewin;
  Pixmap messagepixmap;
  KeySym keysym;
  XComposeStatus composestatus;
  char keys_return[2];
  int x, y;

                                   /* open message window */
  arrow = XCreateFontCursor(display, XC_top_left_arrow);
  vis = DefaultVisual(display, screen);

  mask = CWEventMask | CWBackPixel | CWBorderPixel | CWColormap |
    CWCursor;
  attr.event_mask = ExposureMask | ButtonPressMask |
    ButtonReleaseMask | KeyPressMask | KeyReleaseMask;
  attr.background_pixel = grey.pixel;
  attr.border_pixel = dkgrey.pixel;
  attr.colormap = cmap;
  attr.cursor = arrow;

  messagewin = XCreateWindow(display, rootwin, 0, 0, messagewindow.w,
  messagewindow.h,
    0, depth, InputOutput, vis, mask, &attr);

  messagepixmap =   XCreatePixmap(display, messagewin, messagewindow.w,
  messagewindow.h, depth);

  size_hints.flags = PSize;
  size_hints.width = messagewindow.w;
  size_hints.height = messagewindow.h;

  XSetStandardProperties(display, messagewin, messagewindow.title,
    messagewindow.title, None, 0, 0, &size_hints);

  XMapWindow(display, messagewin);

  XSetForeground(display, gc, grey.pixel);
  XFillRectangle(display, messagepixmap, gc, 0, 0, messagewindow.w,
  messagewindow.h);
  XSetForeground(display, gc, black.pixel);
  XDrawString(display, messagepixmap, gc, 10, 30, string, strlen(string));

                                /* buttons */
  DrawUpButton(messagepixmap, messagewin, &messagewindow.ok);
  XCopyArea(display, messagepixmap, messagewin, gc, 0, 0,
    messagewindow.w, messagewindow.h, 0, 0);

                                /* event handler */
  do{
    XNextEvent(display, &report);
    switch(report.type){
      case Expose:
        XCopyArea(display, messagepixmap, messagewin, gc, 0, 0,
        messagewindow.w, messagewindow.h, 0, 0);
        break;

      case KeyPress:
        keys_return[0] = '\0';
        XLookupString ((XKeyEvent *)&report, keys_return, 1, &keysym,
        &composestatus);
        switch(keys_return[0]){
          case 13:
            XDestroyWindow(display, messagewin);
            XFreePixmap(display, messagepixmap);
            return 0;
            break;
        }
        break;

      case ButtonPress:
        x = report.xbutton.x;
        y = report.xbutton.y;

        if(buttonpressrelease(messagepixmap, messagewin, x, y, &messagewindow.ok)){
          XDestroyWindow(display, messagewin);
          XFreePixmap(display, messagepixmap);
          return 0;
        }
        break;

      default:
        break;
    }                           /* end switch report type */
  }while(1);                             /* end event handler */
}

