/*
  Broadcast 2.0 multitrack audio editing
  (c) 1997 Adam Williams

  This program is distributed with the intent that it will be useful, without
  any warranty.  The code is being updated and has many bugs which are
  constantly being elucidated so changes should not be made without notifying
  the author.
*/

/* listbox.c  edit a list of numbers */

#include "defs.h"
#include "interfac.h"
#include "listbox.h"

struct listwindow_ {
  TWO w;
  TWO h;
  char *title;
  bcbutton delete;
  bcbutton ok;
  bcbutton cancel;
} listwindow;

struct listwindow_ listwindow = {
  250, 360, "List Selection",
  {140, 40, 100, 30, "Delete Group"},
  {10, 310, 100, 30, "OK"},
  {140, 310, 100, 30, "Cancel"}
};

struct listevent_ {
  char button;
  char value;
} listevent;

bcscroll listscroll = {105, 25, 12, 265, 0, 0, 0, 0};
bcbox listbox = {5, 25, 100, 265};
Window listwin;
Pixmap listpixmap;
struct listevent_ listevent = {0, 0};

void drawlist(char *list, TWO totalelements){
  static int i, y;
  static char string[5];

                                               /* clear list box */
  XSetForeground(display, gc, white.pixel);
  XFillRectangle(display, listpixmap, gc, listbox.x, listbox.y,
    listbox.w, listbox.h);
  XSetForeground(display, gc, black.pixel);
  XDrawRectangle(display, listpixmap, gc, listbox.x, listbox.y,
    listbox.w, listbox.h);

                                               /* draw elements */
  for(y = listbox.y + 17, i = listscroll.position;
    i < totalelements && y < listbox.y + listbox.h; i++, y += 20)
  {
    sprintf(string, "%d", i+1);
    XDrawString(display, listpixmap, gc, listbox.x + 5, y,
      string, strlen(string));
    if(list[i]){
      XSetForeground(display, gc, white.pixel);
      XSetFunction(display, gc, GXxor);
      XFillRectangle(display, listpixmap, gc, listbox.x + 1, y - 16,
        listbox.w - 1, 20);
      XSetFunction(display, gc, GXcopy);
      XSetForeground(display, gc, black.pixel);
    }
  }

  XCopyArea(display, listpixmap, listwin, gc, listbox.x, listbox.y - 20,
    listbox.w, listbox.h + 20, listbox.x, listbox.y - 20);
  XFlush(display);
}

/* move up list number of elements */
void listmoveup(TWO number, char *list, TWO totalelements){
  listscroll.position -= number;

  if(listscroll.position < 0) listscroll.position = 0;
  updateyscrollbar(listpixmap, listwin, listscroll.position,
    listscroll.span, listscroll.size, &listscroll);
  drawlist(list, totalelements);
}

/* move down list number of elements */
void listmovedown(TWO number, char *list, TWO totalelements){
  listscroll.position += number;

  if(listscroll.position > totalelements - listscroll.span)
    listscroll.position = totalelements - listscroll.span;
  updateyscrollbar(listpixmap, listwin, listscroll.position,
    listscroll.span, listscroll.size, &listscroll);
  drawlist(list, totalelements);
}

/* select list elements */
void handlelistevent(XEvent *report, char *list, int totalelements)
{
  static int x, y, i, selected;

  switch(report->type){
    case MotionNotify:  /* scroll up or down and select next item if out of bounds */
      x = report->xmotion.x;
      y = report->xmotion.y;

      if(y > listbox.y + listbox.h){
        listmovedown(1, list, totalelements);
        selected = listscroll.position + listscroll.span;
        if(selected >= totalelements) selected = totalelements - 1;
      }
      else
      if(y < listbox.y){
        listmoveup(1, list, totalelements);
      	selected = listscroll.position;
      }else
      {
      	selected = (y - listbox.y) / 20 + listscroll.position;
      	if(selected >= totalelements) selected = totalelements - 1;
			}
			
      list[selected] = listevent.value;        
      drawlist(list, totalelements);
      break;

    case ButtonRelease:
      listevent.button = 0;
      break;

    case ButtonPress:           /* button press inside list box */
      x = report->xbutton.x;
      y = report->xbutton.y;

                                                  /* locate specific element */
      selected = (y - listbox.y) / 20 + listscroll.position;
      if(selected < totalelements) list[selected] ^= 1;
      listevent.value = list[selected];
      listevent.button = 1;

      drawlist(list, totalelements);
      break;
  }
}

void listscrollevent(XEvent *report, char *newlist, TWO totalelements){
  switch(listscroll.event){
    case SPAN:
      yscrollevent(listpixmap, listwin, report, &listscroll);
      drawlist(newlist, totalelements);
      break;
    case ARROW1:
      listmoveup(1, newlist, totalelements);
      break;
    case ARROW2:
      listmovedown(1, newlist, totalelements);
      break;
    case UPPAGE:
      listmoveup(listwindow.h / 20, newlist, totalelements);
      break;
    case DNPAGE:
      listmovedown(listwindow.h / 20, newlist, totalelements);
      break;
  }
}

/* toggle a list of elements on or off */
TWO editlist(char *list, TWO totalelements, char *notice, char operation){
  XEvent report, nextreport;
  Visual *vis;
  XVisualInfo visinfo;
  XSizeHints size_hints;
  XSetWindowAttributes attr;
  Cursor arrow;
  unsigned FOUR mask;
  KeySym keysym;
  XComposeStatus composestatus;
  char keys_return[2];
  char *newlist;
  TWO x, y, i;

  newlist = malloc(totalelements);
  for(i = 0; i < totalelements; i++) newlist[i] = list[i];

                                   /* open list window */
  arrow = XCreateFontCursor(display, XC_top_left_arrow);
  vis = DefaultVisual(display, screen);

  mask = CWEventMask | CWBackPixel | CWBorderPixel | CWColormap |
    CWCursor;
  attr.event_mask = ExposureMask | ButtonPressMask |
    ButtonReleaseMask | KeyPressMask | PointerMotionMask;
  attr.background_pixel = grey.pixel;
  attr.border_pixel = dkgrey.pixel;
  attr.colormap = cmap;
  attr.cursor = arrow;

  listwin = XCreateWindow(display, rootwin, 0, 0, listwindow.w, listwindow.h,
    0, depth, InputOutput, vis, mask, &attr);

  listpixmap =   XCreatePixmap(display, listwin,
    listwindow.w, listwindow.h, depth);

  size_hints.flags = PSize;
  size_hints.width = listwindow.w;
  size_hints.height = listwindow.h;

  XSetStandardProperties(display, listwin, "Edit list", "Edit list", None, 0, 0,
    &size_hints);

  XMapWindow(display, listwin);

  XSetForeground(display, gc, grey.pixel);
  XFillRectangle(display, listpixmap, gc, 0, 0, listwindow.w, listwindow.h);

  updateyscrollbar(listpixmap, listwin, 0, listbox.h / 20, totalelements,
    &listscroll);
  XSetForeground(display, gc, black.pixel);
  XDrawString(display, listpixmap, gc, listbox.x,
    listbox.y - 3, notice, strlen(notice));
  drawlist(newlist, totalelements);

                                /* buttons */
  DrawUpButton(listpixmap, listwin, &listwindow.ok);
  DrawUpButton(listpixmap, listwin, &listwindow.cancel);
  if(operation == EDITGROUP) DrawUpButton(listpixmap, listwin, &listwindow.delete);


  XCopyArea(display, listpixmap, listwin, gc, 0, 0, listwindow.w,
    listwindow.h, 0, 0);
                                /* event handler */
  do{
                     /* this event reader works best */
    do{
      XNextEvent(display, &report);
      if(XPending(display)) XPeekEvent(display, &nextreport);
      else nextreport.type = 0;
    }while(
      (nextreport.type == MotionNotify && report.type == MotionNotify)
    );


    switch(report.type){
      case Expose:
        if(report.xexpose.window == listwin)
          XCopyArea(display, listpixmap, listwin, gc, 0, 0, listwindow.w,
            listwindow.h, 0, 0);
        else handleexpose(&report);
        break;

      case KeyPress:
        keys_return[0] = '\0';
        XLookupString ((XKeyEvent *)&report, keys_return, 1, &keysym,
          &composestatus);

        switch(keysym){
          case XK_Escape:
            XDestroyWindow(display, listwin);
            XFreePixmap(display, listpixmap);
            free(newlist);
            return 1;
            break;

					case XK_Up:
			      listmoveup(1, newlist, totalelements);
						break;

					case XK_Down:
			      listmovedown(1, newlist, totalelements);
						break;

					case XK_Prior:
			      listmoveup(listwindow.h / 20, newlist, totalelements);
						break;

					case XK_Next:
			      listmovedown(listwindow.h / 20, newlist, totalelements);
						break;

          default:
            switch(keys_return[0]){
              case 13:
                XDestroyWindow(display, listwin);
                XFreePixmap(display, listpixmap);
                for(i = 0; i < totalelements; i++)
                  list[i] = newlist[i];
                free(newlist);
                return 0;
                break;
            }
            break;
        }
        break; /* end case keypress */

      case MotionNotify:
        if(listscroll.event) listscrollevent(&report, newlist, totalelements);
        if(listevent.button) handlelistevent(&report, newlist, totalelements);
        break;

      case ButtonRelease:
        if(listscroll.event){
          yscrollevent(listpixmap, listwin, &report, &listscroll);
        }
        if(listevent.button) handlelistevent(&report, newlist, totalelements);
        break;

      case ButtonPress:
        x = report.xbutton.x;
        y = report.xbutton.y;

        if(testbox(x, y, (bcbox *)&listscroll))
        {
          yscrollevent(listpixmap, listwin, &report, &listscroll);
          if(listscroll.event) listscrollevent(&report, newlist, totalelements);
          goto endbuttonpress;
        }
        if(testbox(x, y, &listbox))
        {
          handlelistevent(&report, newlist, totalelements);
          goto endbuttonpress;
        }
        if(buttonpressrelease(listpixmap, listwin, x, y, &listwindow.ok)){
          XDestroyWindow(display, listwin);
          XFreePixmap(display, listpixmap);
          for(i = 0; i < totalelements; i++)
            list[i] = newlist[i];
          free(newlist);
          return 0;
        }
        if(buttonpressrelease(listpixmap, listwin, x, y, &listwindow.cancel)){
          XDestroyWindow(display, listwin);
          XFreePixmap(display, listpixmap);
          free(newlist);
          return 1;
        }
        if(operation == EDITGROUP &&
          buttonpressrelease(listpixmap, listwin, x, y, &listwindow.delete))
        {
          XDestroyWindow(display, listwin);
          XFreePixmap(display, listpixmap);
          free(newlist);
          return 2;
        }
endbuttonpress:
        break;

      default:
        break;
    }                           /* end switch report type */
  }while(1);                       /* end event handler */
}

