/*
  Broadcast 2.0 multitrack audio editing
  (c) 1997 Adam Williams

  This program is distributed with the intent that it will be useful, without
  any warranty.  The code is being updated and has many bugs which are
  constantly being elucidated so changes should not be made without notifying
  the author.
*/

/* menu.c      menu bar handler  */

#include "defs.h"
#include "interfac.h"
#include "graphics.h"
#include "menu.h"
#include "edit.h"
#include "play.h"
#include "file.h"
#include "effects.h"
#include "automate.h"
#include "bcast.h"
#include "cd2drive.h"
#include "compress.h"

typedef struct _MenuItem {
            char text[64];
            char hotkey[10];
            int id;
            int active;
        } MenuItem;

static MenuItem FileMenu[] = {
    { "New...",            "n",       1, 1},
    { "Open...",           "o",      26, 1},
    { "Import...",         "i",      27, 1},
    { "CD 2 Drive...",      "",      57, 1},
    { BAR,                  "",      0,  1},
    { "Save to file",      "s",      3,  1},
    { "Play to file",       "",     32,  1},
    { "Exit",              "q",      4,  1}
};

static MenuItem EditMenu[] = {
    { "Undo",               "z",      5, 1},
    { "Redo",               "Z",     56, 1},
    { BAR,                  "",       0, 1},
    { "Cut",                "x",      6, 1},
    { "Copy",               "c",      7, 1},
    { "Paste",              "v",      8, 1},
    { "Paste silence",      "Space", 51, 1},
    { "Clear",              "Del",    9, 1},
    { BAR,                  "",       0, 1},
    { "Cut autos",          "X",     47, 1},
    { "Copy autos",         "C",     48, 1},
    { "Paste autos",        "V",     49, 1},
    { "Clear autos",        "DEL",   50, 1},
    { BAR,                  "",       0, 1},
    { "Toggle label",       "l",     29, 1},
    { "Shift labels",       "",      52, 1},
};

static MenuItem ViewMenu[] = {
    { "Expand x",             "Dn",      10, 1},
    { "Shrink x",             "Up",      11, 1},
    { "Fit x",                "f",       12, 1},
    { BAR,                    "",         0, 1},
    { "Expand y",             "^Dn",     13, 1},
    { "Shrink y",             "^Up",     14, 1},
    { BAR,                    "",         0, 1},
    { "Expand track",         "^PgDn",   15, 1},
    { "Shrink track",         "^PgUp",   16, 1} };

static MenuItem EffectsMenu[] = {
    { "Mute",                  "m",      31, 1},
    { "Crossfade",             "",       33, 1},
    { "Normalize",             "",       34, 1},
    { "Balance",               "",       35, 1},
    { "Reverb",                "",       46, 1},
    { "Backwards",             "",       54, 1},
    { "Compress",               "",       55, 1}
};

static MenuItem SettingsMenu[] = {
    { "Set output channels",             "",       21, 1},
    { "Set sample rate",                 "",       22, 1},
    { "Set play buffer",                 "",       36, 1},
    { "Automation",                      "a",      40, 1},
    { "Loop playback",                   "L",      45, 1},
    { BAR,                               "",        0, 1},
    { "Add track",                       "t",      23, 1},
    { "Delete track",                    "d",      24, 1},
    { "Add track group",                 "",       41, 1}
};

static MenuItem WindowMenu[] = {
  { "",                                "",       38, 1},
  { "Show titles",                     "",       30, 2},
  { "Show edits",                      "",       53, 1},
  { "High quality refresh",            "",       37, 2},
  { BAR,                               "",       0,  1},
  { "Mute Automation",                 "",       39, 1},
  { "Fade Automation",                 "",       42, 1},
  { "Pan Automation",                  "~",      43, 1},
  { "Inverse Automation",              "",       44, 1},
};

static MenuItem *PanMenu;

typedef struct _BarItem {
            MenuItem *menu;
            char *text;
            int items;
            int w1;                     /* width of menu title */
            int w2;                     /* width of popup window */
            int x2;                     /* x position of hotkeys */
        } BarItem;

static BarItem MenuBar[] = {
    { FileMenu,        "File",         8,        50, 130, 100},
    { EditMenu,        "Edit",         16,       50, 180, 130},
    { ViewMenu,        "View",         9,        50, 160, 100},
    { EffectsMenu,     "Effects",      6,        70, 130, 100},
    { SettingsMenu,    "Settings",     9,        80, 200, 170},
    { WindowMenu,      "Window",       9,        80, 190, 170},
    { 0,               "Pans",         0,        0,   120, 0} }; /* sub menu */

const int menuheight = 22;

void drawmenubar(int selected){
  static int i, x;
  
  DrawUpMenuBox(menupixmap, 0, 0, menuwindow.w-1, menuwindow.h-1);

  for(i = 0, x = 20; i < MENUS; i++){
    if(i == selected)
      DrawDnMenuBox(menupixmap, x, 0, MenuBar[i].w1-1, menuwindow.h-1);

    XDrawString(display, menupixmap, gc, x+5, menuwindow.h-7, MenuBar[i].text,
      strlen(MenuBar[i].text));
    x += MenuBar[i].w1;
  }
  refreshmenubar();
}

void initializemenu(){
  static Window tempwin;
  static int x, y;

  XTranslateCoordinates(display, menuwin, rootwin,
    menuevent.x, menuwindow.h, &x, &y, &tempwin);

  XMoveResizeWindow(display, popupwin, x, y,
    MenuBar[menuevent.menu].w2, menuheight * MenuBar[menuevent.menu].items);
  popuppixmap = XCreatePixmap(display, popupwin,
    MenuBar[menuevent.menu].w2,  menuheight * MenuBar[menuevent.menu].items, depth);
  XRaiseWindow(display, popupwin);
  XMapWindow(display, popupwin);
}

/* initialize the submenu for display pans */
void initializepanmenu(int selected){
  static Window tempwin;
  static int x, y, i;

  menuevent.submenu = 6;            /* set events */
  menuevent.submenuon = 1;
  menuevent.x += MenuBar[menuevent.menu].x2;
  menuevent.y = selected * menuheight + menuwindow.h;

	PanMenu = malloc(sizeof(MenuItem) * outchannels);
	for(i = 0; i < outchannels; i++){
		sprintf(PanMenu[i].text, "Channel %d", i + 1);
		sprintf(PanMenu[i].hotkey, "");
		PanMenu[i].id = i;
		PanMenu[i].active = autoevent.showpan[i] + 1;
	}
	
	MenuBar[menuevent.submenu].items = outchannels;
	MenuBar[menuevent.submenu].menu = PanMenu;

  XTranslateCoordinates(display, popupwin, rootwin,
    MenuBar[menuevent.menu].x2, selected * menuheight, 
    &x, &y, &tempwin);
    
                        /* show window and set event for y */
  XMoveResizeWindow(display, popupsubwin, x, y,
    MenuBar[menuevent.submenu].w2, menuheight * MenuBar[menuevent.submenu].items);
  popupsubpixmap = XCreatePixmap(display, popupsubwin,
    MenuBar[menuevent.submenu].w2,  menuheight * MenuBar[menuevent.submenu].items, depth);
  XRaiseWindow(display, popupsubwin);
  XMapWindow(display, popupsubwin);
}

/* destroy the submenu for display pans */
void destroypanmenu(){
	static int i;

  XUnmapWindow(display, popupsubwin);
  XFreePixmap(display, popupsubpixmap);
  /*for(i = 0; i < outchannels; i++){
  	free(PanMenu[i].text);
  	free(PanMenu[i].hotkey);
  }*/
  menuevent.submenuon = 0;
  menuevent.x -= MenuBar[menuevent.menu].x2;
}

void destroymenu(){
  XUnmapWindow(display, popupwin);
  XFreePixmap(display, popuppixmap);
}

/* draw a menu or a submenu */
void drawmenu(Drawable wdw, Window win, int selected, int menu){
  static int i, y, items, w;
  static MenuItem *item;
  static int x2, x1;

  items = MenuBar[menu].items;
  w = MenuBar[menu].w2;
  item = MenuBar[menu].menu;
  x2 = MenuBar[menu].x2;
  x1 = 5;

  for(i = 0; i < items && x1 == 5; i++)
    if(item[i].active == 2) x1 = 20;


  DrawUpMenuBox(wdw, 0, 0, w-1, menuheight * items - 1);
  for(y = 0, i = 0; i < items; i++, y += menuheight){
    if(selected == i){
      DrawDnMenuBox(wdw, 0, y, w-1, menuheight-1);
    }
    if(strcmp(item->text, BAR)){
                          /* item is active */
      if(item->active){
        XSetForeground(display, gc, black.pixel);
                            /* checked off */
        if(item->active == 2){
          XDrawString(display, wdw, gc, 5, y + menuheight - 5, "X", 1);
        }
      }else                  /* item is inactive */
        XSetForeground(display, gc, dkgrey.pixel);
                            /* draw text */
      XDrawString(display, wdw, gc, x1, y + menuheight - 5,
        item->text, strlen(item->text));
                            /* draw hotkey */
      if(item->hotkey[0] != '~')       /* standard hotkey */
      	XDrawString(display, wdw, gc, x2, y + menuheight - 5,
        	item->hotkey, strlen(item->hotkey));
      else                           /* submenu icon */
        DrawRightArrow(wdw, x2, y + menuheight / 2);
        
    }else
      XDrawLine(display, wdw, gc, 5, y + 15,
        MenuBar[menu].w2 - 5, y + 15);
    item++;
  }
  XCopyArea(display, wdw, win, gc, 0, 0, w, menuheight*items, 0, 0);
  XFlush(display);
}

int menubuttonpress(XEvent *report){
	if(menuevent.submenuon){
		menuevent.buttondown = 1;
		/* submenucommand(); */
		return 0;
	}
	
  if(menuevent.menuon){
		menuevent.buttondown = 1;
    /* menucommand(); */
    return 0;
  }
  
  if(menuevent.baron){
  	if(menuevent.menu != -1){
  		destroymenu();
  		menuevent.menu = -1;
  	  drawmenubar(menuevent.menu);
  	}
    menuevent.baron = 0;
    return 0;
  }

  menuevent.baron = 1;
  menuevent.menuon = 0;
  menuevent.menu = -1;
  menuevent.buttondown = 1;
  menubarmotion(report);
  return 0;
}

void menubarmotion(XEvent *report){
  static int i, x1, x, y, newmenu;
  static Window tempwin;

  x = report->xmotion.x;
  y = report->xmotion.y;
  
  XTranslateCoordinates(display, report->xmotion.window, win,
    x, y, &x, &y, &tempwin);

          /* cursor inside menu bar */
  if(y < menuwindow.h)
  {
    newmenu = -1;
    for(i = 0, x1 = 20; i < MENUS; i++){
      if(x > x1 && x < MenuBar[i].w1 + x1){
        newmenu = i;
        menuevent.x = x1;
      }
      x1 += MenuBar[i].w1;
    }
    if(newmenu != menuevent.menu){
      if(menuevent.menu != -1) destroymenu();
      menuevent.menu = newmenu;
      drawmenubar(menuevent.menu);
      menuevent.item = -1;
      if(menuevent.menu != -1){
        initializemenu();
        drawmenu(popuppixmap, popupwin, menuevent.item, menuevent.menu);
      }
    }
  }else{        /* cursor inside menu */
    if(menuevent.menu != -1){
      menuevent.menuon = 1;
      menuevent.baron = 0;
      menuevent.item = -1;
    }
  }
}

void menumotion(XEvent *report){
  static int i, y1, x, y, newitem;
  static Window tempwin;

  x = report->xmotion.x;
  y = report->xmotion.y;

  XTranslateCoordinates(display, report->xmotion.window, win,
    x, y, &x, &y, &tempwin);

  newitem = -1;
                     /* y is below menu top */
 	if(y >= menuwindow.h){
                    	/* x is inside menu */
  	if(x > menuevent.x){
                    		/* if x is inside menu */
    	if(
    		(MenuBar[menuevent.menu].menu[menuevent.item].id != 43 &&
    		x < menuevent.x + MenuBar[menuevent.menu].w2)
    		||
    		(MenuBar[menuevent.menu].menu[menuevent.item].id == 43 &&
    		x < menuevent.x + MenuBar[menuevent.menu].x2)
    		)
    	{
	    	for(i = 0, y1 = menuwindow.h; i < MenuBar[menuevent.menu].items; 
	      	i++, y1 += menuheight)
	    	{
	      	if(y > y1 && y < y1 + menuheight){
	        	newitem = i;
	      	}
	    	}
	  	}
	  	else
	  	{
	                  	/* x to right of menu so test for panmenu */
	    	if(menuevent.item != -1 && 
	      	MenuBar[menuevent.menu].menu[menuevent.item].id == 43)
	    	{
                    	/* activate pan menu if there is one */
        	newitem = menuevent.item;
	      	initializepanmenu(newitem);
	      	menuevent.subitem = -1;
	      	drawmenu(popupsubpixmap, popupsubwin, menuevent.subitem,
		      	menuevent.submenu);
	    	}
	  	}
  	}
  }
  else         /* y above menu top */
  {
    menuevent.menuon = 0;
    menuevent.baron = 1;
  }

                    /* new item is different from old item */
  if(newitem != menuevent.item)
  {
    if(newitem != -1 && !strcmp(MenuBar[menuevent.menu].menu[newitem].text, BAR))
      newitem = -1;
    if(newitem != -1 && !MenuBar[menuevent.menu].menu[newitem].active)
      newitem = -1;

    menuevent.item = newitem;
    drawmenu(popuppixmap, popupwin, menuevent.item, menuevent.menu);
  }
}

void submenumotion(XEvent *report){
  static int i, y1, x, y, newitem;
  static Window tempwin;

  x = report->xmotion.x;
  y = report->xmotion.y;

  XTranslateCoordinates(display, report->xmotion.window, win,
    x, y, &x, &y, &tempwin);

                             /* x inside submenu */
  if(x > menuevent.x)
  {
                      /* get the subitem */
    newitem = -1;
    if(x < menuevent.x + MenuBar[menuevent.submenu].w2){
      for(i = 0, y1 = menuevent.y; i < MenuBar[menuevent.submenu].items; 
      	i++, y1 += menuheight)
      {
        if(y > y1 && y < y1 + menuheight){
          newitem = i;
        }
      }
    }

    if(newitem != menuevent.subitem){
      if(newitem != -1 && !strcmp(MenuBar[menuevent.submenu].menu[newitem].text, BAR))
        newitem = -1;
      if(newitem != -1 && !MenuBar[menuevent.submenu].menu[newitem].active)
        newitem = -1;

      menuevent.subitem = newitem;
      drawmenu(popupsubpixmap, popupsubwin, menuevent.subitem, menuevent.submenu);
    }
  }else                   /* deactivate sub menu if x outside */
  {
  	if(MenuBar[menuevent.menu].menu[menuevent.item].id == 43)
  	{
  		destroypanmenu();
  	}
  }
}

void menubuttonrelease(XEvent *report){
  static int i, y1, x, y, newitem;

  x = report->xbutton.x;
  y = report->xbutton.y;
  menuevent.buttondown = 0;

  if(report->xbutton.window == popupwin && y > 0){
    menucommand();
  }else
  	if(y > menuwindow.h) menucommand();
  	else
  	if(menuevent.menu > -1){
    	menuevent.baron = 0;
    	menuevent.menuon = 1;
  	}else{
    	menuevent.baron = 0;
    	menuevent.menuon = 0;
  	}
}

void submenubuttonrelease(XEvent *report){
    submenucommand();
}

void submenucommand(){
  static int id, subid, i, result;

  menuevent.baron = 0;
  menuevent.menuon = 0;
  id = -1;
  subid = -1;
  
  if(menuevent.menu != -1 && menuevent.item != -1){
    id = MenuBar[menuevent.menu].menu[menuevent.item].id;
    if(menuevent.subitem != -1)
    	subid = MenuBar[menuevent.submenu].menu[menuevent.subitem].id;
  }

	if(menuevent.submenuon && id == 43) destroypanmenu();
  if(menuevent.menu != -1) destroymenu();
  drawmenubar(-1);

	switch(id){
		case 43:
			result = 0;
			for(i = 0; i < outchannels; i++){
				if(subid == i){
					autoevent.showpan[i] ^= 1;
					MenuBar[menuevent.submenu].menu[i].active = autoevent.showpan[i] + 1;
				}
				if(autoevent.showpan[i]){
					result = 1;
				}
			}
			MenuBar[menuevent.menu].menu[menuevent.item].active = result + 1;
      drawwave(0, 0, wavewindow.w, wavewindow.h, startx, starty, NO_BLANK,
        FLASH, DRAWCURSOR);
      if(playevent.playsound)
        drawcursor(0, 0, wavewindow.w, wavewindow.h, startx, currentposition,
          currentposition);
      refreshwav();
			break;
	}
}

void menucommand(){
  static int id, i, result, totaltracks;
 	int showedits_;

  if(menuevent.menu != -1 && menuevent.item != -1){
    id = MenuBar[menuevent.menu].menu[menuevent.item].id;
  }else id = -1;

                     /* allow view changes to be persistant */
	if(id != 10 && id != 11 && id != 12 && 
	   id != 13 && id != 14 && id != 15 && id != 16)
	{
  	menuevent.baron = 0;
  	menuevent.menuon = 0;

  	if(menuevent.menu != -1) destroymenu();
  	drawmenubar(-1);
	}

  switch(id){
    case 1:
      createnewfile();
      break;
    case 26:
      loadfile();
      break;
    case 27:
      importfile();
      break;
    case 57:
      cd2drive(selectionstart, selectionend);
      break;
    case 3:
      saveasfile();
      break;
    case 32:
      playasfile();
      break;
    case 4:
      saveandexit();
      break;

     /*=========================edit menu==========================*/
    case 5:
      undo();
      break;
    case 56:
      redo();
      break;
    case 6:
      cut(selectionstart, selectionend, CUT);
      break;
    case 7:
      copy(selectionstart, selectionend, COPY);
      break;
    case 8:
      paste(selectionstart, selectionend, PASTE);
      break;
    case 51:
      pastesilence(selectionstart, selectionend);
      break;
    case 9:
      clear(selectionstart, selectionend, CLEAR);
      break;
    case 47:
      cutautomationonly(selectionstart, selectionend, CUTAUTOS);
      break;
    case 48:
      copyautomationonly(selectionstart, selectionend, COPYAUTOS);
      break;
    case 49:
      pasteautomationonly(selectionstart, selectionend, PASTEAUTOS);
      break;
    case 50:
      clearautomationonly(selectionstart, selectionend, CLEARAUTOS);
      break;
    case 29:
      if(!playevent.playsound){
        if(selectionstart == selectionend){
          togglelabel(selectionstart);
        }else{
          togglelabel(selectionstart);
          togglelabel(selectionend);
        }
      }else
        togglelabel(currentposition);
      break;
    case 52:
      shiftlabels ^= 1;
      EditMenu[15].active = 1 + shiftlabels;
      break;

     /*================================view menu===========================*/
    case 10:
      zoomin(selectionstart, selectionend);
      break;
    case 11:
      zoomout(selectionstart, selectionend);
      break;
    case 12:
      fit(selectionstart, selectionend);
      break;
    case 13:
      zoominvert();
      break;
    case 14:
      zoomoutvert();
      break;
    case 15:
      zoominverttrack();
      break;
    case 16:
      zoomoutverttrack();
      break;

     /*================================effects menu===========================*/
    case 31:
      mute(selectionstart, selectionend, MUTE);
      break;
    case 33:
      crossfade(selectionstart, selectionend);
      break;
    case 34:
      normalize(selectionstart, selectionend);
      break;
    case 35:
      balance(selectionstart, selectionend);
      break;
    case 54:
   		backwards(selectionstart, selectionend);
      break;
    case 46:
   		reverb(selectionstart, selectionend);
      break;
    case 55:
   		compress(selectionstart, selectionend);
      break;

  /*==============================Settings Menu========================*/
    case 21:
      changechannels();
      break;
    case 22:
      setsamplerate();
      break;
    case 23:
      addtrack();
      break;
    case 24:
      deletetrack();
      break;
    case 36:
      setplaybuffer();
      break;
    case 40:
      toggleautomation();
      break;
    case 41:
      if(!showtrackgroups){
        showtrackgroups ^= 1;
        if(showtrackgroups) strcpy(WindowMenu[0].text, "Show Console");
        else strcpy(WindowMenu[0].text, "Show Track Groups");
        updatemaintitle("");
      }
      addgroup();
      break;
    case 45:
      toggleloop();
      break;

   /*========================Window menu===========================*/
    case 37:
      highqualityrefresh ^= 1;
      WindowMenu[menuevent.item].active = 1 + highqualityrefresh;
      if(highqualityrefresh){
        drawwave(0, 0, wavewindow.w, wavewindow.h, startx, starty, NO_BLANK,
          FLASH, DRAWCURSOR);
        if(playevent.playsound)
          drawcursor(0, 0, wavewindow.w, wavewindow.h, startx, currentposition,
            currentposition);
      }
      refreshwav();
      break;
    case 30:
      titles ^= 1;
      WindowMenu[menuevent.item].active = 1 + titles;
                     /* update display */
      if(titles){
        if(playevent.playsound)
          drawcursor(0, 0, wavewindow.w, wavewindow.h, startx, currentposition,
            currentposition);
        drawtitles(0, 0, wavewindow.w, wavewindow.h, startx, starty);
      }else
        drawwave(0, 0, wavewindow.w, wavewindow.h, startx, starty, NO_BLANK,
          FLASH, DRAWCURSOR);

      if(playevent.playsound)
        drawcursor(0, 0, wavewindow.w, wavewindow.h, startx, currentposition,
          currentposition);
      refreshwav();
      break;
    case 38:
      showtrackgroups ^= 1;
      if(showtrackgroups){
        strcpy(WindowMenu[menuevent.item].text, "Show Console");
        updatemaintitle("");
      }else{
        strcpy(WindowMenu[menuevent.item].text, "Show Track Groups");
        updatemaintitle("");
      }
      drawconsolecontrols();
      break;
    case 39:
      autoevent.showmute ^= 1;
      WindowMenu[menuevent.item].active = 1 + autoevent.showmute;
      drawwave(0, 0, wavewindow.w, wavewindow.h, startx, starty, NO_BLANK,
        FLASH, DRAWCURSOR);
      if(playevent.playsound)
        drawcursor(0, 0, wavewindow.w, wavewindow.h, startx, currentposition,
          currentposition);
      refreshwav();
      break;
    case 42:
      autoevent.showfade ^= 1;
      WindowMenu[menuevent.item].active = 1 + autoevent.showfade;
      drawwave(0, 0, wavewindow.w, wavewindow.h, startx, starty, NO_BLANK,
        FLASH, DRAWCURSOR);
      if(playevent.playsound)
        drawcursor(0, 0, wavewindow.w, wavewindow.h, startx, currentposition,
          currentposition);
      refreshwav();
      break;
    case 53:
    	showedits ^= 1;
    	WindowMenu[menuevent.item].active = 1 + showedits;
/*       drawwave(0, 0, wavewindow.w, wavewindow.h, startx, starty, NO_BLANK,
        FLASH, DRAWCURSOR);
      if(playevent.playsound)
        drawcursor(0, 0, wavewindow.w, wavewindow.h, startx, currentposition,
          currentposition);
 */
 			showedits_ = showedits;
 			showedits = 1;
  		drawedits(0, 0, wavewindow.w, wavewindow.h, startx, starty);
  		showedits = showedits_;
      refreshwav();
    	break;
    case 43:
      editlist(autoevent.showpan, outchannels, "Select pans to display",
        SELECTPANS);

      result = 0;
      for(i = 0; i < outchannels; i++){
        if(autoevent.showpan[i]) result = 1;
      }
      WindowMenu[menuevent.item].active = 1 + result;
      drawwave(0, 0, wavewindow.w, wavewindow.h, startx, starty, NO_BLANK,
        FLASH, DRAWCURSOR);
      if(playevent.playsound)
        drawcursor(0, 0, wavewindow.w, wavewindow.h, startx, currentposition,
          currentposition);
      refreshwav();
      break;
    case 44:
      autoevent.showinvert ^= 1;
      WindowMenu[menuevent.item].active = 1 + autoevent.showinvert;
      drawwave(0, 0, wavewindow.w, wavewindow.h, startx, starty, NO_BLANK,
        FLASH, DRAWCURSOR);
      if(playevent.playsound)
        drawcursor(0, 0, wavewindow.w, wavewindow.h, startx, currentposition,
          currentposition);
      refreshwav();
      break;
  }
}

void playswapmenus(){
  FileMenu[0].active ^= 1;
  FileMenu[1].active ^= 1;
  FileMenu[2].active ^= 1;
  FileMenu[3].active ^= 1;
  FileMenu[5].active ^= 1;
  FileMenu[6].active ^= 1;
  FileMenu[7].active ^= 1;

  EditMenu[0].active ^= 1;
  EditMenu[1].active ^= 1;
  EditMenu[3].active ^= 1;
  EditMenu[4].active ^= 1;
  EditMenu[5].active ^= 1;
  EditMenu[6].active ^= 1;
  EditMenu[7].active ^= 1;
  EditMenu[9].active ^= 1;
  EditMenu[10].active ^= 1;
  EditMenu[11].active ^= 1;
  EditMenu[12].active ^= 1;

  EffectsMenu[0].active ^= 1;
  EffectsMenu[1].active ^= 1;
  EffectsMenu[2].active ^= 1;
  EffectsMenu[3].active ^= 1;
  EffectsMenu[4].active ^= 1;
  EffectsMenu[5].active ^= 1;
  EffectsMenu[6].active ^= 1;

  SettingsMenu[0].active ^= 1;
  SettingsMenu[1].active ^= 1;
  SettingsMenu[2].active ^= 1;
  SettingsMenu[6].active ^= 1;
  SettingsMenu[7].active ^= 1;
  SettingsMenu[8].active ^= 1;
}

/* initialize menu settings */
void loadmenusettings(){
  int i, result = 0;

  WindowMenu[1].active = 1 + titles;

  SettingsMenu[3].active = 1 + autoevent.automation;
      /* can't initilize playback loop since there isn't a file to get
         begin and end points normally */

      /* don't initialize window titles here */

  /*free(WindowMenu[0].text);
  WindowMenu[0].text = malloc(256);*/
  if(showtrackgroups){
    strcpy(WindowMenu[0].text, "Show Console");
  }else{
    strcpy(WindowMenu[0].text, "Show Track Groups");
  }

	WindowMenu[2].active = 1 + showedits;

  WindowMenu[3].active = 1 + highqualityrefresh;
	
  WindowMenu[5].active = 1 + autoevent.showmute;

  WindowMenu[6].active = 1 + autoevent.showfade;

  for(i = 0; i < outchannels; i++){
    if(autoevent.showpan[i]) result = 1;
  }
  WindowMenu[7].active = 1 + result;

  WindowMenu[8].active = 1 + autoevent.showinvert;

  EditMenu[15].active = 1 + shiftlabels;
}

void toggleautomation(){
  autoevent.automation ^= 1;
  SettingsMenu[3].active = 1 + autoevent.automation;
}

void toggleloop(){
  playevent.loop ^= 1;
  SettingsMenu[4].active = 1 + playevent.loop;

             /* set up looping parameters and draw boundries */
  if(playevent.loop){
    setplayloop();
    if(playevent.playsound)
      drawcursor(0, 0, wavewindow.w, wavewindow.h, startx, currentposition,
        currentposition);

    drawlooppoints(0, 0, wavewindow.w, wavewindow.h, startx);
  }else
    drawwave(0, 0, wavewindow.w, wavewindow.h, startx, starty, NO_BLANK,
      FLASH, DRAWCURSOR);

  if(playevent.playsound)
    drawcursor(0, 0, wavewindow.w, wavewindow.h, startx, currentposition,
      currentposition);
  refreshwav();
}
