/*
  Broadcast 2.0 multitrack audio editing
  (c) 1997 Adam Williams

  This program is distributed with the intent that it will be useful, without
  any warranty.  The code is being updated and has many bugs which are
  constantly being elucidated so changes should not be made without notifying
  the author.
*/

/* movement.c             movement around the waveform and console  */

#include "interfac.h"
#include "movement.h"
#include "database.h"
#include "play.h"
#include "graphics.h"

void fit(long start, long end){
  if(playevent.playsound)
    drawcursor(0, 0, wavewindow.w, wavewindow.h, startx, currentposition,
      currentposition);

  xscale = 1;
  if(start == end){
    start = 0;
    end = longesttrack();
  }

  while((end - start) / xscale > wavewindow.w){
    xscale *= 2;
  }
  startx = (start + end) / 2 - wavewindow.w * xscale / 2;
  if(startx < 0) startx = 0;
  fittime();
  drawwave(0, 0, wavewindow.w, wavewindow.h, startx, starty, NO_BLANK, FLASH,
    DRAWCURSOR);
  drawtime(startx);
  drawlabels(startx);
  updatexscrollbar(wavscroll.xpixmap, wavscroll.xwin, startx,
    wavewindow.w * xscale, longesttrack(), &wavscroll.x);

  if(playevent.playsound)
    drawcursor(0, 0, wavewindow.w, wavewindow.h, startx, currentposition,
      currentposition);
  refreshwav();
}

/* zoom out from wave */

void zoomout(long selectionstart, long selectionend){
  long viewend = startx + wavewindow.w*xscale;

  if(playevent.playsound)
    drawcursor(0, 0, wavewindow.w, wavewindow.h, startx, currentposition,
      currentposition);

  xscale *= 2;
  if(xscale > 262144) xscale = 262144;
  if((selectionstart + selectionend) / 2 >= startx && (selectionstart +
  selectionend) / 2 <= viewend){
    startx = (selectionstart + selectionend) / 2 - wavewindow.w/2*xscale;
  }else{
    startx = (startx + viewend) / 2 - wavewindow.w/2*xscale;
  }
  if(startx < 0) startx=0;
  fittime();
  drawwave(0, 0, wavewindow.w, wavewindow.h, startx, starty, NO_BLANK, FLASH,
    DRAWCURSOR);
  drawtime(startx);
  drawlabels(startx);
  updatexscrollbar(wavscroll.xpixmap, wavscroll.xwin, startx,
    wavewindow.w * xscale, longesttrack(), &wavscroll.x);

  if(playevent.playsound)
    drawcursor(0, 0, wavewindow.w, wavewindow.h, startx, currentposition,
      currentposition);
  refreshwav();
}

/* zoom into wave by factor of 2 */

void zoomin(long selectionstart, long selectionend){
  long viewend = startx + wavewindow.w*xscale;

  if(playevent.playsound)
    drawcursor(0, 0, wavewindow.w, wavewindow.h, startx, currentposition,
      currentposition);

  xscale /= 2;
  if(xscale < 1)  xscale = 1;
  if((selectionstart + selectionend) / 2 >= startx && (selectionstart +
  selectionend) / 2 <= viewend){
    startx = (selectionstart + selectionend) / 2 - wavewindow.w/2*xscale;
  }else{
    startx = (startx + viewend) / 2 - wavewindow.w/2*xscale;
  }
  if(startx < 0) startx=0;
  fittime();
  drawwave(0, 0, wavewindow.w, wavewindow.h, startx, starty, NO_BLANK, FLASH,
    DRAWCURSOR);
  drawtime(startx);
  drawlabels(startx);
  updatexscrollbar(wavscroll.xpixmap, wavscroll.xwin, startx,
    wavewindow.w * xscale, longesttrack(), &wavscroll.x);

  if(playevent.playsound)
    drawcursor(0, 0, wavewindow.w, wavewindow.h, startx, currentposition,
      currentposition);
  refreshwav();
}

void zoomoutvert(){
  if(playevent.playsound)
    drawcursor(0, 0, wavewindow.w, wavewindow.h, startx, currentposition,
      currentposition);

  yscale /= 2;
  if(yscale < 1) yscale = 1;
  drawwave(0, 0, wavewindow.w, wavewindow.h, startx, starty, NO_BLANK, FLASH,
    DRAWCURSOR);
  drawpatches(starty);

  if(playevent.playsound)
    drawcursor(0, 0, wavewindow.w, wavewindow.h, startx, currentposition,
      currentposition);
  refreshwav();
}

/* zoom into wave by factor of 2 */

void zoominvert(){
  if(playevent.playsound)
    drawcursor(0, 0, wavewindow.w, wavewindow.h, startx, currentposition,
      currentposition);

  yscale *= 2;
  if(yscale > 65536) yscale = 65536;
  drawwave(0, 0, wavewindow.w, wavewindow.h, startx, starty, NO_BLANK, FLASH,
    DRAWCURSOR);
  drawpatches(starty);

  if(playevent.playsound)
    drawcursor(0, 0, wavewindow.w, wavewindow.h, startx, currentposition,
      currentposition);
  refreshwav();
}

void zoomoutverttrack(){
  if(playevent.playsound)
    drawcursor(0, 0, wavewindow.w, wavewindow.h, startx, currentposition,
      currentposition);

  yscaletrack /= 2;
  if(yscaletrack < 1) yscaletrack = 1;
  drawwave(0, 0, wavewindow.w, wavewindow.h, startx, starty, NO_BLANK, FLASH,
    DRAWCURSOR);
  drawpatches(starty);
  updateyscrollbar(wavscroll.ypixmap, wavscroll.ywin, starty,
    wavewindow.h / yscaletrack, tracks(), &wavscroll.y);

  if(playevent.playsound)
    drawcursor(0, 0, wavewindow.w, wavewindow.h, startx, currentposition,
      currentposition);
  refreshwav();
}

/* zoom into wave by factor of 2 */

void zoominverttrack(){
  if(playevent.playsound)
    drawcursor(0, 0, wavewindow.w, wavewindow.h, startx, currentposition,
      currentposition);

  yscaletrack *= 2;
  if(yscaletrack > 16384) yscaletrack = 16384;
  drawwave(0, 0, wavewindow.w, wavewindow.h, startx, starty, NO_BLANK, FLASH,
    DRAWCURSOR);
  drawpatches(starty);
  updateyscrollbar(wavscroll.ypixmap, wavscroll.ywin, starty,
    wavewindow.h / yscaletrack, tracks(), &wavscroll.y);

  if(playevent.playsound)
    drawcursor(0, 0, wavewindow.w, wavewindow.h, startx, currentposition,
      currentposition);
  refreshwav();
}

void moveback(long distance){
  static long newblockwidth;
  static int totaltracks;
  static long longest;

  totaltracks = tracks();
  longest = longesttrack();

  if(playevent.playsound)
    drawcursor(0, 0, wavewindow.w, wavewindow.h, startx, currentposition,
      currentposition);

  if(startx > 0){
    if(distance >= wavewindow.w*xscale){
      startx -= distance;

      if(startx < 0) startx = 0;
      drawwave(0, 0, wavewindow.w, wavewindow.h, startx, starty, NO_BLANK,
        FLASH, DRAWCURSOR);
      drawlabels(startx);
      drawtime(startx);
    }else{
                                /* hide cursor and edits */
      drawcursor(0, 0, wavewindow.w, wavewindow.h, startx, selectionstart,
        selectionend);
      drawedits(0, 0, wavewindow.w, wavewindow.h, startx, starty);

                      /* copy existing data over distance */
      if(startx - distance < 0) distance = startx;
      newblockwidth = wavewindow.w - distance/xscale;
      XCopyArea(display, wavpixmap, wavpixmap, gc, 0, 0,
        newblockwidth, wavewindow.h, distance/xscale, 0);

      startx -= distance;
                                /* draw new section of wave */
      drawwave(0, 0, distance/xscale + 1, wavewindow.h, startx, starty, NO_BLANK,
        FLASH, NO_DRAWCURSOR);
                                /* redraw cursor and edits */
      drawcursor(0, 0, wavewindow.w, wavewindow.h, startx, selectionstart, selectionend);
      drawedits(0, 0, wavewindow.w, wavewindow.h, startx, starty);
      drawtime(startx);
      drawlabels(startx);
    }
  }

  updatexscrollbar(wavscroll.xpixmap, wavscroll.xwin, startx,
    wavewindow.w * xscale, longesttrack(), &wavscroll.x);
  if(playevent.playsound)
    drawcursor(0, 0, wavewindow.w, wavewindow.h, startx, currentposition,
      currentposition);
  refreshwav();
	flushevents();
}

void moveforward(long distance){
  static int newblockcolumn;
  static int newblockwidth;
  static long newblockdata;
  static int totaltracks;
  static long longest;

  totaltracks = tracks();
  longest = longesttrack();

  if(playevent.playsound)
    drawcursor(0, 0, wavewindow.w, wavewindow.h, startx, currentposition,
      currentposition);

  if(distance >= wavewindow.w*xscale){
    startx += distance;
    drawwave(0, 0, wavewindow.w, wavewindow.h, startx, starty, NO_BLANK, FLASH,
      DRAWCURSOR);
    drawlabels(startx);
    drawtime(startx);
  }else{
                  /* hide cursor and edits */
    drawcursor(0, 0, wavewindow.w, wavewindow.h, startx, selectionstart, selectionend);
		drawedits(0, 0, wavewindow.w, wavewindow.h, startx, starty);

        /* copy existing data over distance */
    newblockcolumn = wavewindow.w - distance/xscale;
    newblockwidth = wavewindow.w - newblockcolumn;
    newblockdata = startx + wavewindow.w*xscale;

    XCopyArea(display, wavpixmap, wavpixmap, gc, distance/xscale, 0,
      newblockcolumn, wavewindow.h, 0, 0);

    startx += distance;
                  /* draw new section of wave */
    drawwave(newblockcolumn, 0, newblockwidth + newblockcolumn,
      wavewindow.h, newblockdata, starty, NO_BLANK, FLASH, NO_DRAWCURSOR);
      
                  /* redraw cursor and edits*/
    drawcursor(0, 0, wavewindow.w, wavewindow.h, startx, selectionstart, selectionend);
    drawedits(0, 0, wavewindow.w, wavewindow.h, startx, starty);
    drawtime(startx);
    drawlabels(startx);
  }

  updatexscrollbar(wavscroll.xpixmap, wavscroll.xwin, startx,
    wavewindow.w * xscale, longesttrack(), &wavscroll.x);
  if(playevent.playsound)
    drawcursor(0, 0, wavewindow.w, wavewindow.h, startx, currentposition,
      currentposition);
  refreshwav();
	flushevents();
}

void moveup(int tracksmoved){
  int newblock, blockh;
  static int totaltracks;
  static long longest;

  if(tracksmoved == 0) goto end;
  totaltracks = tracks();
  longest = longesttrack();

  if(playevent.playsound)
    drawcursor(0, 0, wavewindow.w, wavewindow.h, startx, currentposition,
      currentposition);

  if(tracksmoved > starty) tracksmoved = starty;

  starty -= tracksmoved;

  newblock = yscaletrack * tracksmoved;

  blockh = wavewindow.h - newblock;
                                /* hide cursor and edits */
  drawcursor(0, 0, wavewindow.w, wavewindow.h, startx, selectionstart, selectionend);
	drawedits(0, 0, wavewindow.w, wavewindow.h, startx, starty + tracksmoved);
	
                                /* move block of waves down */
  XCopyArea(display, wavpixmap, wavpixmap, gc, 0, 0, wavewindow.w, blockh, 0, newblock);
                                /* draw new wave */
  drawwave(0, 0, wavewindow.w, yscaletrack * tracksmoved, startx, starty,
    NO_BLANK, FLASH, NO_DRAWCURSOR);
    
                                /* draw new cursor and edits */
  drawcursor(0, 0, wavewindow.w, wavewindow.h, startx, selectionstart, selectionend);
	drawedits(0, 0, wavewindow.w, wavewindow.h, startx, starty);
  drawpatches(starty);

  updateyscrollbar(wavscroll.ypixmap, wavscroll.ywin, starty,
    wavewindow.h / yscaletrack, tracks(), &wavscroll.y);

  if(playevent.playsound)
    drawcursor(0, 0, wavewindow.w, wavewindow.h, startx, currentposition,
      currentposition);
  refreshwav();
  flushevents();
end:
  ;
}

void movedown(int tracksmoved){
  static int newblock, blockh;
  static int newtrack;
  static int totaltracks;
  static long longest;

  if(tracksmoved == 0) goto end;

  totaltracks = tracks();
  longest = longesttrack();

  if(starty + tracksmoved > totaltracks) tracksmoved = totaltracks - starty;
  if(tracksmoved < 1) goto end;
  starty += tracksmoved;

  if(playevent.playsound)
    drawcursor(0, 0, wavewindow.w, wavewindow.h, startx, currentposition,
      currentposition);

  newblock = yscaletrack * tracksmoved;
  blockh = wavewindow.h - newblock;
                              /* hide cursor and edits*/
  drawcursor(0, 0, wavewindow.w, wavewindow.h, startx, selectionstart, selectionend);
	drawedits(0, 0, wavewindow.w, wavewindow.h, startx, starty - tracksmoved);

                              /* move block of waves up */
  XCopyArea(display, wavpixmap, wavpixmap, gc, 0, newblock, wavewindow.w, blockh, 0, 0);

  newblock = blockh / yscaletrack;
  newtrack = newblock + starty;
  newblock *= yscaletrack;
  blockh = wavewindow.h - newblock;

  blockh = wavewindow.h - newblock;
  if(newtrack < totaltracks){                            /* draw new wave */
    drawwave(0, newblock, wavewindow.w, wavewindow.h, startx,
      newtrack, NO_BLANK, FLASH, NO_DRAWCURSOR);
  }else{
    XSetForeground(display, gc, white.pixel);
    XFillRectangle(display, wavpixmap, gc, 0, newblock, wavewindow.w,
      wavewindow.h-newblock);
    XSetForeground(display, gc, black.pixel);
  }
                              /* draw new cursor and edits */
  drawcursor(0, 0, wavewindow.w, wavewindow.h, startx,
    selectionstart, selectionend);
	drawedits(0, 0, wavewindow.w, wavewindow.h, startx, starty);
  drawpatches(starty);

  updateyscrollbar(wavscroll.ypixmap, wavscroll.ywin, starty,
    wavewindow.h / yscaletrack, tracks(), &wavscroll.y);
  if(playevent.playsound)
    drawcursor(0, 0, wavewindow.w, wavewindow.h, startx, currentposition,
      currentposition);
  refreshwav();
  flushevents();
end:
  ;
}

void consolemoveup(int tracksmoved){
  if(tracksmoved == 0) goto end;

  if(!showtrackgroups){
    consolestarty -= tracksmoved;
    if(consolestarty < 0) consolestarty = 0;
  }else{
    groupstarty -= tracksmoved;
    if(groupstarty < 0) groupstarty = 0;
  }

  drawconsolecontrols();
  flushevents();
end:
  ;
}

void consolemovedown(int tracksmoved){
  static int totaltracks, totalgroups;

  if(tracksmoved == 0) goto end;

  if(!showtrackgroups){
    totaltracks = tracks();
    consolestarty += tracksmoved;
    if(consolestarty > totaltracks) consolestarty = totaltracks;
    drawconsolecontrols();
  }else{
    totalgroups = groups();
    groupstarty += tracksmoved;
    if(groupstarty > totalgroups) groupstarty = totalgroups;
    drawconsolecontrols();
  }

	flushevents();
end:
  ;
}
