/*
  Broadcast 2.0 multitrack audio editing
  (c) 1997 Adam Williams          swilliam@acs2.byu.edu

  This program is distributed with the intent that it will be useful, without
  any warranty.  The code is being updated and has many bugs which are
  constantly being elucidated so changes should not be made without notifying
  the author.
*/

/* pictures.c    widgets for the iterface   */


#include <math.h>
#include "interfac.h"
#include "pictures.h"
#include "defs.h"
#include "fileattr.h"

void DrawTitle(Drawable wdw, Window win, bctitle *title){
	XDrawString(display, wdw, gc, title->x, title->y, title->text, strlen(title->text));
}

void DrawRightArrow(Drawable wdw, int x, int y){
	static int w = 8, h = 16;
  static XPoint point[3];

  y -= h / 2;

  XSetForeground(display, gc, black.pixel);
  point[0].x = x;
  point[0].y = y;
  point[1].x = x + w;
  point[1].y = y + h / 2;
  point[2].x = x;
  point[2].y = y + h;
  XFillPolygon(display, wdw, gc, (XPoint *)point, 3, Nonconvex,
  	CoordModeOrigin);
}

void DrawLeftEditCursor(Drawable wdw, int x, int y){
	XSetFunction(display, gc, GXxor);
	XSetForeground(display, gc, white.pixel);

	XDrawLine(display, wdw, gc, x, 0, x, wavewindow.h);
	DrawLeftEdit(wdw, x, y + 8);
}

void DrawRightEditCursor(Drawable wdw, int x, int y){
	XSetFunction(display, gc, GXxor);
	XSetForeground(display, gc, white.pixel);

	XDrawLine(display, wdw, gc, x, 0, x, wavewindow.h);
	DrawRightEdit(wdw, x, y + 8);
}

void DrawLeftEdit(Drawable wdw, int x, int y){
	static int w = 8, h = 16;
  static XPoint point[3];

  y -= h / 2;
  x -= w;
  XSetFunction(display, gc, GXxor);
  XSetForeground(display, gc, white.pixel);

  point[0].x = x;
  point[0].y = y;
  point[1].x = x + w;
  point[1].y = y + h / 2;
  point[2].x = x;
  point[2].y = y + h;
  XFillPolygon(display, wdw, gc, (XPoint *)point, 3, Nonconvex, CoordModeOrigin);

  XSetForeground(display, gc, black.pixel);
  XSetFunction(display, gc, GXcopy);
}

void DrawRightEdit(Drawable wdw, int x, int y){
	static int w = 8, h = 16;
  static XPoint point[3];

  y -= h / 2;
  XSetFunction(display, gc, GXxor);
  XSetForeground(display, gc, white.pixel);

  point[0].x = x;
    point[0].y = y + h / 2;
    point[1].x = x + w;
    point[1].y = y;
    point[2].x = x + w;
    point[2].y = y + h;
    XFillPolygon(display, wdw, gc, (XPoint *)point, 3, Nonconvex, CoordModeOrigin);

    XSetForeground(display, gc, black.pixel);
    XSetFunction(display, gc, GXcopy);
}

void DrawPatch(Drawable wdw, int x, int y1, struct Track
  *currenttrack, int track)
{
  static char string[5];
  static y;

  y = y1+12;

  if(currenttrack->play) DrawDnPlay(patchpixmap, x, y, 15, 15);
  else DrawUpPlay(patchpixmap, x, y, 15, 15);

  if(currenttrack->record) DrawDnRec(patchpixmap, x+15, y, 15, 15);
  else DrawUpRec(patchpixmap, x+15, y, 15, 15);

  sprintf(string, "%d", track+1);
  XSetForeground(display, gc, black.pixel);
  XDrawString(display, patchpixmap, gc, x + 2, y + 13, string, strlen(string));
}

void DrawUpBox(Drawable wdw, int x1, int y1, int w, int h){
  static int lx,ly,ux,uy;

  lx = x1+1;  ly = y1+1;
  ux = x1+w-1;  uy = y1+h-1;

  XSetForeground(display, gc, grey.pixel);
  XFillRectangle(display, wdw, gc, x1, y1, w, h);
  XSetForeground(display, gc, ltgrey.pixel);
  XDrawLine(display, wdw, gc, x1, y1, x1+w, y1);
  XDrawLine(display, wdw, gc, x1, y1, x1, y1+h);
  XDrawLine(display, wdw, gc, lx, ly, ux, ly);
  XDrawLine(display, wdw, gc, lx, ly, lx, uy);

  XSetForeground(display, gc, dkgrey.pixel);
  XDrawLine(display, wdw, gc, x1+w, y1, x1+w, y1+h);
  XDrawLine(display, wdw, gc, x1, y1+h, x1+w, y1+h);
  XDrawLine(display, wdw, gc, ux, ly, ux, uy);
  XDrawLine(display, wdw, gc, lx, uy, ux, uy);
  XSetForeground(display, gc, black.pixel);
}

void DrawDnBox(Drawable wdw, int x1, int y1, int w, int h){
  static int lx,ly,ux,uy;

  lx = x1+1;  ly = y1+1;
  ux = x1+w-1;  uy = y1+h-1;

  XSetForeground(display, gc, migrey.pixel);
  XFillRectangle(display, wdw, gc, x1, y1, w, h);
  XSetForeground(display, gc, dkgrey.pixel);
  XDrawLine(display, wdw, gc, x1, y1, x1+w, y1);
  XDrawLine(display, wdw, gc, x1, y1, x1, y1+h);
  XDrawLine(display, wdw, gc, lx, ly, ux, ly);
  XDrawLine(display, wdw, gc, lx, ly, lx, uy);

  XSetForeground(display, gc, ltgrey.pixel);
  XDrawLine(display, wdw, gc, x1+w, y1, x1+w, y1+h);
  XDrawLine(display, wdw, gc, x1, y1+h, x1+w, y1+h);
  XDrawLine(display, wdw, gc, ux, ly, ux, uy);
  XDrawLine(display, wdw, gc, lx, uy, ux, uy);
  XSetForeground(display, gc, black.pixel);
}

void DrawThinUpBox(Drawable wdw, int x1, int y1, int w, int h){
  XSetForeground(display, gc, grey.pixel);
  XFillRectangle(display, wdw, gc, x1, y1, w, h);
  XSetForeground(display, gc, ltgrey.pixel);
  XDrawLine(display, wdw, gc, x1, y1, x1+w, y1);
  XDrawLine(display, wdw, gc, x1, y1, x1, y1+h);

  XSetForeground(display, gc, dkgrey.pixel);
  XDrawLine(display, wdw, gc, x1+w, y1, x1+w, y1+h);
  XDrawLine(display, wdw, gc, x1, y1+h, x1+w, y1+h);
  XSetForeground(display, gc, black.pixel);
}

void DrawThinDnBox(Drawable wdw, int x1, int y1, int w, int h){
  XSetForeground(display, gc, migrey.pixel);
  XFillRectangle(display, wdw, gc, x1, y1, w, h);
  XSetForeground(display, gc, dkgrey.pixel);
  XDrawLine(display, wdw, gc, x1, y1, x1+w, y1);
  XDrawLine(display, wdw, gc, x1, y1, x1, y1+h);

  XSetForeground(display, gc, ltgrey.pixel);
  XDrawLine(display, wdw, gc, x1+w, y1, x1+w, y1+h);
  XDrawLine(display, wdw, gc, x1, y1+h, x1+w, y1+h);
  XSetForeground(display, gc, black.pixel);
}

void DrawUpMenuBox(Drawable wdw, int x1, int y1, int w, int h){
  register int lx,ly,ux,uy;

  lx = x1+1;  ly = y1+1;
  ux = x1+w-1;  uy = y1+h-1;

  XSetForeground(display, gc, cyan.pixel);
  XFillRectangle(display, wdw, gc, x1, y1, w, h);
  XSetForeground(display, gc, ltcyan.pixel);
  XDrawLine(display, wdw, gc, x1, y1, x1+w, y1);
  XDrawLine(display, wdw, gc, x1, y1, x1, y1+h);
  XDrawLine(display, wdw, gc, lx, ly, ux, ly);
  XDrawLine(display, wdw, gc, lx, ly, lx, uy);

  XSetForeground(display, gc, dkcyan.pixel);
  XDrawLine(display, wdw, gc, x1+w, y1, x1+w, y1+h);
  XDrawLine(display, wdw, gc, x1, y1+h, x1+w, y1+h);
  XDrawLine(display, wdw, gc, ux, ly, ux, uy);
  XDrawLine(display, wdw, gc, lx, uy, ux, uy);
  XSetForeground(display, gc, black.pixel);
}

void DrawDnMenuBox(Drawable wdw, int x1, int y1, int w, int h){
  register int lx,ly,ux,uy;

  lx = x1+1;  ly = y1+1;
  ux = x1+w-1;  uy = y1+h-1;

  XSetForeground(display, gc, micyan.pixel);
  XFillRectangle(display, wdw, gc, x1, y1, w, h);
  XSetForeground(display, gc, dkcyan.pixel);
  XDrawLine(display, wdw, gc, x1, y1, x1+w, y1);
  XDrawLine(display, wdw, gc, x1, y1, x1, y1+h);
  XDrawLine(display, wdw, gc, lx, ly, ux, ly);
  XDrawLine(display, wdw, gc, lx, ly, lx, uy);

  XSetForeground(display, gc, ltcyan.pixel);
  XDrawLine(display, wdw, gc, x1+w, y1, x1+w, y1+h);
  XDrawLine(display, wdw, gc, x1, y1+h, x1+w, y1+h);
  XDrawLine(display, wdw, gc, ux, ly, ux, uy);
  XDrawLine(display, wdw, gc, lx, uy, ux, uy);
  XSetForeground(display, gc, black.pixel);
}

void DrawUpButton(Drawable wdw, Window win, bcbutton *button){
  DrawUpBox(wdw, button->x, button->y, button->w, button->h);
  XSetForeground(display, gc, black.pixel);
  if(button->h == 30)
    XDrawString(display, wdw, gc, button->x + 5, button->y + 20,
      button->text, strlen(button->text));
  else
    XDrawString(display, wdw, gc, button->x + 5, button->y + 17,
      button->text, strlen(button->text));
  XCopyArea(display, wdw, win, gc, button->x, button->y, button->w + 1,
    button->h + 1, button->x, button->y);
}

void DrawDnButton(Drawable wdw, Window win, bcbutton *button){
  DrawDnBox(wdw, button->x, button->y, button->w, button->h);
  XSetForeground(display, gc, black.pixel);
  if(button->h == 30)
    XDrawString(display, wdw, gc, button->x + 5, button->y + 20,
      button->text, strlen(button->text));
  else
    XDrawString(display, wdw, gc, button->x + 5, button->y + 17,
      button->text, strlen(button->text));
  XCopyArea(display, wdw, win, gc, button->x, button->y, button->w + 1,
    button->h + 1, button->x, button->y);
}

void DrawEqIcon(Drawable wdw, bcbutton *button){
  static XPoint point[4];

  point[0].x = button->x;
  point[0].y = button->y + button->h * 2 / 3;
  point[1].x = button->x;
  point[1].y = button->y + button->h / 3;
  point[2].x = button->x + button->w;
  point[2].y = button->y + button->h / 3;
  point[3].x = button->x + button->w;
  point[3].y = button->y + button->h * 2 / 3;

  XDrawLines(display, wdw, gc, point, 4, CoordModeOrigin);
}

void DrawBpIcon(Drawable wdw, bcbutton *button){
  static XPoint point[7];

  point[0].x = button->x;
  point[0].y = button->y + button->h * 2 / 3;
  point[1].x = button->x + button->w / 3;
  point[1].y = button->y + button->h * 2 / 3;
  point[2].x = button->x + button->w / 3;
  point[2].y = button->y + button->h / 3;
  point[3].x = button->x + button->w * 2 / 3;
  point[3].y = button->y + button->h / 3;
  point[4].x = button->x + button->w * 2 / 3;
  point[4].y = button->y + button->h * 2 / 3;
  point[5].x = button->x + button->w * 2 / 3;
  point[5].y = button->y + button->h * 2 / 3;
  point[6].x = button->x + button->w;
  point[6].y = button->y + button->h * 2 / 3;

  XDrawLines(display, wdw, gc, point, 7, CoordModeOrigin);
}

void DrawUpRadial(Drawable wdw, bcbutton *button){
  XSetForeground(display, gc, grey.pixel);
  XFillRectangle(display, wdw, gc, button->x, button->y, button->w, button->h);
  XSetForeground(display, gc, ltgrey.pixel);
  XDrawArc(display, wdw, gc, button->x, button->y,
    button->w, button->h, 45*64, 180*64);
  XDrawArc(display, wdw, gc, button->x, button->y + 1,
    button->w, button->h, 45*64, 180*64);

  XSetForeground(display, gc, dkgrey.pixel);
  XDrawArc(display, wdw, gc, button->x, button->y,
    button->w, button->h, 225*64, 180*64);
  XDrawArc(display, wdw, gc, button->x, button->y + 1,
    button->w, button->h, 225*64, 180*64);
  XSetForeground(display, gc, black.pixel);
}

void DrawDnRadial(Drawable wdw, bcbutton *button){
  XSetForeground(display, gc, red.pixel);
  XFillArc(display, wdw, gc, button->x, button->y,
    button->w, button->h, 0*64, 360*64);
  XSetForeground(display, gc, dkgrey.pixel);
  XDrawArc(display, wdw, gc, button->x, button->y,
    button->w, button->h, 45*64, 180*64);
  XDrawArc(display, wdw, gc, button->x, button->y + 1,
    button->w, button->h, 45*64, 180*64);

  XSetForeground(display, gc, ltgrey.pixel);
  XDrawArc(display, wdw, gc, button->x, button->y,
    button->w, button->h, 225*64, 180*64);
  XDrawArc(display, wdw, gc, button->x, button->y + 1,
    button->w, button->h, 225*64, 180*64);
  XSetForeground(display, gc, black.pixel);
}

                     /* play for patches */
void DrawUpPlay(Drawable wdw, int x1, int y1, int width, int height){
    int x2, y2, x3, y3;

    x2 = x1;
    y2 = y1 + height;
    x3 = x1 + width;
    y3 = y1 + height / 2;

    XSetForeground(display, gc, grey.pixel);
    XFillRectangle(display, wdw, gc, x1, y1, width+1, height+2);
    XSetForeground(display, gc, ltgrey.pixel);
    XDrawLine(display, wdw, gc, x1, y1, x2, y2);
    XDrawLine(display, wdw, gc, x1+1, y1, x1+1, y2);
    XDrawLine(display, wdw, gc, x1, y1, x3, y3);

    XSetForeground(display, gc, dkgrey.pixel);
    XDrawLine(display, wdw, gc, x3, y3, x2, y2);
    XDrawLine(display, wdw, gc, x3, y3+1, x2, y2+1);
  XSetForeground(display, gc, black.pixel);
}

                     /* play for patches */
void DrawDnPlay(Drawable wdw, int x1, int y1, int width, int height){
    int x2, y2, x3, y3;
    static XPoint point[3];

    x2 = x1;
    y2 = y1 + height;
    x3 = x1 + width;
    y3 = y1 + height / 2;

    XSetForeground(display, gc, grey.pixel);
    XFillRectangle(display, wdw, gc, x1, y1, width+1, height+2);
    XSetForeground(display, gc, green.pixel);
    point[0].x = x1;
    point[0].y = y1;
    point[1].x = x2;
    point[1].y = y2;
    point[2].x = x3;
    point[2].y = y3;
    XFillPolygon(display, wdw, gc, (XPoint *)point, 3, Nonconvex, CoordModeOrigin);

    XSetForeground(display, gc, dkgrey.pixel);
    XDrawLine(display, wdw, gc, x1, y1, x2, y2);
    XDrawLine(display, wdw, gc, x1+1, y1, x1+1, y2);
    XDrawLine(display, wdw, gc, x1, y1, x3, y3);

    XSetForeground(display, gc, ltgrey.pixel);
    XDrawLine(display, wdw, gc, x3, y3, x2, y2);
    XDrawLine(display, wdw, gc, x3, y3+1, x2, y2+1);

  XSetForeground(display, gc, black.pixel);
}

                 /* rec for patches */
void DrawUpRec(Drawable wdw, int x1, int y1, int width, int height){
  XSetForeground(display, gc, grey.pixel);
  XFillRectangle(display, wdw, gc, x1, y1, width+1, height+2);
  XSetForeground(display, gc, ltgrey.pixel);
  XDrawArc(display, wdw, gc, x1, y1, width, height, 45*64, 180*64);
  XDrawArc(display, wdw, gc, x1, y1+1, width, height, 45*64, 180*64);

  XSetForeground(display, gc, dkgrey.pixel);
  XDrawArc(display, wdw, gc, x1, y1, width, height, 225*64, 180*64);
  XDrawArc(display, wdw, gc, x1, y1+1, width, height, 225*64, 180*64);
  XSetForeground(display, gc, black.pixel);
}

                 /* rec for patches */
void DrawDnRec(Drawable wdw, int x1, int y1, int width, int height){
  XSetForeground(display, gc, red.pixel);
  XFillArc(display, wdw, gc, x1, y1, width, height, 0*64, 360*64);
  XSetForeground(display, gc, dkgrey.pixel);
  XDrawArc(display, wdw, gc, x1, y1, width, height, 45*64, 180*64);
  XDrawArc(display, wdw, gc, x1, y1+1, width, height, 45*64, 180*64);

  XSetForeground(display, gc, ltgrey.pixel);
  XDrawArc(display, wdw, gc, x1, y1, width, height, 225*64, 180*64);
  XDrawArc(display, wdw, gc, x1, y1+1, width, height, 225*64, 180*64);
  XSetForeground(display, gc, black.pixel);
}

void DrawMeter(Drawable wdw, Window win, bcbox *box, long meterpeak){
  int meterscale = 32767 / (box->w - 4);
  meterpeak /= meterscale;

  DrawDnBox(wdw, box->x, box->y, box->w + 4, box->h + 4);
  XSetForeground(display, gc, red.pixel);
  XFillRectangle(display, wdw, gc, box->x+2, box->y+2, meterpeak, box->h);
  XCopyArea(display, wdw, win, gc, box->x, box->y,
    box->w+4, box->h+4, box->x, box->y);
}

                     /* record for record window */
void DrawUpRecord(Drawable wdw, Window win, bcbutton *button){
  DrawUpBox(wdw, button->x, button->y, button->w, button->h);
  XSetForeground(display, gc, red.pixel);
  XFillArc(display, wdw, gc, button->x + 10, button->y + 10,
    button->w - 20, button->h - 20, 0*64, 360*64);

  XCopyArea(display, wdw, win, gc, button->x, button->y,
    button->w + 1, button->h + 1, button->x, button->y);
}

                     /* record for record window */
void DrawDnRecord(Drawable wdw, Window win, bcbutton *button){
  DrawDnBox(wdw, button->x, button->y, button->w, button->h);
  XSetForeground(display, gc, red.pixel);
  XFillArc(display, wdw, gc, button->x + 10, button->y + 10,
    button->w - 20, button->h - 20, 0*64, 360*64);

  XCopyArea(display, wdw, win, gc, button->x, button->y,
    button->w + 1, button->h + 1, button->x, button->y);
}

                                /* play for play window */
void DrawUpPlayButton(Drawable wdw, Window win, bcbutton *button){
  static int x2, y2, y3, x, y, w, h;
  static XPoint point[3];

  x = button->x;
  y = button->y;
  w = button->w;
  h = button->h;
  x2 = x + w - 10;
  y2 = y + h - 10;
  y3 = y + h / 2;

  DrawUpBox(wdw, x, y, w, h);
  XSetForeground(display, gc, green.pixel);
  point[0].x = x + 10;
  point[0].y = y + 10;
  point[1].x = x + 10;
  point[1].y = y2;
  point[2].x = x2;
  point[2].y = y3;

  XFillPolygon(display, wdw, gc, (XPoint *)point, 3, Nonconvex, CoordModeOrigin);

  XCopyArea(display, wdw, win, gc, x, y, w+1, h+1, x, y);
  XFlush(display);
}

void DrawDnPlayButton(Drawable wdw, Window win, bcbutton *button){
  static int x2, y2, y3, x, y, w, h;
  static XPoint point[3];

  x = button->x;
  y = button->y;
  w = button->w;
  h = button->h;
  x2 = x + w - 10;
  y2 = y + h - 10;
  y3 = y + h / 2;

  DrawDnBox(wdw, x, y, w, h);
  XSetForeground(display, gc, green.pixel);
  point[0].x = x + 10;
  point[0].y = y + 10;
  point[1].x = x + 10;
  point[1].y = y2;
  point[2].x = x2;
  point[2].y = y3;

  XFillPolygon(display, wdw, gc, (XPoint *)point, 3, Nonconvex, CoordModeOrigin);

  XCopyArea(display, wdw, win, gc, x, y, w+1, h+1, x, y);
  XFlush(display);
}

void DrawUpStopButton(Drawable wdw, Window win, bcbutton *button){
  DrawUpBox(wdw, button->x, button->y, button->w, button->h);
  XFillRectangle(display, wdw, gc, button->x + 10, button->y + 10,
    button->w - 20, button->h - 20);
  XCopyArea(display, wdw, win, gc, button->x, button->y,
    button->w + 1, button->h + 1, button->x, button->y);
  XFlush(display);
}

void DrawDnStopButton(Drawable wdw, Window win, bcbutton *button){
  DrawDnBox(wdw, button->x, button->y, button->w, button->h);
  XFillRectangle(display, wdw, gc, button->x + 10, button->y + 10,
    button->w - 20, button->h - 20);
  XCopyArea(display, wdw, win, gc, button->x, button->y,
    button->w + 1, button->h + 1, button->x, button->y);
  XFlush(display);
}

void DrawDnDogEar(Drawable wdw, Window win, bcbox *box){
  DrawDnBox(wdw, box->x, box->y, box->w, box->h);
  XDrawLine(display, wdw, gc, box->x, box->y + box->h,
    box->x + box->w, box->y);
  XCopyArea(display, wdw, win, gc, box->x, box->y,
    box->w + 1, box->h + 1, box->x, box->y);
}

void DrawUpDogEar(Drawable wdw, Window win, bcbox *box){
  DrawUpBox(wdw, box->x, box->y, box->w, box->h);
  XDrawLine(display, wdw, gc, box->x, box->y + box->h,
    box->x + box->w, box->y);
  XCopyArea(display, wdw, win, gc, box->x, box->y,
    box->w + 1, box->h + 1, box->x, box->y);
}
