/*
  Broadcast 2.0 multitrack audio editing
  (c) 1997 Adam Williams

  This program is distributed with the intent that it will be useful, without
  any warranty.  The code is being updated and has many bugs which are
  constantly being elucidated so changes should not be made without notifying
  the author.
*/

/* play.c         playback of the Nom-ROM powerplant   */


#include "defs.h"
#include "interfac.h"
#include "graphics.h"
#include "console.h"
#include "audio.h"
#include "play.h"
#include "menu.h"

long x;

void drawlooppoints(int column1, int row1, int column2, int row2, long startx)
{
  static long _column1;
  static long _column2;
  static long width;
  static long viewend;

  width = column2 - column1;
  viewend = startx + (long)xscale * width;

  if(playevent.loop){
    XSetForeground(display, gc, green.pixel);
    if(playevent.loopstart >= startx){
      _column1 = (playevent.loopstart - startx) / xscale + column1;
      XDrawLine(display, wavpixmap, gc, _column1, row1, _column1, row2);
    }

    if(playevent.loopend <= viewend){
      _column2 = (playevent.loopend - startx) / xscale + column1 + 1;
      XDrawLine(display, wavpixmap, gc, _column2, row1, _column2, row2);
    }
    XSetForeground(display, gc, black.pixel);
  }
}

        /* set start and end of playback loop */
void setplayloop(){
  if(selectionstart == selectionend){
    playevent.loopstart = 0;
    playevent.loopend = longestplayabletrack();
  }else{
    playevent.loopstart = selectionstart;
    playevent.loopend = selectionend;
  }
}

void playbuttonpress(XEvent *report){
  static int x;
  static int y;

  x = report->xbutton.x;
  y = report->xbutton.y;

  if(testbutton(x, y, &playwindow.play)){
    if(playevent.playsound){
      playevent.pause = 1;
    }else{
      playevent.playsound = 1;
      if(playevent.playsound == 1) playsound(selectionstart, selectionend);
    }
  }
  if(testbutton(x, y, &playwindow.stop)){
    if(playevent.playsound){
      playevent.stopplay = 1;
      playevent.paused = 0;
    }
  }
  if(testbutton(x, y, &playwindow.record) && !playevent.playsound){
    DrawDnRecord(playpixmap, playwin, &playwindow.record);
    do{
      XNextEvent(display, report);
    }while(report->type != ButtonRelease);
    recordsound(selectionstart, selectionend);
    DrawUpRecord(playpixmap, playwin, &playwindow.record);
  }
}

void playhandleevent(){
  XEvent report, nextreport;

                    /* don't busy wait when paused */
	if(playevent.paused){
		XNextEvent(display, &report);
		XPutBackEvent(display, &report);
	}
	
  while(XPending(display)){ /* run down event queue */
    do{
      XNextEvent(display, &report);
      if(XPending(display)) XPeekEvent(display, &nextreport);
      else nextreport.type = 0;
    }while(
      (nextreport.type == KeyPress && report.type == KeyPress) |
      (nextreport.type == MotionNotify && report.type == MotionNotify)
    );

    switch(report.type){
      case Expose:
        handleexpose(&report);
        break;

      case ConfigureNotify:
        drawcursor(0, 0, wavewindow.w, wavewindow.h, startx, currentposition, currentposition);
        mainconfigurenotify(&report);
        drawcursor(0, 0, wavewindow.w, wavewindow.h, startx, currentposition, currentposition);
      break;

      case MotionNotify:
        if(wavscroll.x.event | wavscroll.y.event) wavscrollevent(&report);
        if(consolewindow.scroll.event) consolescrollevent(&report);
        if(consoleevent.consoleadjust) consolemotion(&report);
        if(patchevent.patchadjust) patchmotion(&report);
      	if(menuevent.baron) menubarmotion(&report);
      	if(menuevent.submenuon) submenumotion(&report);
      	else if(menuevent.menuon) menumotion(&report);
      break;

      case ButtonPress:
        mainbuttonpress(&report);
        if(wavscroll.x.event | wavscroll.y.event) wavscrollevent(&report);
        if(consolewindow.scroll.event) consolescrollevent(&report);
        break;

      case ButtonRelease:
        mainbuttonrelease(&report);
        break;

      case KeyPress:
        mainkeypress(&report);
        break;
    } /* end switch report.type */
  } /* end run down event queue */
}

void playlabel(){
                            /* hide cursor */
  drawcursor(0, 0, wavewindow.w, wavewindow.h, startx, currentposition,
    currentposition);
  currentposition = x - getaudiotxlength() / outchannels / 2;
                            /* show cursor */
  drawcursor(0, 0, wavewindow.w, wavewindow.h, startx, currentposition,
    currentposition);
  refreshwav();
  togglelabel(currentposition);
}

int playsound(long start, long end){
  char *buffer;
  int i, j;
  long bufferlength, audiotxlength;
  int frame;
  
            /* set end to longest track if no region is selected */
  if(start == end) end = longesttrack(); /* in case the patch bay is changed */

            /* can occur if no region is selected but cursor is beyond longest
               track */
            /* also if loop is enabled and selection is after looping region
               position start at beginning of looping region */
  if(end < start){
    playevent.playsound = 0;
    return 1;
  }

  if(openaudio()){
    playevent.playsound = 0;
    return 1;
  }

  initplaybackaudio(outchannels);
  frame = outchannels * 2;

  if(!(buffer = malloc(PLAYBUFFER + outchannels * 2))){
    closeaudio();
    return 1;
  }

  if(startmixdown(PLAYBUFFER + outchannels * 2, outchannels)){
    closeaudio();
    free(buffer);
    return 1;
  }

  playswapmenus();

  DrawDnPlayButton(playpixmap, playwin, &playwindow.play);
  DrawUpStopButton(playpixmap, playwin, &playwindow.stop);

  playevent.paused = 0;
  playevent.stopplay = 0;
  playevent.pause = 0;

  x = currentposition = start;          /* set first x here in case loop
                                           starts at different location */
                                                    /* play loop */
  do{                                             /* process loop */
                                    /* initialize play cursor */
    drawcursor(0, 0, wavewindow.w, wavewindow.h, startx, currentposition, currentposition);
    refreshwav();

                           /* initialize first buffer length */
    bufferlength = PLAYBUFFER - PLAYBUFFER / 4;

    bufferlength /= frame;
    bufferlength *= frame;
                                                   /* load first buffer */
    if(!playevent.loop && bufferlength / frame > end - x)
      bufferlength = (end - x) * frame;
    if(playevent.loop && bufferlength / frame > playevent.loopend - x)
      bufferlength = (playevent.loopend - x) * frame;

    if(mixdown(buffer, bufferlength, x)){
              /* hide cursor and exit if error */
      drawcursor(0, 0, wavewindow.w, wavewindow.h, startx, currentposition, currentposition);
       refreshwav();
      closeaudio();
      free(buffer);
      playswapmenus();
      playevent.playsound = 0;
      DrawUpPlayButton(playpixmap, playwin, &playwindow.play);
      DrawDnStopButton(playpixmap, playwin, &playwindow.stop);
      killmixdown();
      return 1;
    }

    do{                                           /* process selection */
      playhandleevent();

      if(playevent.pause){
        playevent.paused ^= 1;
        playevent.pause--;
        if(playevent.paused){
          DrawUpPlayButton(playpixmap, playwin, &playwindow.play);
          audiotxlength = getaudiotxlength();
          if(audiotxlength) pauseaudiooutput();
                                    /* hide cursor */
          drawcursor(0, 0, wavewindow.w, wavewindow.h, startx, currentposition, currentposition);
          currentposition = x - audiotxlength/frame;
                                    /* show cursor */
          drawcursor(0, 0, wavewindow.w, wavewindow.h, startx, currentposition, currentposition);
          refreshwav();
        }else{
          DrawDnPlayButton(playpixmap, playwin, &playwindow.play);
          DrawUpStopButton(playpixmap, playwin, &playwindow.stop);
          if(getaudiotxlength()) resumeaudiooutput();
        }
      }

      if(!playevent.paused && !playevent.stopplay){
        audiotxlength = getaudiotxlength();
                                              /* test playback buffer */

      if(audiotxlength < PLAYBUFFER / 4)
        {
/*
printf("%d\n", audiotxlength);
*/
          writeaudio(buffer, bufferlength);

          x += bufferlength/frame;
                               /* clear old cursor position */
          drawcursor(0, 0, wavewindow.w, wavewindow.h, startx, currentposition, currentposition);
          currentposition = x - bufferlength/frame - audiotxlength/frame;
                              /* draw new cursor position */
          drawcursor(0, 0, wavewindow.w, wavewindow.h, startx, currentposition, currentposition);
          refreshwav();
                   /* read next buffer */
          if(!playevent.loop && bufferlength / frame > end - x)
            bufferlength = (end - x) * frame;
          if(playevent.loop && bufferlength / frame > playevent.loopend - x)
            bufferlength = (playevent.loopend - x) * frame;

          if(mixdown(buffer, bufferlength, x)){
                    /* hide cursor and exit if error */
            drawcursor(0, 0, wavewindow.w, wavewindow.h, startx, currentposition, currentposition);
             refreshwav();
            closeaudio();
            free(buffer);
            playswapmenus();
            playevent.playsound = 0;
            DrawUpPlayButton(playpixmap, playwin, &playwindow.play);
            DrawDnStopButton(playpixmap, playwin, &playwindow.stop);
            killmixdown();
            return 1;
          }
        }
      }
    }while(((!playevent.loop && x < end) ||
      (playevent.loop && x < playevent.loopend))
      && !playevent.stopplay);            /* process selection */

                /* hide cursor */
     drawcursor(0, 0, wavewindow.w, wavewindow.h, startx, currentposition, currentposition);

              /* if looping enabled, reset position here */
    if(playevent.loop) x = currentposition = playevent.loopstart;
  }while(playevent.loop && !playevent.stopplay);               /* process loop */

  if(!playevent.stopplay){
    flushaudio();
  }
  closeaudio();
  free(buffer);
  killmixdown();

  playevent.playsound = 0;
  DrawUpPlayButton(playpixmap, playwin, &playwindow.play);
  DrawDnStopButton(playpixmap, playwin, &playwindow.stop);
   refreshwav();
   playswapmenus();

  return 0;
}
