#include "pot.h"
#include "interfac.h"
#include "units.h"
#include <math.h>

void DrawPot(Drawable wdw, Window win, bcpot *box, int flash)
{
  XSetForeground(display, gc, grey.pixel);       /* clear area */
  XFillRectangle(display, wdw, gc, box->x, box->y, box->w, box->h);

  XSetForeground(display, gc, ltgrey.pixel);     /* upper left */
  XDrawArc(display, wdw, gc, box->x, box->y, box->w, box->h, 45*64, 180*64);
  XDrawArc(display, wdw, gc, box->x, box->y + 1, box->w, box->h, 45*64, 180*64);

  XSetForeground(display, gc, dkgrey.pixel);    /* lower right */
  XDrawArc(display, wdw, gc, box->x, box->y, box->w, box->h, 225*64, 180*64);
  XDrawArc(display, wdw, gc, box->x, box->y + 1, box->w, box->h, 225*64, 180*64);

	XSetForeground(display, gc, yellow.pixel);   /* draw value */
	XSetFont(display, gc, smallfont->fid);
	XDrawString(display, wdw, gc, box->x, box->y + box->h * .6, 
		box->text, strlen(box->text));
	XSetFont(display, gc, menufont->fid);

  XSetForeground(display, gc, black.pixel);
  XDrawLine(display, wdw, gc, box->x + box->w / 2, box->y + box->h / 2, box->x2,
  	box->y2);  /* draw pointer */
	
  if(flash){
    XCopyArea(display, wdw, win, gc, box->x, box->y, 
    	box->w + 2, box->h + 2, box->x, box->y);
    XFlush(display);
  }
}

void initpot(Window win, Pixmap wdw, bcpot *box, int flash)
{
	static float angle;
	
	switch(box->type){       /* get position and text */
		case POT_INT:
			angle = (box->value - box->minvalue) / (box->maxvalue - box->minvalue)
				* 1.5 * 3.14159265 + .25 * 3.14159265;
			sprintf(box->text, "%.0f", box->value);
			break;
		case POT_DB:
			angle = (box->value - box->minvalue) / (box->maxvalue - box->minvalue)
				* 1.5 * 3.14159265 + .25 * 3.14159265;
			if(box->value >= 0) sprintf(box->text, "+%.1f", box->value);
			else if(box->value > INFINITYGAIN) sprintf(box->text, "%.1f", box->value);
			else sprintf(box->text, "oo");
			break;
		case POT_FREQ:
			angle = (box->value - box->minvalue) / (box->maxvalue - box->minvalue)
				* 1.5 * 3.14159265 + .25 * 3.14159265;
			if(tofreq(box->value) < 1000) sprintf(box->text, "%d", tofreq(box->value));
			else sprintf(box->text, "%.1fk", (float)tofreq(box->value) / 1000);
			break;
	}
    /* get angle of pointer */

  box->y2 = cos(angle) * box->h / 2;
  box->x2 = - sin(angle) * box->w / 2;
  
  box->x2 += box->x + box->w / 2;
  box->y2 += box->y + box->h / 2;

	DrawPot(wdw, win, box, flash);    /* draw pot */
}

/* return 1 if pot still selected */
int potevent(Window win, Pixmap wdw, XEvent *report, bcpot *box, int flash)
{
	static float x, y;   /* cursor location */
	static float x1, y1, angle;

	switch(report->type){
		case MotionNotify:
			x = report->xmotion.x;
			y = report->xmotion.y;
			break;
		case ButtonPress:
			x = report->xbutton.x;
			y = report->xbutton.y;
			break;
		case ButtonRelease:
			return 0;
			break;
		default:
			return 1;
			break;
	}
	
	x -= box->x + box->w / 2;
	y -= box->y + box->h / 2;

	/* fix x, y that gives error */

	if(x < 0 && y > 0){
		x1 = y;
		y1 = x;
	}else{
		x1 = x;
		y1 = y;
	}

	if(!y || !x){
		if(x < 0) angle = .5;
		else
		if(x > 0) angle = 1.5;
		else
		if(y < 0) angle = 1;
		else
		if(y > 0) angle = 0;
	}else{
		angle = atan(y1 / x1);
		angle /= 3.14159265;
	}

	/* fix angle */

	if(x < 0 && y < 0){
		angle += .5;
	}else
	if(x > 0 && y > 0){
		angle += 1.5;
	}else
	if(x > 0 && y < 0){
		angle += 1.5;
	}

	angle -= .25;         /* compensate for min pot position */
	if(x < 0 && y > 0) angle = -angle -.5;

	if(angle < 0) angle = 0;
	else
	if(angle > 1.5) angle = 1.5;

	switch(box->type){       /* get new value */
		case POT_INT:
			box->value = angle / 1.5 * (box->maxvalue - box->minvalue) + box->minvalue;
			break;
		case POT_DB:
			box->value = angle / 1.5 * (box->maxvalue - box->minvalue) + box->minvalue;
			if(box->value >= -0.2 && box->value <= 0.2) box->value = 0;  /* notched fader */
			box->value *= 10;
			box->value = (int)box->value;     /* set precision */
			box->value /= 10;
			break;
		case POT_FREQ:
			box->value = angle / 1.5 * (box->maxvalue - box->minvalue) + box->minvalue;
			break;
	}
	initpot(win, wdw, box, flash);       /* draw pot */
	return 1;         /* pot still selected */
}
