/*
  Broadcast 2.0 multitrack audio editing
  (c) 1997 Adam Williams

  This program is distributed with the intent that it will be useful, without
  any warranty.  The code is being updated and has many bugs which are
  constantly being elucidated so changes should not be made without notifying
  the author.
*/

/* progressbox.c         entertainment while you wait */

#include "interfac.h"
#include "progressbox.h"

struct ProgressWindow progresswindow = {
  500, 90, "Progress",
  {10, 24, 480, 20},
  {200, 50, 100, 30, "Cancel"}
};

static long previous;    /* previous position for progress bar */
Window progresswin;
Pixmap progresspixmap;

/* draw progress bar for updateprogress */
/* return 1 if progress has changed */
int DrawProgressBar(Drawable wdw, Window win, bcbox *box, long current,
  long *previous, long total)
{
  static XPoint point[4];
  static TWO w;

  w = (float)current / (float)total * (box->w - 2);
  if(w != *previous){
    XSetForeground(display, gc, white.pixel);
    XFillRectangle(display, wdw, gc, box->x, box->y,
      box->w, box->h);
    XSetForeground(display, gc, black.pixel);
    XDrawRectangle(display, wdw, gc, box->x, box->y,
      box->w, box->h);
    XSetForeground(display, gc, green.pixel);
    XFillRectangle(display, wdw, gc, box->x + 1, box->y + 1,
      w, box->h - 1);
    XSetForeground(display, gc, black.pixel);

    XCopyArea(display, wdw, win, gc, box->x, box->y,
      box->w+1, box->h+1, box->x, box->y);

    *previous = w;
    return 1;
  }else{
    return 0;
  }
}

void initializeprogress(char *string){
  Visual *vis;
  XVisualInfo visinfo;
  XSizeHints size_hints;
  XSetWindowAttributes attr;
  Cursor arrow;
  unsigned long mask;

                                   /* open progress window */
  arrow = XCreateFontCursor(display, XC_top_left_arrow);
  vis = DefaultVisual(display, screen);

  mask = CWEventMask | CWBackPixel | CWBorderPixel | CWColormap |
    CWCursor;
  attr.event_mask = ExposureMask | ButtonPressMask |
    ButtonReleaseMask | KeyPressMask | KeyReleaseMask;
  attr.background_pixel = grey.pixel;
  attr.border_pixel = dkgrey.pixel;
  attr.colormap = cmap;
  attr.cursor = arrow;

  progresswin = XCreateWindow(display, rootwin, 0, 0, progresswindow.w,
    progresswindow.h, 0, depth, InputOutput, vis, mask, &attr);

  progresspixmap =   XCreatePixmap(display, progresswin, progresswindow.w,
    progresswindow.h, depth);

  size_hints.flags = PSize;
  size_hints.width = progresswindow.w;
  size_hints.height = progresswindow.h;

  XSetStandardProperties(display, progresswin, "Progress",
    "Progress", None, 0, 0, &size_hints);

  XMapWindow(display, progresswin);

                                /* draw window contents */
  previous = 0;
  XSetForeground(display, gc, grey.pixel);
  XFillRectangle(display, progresspixmap, gc, 0, 0, progresswindow.w,
    progresswindow.h);
  XSetForeground(display, gc, black.pixel);
  XDrawString(display, progresspixmap, gc, 10, 20, string, strlen(string));
                                /* progressbar */
  DrawProgressBar(progresspixmap, progresswin,
    &progresswindow.progressbar, 0, &previous, 100);

                                /* buttons */
  DrawUpButton(progresspixmap, progresswin, &progresswindow.cancel);
  XCopyArea(display, progresspixmap, progresswin, gc, 0, 0,
    progresswindow.w, progresswindow.h, 0, 0);
    XFlush(display);
}

int updateprogress(long current, long total){
  XEvent report, nextreport;
  KeySym keysym;
  XComposeStatus composestatus;
  char keys_return[2];
  int x, y;

  DrawProgressBar(progresspixmap, progresswin,
    &progresswindow.progressbar, current, &previous, total);
                                /* event handler */
  while(XPending(display)){
    do{
      XNextEvent(display, &report);
      if(XPending(display)) XPeekEvent(display, &nextreport);
      else nextreport.type = 0;
    }while(nextreport.type == Expose && report.type == Expose);

    switch(report.type){
      case Expose:
        XCopyArea(display, progresspixmap, progresswin, gc, 0, 0,
          progresswindow.w, progresswindow.h, 0, 0);
          handleexpose(&report);
        break;

      case KeyPress:
        keys_return[0] = '\0';
        XLookupString ((XKeyEvent *)&report, keys_return, 1, &keysym,
          &composestatus);

        switch(keysym){
          case XK_Escape:
            DrawDnButton(progresspixmap, progresswin, &progresswindow.cancel);
            return 1;
            break;
        }
        break;

      case ButtonPress:
        x = report.xbutton.x;
        y = report.xbutton.y;

        if(buttonpressrelease(progresspixmap, progresswin, x, y, &progresswindow.cancel)){
          return 1;
        }
        break;

      default:
        break;
    }                           /* end switch report type */
  }
  return 0;
}

int destroyprogress(){
  XDestroyWindow(display, progresswin);
  XFreePixmap(display, progresspixmap);
}
