/*
  Broadcast 2.0 multitrack audio editing
  (c) 1997 Adam Williams

  This program is distributed with the intent that it will be useful, without
  any warranty.  The code is being updated and has many bugs which are
  constantly being elucidated so changes should not be made without notifying
  the author.
*/

/* record.c             recording interface and handler  */

#include "defs.h"
#include "audio.h"
#include "record.h"
#include "bcast.h"
#include "file.h"
#include "interfac.h"
#include "database.h"

struct RecWindow{
  int w;
  int h;
  char *title;
  bcbutton restart;
  bcbutton label;
  bcbutton record;
  bcbutton save;
  bcbutton duplex;
  bcslider level;
  bcbox meter;
  bcbox rate;
  int labelx;
  int labely;
  int timex;
  int timey;
} recwindow;

struct RecWindow recwindow = {
  620, 0, "Record",
  {10, 5, 100, 30, "Start Over"},
  {300, 5, 100, 30, "Label"},
  {200, 5, 50, 50, "Record"},
  {450, 5, 130, 30, "Save and exit"},
  {480, 50, 16, 16, "Playback"},
  {5, 100, 128, 14, SLIDER_INT, 0, 128, 0, 0, ""},                /* level */
  {200, 100, 400, 15},                       /* meter */
  {400, 94, 250, 20},                       /* rate text */
  280, 40,
  5, 40
};

struct RecEvent recevent = {0, 0, 0, 0, 0, 0, 0, 0, 0, "", 0 , PCMLOHI, LINE};

Window recwin;
Pixmap recpixmap;

char outfilename[256];       /* filename used for default record directory */
static char recfiletype = PCMLOHI;
long reclabel[MAXRECLABELS];
int reclabels;
char duplexenable = 1;

void recordreclabel(long x, long  reclabel[], int *reclabels){
  static char timestring[15], string[32];

  if(reclabel[(*reclabels)-1] == x && *reclabels > 0){
    (*reclabels)--;
  }else{
    reclabel[(*reclabels)++] = x;
  }

                /* inform user of reclabel */
  XSetForeground(display, gc, grey.pixel);
  XFillRectangle(display, recpixmap, gc, recwindow.labelx, recwindow.labely, 150,
    30);
  XSetForeground(display, gc, BlackPixel(display,
    screen));

  if(*reclabels > 0){
    gettime(reclabel[(*reclabels)-1], timestring);
    sprintf(string, "Label %d    %s", (*reclabels)-1, timestring);
     XDrawString(display, recpixmap, gc, recwindow.labelx, recwindow.labely + 20,
       string, strlen(string));
   }
  XCopyArea(display, recpixmap, recwin, gc, recwindow.labelx, recwindow.labely,
  150, 30, recwindow.labelx, recwindow.labely);
}

void updatetimedisplay(long x){
  static char timestring[15], string[32];

                       /* update time display */
  XSetForeground(display, gc, grey.pixel);
  XFillRectangle(display, recpixmap, gc, recwindow.timex, recwindow.timey, 150, 30);
  XSetForeground(display, gc, black.pixel);

  gettime(x, timestring);
  sprintf(string, "Time: %s", timestring);
  XDrawString(display, recpixmap, gc, recwindow.timex, recwindow.timey+20, string, strlen(string));
  XCopyArea(display, recpixmap, recwin, gc, recwindow.timex, recwindow.timey, 150,
    30, recwindow.timex, recwindow.timey);
}

void recmotion(XEvent *report, int *reclevel){
  if(recevent.leveladjust){
    sliderevent(recwin, recpixmap, report, &recwindow.level, 1);
    *reclevel = recwindow.level.value;
    setrecordgain(recevent.level, *reclevel);
  }
}

void rechandleevent(int reclevel[], int channels){
  XEvent report;
  KeySym keysym;
  XComposeStatus composestatus;
  char keys_return[2];
  int x, y, x1, y1, i;
  int channel;

#ifndef _LINUX_
	if(XPending(display))
#else
             /* flush all the events */
  while(XPending(display))
#endif
  {
    XNextEvent(display, &report);
    x = report.xbutton.x;
    y = report.xbutton.y;

    switch(report.type){
      case Expose:
        if(report.xexpose.window == recwin)
          XCopyArea(display, recpixmap, recwin, gc, 0, 0, recwindow.w, recwindow.h, 0, 0);
        else
          handleexpose(&report);
        break;

      case ButtonPress:
        if(testbutton(x, y, &recwindow.restart)){
          DrawDnButton(recpixmap, recwin, &recwindow.restart);
          recevent.startover = 2;
        }
        if(testbutton(x, y, &recwindow.record)){
          if(recevent.paused) DrawDnRecord(recpixmap, recwin, &recwindow.record);
          else DrawUpRecord(recpixmap, recwin, &recwindow.record);
          recevent.pause = 2;
        }
        if(testbutton(x, y, &recwindow.label)){
          DrawDnButton(recpixmap, recwin, &recwindow.label);
          recevent.labeladd = 2;
        }
        if(testbutton(x, y, &recwindow.save)){
          DrawDnButton(recpixmap, recwin, &recwindow.save);
          recevent.stoprecord = 2;
        }
        if(testbutton(x, y, &recwindow.duplex)/* && recevent.length == 0*/){
          recevent.duplex ^= 1;
          if(recevent.duplex) DrawDnPlay(recpixmap, recwindow.duplex.x,
            recwindow.duplex.y, recwindow.duplex.w, recwindow.duplex.h);
          else
            DrawUpPlay(recpixmap, recwindow.duplex.x,
              recwindow.duplex.y, recwindow.duplex.w, recwindow.duplex.h);

          XCopyArea(display, recpixmap, recwin, gc, recwindow.duplex.x,
            recwindow.duplex.y, recwindow.duplex.w+1, recwindow.duplex.h+5,
            recwindow.duplex.x, recwindow.duplex.y);
        }
#ifndef _LINUX_
        for(recwindow.level.y = 100, i = 0; i < channels;)
        {
          if(testbox(x, y, (bcbox *)&recwindow.level)){
            recevent.leveladjust = 1;
            recevent.level = i;
            recwindow.level.value = reclevel[recevent.level];
            recmotion(&report, &reclevel[recevent.level]);
            i = channels + 1;
          }else{
	          i++;
        	  recwindow.level.y += 20;
        	}
        }
#endif
        break;

      case KeyPress:
        keys_return[0] = '\0';
        XLookupString ((XKeyEvent*)&report, keys_return, 1, &keysym,
          &composestatus);

        switch(keysym){
        
#ifndef _LINUX_
          case XK_Left:
            for(i = 0; i < channels; i++){
              if(reclevel[i] > 0) reclevel[i]--;
              setrecordgain(i, reclevel[i]);
              recwindow.level.y = 100 + 20 * i;
              recwindow.level.value = reclevel[i];
              initslider(recwin, recpixmap, &recwindow.level, 1);
            }
            break;

          case XK_Right:
            for(i = 0; i < channels; i++){
              if(reclevel[i] < recwindow.level.w) reclevel[i]++;
              setrecordgain(i, reclevel[i]);
              recwindow.level.y = 100 + 20 * i;
              recwindow.level.value = reclevel[i];
              initslider(recwin, recpixmap, &recwindow.level, 1);
            }
            break;
#endif

          default:
            switch(keys_return[0]){
              case 's':
                DrawDnButton(recpixmap, recwin, &recwindow.save);
                recevent.stoprecord = 2;
                break;
              case 3:
                DrawDnButton(recpixmap, recwin, &recwindow.restart);
                recevent.startover = 2;
                break;
              case 'l':
                DrawDnButton(recpixmap, recwin, &recwindow.label);
                recevent.labeladd = 2;
                break;
              case 'p':
                if(recevent.paused) DrawDnRecord(recpixmap, recwin, &recwindow.record);
                else DrawUpRecord(recpixmap, recwin, &recwindow.record);
                recevent.pause = 2;
                break;
            }
            break;
        }

#ifndef _LINUX_
      case MotionNotify:
        recmotion(&report, &reclevel[recevent.level]);
        break;
#endif
    } /* end switch report.type */

    if(report.type == ButtonRelease | report.type == KeyRelease){
        if(recevent.startover){                /* reset event flag and redraw button */
          DrawUpButton(recpixmap, recwin, &recwindow.restart);
          DrawUpRecord(recpixmap, recwin, &recwindow.record);
          recevent.startover--;

        }
        if(recevent.labeladd){
          DrawUpButton(recpixmap, recwin, &recwindow.label);
          recevent.labeladd--;
        }
        if(recevent.stoprecord){
          DrawUpButton(recpixmap, recwin, &recwindow.save);
          recevent.stoprecord--;
        }
        if(recevent.leveladjust){
          recevent.leveladjust = 0;
        }
        if(recevent.pause){
          recevent.pause--;
        }
    }
  } /* end XPending */
}

void drawrecordmeter(char *bufferstart, long buffersize, int channels){
  static int channel, y;
  static TWO *_buffer;
  static char *bufferend;
  static int j;
  static long highsample;
  static long lowsample;

  bufferend = bufferstart + buffersize;

  for(recwindow.meter.y = 100, j = 0;
    j < channels;
    j++, recwindow.meter.y += 20)
  {
    _buffer = (TWO *)bufferstart + j;
    highsample = 0;
    lowsample = 0;

    while((char *)_buffer < bufferend){
      if(*_buffer > highsample) highsample = *_buffer;
      else if(*_buffer < lowsample) lowsample = *_buffer;
      _buffer += channels;
    }
    if(highsample < -lowsample) highsample = -lowsample;
    DrawMeter(recpixmap, recwin, &recwindow.meter, highsample);
  }
}

void loadinitrecdir(FILE *in){
  fscanf(in, "%s", outfilename);
}

void saveinitrecdir(FILE *out){
  char string[256];

  cutdirectory(string, outfilename);
  if(strlen(string)) fprintf(out, "RECORDDIR %s\n", string);
}

void loadinitrectype(long value){
  recfiletype = value;
}

long saveinitrectype(){
  return recfiletype;
}

int recordsound(long start, long end){
  FILE *out;
  char *buffer;
  char string[256], string_[256];
  int i, y;
  long filelength, currentposition;
  long audiorxlength = 0;
  int frame;
  int *reclevel;
  struct Track *currenttrack;

  char *playbuffer;
  long playbufferlength, audiotxlength;
  int playframe;
  long playx, playend;

  Visual *vis;
  XVisualInfo visinfo;
  XSizeHints size_hints;
  XSetWindowAttributes attr;
  Cursor arrow;
  unsigned long mask;

                           /* get # of tracks with record permission */
  recevent.channels = recordabletracks();
  if(recevent.channels == 0){
		displayalert("Define some tracks to record on.");
  	return 1;
	}
	
  reclevel = malloc(sizeof(int) * recevent.channels);

  frame = recevent.channels * 2;

  strcpy(recevent.file, outfilename);
  recevent.format = recfiletype;
  if(getfileattributes(recevent.file, &recevent.format, RECORD, "Record")){
    free(reclevel);
    return 1;
  }
  recfiletype = recevent.format;

  strcpy(outfilename, recevent.file);

  if(out = fopen(recevent.file, "rb")){
    fclose(out);
    sprintf(string, "%s exists.  Record over it?", recevent.file);
    if(confirm(string, "Record")){
      free(reclevel);
      return 1;
    }
  }
  updateundo(RECORD);
  changesmade = 1;
  
  if(openaudio()){
    free(reclevel);
    return 1;
  }

  recwindow.h = 120 + recevent.channels * 20;

                        /* delete index file if one exists */
  sprintf(string, "%s%s", recevent.file, SCALEFILEEXT);
  remove(string);

  if(!(out = fopen(recevent.file, "wb"))){
    printf("Can't open output file\n");
    closeaudio();
    free(reclevel);
    return 1;
  }

  if(!(buffer = malloc(RECBUFFER))){
    printf("Not enough memory to open record buffer.\n");
    closeaudio();
    fclose(out);
    free(reclevel);
    return 1;
  }

  initrecordaudio(recevent.audioinput, buffer, recevent.channels);

#ifndef _HPUX_
                           /* initialize reclevel of tracks */
  getrecordgain(recevent.channels, reclevel);
#endif

  if(duplexenable){
    initplaybackaudio(recevent.channels);
    if(startmixdown(PLAYBUFFER + recevent.channels * 2, recevent.channels)){
      duplexenable = 0;
    }else{
      if(!(playbuffer = malloc(PLAYBUFFER + recevent.channels * 2))){
        printf("Not enough memory for duplex.\n");
        duplexenable = 0;
      }
    }
  }

                                   /* initialize screen */
  arrow = XCreateFontCursor(display, XC_top_left_arrow);
  vis = DefaultVisual(display, screen);

  mask = CWEventMask | CWBackPixel | CWBorderPixel | CWColormap |
    CWCursor;
  attr.event_mask = ExposureMask | ButtonPressMask |
    ButtonReleaseMask | KeyPressMask | KeyReleaseMask | PointerMotionMask;
  attr.background_pixel = grey.pixel;
  attr.border_pixel = dkgrey.pixel;
  attr.colormap = cmap;
  attr.cursor = arrow;

  recwin = XCreateWindow(display, rootwin, 0, 0, recwindow.w, recwindow.h,
    0, depth, InputOutput, vis, mask, &attr);

  recpixmap =   XCreatePixmap(display, recwin, recwindow.w, recwindow.h, depth);

  size_hints.flags = PSize;
  size_hints.width = recwindow.w;
  size_hints.height = recwindow.h;

  truncatefilename(string_, recevent.file);
  sprintf(string, "%s %s", recwindow.title, string_);
  XSetStandardProperties(display, recwin, string, recwindow.title, None,
    0, 0, &size_hints);

  XMapWindow(display, recwin);
  XSetForeground(display, gc, grey.pixel);
  XFillRectangle(display, recpixmap, gc, 0, 0, recwindow.w, recwindow.h);

#ifdef _HPUX_
                                        /* draw gain sliders */
  for(recwindow.level.y = 100, i = 0;
    i < recevent.channels; recwindow.level.y += 20, i++)
  {
    recwindow.level.value = reclevel[i];
    initslider(recwin, recpixmap, &recwindow.level, 1);
  }
#endif
                                        /* draw meters */
  drawrecordmeter(buffer, audiorxlength, recevent.channels);

                                        /* draw buttons */
  DrawUpRecord(recpixmap, recwin, &recwindow.record);
  DrawUpButton(recpixmap, recwin, &recwindow.restart);
  DrawUpButton(recpixmap, recwin, &recwindow.label);
  DrawUpButton(recpixmap, recwin, &recwindow.save);

                                /* meter text */
  recwindow.meter.y = 95;
  sprintf(string, "Peak level meter (int)");
  XDrawString(display, recpixmap, gc, recwindow.meter.x + 20,
    recwindow.rate.y, string,
    strlen(string));

                                /* sample rate text */
  sprintf(string, "Sample Rate: %ld", samplerate);
  XDrawString(display, recpixmap, gc, recwindow.rate.x,
    recwindow.rate.y, string, strlen(string));

                               /* status of duplex */
  XDrawString(display, recpixmap, gc, recwindow.duplex.x + 20,
    recwindow.duplex.y + 15, recwindow.duplex.text,
    strlen(recwindow.duplex.text));

  if(recevent.duplex) DrawDnPlay(recpixmap, recwindow.duplex.x,
    recwindow.duplex.y, recwindow.duplex.w, recwindow.duplex.h);
  else
    DrawUpPlay(recpixmap, recwindow.duplex.x,
      recwindow.duplex.y, recwindow.duplex.w, recwindow.duplex.h);

   XCopyArea(display, recpixmap, recwin, gc, 0,
    0, recwindow.w, recwindow.h, 0, 0);

  recevent.stoprecord = 0;
  recevent.leveladjust = 0;
  recevent.labeladd = 0;
  recevent.startover = 0;
  playframe = recevent.channels * 2;
  playend = longesttrack();

  do{                               /* loop for startover */
                           /* initialize play buffer length */
    playbufferlength = PLAYBUFFER - PLAYBUFFER / 4;

    playbufferlength /= playframe;
    playbufferlength *= playframe;
    playx = start;

		if(recevent.format == WAV){
			fwrite(buffer, WAVHEADERLENGTH, 1, out);
		}

    recevent.length = 0;
    if(recevent.startover) recevent.startover--;
    reclabels = 0;
    recevent.pause = 0;
    recevent.paused = 1;                             /* start paused */

    if(duplexenable){
                                           /* load first playback buffer */
      if(playbufferlength / frame > playend - playx)
        playbufferlength = (playend - playx) * frame;
      if(mixdown(playbuffer, playbufferlength, playx)){
        printf("Can't mix down output audio!\n");
      }
      playx += playbufferlength / playframe;
    }

    updatetimedisplay(recevent.length);
    recordreclabel(recevent.length, reclabel, &reclabels);
    recordreclabel(recevent.length, reclabel, &reclabels);
    do{                                           /* loop for record */
      rechandleevent(reclevel, recevent.channels);            /* get events */

      if(recevent.labeladd == 2){   /* reclabel button just pressed */
        recordreclabel(recevent.length, reclabel, &reclabels);
        recevent.labeladd--;        /* command executed */
      }

                                  /* pause button just pressed */
      if(recevent.pause == 1){
        recevent.pause--;
        if(recevent.paused){
          recevent.paused = 0;
          audiorxlength = getaudiorxlength(frame);
          if(audiorxlength > 0) readaudio(buffer, audiorxlength);
          resumeaudiooutput();
        }else{
          recevent.paused = 1;                    /* flush input buffer */
          audiorxlength = getaudiorxlength(frame);
          readaudio(buffer, audiorxlength);
          drawrecordmeter(buffer, audiorxlength, recevent.channels);
          
          if((recevent.format == WAV && !byteorder) ||
            (recevent.format == PCMHILO && byteorder) ||
            (recevent.format == PCMLOHI && !byteorder))
            swapbytes(buffer, audiorxlength);
            
          fwrite(buffer, audiorxlength, 1, out);
          if(recevent.format == WAV) recevent.length=(ftell(out) - WAVHEADERLENGTH) / frame;
          else recevent.length = ftell(out) / frame;
          updatetimedisplay(recevent.length);

                                           /* load new playback buffer */
          if(duplexenable && recevent.duplex){
            pauseaudiooutput();
            resetaudio();
            playx = recevent.length + start;
            if(playbufferlength / frame > playend - playx)
              playbufferlength = (playend - playx) * frame;
            if(mixdown(playbuffer, playbufferlength, playx)){
              printf("Can't mix down output audio!\n");
            }
            playx += playbufferlength / playframe;
          }
        }
      }

                              /* get playback buffer status */
      if(duplexenable && recevent.duplex && !recevent.paused){
        
      if(getaudiotxlength() < PLAYBUFFER / 4 && playx < playend)
			  {
          writeaudio(playbuffer, playbufferlength);

          if(playbufferlength / frame > playend - playx)
            playbufferlength = (playend - playx) * frame;
          if(mixdown(playbuffer, playbufferlength, playx)){
            printf("Can't mix down output audio!\n");
          }
          playx += playbufferlength / playframe;
        }
      }
                              /* get record buffer status */
#ifndef _LINUX_     /* linux bug */
      audiorxlength = getaudiorxlength(frame);
      if(audiorxlength > samplerate * recevent.channels * 2 / 8 
      	|| recevent.stoprecord == 1)
#else
			audiorxlength = samplerate * recevent.channels * 2 / 8;
			audiorxlength /= frame;
			audiorxlength *= frame;
      {
#endif
    
/* printf("%d %d\n", audiorxlength, samplerate * recevent.channels * 2 / 8 ); */

        readaudio(buffer, audiorxlength);
        drawrecordmeter(buffer, audiorxlength, recevent.channels);

        if(!recevent.paused){   /* write only if not paused */
        
          if((recevent.format == WAV && !byteorder) ||
            (recevent.format == PCMHILO && byteorder) ||
            (recevent.format == PCMLOHI && !byteorder))
            swapbytes(buffer, audiorxlength);
            
          fwrite(buffer, audiorxlength, 1, out);
          if(recevent.format == WAV) recevent.length=(ftell(out) - WAVHEADERLENGTH) / frame;
          else recevent.length = ftell(out) / frame;
          updatetimedisplay(recevent.length);
        }
      }
    }while(recevent.stoprecord != 1 && recevent.startover != 1);

    if(recevent.startover == 1){
      fclose(out);
      remove(recevent.file);
      if(!(out = fopen(recevent.file, "wb"))){
        printf("Can't open output file\n");
        closeaudio();
        XDestroyWindow(display, recwin);
        XFreePixmap(display, recpixmap);
        if(duplexenable){
          killmixdown();
          free(playbuffer);
        }
        free(reclevel);
        free(buffer);
        return 1;
      }
    }
  }while(recevent.startover == 1);               /* end startover loop */

                        /* on HPUX if computer can't keep up with the audio,
                           the program hangs because the audio
                           buffer maxes out below playbuffer. */

                        /* write header and close files */
  XDestroyWindow(display, recwin);
  XFreePixmap(display, recpixmap);

  if(recevent.length > 0){
    filelength = ftell(out);
    if(recevent.format == WAV){
      fseek(out, 0, SEEK_SET);
      writeheader(out, filelength, recevent.channels);
    }
    pauseaudiooutput();
    closeaudio();
    fclose(out);

    paste(start, end, RECORD);

    drawpatches(starty);
    drawtime(startx);
    drawplaycontrols();
    drawconsolecontrols();
  }else{
    closeaudio();
    fclose(out);
    remove(recevent.file);
  }

  if(duplexenable){
    killmixdown();
    free(playbuffer);
  }

  free(buffer);
  free(reclevel);
  return 0;
}
