/*
  Broadcast 2.0 multitrack audio editing
  (c) 1997 Adam Williams

  This program is distributed with the intent that it will be useful, without
  any warranty.  The code is being updated and has many bugs which are
  constantly being elucidated so changes should not be made without notifying
  the author.
*/

/* scrollbar.c       scrollbar handlers */

#include "interfac.h"
#include "scrollbar.h"

void xscrollevent(Drawable wdw, Window win, XEvent *report, bcscroll *bar){
  static long x, newx, oldx;   /* new x position and old x position for motion event */
  static long position, span;

  if(bar->size > 0){
    position =
      (float)(bar->w - bar->h * 2) / bar->size * bar->position +
      bar->h;
    span = (float)(bar->w - bar->h * 2) / bar->size * bar->span;
  }else{
    position = bar->h;
    span = bar->w - bar->h * 2;
  }
  if(span > bar->w - bar->h * 2) span = bar->w - bar->h * 2;
  if(span < 5) span = 5;

  switch(report->type){
    case ButtonPress:
      oldx = x = report->xbutton.x;
      x -= bar->x;

      if(x < bar->h) bar->event = ARROW1;
      if(x > bar->w - bar->h) bar->event = ARROW2;
      if(x > bar->h && x < bar->w - bar->h){
        if(x < position) bar->event = UPPAGE;
        else
        if(x > position + span) bar->event = DNPAGE;
        else{
          x = bar->position / ((float)bar->size / (bar->w - bar->h * 2));
          bar->event = SPAN;
        }
      }
      break;

    case MotionNotify:
      if(bar->event == SPAN){
        newx = report->xmotion.x;
        x += newx - oldx;
        oldx = newx;

        bar->position = (float)bar->size / (bar->w - bar->h * 2) * x;

        if(bar->position < 0) bar->position = 0;
        if(bar->position > bar->size - bar->span)
          bar->position = bar->size - bar->span;
        if(bar->position < 0) bar->position = 0;
      }
      break;

    case ButtonRelease:
      bar->event = 0;
      break;
  }
  DrawXScrollBar(wdw, win, bar);
}

void yscrollevent(Drawable wdw, Window win, XEvent *report, bcscroll *bar){
  static long y, newy, oldy;   /* new and old y position on screen */
  static long position, span;

  if(bar->size > 0){
    position =
      (float)(bar->h - bar->w * 2) / bar->size * bar->position +
      bar->w;
    span = (float)(bar->h - bar->w * 2) / bar->size * bar->span;
  }else{
    position = bar->w;
    span = bar->h - bar->w * 2;
  }
  if(span > bar->h - bar->w * 2) span = bar->h - bar->w * 2;
  if(span < 5) span = 5;

  switch(report->type){
    case ButtonPress:
      oldy = y = report->xbutton.y;
      y -= bar->y;

      if(y < bar->w) bar->event = ARROW1;
      if(y > bar->h - bar->w) bar->event = ARROW2;
      if(y > bar->w && y < bar->h - bar->w){
        if(y < position) bar->event = UPPAGE;
        else
        if(y > position + span) bar->event = DNPAGE;
        else{
          y = bar->position / ((float)bar->size / (bar->h - bar->w * 2));
          bar->event = SPAN;
        }
      }
      break;

    case MotionNotify:
      if(bar->event == SPAN){
        newy = report->xmotion.y;
        y += newy - oldy;
        oldy = newy;

        bar->position = (float)bar->size / (bar->h - bar->w * 2) * y;

        if(bar->position < 0) bar->position = 0;
        if(bar->position > bar->size - bar->span)
          bar->position = bar->size - bar->span;
        if(bar->position < 0) bar->position = 0;
      }
      break;

    case ButtonRelease:
      bar->event = 0;
      break;
  }
  DrawYScrollBar(wdw, win, bar);
}

void updatexscrollbar(Drawable wdw, Window win, long position, long span,
  long size, bcscroll *bar)
{
  bar->position = position;
  bar->size = size;
  bar->span = span;
  if(position > size - span) bar->position = bar->size - span;
  if(bar->position < 0) bar->position = 0;
  DrawXScrollBar(wdw, win, bar);
}


void updateyscrollbar(Drawable wdw, Window win, long position, long span,
  long size, bcscroll *bar)
{
  bar->position = position;
  bar->size = size;
  bar->span = span;
  if(position > size - span) bar->position = bar->size - span;
  if(bar->position < 0) bar->position = 0;
  DrawYScrollBar(wdw, win, bar);
}

void DrawXScrollBar(Drawable wdw, Window win, bcscroll *scrollbar){
  static int x1, y1, x2, y2, x3, y3;
  static XPoint point[3];
  static long position, span;

  DrawThinDnBox(wdw, scrollbar->x, scrollbar->y, scrollbar->w, scrollbar->h);

  y1 = scrollbar->y + 1;
  x1 = scrollbar->x + 1;
  y2 = scrollbar->y + scrollbar->h / 2;
  x2 = scrollbar->x + scrollbar->h;
  y3 = scrollbar->y + scrollbar->h - 1;

  point[0].x = x1;
  point[0].y = y2;
  point[1].x = x2;
  point[1].y = y1;
  point[2].x = x2;
  point[2].y = y3;

  if(scrollbar->event == ARROW1) XSetForeground(display, gc, migrey.pixel);
    else  XSetForeground(display, gc, grey.pixel);
  XFillPolygon(display, wdw, gc, (XPoint *)point, 3, Nonconvex,
  CoordModeOrigin);

  if(scrollbar->event == ARROW1) XSetForeground(display, gc, dkgrey.pixel);
    else  XSetForeground(display, gc, ltgrey.pixel);
  XDrawLine(display, wdw, gc, x1, y2, x2, y1);

  if(scrollbar->event == ARROW1) XSetForeground(display, gc, ltgrey.pixel);
    else  XSetForeground(display, gc, dkgrey.pixel);
  XDrawLine(display, wdw, gc, x2, y1, x2, y3);
  XDrawLine(display, wdw, gc, x2, y3, x1, y2);

  x1 = scrollbar->x + scrollbar->w - scrollbar->h;
  x2 = scrollbar->x + scrollbar->w - 1;

  point[0].x = x1;
  point[0].y = y1;
  point[1].x = x2;
  point[1].y = y2;
  point[2].x = x1;
  point[2].y = y3;

  if(scrollbar->event == ARROW2) XSetForeground(display, gc, migrey.pixel);
    else  XSetForeground(display, gc, grey.pixel);
  XFillPolygon(display, wdw, gc, (XPoint *)point, 3, Nonconvex,
  CoordModeOrigin);

  if(scrollbar->event == ARROW2) XSetForeground(display, gc, ltgrey.pixel);
    else  XSetForeground(display, gc, dkgrey.pixel);
  XDrawLine(display, wdw, gc, x2, y2, x1, y1);

  if(scrollbar->event == ARROW2) XSetForeground(display, gc, dkgrey.pixel);
    else  XSetForeground(display, gc, ltgrey.pixel);
  XDrawLine(display, wdw, gc, x1, y3, x1, y1);
  XDrawLine(display, wdw, gc, x1, y1, x2, y2);

  if(scrollbar->size > 0){
    position =
      (float)(scrollbar->w - scrollbar->h * 2) / scrollbar->size
      * scrollbar->position +
      scrollbar->x + scrollbar->h;
    span = (float)(scrollbar->w - scrollbar->h * 2) /
      scrollbar->size * scrollbar->span;
  }else
  {
    position = scrollbar->x + scrollbar->h;
    span = scrollbar->w - scrollbar->h * 2;
  }

  if(span > scrollbar->w - scrollbar->h * 2)
    span = scrollbar->w - scrollbar->h * 2;
  if(span < 5) span = 5;
  if(position < scrollbar->x + scrollbar->h)
    position = scrollbar->x + scrollbar->h;


  if(scrollbar->event == SPAN)
    DrawThinDnBox(wdw, position,
      scrollbar->y + 1, span, scrollbar->h - 2);
  else
    DrawThinUpBox(wdw, position,
      scrollbar->y + 1, span, scrollbar->h - 2);

  XSetForeground(display, gc, black.pixel);
  XCopyArea(display, wdw, win, gc, scrollbar->x, scrollbar->y,
    scrollbar->w+1, scrollbar->h+1, scrollbar->x, scrollbar->y);
}

void DrawYScrollBar(Drawable wdw, Window win, bcscroll *scrollbar){
  static int x1, y1, x2, y2, x3, y3;
  static XPoint point[3];
  static long position, span;

  DrawThinDnBox(wdw, scrollbar->x, scrollbar->y, scrollbar->w, scrollbar->h);

  x1 = scrollbar->x + 1;
  y1 = scrollbar->y + 1;
  x2 = scrollbar->x + scrollbar->w / 2;
  y2 = scrollbar->y + scrollbar->w;
  x3 = scrollbar->x + scrollbar->w - 2;

  point[0].x = x2;
  point[0].y = y1;
  point[1].x = x3;
  point[1].y = y2;
  point[2].x = x1;
  point[2].y = y2;

  if(scrollbar->event == ARROW1) XSetForeground(display, gc, migrey.pixel);
    else  XSetForeground(display, gc, grey.pixel);
  XFillPolygon(display, wdw, gc, (XPoint *)point, 3, Nonconvex,
  CoordModeOrigin);

  if(scrollbar->event == ARROW1) XSetForeground(display, gc, dkgrey.pixel);
    else  XSetForeground(display, gc, ltgrey.pixel);
  XDrawLine(display, wdw, gc, x2, y1, x1, y2);

  if(scrollbar->event == ARROW1) XSetForeground(display, gc, ltgrey.pixel);
    else  XSetForeground(display, gc, dkgrey.pixel);
  XDrawLine(display, wdw, gc, x2, y1, x3, y2);
  XDrawLine(display, wdw, gc, x3, y2, x1, y2);

  y1 = scrollbar->y + scrollbar->h - scrollbar->w;
  y2 = scrollbar->y + scrollbar->h - 2;

  point[0].x = x2;
  point[0].y = y2;
  point[1].x = x3;
  point[1].y = y1;
  point[2].x = x1;
  point[2].y = y1;

  if(scrollbar->event == ARROW2) XSetForeground(display, gc, migrey.pixel);
    else  XSetForeground(display, gc, grey.pixel);
  XFillPolygon(display, wdw, gc, (XPoint *)point, 3, Nonconvex,
  CoordModeOrigin);

  if(scrollbar->event == ARROW2) XSetForeground(display, gc, ltgrey.pixel);
    else  XSetForeground(display, gc, dkgrey.pixel);
  XDrawLine(display, wdw, gc, x3, y1, x2, y2);

  if(scrollbar->event == ARROW2) XSetForeground(display, gc, dkgrey.pixel);
    else  XSetForeground(display, gc, ltgrey.pixel);
  XDrawLine(display, wdw, gc, x3, y1, x1, y1);
  XDrawLine(display, wdw, gc, x1, y1, x2, y2);

  if(scrollbar->size > 0){
    position =
      (float)(scrollbar->h - scrollbar->w * 2) / scrollbar->size
      * scrollbar->position +
      scrollbar->y + scrollbar->w;
    span = (float)(scrollbar->h - scrollbar->w * 2) / scrollbar->size *
       scrollbar->span;
  }else{
    position = scrollbar->y + scrollbar->w;
    span = scrollbar->h - scrollbar->w * 2;
  }

  if(span > scrollbar->h - scrollbar->w * 2)
    span = scrollbar->h - scrollbar->w * 2;
  if(span < 5) span = 5;
  if(position < scrollbar->y + scrollbar->w)
    position = scrollbar->y + scrollbar->w;

  if(scrollbar->event == SPAN)
    DrawThinDnBox(wdw, scrollbar->x + 1, position,
      scrollbar->w - 2, span);
  else
    DrawThinUpBox(wdw, scrollbar->x + 1, position,
      scrollbar->w - 2, span);

  XSetForeground(display, gc, black.pixel);
  XCopyArea(display, wdw, win, gc, scrollbar->x, scrollbar->y,
    scrollbar->w+1, scrollbar->h+1, scrollbar->x, scrollbar->y);
}

