#include "interfac.h"
#include "slider.h"
#include "units.h"

const int hw = 16;    /* handle dimensions */

/* convert slider values to value of text box */
void texttoslider(bctextbox *box, bcslider *slider)
{
	switch(slider->type){
		case SLIDER_INT:
			slider->value = atol(box->text);
			break;
			
		case SLIDER_DB:
			if(!strcmp(box->text, "oo")) slider->value = INFINITYGAIN;
			else slider->value = atof(box->text);
			break;
			
		case SLIDER_FREQ:
			slider->value = fromfreq(atol(box->text));
			break;
	}
}

/* convert textbox values to value of slider */
void slidertotext(bctextbox *box, bcslider *slider)
{
	switch(slider->type){
		case SLIDER_INT:
			sprintf(box->text, "%.0f", slider->value);
			break;
			
		case SLIDER_DB:
			if(slider->value >= 0) sprintf(box->text, "+%.1f", slider->value);
			else 
			if(slider->value == INFINITYGAIN) sprintf(box->text, "oo");
			else 
			sprintf(box->text, "%.1f", slider->value);
			break;
			
		case SLIDER_FREQ:
			sprintf(box->text, "%d", tofreq(slider->value));
			break;
	}
}

void DrawSlider(Drawable wdw, Window win, bcslider *box, int flash)
{
	static int hh;    /* handle dimensions */
  static int x1, y1;  /* position of handle */

  x1 = box->x + box->position - hw / 2;
  y1 = box->y + 2;
  hh = box->h - 4;

  DrawDnBox(wdw, box->x, box->y, box->w, box->h);   /* slider box */

  DrawUpBox(wdw, x1, y1, hw, hh);                   /* handle */
  XDrawLine(display, wdw, gc, x1 + hw / 2, y1,
    x1 + hw / 2, y1 + hh);

  XSetForeground(display, gc, white.pixel);         /* text */
  XDrawString(display, wdw, gc, box->x + 2,
    box->y + box->h - 2, box->text, strlen(box->text));
  XSetForeground(display, gc, black.pixel);

  if(flash) XCopyArea(display, wdw, win, gc, box->x, box->y,    /* flash */
    box->w+2, box->h+2, box->x, box->y);
  XFlush(display);
}

void initslider(Window win, Pixmap wdw, bcslider *box, int flash)
{
	int x, y, w, h;  /* inner dimensions */
	char string[32];  /* db text */
	
	w = box->w - 4 - hw;
	x = 2 + hw / 2;
	
	switch(box->type){       /* get position and text */
		case SLIDER_INT:
			box->position = x + w * ((box->value - box->minvalue) 
				/ (box->maxvalue - box->minvalue));
			sprintf(box->text, "%.0f", box->value);
			break;
		case SLIDER_DB:
			box->position = x + w * (box->value - box->minvalue) 
				/ (box->maxvalue - box->minvalue);
			if(box->value >= 0) sprintf(box->text, "+%.1f", box->value);
			else if(box->value > INFINITYGAIN) sprintf(box->text, "%.1f", box->value);
			else sprintf(box->text, "oo");
			break;
		case SLIDER_FREQ:
			box->position = x + ((box->value - box->minvalue) 
				/ (box->maxvalue - box->minvalue)) * w;
			if(tofreq(box->value) < 10000)
				sprintf(box->text, "%d", tofreq(box->value));
			else
				sprintf(box->text, "%.1f", (float)tofreq(box->value) / 1000);
			break;
	}
	DrawSlider(wdw, win, box, flash);    /* draw slider */
}

/* return 1 if slider still selected */
int sliderevent(Window win, Pixmap wdw, XEvent *report, bcslider *box, int flash)
{
	static float x, w;   /* offset */

	w = box->w - 4 - hw;
	switch(report->type){
		case MotionNotify:
			x = report->xmotion.x - box->x - 2 - hw / 2;
			break;
		case ButtonPress:
			x = report->xbutton.x - box->x - 2 - hw / 2;
			break;
		case ButtonRelease:
			return 0;
			break;
		default:
			return 1;
			break;
	}
	if(x < 0) x = 0;
	if(x > box->w - 4 - hw) x = box->w - 4 - hw;
	
	switch(box->type){       /* get new value */
		case SLIDER_INT:
			box->value = x / w * (box->maxvalue - box->minvalue) + box->minvalue;
			break;
		case SLIDER_DB:
			box->value = x / w * (box->maxvalue - box->minvalue) + box->minvalue;
			if(box->value >= -0.2 && box->value <= 0.2) box->value = 0;  /* notched fader */
			box->value *= 10;
			box->value = (int)box->value;     /* set precision */
			box->value /= 10;
			break;
		case SLIDER_FREQ:
			box->value = x / w * (box->maxvalue - box->minvalue) + box->minvalue;
			break;
	}
	initslider(win, wdw, box, flash);       /* draw slider */
	return 1;         /* slider still selected */
}
