/*
  Broadcast 2.0 multitrack audio editing
  (c) 1997 Adam Williams

  This program is distributed with the intent that it will be useful, without
  any warranty.  The code is being updated and has many bugs which are
  constantly being elucidated so changes should not be made without notifying
  the author.
*/

/* textbox.c      display and edit a single line of text */

#include "textbox.h"

static int x1, x2, start, selecttext, selectword;
static long time_;

/* return char position in string of pixel x */
int charof(char *string, int len, int x){
  for(; len > 0 && XTextWidth(menufont, string, len) > x; len--)
    ;

  return len;
}

/* draw a text box or update one without cursor selection */
void create_text_box(Drawable wdw, Window win, bctextbox *box){
  static int len;                          /* length of string to print */

  len = strlen(&box->text[start]);       /* cut length until it fits in box */
  while(XTextWidth(menufont, &box->text[start], len) > box->w - 8) len--;

  XSetForeground(display, gc, white.pixel);    /* fill background */
  XFillRectangle(display, wdw, gc, box->x, box->y, box->w, box->h);

  XSetForeground(display, gc, black.pixel);    /* draw border */
  XDrawRectangle(display, wdw, gc, box->x, box->y, box->w, box->h);


	if(box->h > 20)
	  XDrawString(display, wdw, gc, box->x + 5,    /* draw unhighlighted text */
	    box->y + box->h - 10, &box->text[start], strlen(&box->text[start]));
  else
	  XDrawString(display, wdw, gc, box->x + 5,    /* draw unhighlighted text */
	    box->y + box->h - 2, &box->text[start], strlen(&box->text[start]));
    
    
  XCopyArea(display, wdw, win, gc, box->x, box->y,
    box->w + 1, box->h + 1, box->x, box->y);   /* flash on screen */
}

/* update text in a text box, also updating the cursor selection */
void update_text_box(Drawable wdw, Window win, bctextbox *box){
  static int column1, column2;                 /* columns for highlight box */
  static int len;                          /* length of string to print */

  XSetForeground(display, gc, white.pixel);    /* fill background */
  XFillRectangle(display, wdw, gc, box->x, box->y, box->w, box->h);

  XSetForeground(display, gc, black.pixel);    /* draw border */
  XDrawRectangle(display, wdw, gc, box->x, box->y, box->w, box->h);

  len = strlen(&box->text[start]);       /* cut length until it fits in box */
  while(XTextWidth(menufont, &box->text[start], len) > box->w - 8) len--;

  XDrawString(display, wdw, gc, box->x + 5,
    box->y + box->h - 10, &box->text[start], len);
                                       /* draw selection */
  XSetFunction(display, gc, GXxor);
  XSetForeground(display, gc, white.pixel);    /* set inverse */

  if(start <= x1){        /* x1 after start */
    column1 = XTextWidth(menufont, &box->text[start], x1 - start) + 2;
  }else{                /* x1 before view */
    column1 = -1;
  }

  if(x2 >= start){   /* x2 after start */
    column2 = XTextWidth(menufont, &box->text[start], x2 - start) + 2;
  }else{
    column2 = -1;         /* x2 before view */
  }

  if(column1 < box->w - 2 && column2 >= 2){  /* selection in view */
    if(column1 < 2) column1 = 2;      /* x1 before view */
    if(column2 > box->w - 2) column2 = box->w;  /* x2 after view */

    XFillRectangle(display, wdw, gc, box->x + column1 + 2, box->y + 2,
      column2 - column1 + 1, box->h - 4);
  }

  XSetFunction(display, gc, GXcopy);
  XCopyArea(display, wdw, win, gc, box->x, box->y,
    box->w + 1, box->h + 1, box->x, box->y);   /* flash on screen */
}

/* draw a text box with cursor and highlighted */
void activate_box(Drawable wdw, Window win, bctextbox *box){
  x1 =0;
  x2 = strlen(box->text);     /* set cursor selection for entire string */
  start = 0;
  selecttext = 0;
  selectword = 0;
  time_ = 0;

  XSetForeground(display, gc, red.pixel);

  XDrawRectangle(display, wdw, gc, box->x - 1, box->y - 1,
    box->w + 2, box->h + 2);
  XCopyArea(display, wdw, win, gc, box->x - 2, box->y - 2,
    box->w + 5, box->h + 5, box->x - 2, box->y - 2);

  update_text_box(wdw, win, box);
}

/* set the active box out of range and draw a text box with no cursor */
void deactivate_box(Drawable wdw, Window win, bctextbox *box){
  XSetForeground(display, gc, grey.pixel);
  selecttext = 0;
  selectword = 0;
  x1 = x2 = -1;

  XDrawRectangle(display, wdw, gc, box->x - 1, box->y - 1,
    box->w + 2, box->h + 2);
  XCopyArea(display, wdw, win, gc, box->x - 2, box->y - 2,
    box->w + 5, box->h + 5, box->x - 2, box->y - 2);

  create_text_box(wdw, win, box);
}

/* return 1 if the data has been changed */
/* calling routine must be sure the box is active */
int handle_text_event(Drawable wdw, Window win, XEvent *report, bctextbox *box)
{
  static char keys_return[2];
  static XComposeStatus composestatus;
  static KeySym keysym;
  static int x, y, i, x1_, x2_, center, center_;
  static int wordx1, wordx2;   /* original word selected */

  do{           /* loop if making selection */
    switch(report->type){
      case KeyPress:
        keys_return[0] = '\0';
        XLookupString ((XKeyEvent *)report, keys_return, 1, &keysym,
          &composestatus);

        switch(keysym){
        	case XK_Alt_L:    /* block out the alt */
        		break;
        		
        	case XK_Alt_R:
        		break;
        		
          case XK_Shift_L:    /* block out the shift */
            break;

          case XK_Shift_R:    /* block out the shift */
            break;

          case XK_Left:
            if(x1 < x2) x2 = x1;

            if(x1 > 0){
              x1--;
              x2--;
            }

            if(x1 < start) start = x1;
            update_text_box(wdw, win, box);
            break;

          case XK_Right:
            if(x1 < x2) x1 = x2;

            if(x2 < strlen(box->text)){
              x2++;
              x1++;
            }

            while(XTextWidth(menufont, &box->text[start], x2 - start) + 2 >
              box->w - 2) start++;
            update_text_box(wdw, win, box);
            break;

          default:
            switch(keys_return[0]){
              case 8:
                if(x1 == x2){                    /* single character */
                  if(x1 > 0){
                    for(i = x1 - 1; i < strlen(box->text); i++){
                      box->text[i] = box->text[i+1];
                    }
                    x1--;
                    x2--;
                    if(start) start--;     /* scroll left if extended */
                  }
                }else{                     /* multiple characters */
                  for(i = x1; x2 < strlen(box->text); i++, x2++){
                    box->text[i] = box->text[x2];
                  }
                  box->text[i] = 0;
                  x2 = x1;
                }

                if(x2 < start) start = x2;

                update_text_box(wdw, win, box);
                break;

              default:
                if(x1 == x2){                     /* insert */
                  for(i = strlen(box->text) + 1; i > x1; i--){
                    box->text[i] = box->text[i - 1];
                  }
                }else{                            /* replace */
                  for(i = x1+1; x2 <= strlen(box->text);){
                    box->text[i++] = box->text[x2++];
                  }
                  x2 = x1;
                }

                                        /* non standard characters */
                if(keysym == XK_underscore) box->text[x1] = '_';
                else
                if(keysym == XK_asciitilde) box->text[x1] = '~';
                else
                box->text[x1] = keys_return[0];

                x1++;
                x2++;

                                        /* fit text into window */
                while(XTextWidth(menufont, &box->text[start], x2 - start) + 2 >
                  box->w - 8) start++;

                update_text_box(wdw, win, box);
                break;
            }                     /* end case default */
          break;
        }
      break;                                      /* end case keypress */

      case ButtonPress:
        x = report->xbutton.x;

        if(report->xbutton.time - time_ < 300){    /* double click */
          static int len;

          len = strlen(box->text);
          selectword = 1;                        /* set center of selection */
          center = x1 = x2 = start +
            charof(&box->text[start], len - start, x - box->x - 2);
                                                 /* set word of selection */
          while(isalnum(box->text[x1]) && x1 > 0) x1--;
          while(isalnum(box->text[x2]) && x2 < len) x2++;

          wordx1 = x1;
          wordx2 = x2;                  /* set original word selected */
        }else{                                 /* set center of selection */
          center = x1 = x2 = start +
            charof(&box->text[start], strlen(&box->text[start]), x - box->x - 2);
          selecttext = 1; /* select on */
        }

        time_ = report->xbutton.time;
        update_text_box(wdw, win, box);
        break;

      case ButtonRelease:
        selecttext = 0;
        selectword = 0;
        break;

      case MotionNotify:
        x = report->xmotion.x;
        x1_ = x1;
        x2_ = x2;

        if(selecttext || selectword){       /* decide which side to extend */
          static int center_;

          center_ = start +
            charof(&box->text[start], strlen(&box->text[start]), x - box->x - 2);

          if(center_ < center){

            if(selectword){
               while(center_ > 0 && isalnum(box->text[center_])) center_--;

              x1 = center_;
              x2 = wordx2;        /* reset right side */
            }else{
              x1 = center_;       /* extend left side */
              x2 = center;        /* reset right side */
            }
          }else
          if(center_ > center){
            if(selectword){        /* extend right word */
              static int len;

              len = strlen(box->text);
              while(center_ < len && isalnum(box->text[center_])) center_++;

              x2 = center_;
              x1 = wordx1;
            }else{
              x2 = center_;        /* extend right side */
              x1 = center;         /* reset right side */
            }
          }
        }

        if(x1_ != x1 || x2_ != x2) update_text_box(wdw, win, box);
        x1_ = x1; x2_ = x2;
        break;

      default:
        break;
    }                           /* end switch report type */
    if(selecttext || selectword) XNextEvent(display, report);
  }while(selecttext || selectword);      /* handle selecttext events */
}
