#include "undo.h"
#include "interfac.h"
#include "graphics.h"

static int autostweeked = 0;    /* if last undo update was a tweek */
static int editstweeked = 0;
static int consoletweeked = 0;
static int currentundo = 0;   /* # of pointer to the undo of the next edit */
static int totalundos = 0;    /* total number of undos in the stack */
static char *undostack[MAXUNDOS];  /* pointers to undofiles in RAM */

typedef struct {
  long selectionstart;
  long selectionend;
  char changesmade;
  long undofilelength;
} bcundobuffer;

bcundobuffer *undobuffer[MAXUNDOS];  /* environment information */


int pushundo(){
	FILE *in;
	long filelength;
	int i;
	
	if(currentundo == MAXUNDOS){
		free(undobuffer[0]);  /* delete oldest undo */
		free(undostack[0]);

		for(i = 1; i < currentundo; i++){ /* shift all undos back */
			undobuffer[i-1] = undobuffer[i];
			undostack[i-1] = undostack[i];
		}
		currentundo--;
		totalundos--;
	}

	if(currentundo < totalundos){ /* delete future undos */
		for(i = currentundo + 1; i <= totalundos; i++){
			free(undobuffer[i]);
			free(undostack[i]);
		}
		totalundos = currentundo;
	}
	
	undobuffer[currentundo] = malloc(sizeof(bcundobuffer));
  undobuffer[currentundo]->selectionstart = selectionstart;
  undobuffer[currentundo]->selectionend = selectionend;
  undobuffer[currentundo]->changesmade = changesmade;

  if(!(in = fopen(UNDOFILENAME, "rb"))){
  	printf("Can't open undo file for archiving.\n");
  	return 1;
  }
  fseek(in, 0, SEEK_END);
  filelength = ftell(in);
  fseek(in, 0, SEEK_SET);
  undobuffer[currentundo]->undofilelength = filelength;
  undostack[currentundo] = malloc(filelength+1);
  fread(undostack[currentundo], filelength, 1, in);
  fclose(in);
  currentundo++;
  totalundos++;
  
  return 0;
}

int pullundo(){
	FILE *out;
	
	if(currentundo == 0) return 1;     /* no more undos to load */
	if(currentundo == totalundos){
		saveasedits(UNDOFILENAME); /* save current status to undo undo*/
		pushundo(UNDO);
		currentundo--;
	}
	currentundo--;  /* go back to previous status */

	if(!(out = fopen(UNDOFILENAME, "wb"))){
  	printf("Can't open undo file for retrieving.\n");
  	return 1;
  }
  fwrite(undostack[currentundo], undobuffer[currentundo]->undofilelength, 1, out);
	fflush(out);
  fclose(out);
  
  return 0;
}

int redo(){
	FILE *out;

	if(totalundos == 0 || currentundo >= totalundos - 1) return 1;

	currentundo++;
/* printf("%d %d\n", currentundo, totalundos); */
	
  selectionstart = undobuffer[currentundo]->selectionstart;
  selectionend = undobuffer[currentundo]->selectionend;
  changesmade = undobuffer[currentundo]->changesmade;

	if(!(out = fopen(UNDOFILENAME, "wb"))){
  	printf("Can't open undo file for retrieving.\n");
  	return 1;
  }
  fwrite(undostack[currentundo], undobuffer[currentundo]->undofilelength, 1, out);
	fflush(out);
  fclose(out);
  

	loadedits(UNDOFILENAME, UNDO);
  autostweeked = editstweeked = consoletweeked = 0;    /* update undo on next tweek */
  drawwave(0, 0, wavewindow.w, wavewindow.h, startx, starty, NO_BLANK, FLASH,
    DRAWCURSOR);
  drawpatches(starty);
  drawlabels(startx);
  fittime();
  drawtime(startx);
  drawplaycontrols();
  drawconsolecontrols();
  updatexscrollbar(wavscroll.xpixmap, wavscroll.xwin, startx,
    wavewindow.w * xscale, longesttrack(), &wavscroll.x);
  updateyscrollbar(wavscroll.ypixmap, wavscroll.ywin, starty,
    wavewindow.h / yscaletrack, tracks(), &wavscroll.y);
    
  return 0;
}

int undo(){
	if(totalundos == 0) return 1;

  if(pullundo()) return 1;
/* printf("%d %d\n", currentundo, totalundos); */

  autostweeked = editstweeked = consoletweeked = 0;    /* update undo on next tweek */

  selectionstart = undobuffer[currentundo]->selectionstart;
  selectionend = undobuffer[currentundo]->selectionend;
  changesmade = undobuffer[currentundo]->changesmade;

  loadedits(UNDOFILENAME, UNDO);

  drawwave(0, 0, wavewindow.w, wavewindow.h, startx, starty, NO_BLANK, FLASH,
    DRAWCURSOR);
  drawpatches(starty);
  drawlabels(startx);
  fittime();
  drawtime(startx);
  drawplaycontrols();
  drawconsolecontrols();
  updatexscrollbar(wavscroll.xpixmap, wavscroll.xwin, startx,
    wavewindow.w * xscale, longesttrack(), &wavscroll.x);
  updateyscrollbar(wavscroll.ypixmap, wavscroll.ywin, starty,
    wavewindow.h / yscaletrack, tracks(), &wavscroll.y);
  return 0;
}

int updateundo(char command){
	if((command != TWEEKAUTOS &&
		command != TWEEKEDITS &&
		command != TWEEKCONSOLE) || !changesmade)
	{
		autostweeked = consoletweeked = editstweeked = 0;
	}  /* update undo on next tweek */

  switch(command){
/* tweeks */
  	case TWEEKAUTOS:
  		/* if(!autostweeked){ */
	  		autostweeked = 1;  /* don't update undo on next autostweek */
	  		editstweeked = consoletweeked = 0;  /* update undo if others tweeked */
	  		saveasedits(UNDOFILENAME);
	  		pushundo(command);
  		/* } */
  		break;
  	case TWEEKCONSOLE:
  		if(!consoletweeked){
	  		consoletweeked = 1;
	  		editstweeked = autostweeked = 0;
	  		saveasedits(UNDOFILENAME);
	  		pushundo(command);
  		}
  		break;
  	case TWEEKEDITS:
  		/* if(!editstweeked){ */
	  		editstweeked = 1;
	  		consoletweeked = autostweeked = 0;
	  		saveasedits(UNDOFILENAME);
	  		pushundo(command);
  		/* } */
  		break;
  		
	/* edits */
    default:
      saveasedits(UNDOFILENAME);
      pushundo(command);
      break;
  }
  
  return 0;
}
